/********************************************************************************
** Form generated from reading UI file 'csdtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CSDTAB_H
#define UI_CSDTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorpalette.h"
#include "curveplacement.h"
#include "fftoptions.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_CSDTab
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *TextLabel1_6_2_4;
    Kst::VectorSelector *_vector;
    QLabel *textLabel1;
    QSpinBox *_windowSize;
    Kst::FFTOptions *_FFTOptions;
    QGroupBox *_imageOptionsGroup;
    QGridLayout *gridLayout1;
    Kst::ColorPalette *_colorPalette;
    Kst::CurvePlacement *_curvePlacement;

    void setupUi(QWidget *CSDTab)
    {
        if (CSDTab->objectName().isEmpty())
            CSDTab->setObjectName("CSDTab");
        CSDTab->resize(549, 590);
        verticalLayout = new QVBoxLayout(CSDTab);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(CSDTab);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        TextLabel1_6_2_4 = new QLabel(groupBox);
        TextLabel1_6_2_4->setObjectName("TextLabel1_6_2_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel1_6_2_4->sizePolicy().hasHeightForWidth());
        TextLabel1_6_2_4->setSizePolicy(sizePolicy);
        TextLabel1_6_2_4->setWordWrap(false);

        gridLayout->addWidget(TextLabel1_6_2_4, 0, 0, 1, 1);

        _vector = new Kst::VectorSelector(groupBox);
        _vector->setObjectName("_vector");

        gridLayout->addWidget(_vector, 0, 1, 1, 1);

        textLabel1 = new QLabel(groupBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        _windowSize = new QSpinBox(groupBox);
        _windowSize->setObjectName("_windowSize");
        _windowSize->setMinimum(1);
        _windowSize->setMaximum(999999999);
        _windowSize->setValue(500);

        gridLayout->addWidget(_windowSize, 1, 1, 1, 1);


        verticalLayout->addWidget(groupBox);

        _FFTOptions = new Kst::FFTOptions(CSDTab);
        _FFTOptions->setObjectName("_FFTOptions");

        verticalLayout->addWidget(_FFTOptions);

        _imageOptionsGroup = new QGroupBox(CSDTab);
        _imageOptionsGroup->setObjectName("_imageOptionsGroup");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_imageOptionsGroup->sizePolicy().hasHeightForWidth());
        _imageOptionsGroup->setSizePolicy(sizePolicy1);
        gridLayout1 = new QGridLayout(_imageOptionsGroup);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        _colorPalette = new Kst::ColorPalette(_imageOptionsGroup);
        _colorPalette->setObjectName("_colorPalette");

        gridLayout1->addWidget(_colorPalette, 0, 0, 1, 1);


        verticalLayout->addWidget(_imageOptionsGroup);

        _curvePlacement = new Kst::CurvePlacement(CSDTab);
        _curvePlacement->setObjectName("_curvePlacement");

        verticalLayout->addWidget(_curvePlacement);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(_windowSize);
#endif // QT_CONFIG(shortcut)

        retranslateUi(CSDTab);

        QMetaObject::connectSlotsByName(CSDTab);
    } // setupUi

    void retranslateUi(QWidget *CSDTab)
    {
        groupBox->setTitle(QCoreApplication::translate("CSDTab", "Spectrogram Contents", nullptr));
        TextLabel1_6_2_4->setText(QCoreApplication::translate("CSDTab", "Data vector:", nullptr));
        textLabel1->setText(QCoreApplication::translate("CSDTab", "Window size:", nullptr));
        _imageOptionsGroup->setTitle(QCoreApplication::translate("CSDTab", "Image Options", nullptr));
        (void)CSDTab;
    } // retranslateUi

};

namespace Ui {
    class CSDTab: public Ui_CSDTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CSDTAB_H
