/********************************************************************************
** Form generated from reading UI file 'stringtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STRINGTAB_H
#define UI_STRINGTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "combobox.h"
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_StringTab
{
public:
    QGridLayout *gridLayout;
    QGroupBox *_dataStringGroup;
    QGridLayout *gridLayout_2;
    QLabel *_fileNameLabel;
    QHBoxLayout *_3;
    QSpacerItem *spacerItem;
    QPushButton *_connect;
    QPushButton *_configure;
    QLabel *label_5;
    QLabel *_frameLabel;
    QSpinBox *_frame;
    QCheckBox *_lastFrame;
    Kst::DataSourceSelector *_fileName;
    Kst::ComboBox *_field;
    QGroupBox *_generatedStringGroup;
    QGridLayout *_4;
    QHBoxLayout *_5;
    QLabel *label_3;
    QLineEdit *_stringValue;

    void setupUi(QWidget *StringTab)
    {
        if (StringTab->objectName().isEmpty())
            StringTab->setObjectName("StringTab");
        StringTab->resize(387, 270);
        gridLayout = new QGridLayout(StringTab);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _dataStringGroup = new QGroupBox(StringTab);
        _dataStringGroup->setObjectName("_dataStringGroup");
        _dataStringGroup->setCheckable(true);
        gridLayout_2 = new QGridLayout(_dataStringGroup);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        _fileNameLabel = new QLabel(_dataStringGroup);
        _fileNameLabel->setObjectName("_fileNameLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_fileNameLabel->sizePolicy().hasHeightForWidth());
        _fileNameLabel->setSizePolicy(sizePolicy);
        _fileNameLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _fileNameLabel->setWordWrap(false);

        gridLayout_2->addWidget(_fileNameLabel, 0, 0, 1, 1);

        _3 = new QHBoxLayout();
        _3->setSpacing(6);
        _3->setObjectName("_3");
        _3->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(21, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _3->addItem(spacerItem);

        _connect = new QPushButton(_dataStringGroup);
        _connect->setObjectName("_connect");
        _connect->setEnabled(false);

        _3->addWidget(_connect);

        _configure = new QPushButton(_dataStringGroup);
        _configure->setObjectName("_configure");
        _configure->setEnabled(false);

        _3->addWidget(_configure);


        gridLayout_2->addLayout(_3, 4, 0, 1, 3);

        label_5 = new QLabel(_dataStringGroup);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_5->setWordWrap(false);

        gridLayout_2->addWidget(label_5, 1, 0, 1, 1);

        _frameLabel = new QLabel(_dataStringGroup);
        _frameLabel->setObjectName("_frameLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_frameLabel->sizePolicy().hasHeightForWidth());
        _frameLabel->setSizePolicy(sizePolicy1);
        _frameLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_frameLabel, 2, 0, 1, 1);

        _frame = new QSpinBox(_dataStringGroup);
        _frame->setObjectName("_frame");
        _frame->setMaximum(10000000);

        gridLayout_2->addWidget(_frame, 2, 1, 1, 1);

        _lastFrame = new QCheckBox(_dataStringGroup);
        _lastFrame->setObjectName("_lastFrame");

        gridLayout_2->addWidget(_lastFrame, 2, 2, 1, 1);

        _fileName = new Kst::DataSourceSelector(_dataStringGroup);
        _fileName->setObjectName("_fileName");

        gridLayout_2->addWidget(_fileName, 0, 1, 1, 2);

        _field = new Kst::ComboBox(_dataStringGroup);
        _field->setObjectName("_field");
        _field->setEnabled(false);

        gridLayout_2->addWidget(_field, 1, 1, 1, 2);


        gridLayout->addWidget(_dataStringGroup, 1, 0, 1, 2);

        _generatedStringGroup = new QGroupBox(StringTab);
        _generatedStringGroup->setObjectName("_generatedStringGroup");
        _generatedStringGroup->setCheckable(true);
        _generatedStringGroup->setChecked(false);
        _4 = new QGridLayout(_generatedStringGroup);
        _4->setSpacing(6);
        _4->setContentsMargins(11, 11, 11, 11);
        _4->setObjectName("_4");
        _4->setContentsMargins(-1, 0, -1, -1);
        _5 = new QHBoxLayout();
        _5->setSpacing(6);
        _5->setObjectName("_5");
        _5->setContentsMargins(0, 0, 0, 0);
        label_3 = new QLabel(_generatedStringGroup);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setWordWrap(false);

        _5->addWidget(label_3);

        _stringValue = new QLineEdit(_generatedStringGroup);
        _stringValue->setObjectName("_stringValue");

        _5->addWidget(_stringValue);


        _4->addLayout(_5, 0, 0, 1, 2);


        gridLayout->addWidget(_generatedStringGroup, 0, 0, 1, 2);

#if QT_CONFIG(shortcut)
        label_5->setBuddy(_field);
        label_3->setBuddy(_stringValue);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_generatedStringGroup, _stringValue);
        QWidget::setTabOrder(_stringValue, _dataStringGroup);
        QWidget::setTabOrder(_dataStringGroup, _connect);
        QWidget::setTabOrder(_connect, _configure);

        retranslateUi(StringTab);

        QMetaObject::connectSlotsByName(StringTab);
    } // setupUi

    void retranslateUi(QWidget *StringTab)
    {
        _dataStringGroup->setTitle(QCoreApplication::translate("StringTab", "Read fro&m data source", nullptr));
        _fileNameLabel->setText(QCoreApplication::translate("StringTab", "&File name:", nullptr));
        _connect->setText(QCoreApplication::translate("StringTab", "&Connect", nullptr));
        _configure->setText(QCoreApplication::translate("StringTab", "Co&nfigure...", nullptr));
        label_5->setText(QCoreApplication::translate("StringTab", "Fie&ld:", nullptr));
        _frameLabel->setText(QCoreApplication::translate("StringTab", "Frame:", nullptr));
        _lastFrame->setText(QCoreApplication::translate("StringTab", "Last Frame", nullptr));
        _generatedStringGroup->setTitle(QCoreApplication::translate("StringTab", "&Generate", nullptr));
        label_3->setText(QCoreApplication::translate("StringTab", "&Value: ", nullptr));
        _stringValue->setText(QString());
        (void)StringTab;
    } // retranslateUi

};

namespace Ui {
    class StringTab: public Ui_StringTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STRINGTAB_H
