/********************************************************************************
** Form generated from reading UI file 'stringselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STRINGSELECTOR_H
#define UI_STRINGSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "combobox.h"

QT_BEGIN_NAMESPACE

class Ui_StringSelector
{
public:
    QHBoxLayout *horizontalLayout;
    Kst::ComboBox *_string;
    QToolButton *_selectString;
    QToolButton *_newString;
    QToolButton *_editString;

    void setupUi(QWidget *StringSelector)
    {
        if (StringSelector->objectName().isEmpty())
            StringSelector->setObjectName("StringSelector");
        StringSelector->resize(296, 38);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(StringSelector->sizePolicy().hasHeightForWidth());
        StringSelector->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(StringSelector);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        _string = new Kst::ComboBox(StringSelector);
        _string->setObjectName("_string");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_string->sizePolicy().hasHeightForWidth());
        _string->setSizePolicy(sizePolicy1);
        _string->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout->addWidget(_string);

        _selectString = new QToolButton(StringSelector);
        _selectString->setObjectName("_selectString");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_selectString->sizePolicy().hasHeightForWidth());
        _selectString->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(_selectString);

        _newString = new QToolButton(StringSelector);
        _newString->setObjectName("_newString");
        sizePolicy2.setHeightForWidth(_newString->sizePolicy().hasHeightForWidth());
        _newString->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(_newString);

        _editString = new QToolButton(StringSelector);
        _editString->setObjectName("_editString");
        _editString->setEnabled(false);
        sizePolicy2.setHeightForWidth(_editString->sizePolicy().hasHeightForWidth());
        _editString->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(_editString);

        QWidget::setTabOrder(_string, _selectString);
        QWidget::setTabOrder(_selectString, _newString);
        QWidget::setTabOrder(_newString, _editString);

        retranslateUi(StringSelector);

        QMetaObject::connectSlotsByName(StringSelector);
    } // setupUi

    void retranslateUi(QWidget *StringSelector)
    {
#if QT_CONFIG(whatsthis)
        StringSelector->setWhatsThis(QCoreApplication::translate("StringSelector", "Select a string to utilize.  If you wish to create a new string, select the icon to the right.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _selectString->setText(QCoreApplication::translate("StringSelector", "...", nullptr));
#if QT_CONFIG(tooltip)
        _newString->setToolTip(QCoreApplication::translate("StringSelector", "Create a new string.", nullptr));
#endif // QT_CONFIG(tooltip)
        _newString->setText(QString());
#if QT_CONFIG(tooltip)
        _editString->setToolTip(QCoreApplication::translate("StringSelector", "Edit selected string.", nullptr));
#endif // QT_CONFIG(tooltip)
        _editString->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class StringSelector: public Ui_StringSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STRINGSELECTOR_H
