/********************************************************************************
** Form generated from reading UI file 'changedatasampledialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHANGEDATASAMPLEDIALOG_H
#define UI_CHANGEDATASAMPLEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "datarange.h"

QT_BEGIN_NAMESPACE

class Ui_ChangeDataSampleDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QGroupBox *buttonGroup1;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *_vectorList;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *_addAll;
    QPushButton *_add;
    QPushButton *_remove;
    QPushButton *_removeAll;
    QSpacerItem *spacerItem;
    QListWidget *_selectedVectorList;
    Kst::DataRange *_dataRange;
    QDialogButtonBox *_buttonBox;

    void setupUi(QDialog *ChangeDataSampleDialog)
    {
        if (ChangeDataSampleDialog->objectName().isEmpty())
            ChangeDataSampleDialog->setObjectName("ChangeDataSampleDialog");
        verticalLayout_3 = new QVBoxLayout(ChangeDataSampleDialog);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        buttonGroup1 = new QGroupBox(ChangeDataSampleDialog);
        buttonGroup1->setObjectName("buttonGroup1");
        verticalLayout_2 = new QVBoxLayout(buttonGroup1);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _vectorList = new QListWidget(buttonGroup1);
        _vectorList->setObjectName("_vectorList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_vectorList->sizePolicy().hasHeightForWidth());
        _vectorList->setSizePolicy(sizePolicy);
        _vectorList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout_2->addWidget(_vectorList);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        _addAll = new QPushButton(buttonGroup1);
        _addAll->setObjectName("_addAll");

        verticalLayout->addWidget(_addAll);

        _add = new QPushButton(buttonGroup1);
        _add->setObjectName("_add");

        verticalLayout->addWidget(_add);

        _remove = new QPushButton(buttonGroup1);
        _remove->setObjectName("_remove");

        verticalLayout->addWidget(_remove);

        _removeAll = new QPushButton(buttonGroup1);
        _removeAll->setObjectName("_removeAll");

        verticalLayout->addWidget(_removeAll);

        spacerItem = new QSpacerItem(80, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);


        horizontalLayout_2->addLayout(verticalLayout);

        _selectedVectorList = new QListWidget(buttonGroup1);
        _selectedVectorList->setObjectName("_selectedVectorList");
        sizePolicy.setHeightForWidth(_selectedVectorList->sizePolicy().hasHeightForWidth());
        _selectedVectorList->setSizePolicy(sizePolicy);
        _selectedVectorList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout_2->addWidget(_selectedVectorList);


        verticalLayout_2->addLayout(horizontalLayout_2);


        verticalLayout_3->addWidget(buttonGroup1);

        _dataRange = new Kst::DataRange(ChangeDataSampleDialog);
        _dataRange->setObjectName("_dataRange");

        verticalLayout_3->addWidget(_dataRange);

        _buttonBox = new QDialogButtonBox(ChangeDataSampleDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(_buttonBox);


        retranslateUi(ChangeDataSampleDialog);

        QMetaObject::connectSlotsByName(ChangeDataSampleDialog);
    } // setupUi

    void retranslateUi(QDialog *ChangeDataSampleDialog)
    {
        ChangeDataSampleDialog->setWindowTitle(QCoreApplication::translate("ChangeDataSampleDialog", "Change Data Samples", nullptr));
        buttonGroup1->setTitle(QCoreApplication::translate("ChangeDataSampleDialog", "Vector Selection", nullptr));
#if QT_CONFIG(whatsthis)
        _vectorList->setWhatsThis(QCoreApplication::translate("ChangeDataSampleDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        _addAll->setWhatsThis(QCoreApplication::translate("ChangeDataSampleDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _addAll->setText(QCoreApplication::translate("ChangeDataSampleDialog", "Add A&ll", nullptr));
#if QT_CONFIG(whatsthis)
        _add->setWhatsThis(QCoreApplication::translate("ChangeDataSampleDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _add->setText(QCoreApplication::translate("ChangeDataSampleDialog", "&Add", nullptr));
#if QT_CONFIG(whatsthis)
        _remove->setWhatsThis(QCoreApplication::translate("ChangeDataSampleDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _remove->setText(QCoreApplication::translate("ChangeDataSampleDialog", "&Remove", nullptr));
#if QT_CONFIG(whatsthis)
        _removeAll->setWhatsThis(QCoreApplication::translate("ChangeDataSampleDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _removeAll->setText(QCoreApplication::translate("ChangeDataSampleDialog", "R&emove All", nullptr));
#if QT_CONFIG(whatsthis)
        _selectedVectorList->setWhatsThis(QCoreApplication::translate("ChangeDataSampleDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class ChangeDataSampleDialog: public Ui_ChangeDataSampleDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHANGEDATASAMPLEDIALOG_H
