/****************************************************************************
** Meta object code from reading C++ file 'dialogpage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstapp/dialogpage.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dialogpage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst10DialogPageE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DialogPage::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst10DialogPageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DialogPage",
        "ok",
        "",
        "apply",
        "cancel",
        "modified"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ok'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'apply'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cancel'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modified'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DialogPage, qt_meta_tag_ZN3Kst10DialogPageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DialogPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10DialogPageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10DialogPageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst10DialogPageE_t>.metaTypes,
    nullptr
} };

void Kst::DialogPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DialogPage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ok(); break;
        case 1: _t->apply(); break;
        case 2: _t->cancel(); break;
        case 3: _t->modified(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DialogPage::*)()>(_a, &DialogPage::ok, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DialogPage::*)()>(_a, &DialogPage::apply, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DialogPage::*)()>(_a, &DialogPage::cancel, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DialogPage::*)()>(_a, &DialogPage::modified, 3))
            return;
    }
}

const QMetaObject *Kst::DialogPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DialogPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10DialogPageE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Kst::DialogPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Kst::DialogPage::ok()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Kst::DialogPage::apply()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Kst::DialogPage::cancel()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Kst::DialogPage::modified()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN3Kst13DialogPageTabE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DialogPageTab::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst13DialogPageTabE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DialogPageTab"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DialogPageTab, qt_meta_tag_ZN3Kst13DialogPageTabE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DialogPageTab::staticMetaObject = { {
    QMetaObject::SuperData::link<DialogPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst13DialogPageTabE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst13DialogPageTabE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst13DialogPageTabE_t>.metaTypes,
    nullptr
} };

void Kst::DialogPageTab::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DialogPageTab *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Kst::DialogPageTab::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DialogPageTab::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst13DialogPageTabE_t>.strings))
        return static_cast<void*>(this);
    return DialogPage::qt_metacast(_clname);
}

int Kst::DialogPageTab::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DialogPage::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
