/********************************************************************************
** Form generated from reading UI file 'movingaverageconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOVINGAVERAGECONFIG_H
#define UI_MOVINGAVERAGECONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_MovingAverageConfig
{
public:
    QGridLayout *gridLayout;
    Kst::VectorSelector *_vector;
    QLabel *label_4;
    Kst::ScalarSelector *_scalarSamples;
    QLabel *label_3;
    QCheckBox *_weighted;

    void setupUi(QWidget *MovingAverageConfig)
    {
        if (MovingAverageConfig->objectName().isEmpty())
            MovingAverageConfig->setObjectName("MovingAverageConfig");
        MovingAverageConfig->resize(488, 92);
        gridLayout = new QGridLayout(MovingAverageConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _vector = new Kst::VectorSelector(MovingAverageConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_vector, 0, 1, 1, 4);

        label_4 = new QLabel(MovingAverageConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        _scalarSamples = new Kst::ScalarSelector(MovingAverageConfig);
        _scalarSamples->setObjectName("_scalarSamples");
        sizePolicy.setHeightForWidth(_scalarSamples->sizePolicy().hasHeightForWidth());
        _scalarSamples->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_scalarSamples, 1, 1, 1, 3);

        label_3 = new QLabel(MovingAverageConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _weighted = new QCheckBox(MovingAverageConfig);
        _weighted->setObjectName("_weighted");
        sizePolicy1.setHeightForWidth(_weighted->sizePolicy().hasHeightForWidth());
        _weighted->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_weighted, 1, 4, 1, 1);


        retranslateUi(MovingAverageConfig);

        QMetaObject::connectSlotsByName(MovingAverageConfig);
    } // setupUi

    void retranslateUi(QWidget *MovingAverageConfig)
    {
        label_4->setText(QCoreApplication::translate("MovingAverageConfig", "Samples: ", nullptr));
#if QT_CONFIG(tooltip)
        _scalarSamples->setToolTip(QCoreApplication::translate("MovingAverageConfig", "The length of the averaging filter.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("MovingAverageConfig", "Y: ", nullptr));
        _weighted->setText(QCoreApplication::translate("MovingAverageConfig", "Weighted", nullptr));
        (void)MovingAverageConfig;
    } // retranslateUi

};

namespace Ui {
    class MovingAverageConfig: public Ui_MovingAverageConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOVINGAVERAGECONFIG_H
