/********************************************************************************
** Form generated from reading UI file 'datamanager.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATAMANAGER_H
#define UI_DATAMANAGER_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DataManager
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *_filterColumn;
    QLineEdit *_filterText;
    QCheckBox *_caseSensitive;
    QSplitter *_splitter;
    QTreeView *_session;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *_edit;
    QPushButton *_delete;
    QPushButton *_purge;
    QDialogButtonBox *_buttonBox;

    void setupUi(QDialog *DataManager)
    {
        if (DataManager->objectName().isEmpty())
            DataManager->setObjectName("DataManager");
        DataManager->resize(744, 231);
        verticalLayout = new QVBoxLayout(DataManager);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(DataManager);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        _filterColumn = new QComboBox(DataManager);
        _filterColumn->addItem(QString());
        _filterColumn->addItem(QString());
        _filterColumn->addItem(QString());
        _filterColumn->addItem(QString());
        _filterColumn->addItem(QString());
        _filterColumn->setObjectName("_filterColumn");

        horizontalLayout->addWidget(_filterColumn);

        _filterText = new QLineEdit(DataManager);
        _filterText->setObjectName("_filterText");
        _filterText->setFocusPolicy(Qt::ClickFocus);

        horizontalLayout->addWidget(_filterText);

        _caseSensitive = new QCheckBox(DataManager);
        _caseSensitive->setObjectName("_caseSensitive");

        horizontalLayout->addWidget(_caseSensitive);


        verticalLayout->addLayout(horizontalLayout);

        _splitter = new QSplitter(DataManager);
        _splitter->setObjectName("_splitter");
        _splitter->setLayoutDirection(Qt::LeftToRight);
        _splitter->setOrientation(Qt::Horizontal);
        _splitter->setChildrenCollapsible(false);
        _session = new QTreeView(_splitter);
        _session->setObjectName("_session");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_session->sizePolicy().hasHeightForWidth());
        _session->setSizePolicy(sizePolicy);
        _session->setAlternatingRowColors(true);
        _splitter->addWidget(_session);

        verticalLayout->addWidget(_splitter);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(601, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        _edit = new QPushButton(DataManager);
        _edit->setObjectName("_edit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_edit->sizePolicy().hasHeightForWidth());
        _edit->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(_edit);

        _delete = new QPushButton(DataManager);
        _delete->setObjectName("_delete");
        sizePolicy1.setHeightForWidth(_delete->sizePolicy().hasHeightForWidth());
        _delete->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(_delete);

        _purge = new QPushButton(DataManager);
        _purge->setObjectName("_purge");
        sizePolicy1.setHeightForWidth(_purge->sizePolicy().hasHeightForWidth());
        _purge->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(_purge);

        _buttonBox = new QDialogButtonBox(DataManager);
        _buttonBox->setObjectName("_buttonBox");
        sizePolicy1.setHeightForWidth(_buttonBox->sizePolicy().hasHeightForWidth());
        _buttonBox->setSizePolicy(sizePolicy1);
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_2->addWidget(_buttonBox);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(DataManager);
        QObject::connect(_buttonBox, &QDialogButtonBox::accepted, DataManager, qOverload<>(&QDialog::accept));
        QObject::connect(_buttonBox, &QDialogButtonBox::rejected, DataManager, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DataManager);
    } // setupUi

    void retranslateUi(QDialog *DataManager)
    {
        DataManager->setWindowTitle(QCoreApplication::translate("DataManager", "Data Manager", nullptr));
        label->setText(QCoreApplication::translate("DataManager", "Filter on:", nullptr));
        _filterColumn->setItemText(0, QCoreApplication::translate("DataManager", "All Columns", nullptr));
        _filterColumn->setItemText(1, QCoreApplication::translate("DataManager", "Name", "the name of the item (noun)"));
        _filterColumn->setItemText(2, QCoreApplication::translate("DataManager", "Type", "the type of the item (noun)"));
        _filterColumn->setItemText(3, QCoreApplication::translate("DataManager", "Samples", nullptr));
        _filterColumn->setItemText(4, QCoreApplication::translate("DataManager", "Properties", nullptr));

        _caseSensitive->setText(QCoreApplication::translate("DataManager", "Case sensitive", nullptr));
        _edit->setText(QCoreApplication::translate("DataManager", "&Edit", nullptr));
        _delete->setText(QCoreApplication::translate("DataManager", "Delete", nullptr));
        _purge->setText(QCoreApplication::translate("DataManager", "Purge", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DataManager: public Ui_DataManager {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATAMANAGER_H
