/********************************************************************************
** Form generated from reading UI file 'legendtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LEGENDTAB_H
#define UI_LEGENDTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_LegendTab
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label;
    QLineEdit *_title;
    QCheckBox *_autoContents;
    QCheckBox *_displayVertically;
    QGroupBox *_contentGroupBox;
    QGridLayout *gridLayout1;
    QLabel *TextLabel4_4_2;
    QSpacerItem *spacerItem;
    QLabel *TextLabel3_3_2;
    QListWidget *_availableRelationList;
    QListWidget *_displayedRelationList;
    QPushButton *_up;
    QPushButton *_remove;
    QPushButton *_add;
    QPushButton *_down;
    QSpacerItem *spacerItem1;
    QGroupBox *_topLabelGroup;
    QGridLayout *gridLayout_1;
    QLabel *_Label_11;
    QDoubleSpinBox *_fontSize;
    QLabel *label_2;
    QPushButton *_bold;
    QPushButton *_italic;
    Kst::ColorButton *_labelColor;
    QLabel *_Label_12;
    QFontComboBox *_family;

    void setupUi(QWidget *LegendTab)
    {
        if (LegendTab->objectName().isEmpty())
            LegendTab->setObjectName("LegendTab");
        LegendTab->resize(423, 363);
        gridLayout = new QGridLayout(LegendTab);
        gridLayout->setObjectName("gridLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        label = new QLabel(LegendTab);
        label->setObjectName("label");

        hboxLayout->addWidget(label);

        _title = new QLineEdit(LegendTab);
        _title->setObjectName("_title");

        hboxLayout->addWidget(_title);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 2);

        _autoContents = new QCheckBox(LegendTab);
        _autoContents->setObjectName("_autoContents");
        _autoContents->setChecked(true);
        _autoContents->setTristate(false);

        gridLayout->addWidget(_autoContents, 1, 0, 1, 1);

        _displayVertically = new QCheckBox(LegendTab);
        _displayVertically->setObjectName("_displayVertically");

        gridLayout->addWidget(_displayVertically, 1, 1, 1, 1);

        _contentGroupBox = new QGroupBox(LegendTab);
        _contentGroupBox->setObjectName("_contentGroupBox");
        _contentGroupBox->setEnabled(false);
        gridLayout1 = new QGridLayout(_contentGroupBox);
        gridLayout1->setObjectName("gridLayout1");
        TextLabel4_4_2 = new QLabel(_contentGroupBox);
        TextLabel4_4_2->setObjectName("TextLabel4_4_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel4_4_2->sizePolicy().hasHeightForWidth());
        TextLabel4_4_2->setSizePolicy(sizePolicy);
        TextLabel4_4_2->setWordWrap(false);

        gridLayout1->addWidget(TextLabel4_4_2, 0, 0, 1, 1);

        spacerItem = new QSpacerItem(128, 71, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout1->addItem(spacerItem, 0, 1, 2, 3);

        TextLabel3_3_2 = new QLabel(_contentGroupBox);
        TextLabel3_3_2->setObjectName("TextLabel3_3_2");
        sizePolicy.setHeightForWidth(TextLabel3_3_2->sizePolicy().hasHeightForWidth());
        TextLabel3_3_2->setSizePolicy(sizePolicy);
        TextLabel3_3_2->setWordWrap(false);

        gridLayout1->addWidget(TextLabel3_3_2, 0, 4, 1, 1);

        _availableRelationList = new QListWidget(_contentGroupBox);
        _availableRelationList->setObjectName("_availableRelationList");

        gridLayout1->addWidget(_availableRelationList, 1, 0, 5, 1);

        _displayedRelationList = new QListWidget(_contentGroupBox);
        _displayedRelationList->setObjectName("_displayedRelationList");

        gridLayout1->addWidget(_displayedRelationList, 1, 4, 5, 1);

        _up = new QPushButton(_contentGroupBox);
        _up->setObjectName("_up");
        _up->setEnabled(false);
        sizePolicy.setHeightForWidth(_up->sizePolicy().hasHeightForWidth());
        _up->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_up, 2, 2, 1, 1);

        _remove = new QPushButton(_contentGroupBox);
        _remove->setObjectName("_remove");
        _remove->setEnabled(false);
        sizePolicy.setHeightForWidth(_remove->sizePolicy().hasHeightForWidth());
        _remove->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_remove, 3, 1, 1, 1);

        _add = new QPushButton(_contentGroupBox);
        _add->setObjectName("_add");
        _add->setEnabled(false);
        sizePolicy.setHeightForWidth(_add->sizePolicy().hasHeightForWidth());
        _add->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_add, 3, 3, 1, 1);

        _down = new QPushButton(_contentGroupBox);
        _down->setObjectName("_down");
        _down->setEnabled(false);
        sizePolicy.setHeightForWidth(_down->sizePolicy().hasHeightForWidth());
        _down->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_down, 4, 2, 1, 1);

        spacerItem1 = new QSpacerItem(128, 61, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout1->addItem(spacerItem1, 5, 1, 1, 3);


        gridLayout->addWidget(_contentGroupBox, 2, 0, 1, 2);

        _topLabelGroup = new QGroupBox(LegendTab);
        _topLabelGroup->setObjectName("_topLabelGroup");
        gridLayout_1 = new QGridLayout(_topLabelGroup);
        gridLayout_1->setObjectName("gridLayout_1");
        _Label_11 = new QLabel(_topLabelGroup);
        _Label_11->setObjectName("_Label_11");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_Label_11->sizePolicy().hasHeightForWidth());
        _Label_11->setSizePolicy(sizePolicy1);
        _Label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_1->addWidget(_Label_11, 0, 0, 1, 1);

        _fontSize = new QDoubleSpinBox(_topLabelGroup);
        _fontSize->setObjectName("_fontSize");
        _fontSize->setDecimals(1);
        _fontSize->setMinimum(-25.000000000000000);
        _fontSize->setSingleStep(1.000000000000000);

        gridLayout_1->addWidget(_fontSize, 0, 1, 1, 1);

        label_2 = new QLabel(_topLabelGroup);
        label_2->setObjectName("label_2");
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        gridLayout_1->addWidget(label_2, 0, 2, 1, 1);

        _bold = new QPushButton(_topLabelGroup);
        _bold->setObjectName("_bold");
        _bold->setCheckable(true);

        gridLayout_1->addWidget(_bold, 0, 3, 1, 1);

        _italic = new QPushButton(_topLabelGroup);
        _italic->setObjectName("_italic");
        _italic->setCheckable(true);

        gridLayout_1->addWidget(_italic, 0, 4, 1, 1);

        _labelColor = new Kst::ColorButton(_topLabelGroup);
        _labelColor->setObjectName("_labelColor");

        gridLayout_1->addWidget(_labelColor, 0, 5, 1, 1);

        _Label_12 = new QLabel(_topLabelGroup);
        _Label_12->setObjectName("_Label_12");
        _Label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_1->addWidget(_Label_12, 1, 0, 1, 1);

        _family = new QFontComboBox(_topLabelGroup);
        _family->setObjectName("_family");

        gridLayout_1->addWidget(_family, 1, 1, 1, 5);


        gridLayout->addWidget(_topLabelGroup, 3, 0, 1, 2);

#if QT_CONFIG(shortcut)
        TextLabel4_4_2->setBuddy(_availableRelationList);
        TextLabel3_3_2->setBuddy(_displayedRelationList);
        _Label_11->setBuddy(_fontSize);
        _Label_12->setBuddy(_family);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_title, _autoContents);
        QWidget::setTabOrder(_autoContents, _displayVertically);
        QWidget::setTabOrder(_displayVertically, _availableRelationList);
        QWidget::setTabOrder(_availableRelationList, _displayedRelationList);
        QWidget::setTabOrder(_displayedRelationList, _up);
        QWidget::setTabOrder(_up, _remove);
        QWidget::setTabOrder(_remove, _add);
        QWidget::setTabOrder(_add, _down);
        QWidget::setTabOrder(_down, _fontSize);
        QWidget::setTabOrder(_fontSize, _bold);
        QWidget::setTabOrder(_bold, _italic);
        QWidget::setTabOrder(_italic, _labelColor);
        QWidget::setTabOrder(_labelColor, _family);

        retranslateUi(LegendTab);

        QMetaObject::connectSlotsByName(LegendTab);
    } // setupUi

    void retranslateUi(QWidget *LegendTab)
    {
        label->setText(QCoreApplication::translate("LegendTab", "Title:", nullptr));
        _autoContents->setText(QCoreApplication::translate("LegendTab", "Use automatic plot-based contents", nullptr));
        _displayVertically->setText(QCoreApplication::translate("LegendTab", "Display &vertically", nullptr));
        TextLabel4_4_2->setText(QCoreApplication::translate("LegendTab", "A&vailable objects:", nullptr));
        TextLabel3_3_2->setText(QCoreApplication::translate("LegendTab", "Displayed obje&cts:", nullptr));
#if QT_CONFIG(tooltip)
        _up->setToolTip(QCoreApplication::translate("LegendTab", "Raise in plot order: Alt+up", nullptr));
#endif // QT_CONFIG(tooltip)
        _up->setText(QString());
#if QT_CONFIG(shortcut)
        _up->setShortcut(QCoreApplication::translate("LegendTab", "Alt+Up", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        _remove->setToolTip(QCoreApplication::translate("LegendTab", "Remove: Alt+r", nullptr));
#endif // QT_CONFIG(tooltip)
        _remove->setText(QString());
#if QT_CONFIG(tooltip)
        _add->setToolTip(QCoreApplication::translate("LegendTab", "Select: Alt+s", nullptr));
#endif // QT_CONFIG(tooltip)
        _add->setText(QString());
#if QT_CONFIG(tooltip)
        _down->setToolTip(QCoreApplication::translate("LegendTab", "Lower in plot order: Alt+down", nullptr));
#endif // QT_CONFIG(tooltip)
        _down->setText(QString());
#if QT_CONFIG(shortcut)
        _down->setShortcut(QCoreApplication::translate("LegendTab", "Alt+Down", nullptr));
#endif // QT_CONFIG(shortcut)
        _topLabelGroup->setTitle(QCoreApplication::translate("LegendTab", "Legend Font Settings", nullptr));
        _Label_11->setText(QCoreApplication::translate("LegendTab", "Font si&ze:", nullptr));
        label_2->setText(QCoreApplication::translate("LegendTab", " points", nullptr));
#if QT_CONFIG(tooltip)
        _bold->setToolTip(QCoreApplication::translate("LegendTab", "Bold", "Bold Font"));
#endif // QT_CONFIG(tooltip)
        _bold->setText(QString());
#if QT_CONFIG(tooltip)
        _italic->setToolTip(QCoreApplication::translate("LegendTab", "Italic", "Italic font"));
#endif // QT_CONFIG(tooltip)
        _italic->setText(QString());
        _Label_12->setText(QCoreApplication::translate("LegendTab", "Fo&nt family", nullptr));
        (void)LegendTab;
    } // retranslateUi

};

namespace Ui {
    class LegendTab: public Ui_LegendTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LEGENDTAB_H
