/********************************************************************************
** Form generated from reading UI file 'crossspectrumconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CROSSSPECTRUMCONFIG_H
#define UI_CROSSSPECTRUMCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_CrossSpectrumConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vectorOne;
    QLabel *label_5;
    Kst::VectorSelector *_vectorTwo;
    QLabel *label_4;
    Kst::ScalarSelector *_scalarFFT;
    QLabel *label_6;
    Kst::ScalarSelector *_scalarRate;

    void setupUi(QWidget *CrossSpectrumConfig)
    {
        if (CrossSpectrumConfig->objectName().isEmpty())
            CrossSpectrumConfig->setObjectName("CrossSpectrumConfig");
        CrossSpectrumConfig->resize(425, 104);
        CrossSpectrumConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(CrossSpectrumConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(CrossSpectrumConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _vectorOne = new Kst::VectorSelector(CrossSpectrumConfig);
        _vectorOne->setObjectName("_vectorOne");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorOne->sizePolicy().hasHeightForWidth());
        _vectorOne->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorOne, 0, 1, 1, 1);

        label_5 = new QLabel(CrossSpectrumConfig);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy2);
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_5->setWordWrap(false);

        gridLayout->addWidget(label_5, 1, 0, 1, 1);

        _vectorTwo = new Kst::VectorSelector(CrossSpectrumConfig);
        _vectorTwo->setObjectName("_vectorTwo");
        sizePolicy1.setHeightForWidth(_vectorTwo->sizePolicy().hasHeightForWidth());
        _vectorTwo->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorTwo, 1, 1, 1, 1);

        label_4 = new QLabel(CrossSpectrumConfig);
        label_4->setObjectName("label_4");
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 2, 0, 1, 1);

        _scalarFFT = new Kst::ScalarSelector(CrossSpectrumConfig);
        _scalarFFT->setObjectName("_scalarFFT");

        gridLayout->addWidget(_scalarFFT, 2, 1, 1, 1);

        label_6 = new QLabel(CrossSpectrumConfig);
        label_6->setObjectName("label_6");
        sizePolicy2.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy2);
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_6->setWordWrap(false);

        gridLayout->addWidget(label_6, 3, 0, 1, 1);

        _scalarRate = new Kst::ScalarSelector(CrossSpectrumConfig);
        _scalarRate->setObjectName("_scalarRate");

        gridLayout->addWidget(_scalarRate, 3, 1, 1, 1);


        retranslateUi(CrossSpectrumConfig);

        QMetaObject::connectSlotsByName(CrossSpectrumConfig);
    } // setupUi

    void retranslateUi(QWidget *CrossSpectrumConfig)
    {
        label_3->setText(QCoreApplication::translate("CrossSpectrumConfig", "Input Vector One", nullptr));
        label_5->setText(QCoreApplication::translate("CrossSpectrumConfig", "Input Vector Two", nullptr));
        label_4->setText(QCoreApplication::translate("CrossSpectrumConfig", "FFT Length = 2^", nullptr));
        label_6->setText(QCoreApplication::translate("CrossSpectrumConfig", "Sample Rate", nullptr));
        (void)CrossSpectrumConfig;
    } // retranslateUi

};

namespace Ui {
    class CrossSpectrumConfig: public Ui_CrossSpectrumConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CROSSSPECTRUMCONFIG_H
