/********************************************************************************
** Form generated from reading UI file 'fitgradient_weightedconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FITGRADIENT_WEIGHTEDCONFIG_H
#define UI_FITGRADIENT_WEIGHTEDCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FitGradient_WeightedConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_4;
    Kst::VectorSelector *_vectorX;
    QLabel *label_3;
    Kst::VectorSelector *_vectorY;
    QLabel *label_5;
    Kst::VectorSelector *_vectorWeights;

    void setupUi(QWidget *FitGradient_WeightedConfig)
    {
        if (FitGradient_WeightedConfig->objectName().isEmpty())
            FitGradient_WeightedConfig->setObjectName("FitGradient_WeightedConfig");
        FitGradient_WeightedConfig->resize(420, 99);
        FitGradient_WeightedConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(FitGradient_WeightedConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(FitGradient_WeightedConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        _vectorX = new Kst::VectorSelector(FitGradient_WeightedConfig);
        _vectorX->setObjectName("_vectorX");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorX->sizePolicy().hasHeightForWidth());
        _vectorX->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorX, 0, 1, 1, 1);

        label_3 = new QLabel(FitGradient_WeightedConfig);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        _vectorY = new Kst::VectorSelector(FitGradient_WeightedConfig);
        _vectorY->setObjectName("_vectorY");
        sizePolicy1.setHeightForWidth(_vectorY->sizePolicy().hasHeightForWidth());
        _vectorY->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorY, 1, 1, 1, 1);

        label_5 = new QLabel(FitGradient_WeightedConfig);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy2);
        label_5->setWordWrap(false);

        gridLayout->addWidget(label_5, 2, 0, 1, 1);

        _vectorWeights = new Kst::VectorSelector(FitGradient_WeightedConfig);
        _vectorWeights->setObjectName("_vectorWeights");
        sizePolicy1.setHeightForWidth(_vectorWeights->sizePolicy().hasHeightForWidth());
        _vectorWeights->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorWeights, 2, 1, 1, 1);


        retranslateUi(FitGradient_WeightedConfig);

        QMetaObject::connectSlotsByName(FitGradient_WeightedConfig);
    } // setupUi

    void retranslateUi(QWidget *FitGradient_WeightedConfig)
    {
        label_4->setText(QCoreApplication::translate("FitGradient_WeightedConfig", "X:", "The X axis of a plot"));
        label_3->setText(QCoreApplication::translate("FitGradient_WeightedConfig", "Y:", "The Y axis of a plot"));
        label_5->setText(QCoreApplication::translate("FitGradient_WeightedConfig", "Weights:", "statistical weights"));
        (void)FitGradient_WeightedConfig;
    } // retranslateUi

};

namespace Ui {
    class FitGradient_WeightedConfig: public Ui_FitGradient_WeightedConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FITGRADIENT_WEIGHTEDCONFIG_H
