/********************************************************************************
** Form generated from reading UI file 'cumulativeaverageconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CUMULATIVEAVERAGECONFIG_H
#define UI_CUMULATIVEAVERAGECONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_CumulativeAverageConfig
{
public:
    QGridLayout *gridLayout;
    Kst::VectorSelector *_vector;
    QLabel *label_3;

    void setupUi(QWidget *CumulativeAverageConfig)
    {
        if (CumulativeAverageConfig->objectName().isEmpty())
            CumulativeAverageConfig->setObjectName("CumulativeAverageConfig");
        CumulativeAverageConfig->resize(425, 55);
        CumulativeAverageConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(CumulativeAverageConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _vector = new Kst::VectorSelector(CumulativeAverageConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_vector, 0, 1, 1, 1);

        label_3 = new QLabel(CumulativeAverageConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);


        retranslateUi(CumulativeAverageConfig);

        QMetaObject::connectSlotsByName(CumulativeAverageConfig);
    } // setupUi

    void retranslateUi(QWidget *CumulativeAverageConfig)
    {
        label_3->setText(QCoreApplication::translate("CumulativeAverageConfig", "Y: ", nullptr));
        (void)CumulativeAverageConfig;
    } // retranslateUi

};

namespace Ui {
    class CumulativeAverageConfig: public Ui_CumulativeAverageConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CUMULATIVEAVERAGECONFIG_H
