/********************************************************************************
** Form generated from reading UI file 'datarange.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATARANGE_H
#define UI_DATARANGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DataRange
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBox2;
    QGridLayout *gridLayoutDR;
    QCheckBox *_readToEnd;
    QCheckBox *_countFromEnd;
    QLineEdit *_last;
    QLabel *label_3;
    QCheckBox *_doFilter;
    QLabel *_lastLabel;
    QLineEdit *_range;
    QComboBox *_startUnits;
    QHBoxLayout *horizontalLayout;
    QCheckBox *_doSkip;
    QSpinBox *_skip;
    QLabel *label_2;
    QLabel *_rangeLabel;
    QComboBox *_rangeUnits;
    QLineEdit *_start;
    QLabel *_startLabel;

    void setupUi(QWidget *DataRange)
    {
        if (DataRange->objectName().isEmpty())
            DataRange->setObjectName("DataRange");
        DataRange->resize(597, 138);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DataRange->sizePolicy().hasHeightForWidth());
        DataRange->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(DataRange);
        gridLayout->setSpacing(0);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        groupBox2 = new QGroupBox(DataRange);
        groupBox2->setObjectName("groupBox2");
        gridLayoutDR = new QGridLayout(groupBox2);
        gridLayoutDR->setSpacing(6);
        gridLayoutDR->setContentsMargins(11, 11, 11, 11);
        gridLayoutDR->setObjectName("gridLayoutDR");
        _readToEnd = new QCheckBox(groupBox2);
        _readToEnd->setObjectName("_readToEnd");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_readToEnd->sizePolicy().hasHeightForWidth());
        _readToEnd->setSizePolicy(sizePolicy1);
        _readToEnd->setChecked(true);

        gridLayoutDR->addWidget(_readToEnd, 2, 6, 1, 1);

        _countFromEnd = new QCheckBox(groupBox2);
        _countFromEnd->setObjectName("_countFromEnd");

        gridLayoutDR->addWidget(_countFromEnd, 0, 6, 1, 1);

        _last = new QLineEdit(groupBox2);
        _last->setObjectName("_last");
        _last->setEnabled(false);

        gridLayoutDR->addWidget(_last, 0, 3, 1, 1);

        label_3 = new QLabel(groupBox2);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setWordWrap(false);

        gridLayoutDR->addWidget(label_3, 0, 5, 1, 1);

        _doFilter = new QCheckBox(groupBox2);
        _doFilter->setObjectName("_doFilter");
        _doFilter->setEnabled(false);
        sizePolicy1.setHeightForWidth(_doFilter->sizePolicy().hasHeightForWidth());
        _doFilter->setSizePolicy(sizePolicy1);

        gridLayoutDR->addWidget(_doFilter, 3, 4, 1, 3);

        _lastLabel = new QLabel(groupBox2);
        _lastLabel->setObjectName("_lastLabel");
        _lastLabel->setEnabled(false);

        gridLayoutDR->addWidget(_lastLabel, 0, 2, 1, 1);

        _range = new QLineEdit(groupBox2);
        _range->setObjectName("_range");
        _range->setEnabled(false);

        gridLayoutDR->addWidget(_range, 2, 1, 1, 3);

        _startUnits = new QComboBox(groupBox2);
        _startUnits->setObjectName("_startUnits");

        gridLayoutDR->addWidget(_startUnits, 0, 4, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        _doSkip = new QCheckBox(groupBox2);
        _doSkip->setObjectName("_doSkip");
        sizePolicy1.setHeightForWidth(_doSkip->sizePolicy().hasHeightForWidth());
        _doSkip->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(_doSkip);

        _skip = new QSpinBox(groupBox2);
        _skip->setObjectName("_skip");
        _skip->setEnabled(false);
        _skip->setMinimum(1);
        _skip->setMaximum(2147483647);

        horizontalLayout->addWidget(_skip);


        gridLayoutDR->addLayout(horizontalLayout, 3, 0, 1, 4);

        label_2 = new QLabel(groupBox2);
        label_2->setObjectName("label_2");
        sizePolicy2.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy2);
        label_2->setWordWrap(false);

        gridLayoutDR->addWidget(label_2, 2, 5, 1, 1);

        _rangeLabel = new QLabel(groupBox2);
        _rangeLabel->setObjectName("_rangeLabel");
        _rangeLabel->setEnabled(false);
        sizePolicy2.setHeightForWidth(_rangeLabel->sizePolicy().hasHeightForWidth());
        _rangeLabel->setSizePolicy(sizePolicy2);
        _rangeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _rangeLabel->setWordWrap(false);

        gridLayoutDR->addWidget(_rangeLabel, 2, 0, 1, 1);

        _rangeUnits = new QComboBox(groupBox2);
        _rangeUnits->setObjectName("_rangeUnits");
        _rangeUnits->setEnabled(false);

        gridLayoutDR->addWidget(_rangeUnits, 2, 4, 1, 1);

        _start = new QLineEdit(groupBox2);
        _start->setObjectName("_start");

        gridLayoutDR->addWidget(_start, 0, 1, 1, 1);

        _startLabel = new QLabel(groupBox2);
        _startLabel->setObjectName("_startLabel");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_startLabel->sizePolicy().hasHeightForWidth());
        _startLabel->setSizePolicy(sizePolicy3);
        _startLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _startLabel->setWordWrap(false);

        gridLayoutDR->addWidget(_startLabel, 0, 0, 1, 1);


        gridLayout->addWidget(groupBox2, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        _lastLabel->setBuddy(_last);
        _rangeLabel->setBuddy(_range);
        _startLabel->setBuddy(_start);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_start, _last);
        QWidget::setTabOrder(_last, _startUnits);
        QWidget::setTabOrder(_startUnits, _countFromEnd);
        QWidget::setTabOrder(_countFromEnd, _range);
        QWidget::setTabOrder(_range, _rangeUnits);
        QWidget::setTabOrder(_rangeUnits, _readToEnd);
        QWidget::setTabOrder(_readToEnd, _doSkip);
        QWidget::setTabOrder(_doSkip, _skip);

        retranslateUi(DataRange);

        QMetaObject::connectSlotsByName(DataRange);
    } // setupUi

    void retranslateUi(QWidget *DataRange)
    {
        groupBox2->setTitle(QCoreApplication::translate("DataRange", "Data Range", nullptr));
#if QT_CONFIG(whatsthis)
        _readToEnd->setWhatsThis(QCoreApplication::translate("DataRange", "Select this to read all of the data up to the end of the file.  ", nullptr));
#endif // QT_CONFIG(whatsthis)
        _readToEnd->setText(QCoreApplication::translate("DataRange", "Read to &end", nullptr));
#if QT_CONFIG(whatsthis)
        _countFromEnd->setWhatsThis(QCoreApplication::translate("DataRange", "Select this to read data from the end of the file.  This is useful for scrolling displays when the file is being appended by another application.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _countFromEnd->setText(QCoreApplication::translate("DataRange", "Co&unt from end", nullptr));
        _last->setText(QCoreApplication::translate("DataRange", "0", nullptr));
        label_3->setText(QCoreApplication::translate("DataRange", "or", nullptr));
#if QT_CONFIG(whatsthis)
        _doFilter->setWhatsThis(QCoreApplication::translate("DataRange", "Select this to average the skipped frames together.  Provides simple low pass filtering.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _doFilter->setText(QCoreApplication::translate("DataRange", "Boxcar &filter first", nullptr));
        _lastLabel->setText(QCoreApplication::translate("DataRange", "&Last", nullptr));
#if QT_CONFIG(tooltip)
        _doSkip->setToolTip(QCoreApplication::translate("DataRange", "Save memory by sparsely sampling the data", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _doSkip->setWhatsThis(QCoreApplication::translate("DataRange", "Select this to skip frames, rather than reading all of the data.  This is useful for reading very long but slowly changing data files.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _doSkip->setText(QCoreApplication::translate("DataRange", "Read &1 sample per:", nullptr));
#if QT_CONFIG(whatsthis)
        _skip->setWhatsThis(QCoreApplication::translate("DataRange", "Number of frames to skip, if \"Read 1 sample per\" is selected.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _skip->setSuffix(QCoreApplication::translate("DataRange", " frame(s)", nullptr));
        label_2->setText(QCoreApplication::translate("DataRange", "or", nullptr));
#if QT_CONFIG(whatsthis)
        _rangeLabel->setWhatsThis(QCoreApplication::translate("DataRange", "Select this to average the skipped frames together.  Provides simple low pass filtering.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _rangeLabel->setText(QCoreApplication::translate("DataRange", "Ra&nge:", nullptr));
        _start->setText(QCoreApplication::translate("DataRange", "0", nullptr));
        _startLabel->setText(QCoreApplication::translate("DataRange", "&Start:", nullptr));
        (void)DataRange;
    } // retranslateUi

};

namespace Ui {
    class DataRange: public Ui_DataRange {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATARANGE_H
