/********************************************************************************
** Form generated from reading UI file 'markerstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MARKERSTAB_H
#define UI_MARKERSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"
#include "curveselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_MarkersTab
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *_markersGroup;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *hboxLayout1;
    QLabel *label;
    QLineEdit *_newMarker;
    QPushButton *_addMarker;
    QSpacerItem *spacerItem;
    QVBoxLayout *vboxLayout2;
    QLabel *label_2;
    QListWidget *_currentMarkersList;
    QVBoxLayout *vboxLayout3;
    QSpacerItem *spacerItem1;
    QPushButton *_clearMarkers;
    QPushButton *_removeMarker;
    QSpacerItem *spacerItem2;
    QGroupBox *_axisMajorGridGroup;
    QGridLayout *gridLayout;
    QLabel *_Label_3;
    QComboBox *_markerLineStyle;
    QLabel *label_10;
    Kst::ColorButton *_markerLineColor;
    QLabel *label_11;
    QDoubleSpinBox *_markerLineWidth;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout2;
    QCheckBox *_curveMarkers;
    Kst::CurveSelector *_curve;
    QRadioButton *_hidden;
    QLabel *label_3;
    QRadioButton *_risingEdge;
    QRadioButton *_fallingEdge;
    QRadioButton *_both;
    QCheckBox *_vectorMarkers;
    Kst::VectorSelector *_vector;
    QSpacerItem *spacerItem3;

    void setupUi(QWidget *MarkersTab)
    {
        if (MarkersTab->objectName().isEmpty())
            MarkersTab->setObjectName("MarkersTab");
        MarkersTab->resize(609, 549);
        verticalLayout = new QVBoxLayout(MarkersTab);
        verticalLayout->setObjectName("verticalLayout");
        _markersGroup = new QGroupBox(MarkersTab);
        _markersGroup->setObjectName("_markersGroup");
        vboxLayout = new QVBoxLayout(_markersGroup);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(verticalSpacer);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        label = new QLabel(_markersGroup);
        label->setObjectName("label");

        hboxLayout1->addWidget(label);

        _newMarker = new QLineEdit(_markersGroup);
        _newMarker->setObjectName("_newMarker");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_newMarker->sizePolicy().hasHeightForWidth());
        _newMarker->setSizePolicy(sizePolicy);

        hboxLayout1->addWidget(_newMarker);

        _addMarker = new QPushButton(_markersGroup);
        _addMarker->setObjectName("_addMarker");

        hboxLayout1->addWidget(_addMarker);


        vboxLayout1->addLayout(hboxLayout1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setObjectName("vboxLayout2");
        label_2 = new QLabel(_markersGroup);
        label_2->setObjectName("label_2");

        vboxLayout2->addWidget(label_2);

        _currentMarkersList = new QListWidget(_markersGroup);
        _currentMarkersList->setObjectName("_currentMarkersList");
        _currentMarkersList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout2->addWidget(_currentMarkersList);


        hboxLayout->addLayout(vboxLayout2);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setObjectName("vboxLayout3");
        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacerItem1);

        _clearMarkers = new QPushButton(_markersGroup);
        _clearMarkers->setObjectName("_clearMarkers");

        vboxLayout3->addWidget(_clearMarkers);

        _removeMarker = new QPushButton(_markersGroup);
        _removeMarker->setObjectName("_removeMarker");

        vboxLayout3->addWidget(_removeMarker);

        spacerItem2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacerItem2);


        hboxLayout->addLayout(vboxLayout3);


        vboxLayout->addLayout(hboxLayout);


        verticalLayout->addWidget(_markersGroup);

        _axisMajorGridGroup = new QGroupBox(MarkersTab);
        _axisMajorGridGroup->setObjectName("_axisMajorGridGroup");
        gridLayout = new QGridLayout(_axisMajorGridGroup);
        gridLayout->setObjectName("gridLayout");
        _Label_3 = new QLabel(_axisMajorGridGroup);
        _Label_3->setObjectName("_Label_3");
        _Label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_3, 0, 0, 1, 1);

        _markerLineStyle = new QComboBox(_axisMajorGridGroup);
        _markerLineStyle->setObjectName("_markerLineStyle");
        sizePolicy.setHeightForWidth(_markerLineStyle->sizePolicy().hasHeightForWidth());
        _markerLineStyle->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_markerLineStyle, 0, 1, 1, 1);

        label_10 = new QLabel(_axisMajorGridGroup);
        label_10->setObjectName("label_10");
        label_10->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_10, 0, 2, 1, 1);

        _markerLineColor = new Kst::ColorButton(_axisMajorGridGroup);
        _markerLineColor->setObjectName("_markerLineColor");

        gridLayout->addWidget(_markerLineColor, 0, 3, 1, 1);

        label_11 = new QLabel(_axisMajorGridGroup);
        label_11->setObjectName("label_11");
        label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_11, 0, 4, 1, 1);

        _markerLineWidth = new QDoubleSpinBox(_axisMajorGridGroup);
        _markerLineWidth->setObjectName("_markerLineWidth");

        gridLayout->addWidget(_markerLineWidth, 0, 5, 1, 1);


        verticalLayout->addWidget(_axisMajorGridGroup);

        groupBox_2 = new QGroupBox(MarkersTab);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout2 = new QGridLayout(groupBox_2);
        gridLayout2->setObjectName("gridLayout2");
        _curveMarkers = new QCheckBox(groupBox_2);
        _curveMarkers->setObjectName("_curveMarkers");

        gridLayout2->addWidget(_curveMarkers, 0, 0, 1, 2);

        _curve = new Kst::CurveSelector(groupBox_2);
        _curve->setObjectName("_curve");

        gridLayout2->addWidget(_curve, 0, 2, 1, 2);

        _hidden = new QRadioButton(groupBox_2);
        _hidden->setObjectName("_hidden");
        _hidden->setMaximumSize(QSize(0, 0));
        _hidden->setText(QString::fromUtf8("SHOULD NOT BE &VISIBLE"));

        gridLayout2->addWidget(_hidden, 1, 4, 1, 1);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");

        gridLayout2->addWidget(label_3, 2, 0, 1, 1);

        _risingEdge = new QRadioButton(groupBox_2);
        _risingEdge->setObjectName("_risingEdge");

        gridLayout2->addWidget(_risingEdge, 2, 1, 1, 1);

        _fallingEdge = new QRadioButton(groupBox_2);
        _fallingEdge->setObjectName("_fallingEdge");

        gridLayout2->addWidget(_fallingEdge, 2, 2, 1, 1);

        _both = new QRadioButton(groupBox_2);
        _both->setObjectName("_both");

        gridLayout2->addWidget(_both, 2, 3, 1, 1);

        _vectorMarkers = new QCheckBox(groupBox_2);
        _vectorMarkers->setObjectName("_vectorMarkers");

        gridLayout2->addWidget(_vectorMarkers, 3, 0, 1, 2);

        _vector = new Kst::VectorSelector(groupBox_2);
        _vector->setObjectName("_vector");

        gridLayout2->addWidget(_vector, 3, 2, 1, 2);


        verticalLayout->addWidget(groupBox_2);

        spacerItem3 = new QSpacerItem(20, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem3);

#if QT_CONFIG(shortcut)
        label->setBuddy(_newMarker);
        label_2->setBuddy(_currentMarkersList);
        _Label_3->setBuddy(_markerLineStyle);
        label_10->setBuddy(_markerLineColor);
        label_11->setBuddy(_markerLineWidth);
#endif // QT_CONFIG(shortcut)

        retranslateUi(MarkersTab);

        QMetaObject::connectSlotsByName(MarkersTab);
    } // setupUi

    void retranslateUi(QWidget *MarkersTab)
    {
        _markersGroup->setTitle(QCoreApplication::translate("MarkersTab", "Axis Markers", nullptr));
        label->setText(QCoreApplication::translate("MarkersTab", "New mar&ker point: ", nullptr));
        _addMarker->setText(QCoreApplication::translate("MarkersTab", "&Add", nullptr));
        label_2->setText(QCoreApplication::translate("MarkersTab", "C&urrent markers", nullptr));
        _clearMarkers->setText(QCoreApplication::translate("MarkersTab", "&Clear", nullptr));
        _removeMarker->setText(QCoreApplication::translate("MarkersTab", "&Remove", nullptr));
        _axisMajorGridGroup->setTitle(QCoreApplication::translate("MarkersTab", "Marker Appearance", nullptr));
        _Label_3->setText(QCoreApplication::translate("MarkersTab", "Li&ne style:", nullptr));
        label_10->setText(QCoreApplication::translate("MarkersTab", "Line &color:", nullptr));
        label_11->setText(QCoreApplication::translate("MarkersTab", "Line &width:", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("MarkersTab", "Automatic Markers", nullptr));
        _curveMarkers->setText(QCoreApplication::translate("MarkersTab", "Use curve as marker &source:", nullptr));
        label_3->setText(QCoreApplication::translate("MarkersTab", "Create markers on:", nullptr));
        _risingEdge->setText(QCoreApplication::translate("MarkersTab", "R&ising edge", nullptr));
        _fallingEdge->setText(QCoreApplication::translate("MarkersTab", "&Falling edge", nullptr));
        _both->setText(QCoreApplication::translate("MarkersTab", "&Both", nullptr));
        _vectorMarkers->setText(QCoreApplication::translate("MarkersTab", "Use &vector as marker source:", nullptr));
        (void)MarkersTab;
    } // retranslateUi

};

namespace Ui {
    class MarkersTab: public Ui_MarkersTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MARKERSTAB_H
