/********************************************************************************
** Form generated from reading UI file 'circledimensionstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CIRCLEDIMENSIONSTAB_H
#define UI_CIRCLEDIMENSIONSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CircleDimensionsTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QDoubleSpinBox *_radius;
    QLabel *label;
    QLabel *label_2;
    QDoubleSpinBox *_y;
    QCheckBox *_lockPosToData;
    QDoubleSpinBox *_x;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *CircleDimensionsTab)
    {
        if (CircleDimensionsTab->objectName().isEmpty())
            CircleDimensionsTab->setObjectName("CircleDimensionsTab");
        CircleDimensionsTab->resize(331, 119);
        verticalLayout = new QVBoxLayout(CircleDimensionsTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(CircleDimensionsTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        _radius = new QDoubleSpinBox(CircleDimensionsTab);
        _radius->setObjectName("_radius");
        _radius->setDecimals(4);
        _radius->setMaximum(1.000000000000000);
        _radius->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_radius, 2, 1, 1, 1);

        label = new QLabel(CircleDimensionsTab);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        label_2 = new QLabel(CircleDimensionsTab);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_2, 1, 2, 1, 1);

        _y = new QDoubleSpinBox(CircleDimensionsTab);
        _y->setObjectName("_y");
        _y->setDecimals(4);
        _y->setMaximum(1.000000000000000);
        _y->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_y, 1, 3, 1, 1);

        _lockPosToData = new QCheckBox(CircleDimensionsTab);
        _lockPosToData->setObjectName("_lockPosToData");

        gridLayout->addWidget(_lockPosToData, 0, 1, 1, 3);

        _x = new QDoubleSpinBox(CircleDimensionsTab);
        _x->setObjectName("_x");
        _x->setDecimals(4);
        _x->setMaximum(1.000000000000000);
        _x->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_x, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 29, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(_radius);
        label->setBuddy(_x);
        label_2->setBuddy(_y);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_lockPosToData, _x);
        QWidget::setTabOrder(_x, _y);
        QWidget::setTabOrder(_y, _radius);

        retranslateUi(CircleDimensionsTab);
        QObject::connect(_lockPosToData, SIGNAL(stateChanged(int)), CircleDimensionsTab, SLOT(modified()));
        QObject::connect(_y, SIGNAL(valueChanged(double)), CircleDimensionsTab, SLOT(modified()));
        QObject::connect(_x, SIGNAL(valueChanged(double)), CircleDimensionsTab, SLOT(modified()));
        QObject::connect(_radius, SIGNAL(valueChanged(double)), CircleDimensionsTab, SLOT(modified()));

        QMetaObject::connectSlotsByName(CircleDimensionsTab);
    } // setupUi

    void retranslateUi(QWidget *CircleDimensionsTab)
    {
        label_3->setText(QCoreApplication::translate("CircleDimensionsTab", "&Radius", nullptr));
        label->setText(QCoreApplication::translate("CircleDimensionsTab", "&Center", nullptr));
        label_2->setText(QCoreApplication::translate("CircleDimensionsTab", " , ", nullptr));
        _lockPosToData->setText(QCoreApplication::translate("CircleDimensionsTab", "Lock Position To Data", nullptr));
        (void)CircleDimensionsTab;
    } // retranslateUi

};

namespace Ui {
    class CircleDimensionsTab: public Ui_CircleDimensionsTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CIRCLEDIMENSIONSTAB_H
