/********************************************************************************
** Form generated from reading UI file 'boxcarhighpassconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BOXCARHIGHPASSCONFIG_H
#define UI_BOXCARHIGHPASSCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_BoxcarHighpassConfig
{
public:
    QGridLayout *gridLayout;
    Kst::ScalarSelector *_scalarSamples;
    QLabel *label_5;
    QLabel *label_2;
    QLabel *label;
    QLabel *label_4;
    Kst::VectorSelector *_vector;
    QDoubleSpinBox *_sampleRateSpin;
    QLabel *_cutoffLabel;
    QLabel *label_3;
    QSpinBox *_stagesSpin;
    QLabel *label_6;

    void setupUi(QWidget *BoxcarHighpassConfig)
    {
        if (BoxcarHighpassConfig->objectName().isEmpty())
            BoxcarHighpassConfig->setObjectName("BoxcarHighpassConfig");
        BoxcarHighpassConfig->resize(514, 92);
        gridLayout = new QGridLayout(BoxcarHighpassConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _scalarSamples = new Kst::ScalarSelector(BoxcarHighpassConfig);
        _scalarSamples->setObjectName("_scalarSamples");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_scalarSamples->sizePolicy().hasHeightForWidth());
        _scalarSamples->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_scalarSamples, 1, 1, 1, 4);

        label_5 = new QLabel(BoxcarHighpassConfig);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 2, 3, 1, 2);

        label_2 = new QLabel(BoxcarHighpassConfig);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label_2, 1, 5, 1, 1);

        label = new QLabel(BoxcarHighpassConfig);
        label->setObjectName("label");
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label, 2, 0, 1, 1);

        label_4 = new QLabel(BoxcarHighpassConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        _vector = new Kst::VectorSelector(BoxcarHighpassConfig);
        _vector->setObjectName("_vector");
        sizePolicy.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_vector, 0, 1, 1, 6);

        _sampleRateSpin = new QDoubleSpinBox(BoxcarHighpassConfig);
        _sampleRateSpin->setObjectName("_sampleRateSpin");
        _sampleRateSpin->setDecimals(4);
        _sampleRateSpin->setMaximum(1000000000000000.000000000000000);

        gridLayout->addWidget(_sampleRateSpin, 2, 1, 1, 1);

        _cutoffLabel = new QLabel(BoxcarHighpassConfig);
        _cutoffLabel->setObjectName("_cutoffLabel");

        gridLayout->addWidget(_cutoffLabel, 2, 5, 1, 2);

        label_3 = new QLabel(BoxcarHighpassConfig);
        label_3->setObjectName("label_3");
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _stagesSpin = new QSpinBox(BoxcarHighpassConfig);
        _stagesSpin->setObjectName("_stagesSpin");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_stagesSpin->sizePolicy().hasHeightForWidth());
        _stagesSpin->setSizePolicy(sizePolicy3);
        _stagesSpin->setMinimum(1);
        _stagesSpin->setMaximum(100);
        _stagesSpin->setValue(3);

        gridLayout->addWidget(_stagesSpin, 1, 6, 1, 1);

        label_6 = new QLabel(BoxcarHighpassConfig);
        label_6->setObjectName("label_6");
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label_6, 2, 2, 1, 1);


        retranslateUi(BoxcarHighpassConfig);

        QMetaObject::connectSlotsByName(BoxcarHighpassConfig);
    } // setupUi

    void retranslateUi(QWidget *BoxcarHighpassConfig)
    {
#if QT_CONFIG(tooltip)
        _scalarSamples->setToolTip(QCoreApplication::translate("BoxcarHighpassConfig", "The length of the first stage of the averaging filter.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_5->setText(QCoreApplication::translate("BoxcarHighpassConfig", "Cutoff Frequency: ", nullptr));
        label_2->setText(QCoreApplication::translate("BoxcarHighpassConfig", "Stages:", nullptr));
        label->setText(QCoreApplication::translate("BoxcarHighpassConfig", "Sample Rate: ", nullptr));
        label_4->setText(QCoreApplication::translate("BoxcarHighpassConfig", "Filter Width:", nullptr));
#if QT_CONFIG(tooltip)
        _sampleRateSpin->setToolTip(QCoreApplication::translate("BoxcarHighpassConfig", "Used only for calculation of Cutoff Frequency", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        _cutoffLabel->setToolTip(QCoreApplication::translate("BoxcarHighpassConfig", "The 50% point of the filter.", nullptr));
#endif // QT_CONFIG(tooltip)
        _cutoffLabel->setText(QCoreApplication::translate("BoxcarHighpassConfig", "TextLabel", nullptr));
        label_3->setText(QCoreApplication::translate("BoxcarHighpassConfig", "Y: ", nullptr));
#if QT_CONFIG(tooltip)
        _stagesSpin->setToolTip(QCoreApplication::translate("BoxcarHighpassConfig", "Stages of the filter.  More stages reduces stop band leakage.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_6->setText(QCoreApplication::translate("BoxcarHighpassConfig", "Hz", nullptr));
        (void)BoxcarHighpassConfig;
    } // retranslateUi

};

namespace Ui {
    class BoxcarHighpassConfig: public Ui_BoxcarHighpassConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BOXCARHIGHPASSCONFIG_H
