/********************************************************************************
** Form generated from reading UI file 'histogramtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HISTOGRAMTAB_H
#define UI_HISTOGRAMTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "curveappearance.h"
#include "curveplacement.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_HistogramTab
{
public:
    QGridLayout *gridLayout_2;
    Kst::CurveAppearance *_curveAppearance;
    Kst::CurvePlacement *_curvePlacement;
    QHBoxLayout *horizontalLayout;
    QGroupBox *ButtonGroup1_3;
    QGridLayout *gridLayout;
    QLabel *_vectorLabel;
    Kst::VectorSelector *_vector;
    QLabel *TextLabel1_4_2_2;
    QLineEdit *_min;
    QLabel *TextLabel4_5_2;
    QLineEdit *_max;
    QLabel *TextLabel5_2;
    QSpinBox *_numberOfBins;
    QHBoxLayout *hboxLayout;
    QPushButton *AutoBin;
    QCheckBox *_realTimeAutoBin;
    QGroupBox *ButtonGroup5;
    QGridLayout *gridLayout1;
    QRadioButton *_normalizationIsFraction;
    QRadioButton *_normalizationIsNumber;
    QRadioButton *_normalizationMaximumOne;
    QRadioButton *_normalizationIsPercent;

    void setupUi(QWidget *HistogramTab)
    {
        if (HistogramTab->objectName().isEmpty())
            HistogramTab->setObjectName("HistogramTab");
        HistogramTab->resize(660, 227);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(HistogramTab->sizePolicy().hasHeightForWidth());
        HistogramTab->setSizePolicy(sizePolicy);
        gridLayout_2 = new QGridLayout(HistogramTab);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        _curveAppearance = new Kst::CurveAppearance(HistogramTab);
        _curveAppearance->setObjectName("_curveAppearance");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_curveAppearance->sizePolicy().hasHeightForWidth());
        _curveAppearance->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(_curveAppearance, 1, 0, 1, 1);

        _curvePlacement = new Kst::CurvePlacement(HistogramTab);
        _curvePlacement->setObjectName("_curvePlacement");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_curvePlacement->sizePolicy().hasHeightForWidth());
        _curvePlacement->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(_curvePlacement, 1, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        ButtonGroup1_3 = new QGroupBox(HistogramTab);
        ButtonGroup1_3->setObjectName("ButtonGroup1_3");
        sizePolicy.setHeightForWidth(ButtonGroup1_3->sizePolicy().hasHeightForWidth());
        ButtonGroup1_3->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(ButtonGroup1_3);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _vectorLabel = new QLabel(ButtonGroup1_3);
        _vectorLabel->setObjectName("_vectorLabel");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_vectorLabel->sizePolicy().hasHeightForWidth());
        _vectorLabel->setSizePolicy(sizePolicy3);
        _vectorLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _vectorLabel->setWordWrap(false);

        gridLayout->addWidget(_vectorLabel, 0, 0, 1, 1);

        _vector = new Kst::VectorSelector(ButtonGroup1_3);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy4);

        gridLayout->addWidget(_vector, 0, 1, 1, 4);

        TextLabel1_4_2_2 = new QLabel(ButtonGroup1_3);
        TextLabel1_4_2_2->setObjectName("TextLabel1_4_2_2");
        sizePolicy3.setHeightForWidth(TextLabel1_4_2_2->sizePolicy().hasHeightForWidth());
        TextLabel1_4_2_2->setSizePolicy(sizePolicy3);
        TextLabel1_4_2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel1_4_2_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel1_4_2_2, 1, 0, 1, 1);

        _min = new QLineEdit(ButtonGroup1_3);
        _min->setObjectName("_min");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(_min->sizePolicy().hasHeightForWidth());
        _min->setSizePolicy(sizePolicy5);
        _min->setText(QString::fromUtf8("-10.0"));

        gridLayout->addWidget(_min, 1, 1, 1, 2);

        TextLabel4_5_2 = new QLabel(ButtonGroup1_3);
        TextLabel4_5_2->setObjectName("TextLabel4_5_2");
        sizePolicy3.setHeightForWidth(TextLabel4_5_2->sizePolicy().hasHeightForWidth());
        TextLabel4_5_2->setSizePolicy(sizePolicy3);
        TextLabel4_5_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel4_5_2, 1, 3, 1, 1);

        _max = new QLineEdit(ButtonGroup1_3);
        _max->setObjectName("_max");
        sizePolicy5.setHeightForWidth(_max->sizePolicy().hasHeightForWidth());
        _max->setSizePolicy(sizePolicy5);
        _max->setText(QString::fromUtf8("10.0"));

        gridLayout->addWidget(_max, 1, 4, 1, 1);

        TextLabel5_2 = new QLabel(ButtonGroup1_3);
        TextLabel5_2->setObjectName("TextLabel5_2");
        sizePolicy3.setHeightForWidth(TextLabel5_2->sizePolicy().hasHeightForWidth());
        TextLabel5_2->setSizePolicy(sizePolicy3);
        TextLabel5_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel5_2->setWordWrap(false);

        gridLayout->addWidget(TextLabel5_2, 2, 0, 1, 1);

        _numberOfBins = new QSpinBox(ButtonGroup1_3);
        _numberOfBins->setObjectName("_numberOfBins");
        sizePolicy5.setHeightForWidth(_numberOfBins->sizePolicy().hasHeightForWidth());
        _numberOfBins->setSizePolicy(sizePolicy5);
        _numberOfBins->setMaximum(1000000);
        _numberOfBins->setSingleStep(5);
        _numberOfBins->setValue(60);

        gridLayout->addWidget(_numberOfBins, 2, 1, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        AutoBin = new QPushButton(ButtonGroup1_3);
        AutoBin->setObjectName("AutoBin");
        sizePolicy3.setHeightForWidth(AutoBin->sizePolicy().hasHeightForWidth());
        AutoBin->setSizePolicy(sizePolicy3);

        hboxLayout->addWidget(AutoBin);

        _realTimeAutoBin = new QCheckBox(ButtonGroup1_3);
        _realTimeAutoBin->setObjectName("_realTimeAutoBin");

        hboxLayout->addWidget(_realTimeAutoBin);


        gridLayout->addLayout(hboxLayout, 2, 2, 1, 3);


        horizontalLayout->addWidget(ButtonGroup1_3);

        ButtonGroup5 = new QGroupBox(HistogramTab);
        ButtonGroup5->setObjectName("ButtonGroup5");
        sizePolicy1.setHeightForWidth(ButtonGroup5->sizePolicy().hasHeightForWidth());
        ButtonGroup5->setSizePolicy(sizePolicy1);
        gridLayout1 = new QGridLayout(ButtonGroup5);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        _normalizationIsFraction = new QRadioButton(ButtonGroup5);
        _normalizationIsFraction->setObjectName("_normalizationIsFraction");

        gridLayout1->addWidget(_normalizationIsFraction, 0, 2, 1, 1);

        _normalizationIsNumber = new QRadioButton(ButtonGroup5);
        _normalizationIsNumber->setObjectName("_normalizationIsNumber");
        _normalizationIsNumber->setChecked(true);

        gridLayout1->addWidget(_normalizationIsNumber, 0, 0, 1, 1);

        _normalizationMaximumOne = new QRadioButton(ButtonGroup5);
        _normalizationMaximumOne->setObjectName("_normalizationMaximumOne");

        gridLayout1->addWidget(_normalizationMaximumOne, 1, 2, 1, 1);

        _normalizationIsPercent = new QRadioButton(ButtonGroup5);
        _normalizationIsPercent->setObjectName("_normalizationIsPercent");

        gridLayout1->addWidget(_normalizationIsPercent, 1, 0, 1, 1);


        horizontalLayout->addWidget(ButtonGroup5);


        gridLayout_2->addLayout(horizontalLayout, 0, 0, 1, 2);

#if QT_CONFIG(shortcut)
        TextLabel1_4_2_2->setBuddy(_min);
        TextLabel4_5_2->setBuddy(_max);
        TextLabel5_2->setBuddy(_numberOfBins);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_vector, _min);
        QWidget::setTabOrder(_min, _max);
        QWidget::setTabOrder(_max, _numberOfBins);
        QWidget::setTabOrder(_numberOfBins, AutoBin);
        QWidget::setTabOrder(AutoBin, _realTimeAutoBin);
        QWidget::setTabOrder(_realTimeAutoBin, _normalizationIsNumber);
        QWidget::setTabOrder(_normalizationIsNumber, _curveAppearance);

        retranslateUi(HistogramTab);

        QMetaObject::connectSlotsByName(HistogramTab);
    } // setupUi

    void retranslateUi(QWidget *HistogramTab)
    {
        ButtonGroup1_3->setTitle(QCoreApplication::translate("HistogramTab", "Histogram Properties", nullptr));
        _vectorLabel->setText(QCoreApplication::translate("HistogramTab", "Data vecto&r:", nullptr));
        TextLabel1_4_2_2->setText(QCoreApplication::translate("HistogramTab", "Bins from&:", nullptr));
#if QT_CONFIG(whatsthis)
        _min->setWhatsThis(QCoreApplication::translate("HistogramTab", "The lower limit of the lowest bin.  Values below this limit will not be counted in any bin.", nullptr));
#endif // QT_CONFIG(whatsthis)
        TextLabel4_5_2->setText(QCoreApplication::translate("HistogramTab", "to:", "from: 333 to: 555"));
#if QT_CONFIG(whatsthis)
        _max->setWhatsThis(QCoreApplication::translate("HistogramTab", "The upper limit of the highest bin.  Values above this limit will not be counted in any bin.", nullptr));
#endif // QT_CONFIG(whatsthis)
        TextLabel5_2->setText(QCoreApplication::translate("HistogramTab", "Num bin&s:", nullptr));
#if QT_CONFIG(whatsthis)
        _numberOfBins->setWhatsThis(QCoreApplication::translate("HistogramTab", "The number of bins in the Histogram.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        AutoBin->setWhatsThis(QCoreApplication::translate("HistogramTab", "Set the upper and lower limits, and the number of bins to a reasonable default.", nullptr));
#endif // QT_CONFIG(whatsthis)
        AutoBin->setText(QCoreApplication::translate("HistogramTab", "A&uto Bin", nullptr));
        _realTimeAutoBin->setText(QCoreApplication::translate("HistogramTab", "Real&-time auto bin", nullptr));
        ButtonGroup5->setTitle(QCoreApplication::translate("HistogramTab", "Y-axis Normalization", nullptr));
#if QT_CONFIG(whatsthis)
        _normalizationIsFraction->setWhatsThis(QCoreApplication::translate("HistogramTab", "The Y axis of the histogram reports the fraction of samples in the bin.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _normalizationIsFraction->setText(QCoreApplication::translate("HistogramTab", "&Fraction in bin", nullptr));
#if QT_CONFIG(whatsthis)
        _normalizationIsNumber->setWhatsThis(QCoreApplication::translate("HistogramTab", "The Y axis of the histogram reports the number of samples in the bin.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _normalizationIsNumber->setText(QCoreApplication::translate("HistogramTab", "Nu&mber in bin", nullptr));
#if QT_CONFIG(whatsthis)
        _normalizationMaximumOne->setWhatsThis(QCoreApplication::translate("HistogramTab", "The Y axis of the histogram is normalized so that the highest bin is 1.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _normalizationMaximumOne->setText(QCoreApplication::translate("HistogramTab", "Peak bin = &1.0", nullptr));
#if QT_CONFIG(shortcut)
        _normalizationMaximumOne->setShortcut(QCoreApplication::translate("HistogramTab", "Alt+1", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        _normalizationIsPercent->setWhatsThis(QCoreApplication::translate("HistogramTab", "The Y axis of the histogram reports the percent of the samples in the bin.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _normalizationIsPercent->setText(QCoreApplication::translate("HistogramTab", "Percen&t in bin", nullptr));
        (void)HistogramTab;
    } // retranslateUi

};

namespace Ui {
    class HistogramTab: public Ui_HistogramTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HISTOGRAMTAB_H
