/********************************************************************************
** Form generated from reading UI file 'labelcreator.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LABELCREATOR_H
#define UI_LABELCREATOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include "colorbutton.h"
#include "labelbuilder.h"

QT_BEGIN_NAMESPACE

class Ui_LabelCreator
{
public:
    QGridLayout *gridLayout_3;
    Kst::LabelBuilder *_labelText;
    QGridLayout *gridLayout_2;
    QCheckBox *_lockPosToData;
    QRadioButton *_right;
    QRadioButton *_left;
    QLabel *label_4;
    QGridLayout *gridLayout;
    QLabel *_Label_12;
    QFontComboBox *_family;
    Kst::ColorButton *_labelColor;
    QLabel *_Label_11;
    QPushButton *_italic;
    QPushButton *_bold;
    QLabel *label;
    QDoubleSpinBox *_labelFontScale;
    QCheckBox *_saveAsDefault;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LabelCreator)
    {
        if (LabelCreator->objectName().isEmpty())
            LabelCreator->setObjectName("LabelCreator");
        LabelCreator->resize(401, 279);
        gridLayout_3 = new QGridLayout(LabelCreator);
        gridLayout_3->setObjectName("gridLayout_3");
        _labelText = new Kst::LabelBuilder(LabelCreator);
        _labelText->setObjectName("_labelText");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_labelText->sizePolicy().hasHeightForWidth());
        _labelText->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(_labelText, 0, 0, 1, 2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        _lockPosToData = new QCheckBox(LabelCreator);
        _lockPosToData->setObjectName("_lockPosToData");

        gridLayout_2->addWidget(_lockPosToData, 0, 3, 1, 1);

        _right = new QRadioButton(LabelCreator);
        _right->setObjectName("_right");

        gridLayout_2->addWidget(_right, 0, 2, 1, 1);

        _left = new QRadioButton(LabelCreator);
        _left->setObjectName("_left");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_left->sizePolicy().hasHeightForWidth());
        _left->setSizePolicy(sizePolicy1);
        _left->setChecked(true);

        gridLayout_2->addWidget(_left, 0, 1, 1, 1);

        label_4 = new QLabel(LabelCreator);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);
        label_4->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(label_4, 0, 0, 1, 1);


        gridLayout_3->addLayout(gridLayout_2, 1, 0, 1, 2);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        _Label_12 = new QLabel(LabelCreator);
        _Label_12->setObjectName("_Label_12");
        sizePolicy2.setHeightForWidth(_Label_12->sizePolicy().hasHeightForWidth());
        _Label_12->setSizePolicy(sizePolicy2);
        _Label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_12, 1, 0, 1, 1);

        _family = new QFontComboBox(LabelCreator);
        _family->setObjectName("_family");

        gridLayout->addWidget(_family, 1, 1, 1, 7);

        _labelColor = new Kst::ColorButton(LabelCreator);
        _labelColor->setObjectName("_labelColor");

        gridLayout->addWidget(_labelColor, 0, 7, 1, 1);

        _Label_11 = new QLabel(LabelCreator);
        _Label_11->setObjectName("_Label_11");
        _Label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_11, 0, 0, 1, 1);

        _italic = new QPushButton(LabelCreator);
        _italic->setObjectName("_italic");
        sizePolicy1.setHeightForWidth(_italic->sizePolicy().hasHeightForWidth());
        _italic->setSizePolicy(sizePolicy1);
        _italic->setCheckable(true);

        gridLayout->addWidget(_italic, 0, 6, 1, 1);

        _bold = new QPushButton(LabelCreator);
        _bold->setObjectName("_bold");
        sizePolicy1.setHeightForWidth(_bold->sizePolicy().hasHeightForWidth());
        _bold->setSizePolicy(sizePolicy1);
        _bold->setCheckable(true);

        gridLayout->addWidget(_bold, 0, 5, 1, 1);

        label = new QLabel(LabelCreator);
        label->setObjectName("label");
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(label, 0, 4, 1, 1);

        _labelFontScale = new QDoubleSpinBox(LabelCreator);
        _labelFontScale->setObjectName("_labelFontScale");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_labelFontScale->sizePolicy().hasHeightForWidth());
        _labelFontScale->setSizePolicy(sizePolicy3);
        _labelFontScale->setDecimals(1);
        _labelFontScale->setMinimum(-25.000000000000000);
        _labelFontScale->setSingleStep(1.000000000000000);

        gridLayout->addWidget(_labelFontScale, 0, 1, 1, 3);


        gridLayout_3->addLayout(gridLayout, 2, 0, 1, 2);

        _saveAsDefault = new QCheckBox(LabelCreator);
        _saveAsDefault->setObjectName("_saveAsDefault");

        gridLayout_3->addWidget(_saveAsDefault, 3, 0, 1, 1);

        buttonBox = new QDialogButtonBox(LabelCreator);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout_3->addWidget(buttonBox, 3, 1, 1, 1);

#if QT_CONFIG(shortcut)
        _Label_12->setBuddy(_family);
        _Label_11->setBuddy(_labelFontScale);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_labelFontScale, _labelColor);
        QWidget::setTabOrder(_labelColor, _family);
        QWidget::setTabOrder(_family, _saveAsDefault);
        QWidget::setTabOrder(_saveAsDefault, buttonBox);

        retranslateUi(LabelCreator);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, LabelCreator, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, LabelCreator, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(LabelCreator);
    } // setupUi

    void retranslateUi(QDialog *LabelCreator)
    {
        LabelCreator->setWindowTitle(QCoreApplication::translate("LabelCreator", "Dialog", nullptr));
#if QT_CONFIG(tooltip)
        LabelCreator->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        _lockPosToData->setText(QCoreApplication::translate("LabelCreator", "Lock Position To Data", nullptr));
        _right->setText(QCoreApplication::translate("LabelCreator", "Right", "The right side"));
        _left->setText(QCoreApplication::translate("LabelCreator", "&Left", "The left side"));
        label_4->setText(QCoreApplication::translate("LabelCreator", "Reference Point:", nullptr));
        _Label_12->setText(QCoreApplication::translate("LabelCreator", "Fo&nt family:", nullptr));
#if QT_CONFIG(tooltip)
        _labelColor->setToolTip(QCoreApplication::translate("LabelCreator", "Color", "The color of the object"));
#endif // QT_CONFIG(tooltip)
        _Label_11->setText(QCoreApplication::translate("LabelCreator", "Font si&ze:", nullptr));
#if QT_CONFIG(tooltip)
        _italic->setToolTip(QCoreApplication::translate("LabelCreator", "Italic", nullptr));
#endif // QT_CONFIG(tooltip)
        _italic->setText(QString());
#if QT_CONFIG(tooltip)
        _bold->setToolTip(QCoreApplication::translate("LabelCreator", "Bold", "bold font"));
#endif // QT_CONFIG(tooltip)
        _bold->setText(QString());
        label->setText(QCoreApplication::translate("LabelCreator", " points", nullptr));
        _saveAsDefault->setText(QCoreApplication::translate("LabelCreator", "Save as default", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LabelCreator: public Ui_LabelCreator {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LABELCREATOR_H
