/********************************************************************************
** Form generated from reading UI file 'eventmonitortab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EVENTMONITORTAB_H
#define UI_EVENTMONITORTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_EventMonitorTab
{
public:
    QGridLayout *gridLayout;
    QGroupBox *_emailGroupBox;
    QHBoxLayout *hboxLayout;
    QCheckBox *_emailNotify;
    QLineEdit *_emailRecipients;
    QGroupBox *_equationGroupBox;
    QGridLayout *gridLayout1;
    QLabel *textLabel2;
    QLabel *textLabel1;
    QComboBox *_equationOperator;
    Kst::ScalarSelector *_scalarSelector;
    QLabel *textLabel3;
    QLineEdit *_equation;
    Kst::VectorSelector *_vectorSelector;
    QLabel *textLabel3_2;
    QLabel *textLabel3_2_3;
    QLineEdit *_description;
    QGroupBox *_KstScriptGroupBox;
    QHBoxLayout *hboxLayout1;
    QCheckBox *_executeScript;
    QTextEdit *_script;
    QGroupBox *_ELOGGroupBox;
    QHBoxLayout *hboxLayout2;
    QCheckBox *_ELOGNotify;
    QPushButton *_ELOGConfigure;
    QSpacerItem *spacerItem;
    QGroupBox *_logButtonGroup;
    QGridLayout *gridLayout2;
    QSpacerItem *spacerItem1;
    QRadioButton *_debugLogWarning;
    QRadioButton *_debugLogError;
    QRadioButton *_debugLogNotice;
    QCheckBox *_debugLog;

    void setupUi(QWidget *EventMonitorTab)
    {
        if (EventMonitorTab->objectName().isEmpty())
            EventMonitorTab->setObjectName("EventMonitorTab");
        EventMonitorTab->resize(454, 443);
        gridLayout = new QGridLayout(EventMonitorTab);
        gridLayout->setSpacing(0);
        gridLayout->setContentsMargins(9, 9, 9, 9);
        gridLayout->setObjectName("gridLayout");
        _emailGroupBox = new QGroupBox(EventMonitorTab);
        _emailGroupBox->setObjectName("_emailGroupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_emailGroupBox->sizePolicy().hasHeightForWidth());
        _emailGroupBox->setSizePolicy(sizePolicy);
        hboxLayout = new QHBoxLayout(_emailGroupBox);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        _emailNotify = new QCheckBox(_emailGroupBox);
        _emailNotify->setObjectName("_emailNotify");

        hboxLayout->addWidget(_emailNotify);

        _emailRecipients = new QLineEdit(_emailGroupBox);
        _emailRecipients->setObjectName("_emailRecipients");
        _emailRecipients->setEnabled(false);

        hboxLayout->addWidget(_emailRecipients);


        gridLayout->addWidget(_emailGroupBox, 2, 0, 1, 1);

        _equationGroupBox = new QGroupBox(EventMonitorTab);
        _equationGroupBox->setObjectName("_equationGroupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_equationGroupBox->sizePolicy().hasHeightForWidth());
        _equationGroupBox->setSizePolicy(sizePolicy1);
        gridLayout1 = new QGridLayout(_equationGroupBox);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        textLabel2 = new QLabel(_equationGroupBox);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout1->addWidget(textLabel2, 1, 0, 1, 1);

        textLabel1 = new QLabel(_equationGroupBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout1->addWidget(textLabel1, 0, 0, 1, 1);

        _equationOperator = new QComboBox(_equationGroupBox);
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->addItem(QString());
        _equationOperator->setObjectName("_equationOperator");

        gridLayout1->addWidget(_equationOperator, 0, 1, 1, 1);

        _scalarSelector = new Kst::ScalarSelector(_equationGroupBox);
        _scalarSelector->setObjectName("_scalarSelector");

        gridLayout1->addWidget(_scalarSelector, 2, 1, 1, 1);

        textLabel3 = new QLabel(_equationGroupBox);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout1->addWidget(textLabel3, 2, 0, 1, 1);

        _equation = new QLineEdit(_equationGroupBox);
        _equation->setObjectName("_equation");

        gridLayout1->addWidget(_equation, 3, 1, 1, 1);

        _vectorSelector = new Kst::VectorSelector(_equationGroupBox);
        _vectorSelector->setObjectName("_vectorSelector");

        gridLayout1->addWidget(_vectorSelector, 1, 1, 1, 1);

        textLabel3_2 = new QLabel(_equationGroupBox);
        textLabel3_2->setObjectName("textLabel3_2");
        textLabel3_2->setWordWrap(false);

        gridLayout1->addWidget(textLabel3_2, 3, 0, 1, 1);

        textLabel3_2_3 = new QLabel(_equationGroupBox);
        textLabel3_2_3->setObjectName("textLabel3_2_3");
        textLabel3_2_3->setWordWrap(false);

        gridLayout1->addWidget(textLabel3_2_3, 4, 0, 1, 1);

        _description = new QLineEdit(_equationGroupBox);
        _description->setObjectName("_description");

        gridLayout1->addWidget(_description, 4, 1, 1, 1);


        gridLayout->addWidget(_equationGroupBox, 0, 0, 1, 1);

        _KstScriptGroupBox = new QGroupBox(EventMonitorTab);
        _KstScriptGroupBox->setObjectName("_KstScriptGroupBox");
        hboxLayout1 = new QHBoxLayout(_KstScriptGroupBox);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        _executeScript = new QCheckBox(_KstScriptGroupBox);
        _executeScript->setObjectName("_executeScript");

        hboxLayout1->addWidget(_executeScript);

        _script = new QTextEdit(_KstScriptGroupBox);
        _script->setObjectName("_script");
        _script->setEnabled(false);

        hboxLayout1->addWidget(_script);


        gridLayout->addWidget(_KstScriptGroupBox, 4, 0, 1, 1);

        _ELOGGroupBox = new QGroupBox(EventMonitorTab);
        _ELOGGroupBox->setObjectName("_ELOGGroupBox");
        hboxLayout2 = new QHBoxLayout(_ELOGGroupBox);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        _ELOGNotify = new QCheckBox(_ELOGGroupBox);
        _ELOGNotify->setObjectName("_ELOGNotify");

        hboxLayout2->addWidget(_ELOGNotify);

        _ELOGConfigure = new QPushButton(_ELOGGroupBox);
        _ELOGConfigure->setObjectName("_ELOGConfigure");
        _ELOGConfigure->setEnabled(false);

        hboxLayout2->addWidget(_ELOGConfigure);

        spacerItem = new QSpacerItem(241, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        hboxLayout2->addItem(spacerItem);


        gridLayout->addWidget(_ELOGGroupBox, 3, 0, 1, 1);

        _logButtonGroup = new QGroupBox(EventMonitorTab);
        _logButtonGroup->setObjectName("_logButtonGroup");
        sizePolicy.setHeightForWidth(_logButtonGroup->sizePolicy().hasHeightForWidth());
        _logButtonGroup->setSizePolicy(sizePolicy);
        gridLayout2 = new QGridLayout(_logButtonGroup);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName("gridLayout2");
        spacerItem1 = new QSpacerItem(120, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout2->addItem(spacerItem1, 0, 5, 1, 1);

        _debugLogWarning = new QRadioButton(_logButtonGroup);
        _debugLogWarning->setObjectName("_debugLogWarning");
        _debugLogWarning->setEnabled(false);

        gridLayout2->addWidget(_debugLogWarning, 0, 3, 1, 1);

        _debugLogError = new QRadioButton(_logButtonGroup);
        _debugLogError->setObjectName("_debugLogError");
        _debugLogError->setEnabled(false);

        gridLayout2->addWidget(_debugLogError, 0, 4, 1, 1);

        _debugLogNotice = new QRadioButton(_logButtonGroup);
        _debugLogNotice->setObjectName("_debugLogNotice");
        _debugLogNotice->setEnabled(false);
        _debugLogNotice->setChecked(true);

        gridLayout2->addWidget(_debugLogNotice, 0, 2, 1, 1);

        _debugLog = new QCheckBox(_logButtonGroup);
        _debugLog->setObjectName("_debugLog");

        gridLayout2->addWidget(_debugLog, 0, 0, 1, 1);


        gridLayout->addWidget(_logButtonGroup, 1, 0, 1, 1);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(_equationOperator);
        textLabel3_2->setBuddy(_equation);
        textLabel3_2_3->setBuddy(_description);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_equationOperator, _vectorSelector);
        QWidget::setTabOrder(_vectorSelector, _scalarSelector);
        QWidget::setTabOrder(_scalarSelector, _equation);
        QWidget::setTabOrder(_equation, _description);
        QWidget::setTabOrder(_description, _debugLog);
        QWidget::setTabOrder(_debugLog, _debugLogNotice);
        QWidget::setTabOrder(_debugLogNotice, _debugLogWarning);
        QWidget::setTabOrder(_debugLogWarning, _debugLogError);
        QWidget::setTabOrder(_debugLogError, _emailNotify);
        QWidget::setTabOrder(_emailNotify, _emailRecipients);
        QWidget::setTabOrder(_emailRecipients, _ELOGNotify);
        QWidget::setTabOrder(_ELOGNotify, _ELOGConfigure);

        retranslateUi(EventMonitorTab);
        QObject::connect(_emailNotify, &QCheckBox::toggled, _emailRecipients, &QLineEdit::setEnabled);
        QObject::connect(_ELOGNotify, &QCheckBox::toggled, _ELOGConfigure, &QPushButton::setEnabled);
        QObject::connect(_executeScript, &QCheckBox::toggled, _script, &QTextEdit::setEnabled);
        QObject::connect(_debugLog, &QCheckBox::toggled, _debugLogNotice, &QRadioButton::setEnabled);
        QObject::connect(_debugLog, &QCheckBox::toggled, _debugLogWarning, &QRadioButton::setEnabled);
        QObject::connect(_debugLog, &QCheckBox::toggled, _debugLogError, &QRadioButton::setEnabled);

        QMetaObject::connectSlotsByName(EventMonitorTab);
    } // setupUi

    void retranslateUi(QWidget *EventMonitorTab)
    {
        _emailGroupBox->setTitle(QCoreApplication::translate("EventMonitorTab", "E-Mail Notification", nullptr));
        _emailNotify->setText(QCoreApplication::translate("EventMonitorTab", "N&otify:", nullptr));
        _equationGroupBox->setTitle(QCoreApplication::translate("EventMonitorTab", "Expression", nullptr));
        textLabel2->setText(QCoreApplication::translate("EventMonitorTab", "Vectors:", nullptr));
        textLabel1->setText(QCoreApplication::translate("EventMonitorTab", "Operators:", nullptr));
        _equationOperator->setItemText(0, QCoreApplication::translate("EventMonitorTab", "+", nullptr));
        _equationOperator->setItemText(1, QCoreApplication::translate("EventMonitorTab", "-", nullptr));
        _equationOperator->setItemText(2, QCoreApplication::translate("EventMonitorTab", "*", nullptr));
        _equationOperator->setItemText(3, QCoreApplication::translate("EventMonitorTab", "/", nullptr));
        _equationOperator->setItemText(4, QCoreApplication::translate("EventMonitorTab", "^", nullptr));
        _equationOperator->setItemText(5, QCoreApplication::translate("EventMonitorTab", "PI", "Number: 3.14159265...."));
        _equationOperator->setItemText(6, QCoreApplication::translate("EventMonitorTab", "e", "Euler's number "));
        _equationOperator->setItemText(7, QCoreApplication::translate("EventMonitorTab", "=", nullptr));
        _equationOperator->setItemText(8, QCoreApplication::translate("EventMonitorTab", "<", nullptr));
        _equationOperator->setItemText(9, QCoreApplication::translate("EventMonitorTab", "<=", nullptr));
        _equationOperator->setItemText(10, QCoreApplication::translate("EventMonitorTab", ">", nullptr));
        _equationOperator->setItemText(11, QCoreApplication::translate("EventMonitorTab", ">=", nullptr));
        _equationOperator->setItemText(12, QCoreApplication::translate("EventMonitorTab", "&", nullptr));
        _equationOperator->setItemText(13, QCoreApplication::translate("EventMonitorTab", "|", nullptr));
        _equationOperator->setItemText(14, QCoreApplication::translate("EventMonitorTab", "!", nullptr));
        _equationOperator->setItemText(15, QCoreApplication::translate("EventMonitorTab", "STEP()", "A mathematical function"));
        _equationOperator->setItemText(16, QCoreApplication::translate("EventMonitorTab", "ABS()", nullptr));
        _equationOperator->setItemText(17, QCoreApplication::translate("EventMonitorTab", "SQRT()", nullptr));
        _equationOperator->setItemText(18, QCoreApplication::translate("EventMonitorTab", "CBRT()", nullptr));
        _equationOperator->setItemText(19, QCoreApplication::translate("EventMonitorTab", "SIN()", nullptr));
        _equationOperator->setItemText(20, QCoreApplication::translate("EventMonitorTab", "COS()", nullptr));
        _equationOperator->setItemText(21, QCoreApplication::translate("EventMonitorTab", "TAN()", "a mathematical function"));
        _equationOperator->setItemText(22, QCoreApplication::translate("EventMonitorTab", "ASIN()", nullptr));
        _equationOperator->setItemText(23, QCoreApplication::translate("EventMonitorTab", "ACOS()", nullptr));
        _equationOperator->setItemText(24, QCoreApplication::translate("EventMonitorTab", "ATAN()", nullptr));
        _equationOperator->setItemText(25, QCoreApplication::translate("EventMonitorTab", "SEC()", nullptr));
        _equationOperator->setItemText(26, QCoreApplication::translate("EventMonitorTab", "CSC()", nullptr));
        _equationOperator->setItemText(27, QCoreApplication::translate("EventMonitorTab", "COT()", nullptr));
        _equationOperator->setItemText(28, QCoreApplication::translate("EventMonitorTab", "SINH()", nullptr));
        _equationOperator->setItemText(29, QCoreApplication::translate("EventMonitorTab", "COSH()", nullptr));
        _equationOperator->setItemText(30, QCoreApplication::translate("EventMonitorTab", "TANH()", nullptr));
        _equationOperator->setItemText(31, QCoreApplication::translate("EventMonitorTab", "EXP()", nullptr));
        _equationOperator->setItemText(32, QCoreApplication::translate("EventMonitorTab", "LN()", nullptr));
        _equationOperator->setItemText(33, QCoreApplication::translate("EventMonitorTab", "LOG()", nullptr));

        textLabel3->setText(QCoreApplication::translate("EventMonitorTab", "Scalars:", nullptr));
        textLabel3_2->setText(QCoreApplication::translate("EventMonitorTab", "Expression:", nullptr));
        textLabel3_2_3->setText(QCoreApplication::translate("EventMonitorTab", "Description:", nullptr));
        _KstScriptGroupBox->setTitle(QCoreApplication::translate("EventMonitorTab", "KstScript", nullptr));
        _executeScript->setText(QCoreApplication::translate("EventMonitorTab", "Execute:", nullptr));
        _ELOGGroupBox->setTitle(QCoreApplication::translate("EventMonitorTab", "ELOG", nullptr));
        _ELOGNotify->setText(QCoreApplication::translate("EventMonitorTab", "No&tify:", nullptr));
        _ELOGConfigure->setText(QCoreApplication::translate("EventMonitorTab", "&Configure...", nullptr));
        _logButtonGroup->setTitle(QCoreApplication::translate("EventMonitorTab", "Kst Debug Log", nullptr));
        _debugLogWarning->setText(QCoreApplication::translate("EventMonitorTab", "&Warning", nullptr));
        _debugLogError->setText(QCoreApplication::translate("EventMonitorTab", "&Error", nullptr));
        _debugLogNotice->setText(QCoreApplication::translate("EventMonitorTab", "&Notice", nullptr));
        _debugLog->setText(QCoreApplication::translate("EventMonitorTab", "&Log as:", nullptr));
        (void)EventMonitorTab;
    } // retranslateUi

};

namespace Ui {
    class EventMonitorTab: public Ui_EventMonitorTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EVENTMONITORTAB_H
