/********************************************************************************
** Form generated from reading UI file 'differentiatecurvesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIFFERENTIATECURVESDIALOG_H
#define UI_DIFFERENTIATECURVESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DifferentiateCurvesDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *cycleOrderGroup;
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QPushButton *_up;
    QPushButton *_down;
    QPushButton *_add;
    QPushButton *_remove;
    QVBoxLayout *vboxLayout1;
    QLabel *textLabel1;
    QListWidget *_availableListBox;
    QVBoxLayout *vboxLayout2;
    QLabel *textLabel2;
    QListWidget *_selectedListBox;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;
    QGroupBox *optionsGroup;
    QGridLayout *gridLayout2;
    QGridLayout *gridLayout3;
    QSpinBox *_maxLineWidth;
    QLabel *textLabel3;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout4;
    QLabel *textLabel4;
    QComboBox *_pointDensity;
    QSpacerItem *horizontalSpacer_2;
    QDialogButtonBox *_buttonBox;

    void setupUi(QDialog *DifferentiateCurvesDialog)
    {
        if (DifferentiateCurvesDialog->objectName().isEmpty())
            DifferentiateCurvesDialog->setObjectName("DifferentiateCurvesDialog");
        vboxLayout = new QVBoxLayout(DifferentiateCurvesDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        cycleOrderGroup = new QGroupBox(DifferentiateCurvesDialog);
        cycleOrderGroup->setObjectName("cycleOrderGroup");
        gridLayout = new QGridLayout(cycleOrderGroup);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        _up = new QPushButton(cycleOrderGroup);
        _up->setObjectName("_up");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_up->sizePolicy().hasHeightForWidth());
        _up->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_up, 1, 1, 1, 1);

        _down = new QPushButton(cycleOrderGroup);
        _down->setObjectName("_down");
        sizePolicy.setHeightForWidth(_down->sizePolicy().hasHeightForWidth());
        _down->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_down, 3, 1, 1, 1);

        _add = new QPushButton(cycleOrderGroup);
        _add->setObjectName("_add");
        sizePolicy.setHeightForWidth(_add->sizePolicy().hasHeightForWidth());
        _add->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_add, 2, 2, 1, 1);

        _remove = new QPushButton(cycleOrderGroup);
        _remove->setObjectName("_remove");
        sizePolicy.setHeightForWidth(_remove->sizePolicy().hasHeightForWidth());
        _remove->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_remove, 2, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 1, 1, 1, 1);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(cycleOrderGroup);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        vboxLayout1->addWidget(textLabel1);

        _availableListBox = new QListWidget(cycleOrderGroup);
        _availableListBox->setObjectName("_availableListBox");

        vboxLayout1->addWidget(_availableListBox);


        gridLayout->addLayout(vboxLayout1, 0, 0, 3, 1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        textLabel2 = new QLabel(cycleOrderGroup);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        vboxLayout2->addWidget(textLabel2);

        _selectedListBox = new QListWidget(cycleOrderGroup);
        _selectedListBox->setObjectName("_selectedListBox");

        vboxLayout2->addWidget(_selectedListBox);


        gridLayout->addLayout(vboxLayout2, 0, 2, 3, 1);

        spacerItem = new QSpacerItem(21, 61, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 0, 1, 1, 1);

        spacerItem1 = new QSpacerItem(20, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem1, 2, 1, 1, 1);


        vboxLayout->addWidget(cycleOrderGroup);

        optionsGroup = new QGroupBox(DifferentiateCurvesDialog);
        optionsGroup->setObjectName("optionsGroup");
        gridLayout2 = new QGridLayout(optionsGroup);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        gridLayout3 = new QGridLayout();
        gridLayout3->setSpacing(6);
        gridLayout3->setObjectName("gridLayout3");
        gridLayout3->setContentsMargins(0, 0, 0, 0);
        _maxLineWidth = new QSpinBox(optionsGroup);
        _maxLineWidth->setObjectName("_maxLineWidth");
        _maxLineWidth->setMinimum(1);

        gridLayout3->addWidget(_maxLineWidth, 0, 1, 1, 1);

        textLabel3 = new QLabel(optionsGroup);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout3->addWidget(textLabel3, 0, 0, 1, 1);


        gridLayout2->addLayout(gridLayout3, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(horizontalSpacer, 0, 1, 1, 1);

        gridLayout4 = new QGridLayout();
        gridLayout4->setSpacing(6);
        gridLayout4->setObjectName("gridLayout4");
        gridLayout4->setContentsMargins(0, 0, 0, 0);
        textLabel4 = new QLabel(optionsGroup);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout4->addWidget(textLabel4, 0, 0, 1, 1);

        _pointDensity = new QComboBox(optionsGroup);
        _pointDensity->addItem(QString());
        _pointDensity->addItem(QString());
        _pointDensity->addItem(QString());
        _pointDensity->addItem(QString());
        _pointDensity->setObjectName("_pointDensity");

        gridLayout4->addWidget(_pointDensity, 0, 1, 1, 1);


        gridLayout2->addLayout(gridLayout4, 0, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(horizontalSpacer_2, 0, 3, 1, 1);


        vboxLayout->addWidget(optionsGroup);

        _buttonBox = new QDialogButtonBox(DifferentiateCurvesDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(_buttonBox);

        QWidget::setTabOrder(_availableListBox, _up);
        QWidget::setTabOrder(_up, _remove);
        QWidget::setTabOrder(_remove, _add);
        QWidget::setTabOrder(_add, _down);
        QWidget::setTabOrder(_down, _selectedListBox);
        QWidget::setTabOrder(_selectedListBox, _maxLineWidth);
        QWidget::setTabOrder(_maxLineWidth, _pointDensity);
        QWidget::setTabOrder(_pointDensity, _buttonBox);

        retranslateUi(DifferentiateCurvesDialog);

        QMetaObject::connectSlotsByName(DifferentiateCurvesDialog);
    } // setupUi

    void retranslateUi(QDialog *DifferentiateCurvesDialog)
    {
        DifferentiateCurvesDialog->setWindowTitle(QCoreApplication::translate("DifferentiateCurvesDialog", "Differentiate Curves", nullptr));
        cycleOrderGroup->setTitle(QCoreApplication::translate("DifferentiateCurvesDialog", "Property Selection and Cycle Order", nullptr));
#if QT_CONFIG(tooltip)
        _up->setToolTip(QCoreApplication::translate("DifferentiateCurvesDialog", "Raise in Selection Order: Alt+up", nullptr));
#endif // QT_CONFIG(tooltip)
        _up->setText(QString());
#if QT_CONFIG(shortcut)
        _up->setShortcut(QCoreApplication::translate("DifferentiateCurvesDialog", "Alt+Up", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        _down->setToolTip(QCoreApplication::translate("DifferentiateCurvesDialog", "Lower in Selection Order: Alt+down", nullptr));
#endif // QT_CONFIG(tooltip)
        _down->setText(QString());
#if QT_CONFIG(shortcut)
        _down->setShortcut(QCoreApplication::translate("DifferentiateCurvesDialog", "Alt+Down", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        _add->setToolTip(QCoreApplication::translate("DifferentiateCurvesDialog", "Select: Alt+s", nullptr));
#endif // QT_CONFIG(tooltip)
        _add->setText(QString());
#if QT_CONFIG(tooltip)
        _remove->setToolTip(QCoreApplication::translate("DifferentiateCurvesDialog", "Remove: Alt+r", nullptr));
#endif // QT_CONFIG(tooltip)
        _remove->setText(QString());
        textLabel1->setText(QCoreApplication::translate("DifferentiateCurvesDialog", "Available:", "The items which are available"));
        textLabel2->setText(QCoreApplication::translate("DifferentiateCurvesDialog", "Selected:", "the items which have been selected"));
        optionsGroup->setTitle(QCoreApplication::translate("DifferentiateCurvesDialog", "Options", nullptr));
        textLabel3->setText(QCoreApplication::translate("DifferentiateCurvesDialog", "Maximum line width:", nullptr));
        textLabel4->setText(QCoreApplication::translate("DifferentiateCurvesDialog", "Point density:", nullptr));
        _pointDensity->setItemText(0, QCoreApplication::translate("DifferentiateCurvesDialog", "All", "all of the points"));
        _pointDensity->setItemText(1, QCoreApplication::translate("DifferentiateCurvesDialog", "High", "high point density"));
        _pointDensity->setItemText(2, QCoreApplication::translate("DifferentiateCurvesDialog", "Medium", "medium point density"));
        _pointDensity->setItemText(3, QCoreApplication::translate("DifferentiateCurvesDialog", "Low", "low point density"));

    } // retranslateUi

};

namespace Ui {
    class DifferentiateCurvesDialog: public Ui_DifferentiateCurvesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIFFERENTIATECURVESDIALOG_H
