/********************************************************************************
** Form generated from reading UI file 'powerspectrumtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_POWERSPECTRUMTAB_H
#define UI_POWERSPECTRUMTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "curveappearance.h"
#include "curveplacement.h"
#include "fftoptions.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_PowerSpectrumTab
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *_vectorLabel;
    Kst::VectorSelector *_vector;
    Kst::FFTOptions *_FFTOptions;
    Kst::CurveAppearance *_curveAppearance;
    Kst::CurvePlacement *_curvePlacement;

    void setupUi(QWidget *PowerSpectrumTab)
    {
        if (PowerSpectrumTab->objectName().isEmpty())
            PowerSpectrumTab->setObjectName("PowerSpectrumTab");
        PowerSpectrumTab->resize(687, 525);
        gridLayout = new QGridLayout(PowerSpectrumTab);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        groupBox = new QGroupBox(PowerSpectrumTab);
        groupBox->setObjectName("groupBox");
        horizontalLayout_2 = new QHBoxLayout(groupBox);
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        _vectorLabel = new QLabel(groupBox);
        _vectorLabel->setObjectName("_vectorLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_vectorLabel->sizePolicy().hasHeightForWidth());
        _vectorLabel->setSizePolicy(sizePolicy);
        _vectorLabel->setWordWrap(false);

        horizontalLayout->addWidget(_vectorLabel);

        _vector = new Kst::VectorSelector(groupBox);
        _vector->setObjectName("_vector");

        horizontalLayout->addWidget(_vector);


        horizontalLayout_2->addLayout(horizontalLayout);


        gridLayout->addWidget(groupBox, 0, 0, 1, 2);

        _FFTOptions = new Kst::FFTOptions(PowerSpectrumTab);
        _FFTOptions->setObjectName("_FFTOptions");

        gridLayout->addWidget(_FFTOptions, 1, 0, 1, 2);

        _curveAppearance = new Kst::CurveAppearance(PowerSpectrumTab);
        _curveAppearance->setObjectName("_curveAppearance");

        gridLayout->addWidget(_curveAppearance, 2, 0, 1, 1);

        _curvePlacement = new Kst::CurvePlacement(PowerSpectrumTab);
        _curvePlacement->setObjectName("_curvePlacement");

        gridLayout->addWidget(_curvePlacement, 2, 1, 1, 1);

        QWidget::setTabOrder(_FFTOptions, _curveAppearance);
        QWidget::setTabOrder(_curveAppearance, _curvePlacement);

        retranslateUi(PowerSpectrumTab);

        QMetaObject::connectSlotsByName(PowerSpectrumTab);
    } // setupUi

    void retranslateUi(QWidget *PowerSpectrumTab)
    {
        groupBox->setTitle(QCoreApplication::translate("PowerSpectrumTab", "Power Spectrum Contents", nullptr));
        _vectorLabel->setText(QCoreApplication::translate("PowerSpectrumTab", "Data vecto&r:", nullptr));
        (void)PowerSpectrumTab;
    } // retranslateUi

};

namespace Ui {
    class PowerSpectrumTab: public Ui_PowerSpectrumTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_POWERSPECTRUMTAB_H
