/********************************************************************************
** Form generated from reading UI file 'matrixtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MATRIXTAB_H
#define UI_MATRIXTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "combobox.h"
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_MatrixTab
{
public:
    QGridLayout *gridLayout;
    QGroupBox *_scalingGroup;
    QGridLayout *gridLayout1;
    QLabel *textLabel7;
    QLabel *textLabel5_2_2;
    QVBoxLayout *vboxLayout;
    QLineEdit *_minX;
    QLineEdit *_xStep;
    QLabel *textLabel6;
    QLabel *textLabel7_2;
    QVBoxLayout *vboxLayout1;
    QLineEdit *_minY;
    QLineEdit *_yStep;
    QGroupBox *_gradientGroup;
    QGridLayout *gridLayout2;
    QGridLayout *QGridLayout1;
    QLabel *textLabel5;
    QRadioButton *_gradientX;
    QSpacerItem *spacerItem;
    QRadioButton *_gradientY;
    QGridLayout *QGridLayout2;
    QSpinBox *_nX;
    QSpinBox *_nY;
    QLineEdit *_gradientZAtMin;
    QLabel *textLabel6_2;
    QLabel *textLabel8;
    QLabel *textLabel8_2;
    QLabel *textLabel7_3;
    QLineEdit *_gradientZAtMax;
    QGroupBox *_partialMatrixGroup;
    QGridLayout *gridLayout3;
    QGridLayout *QGridLayout3;
    QLabel *textLabel4_4;
    QSpinBox *_xNumSteps;
    QLabel *textLabel3;
    QLabel *textLabel4;
    QLabel *textLabel2_2;
    QSpinBox *_xStart;
    QSpinBox *_yNumSteps;
    QCheckBox *_xStartCountFromEnd;
    QCheckBox *_xNumStepsReadToEnd;
    QLabel *textLabel4_2;
    QLabel *textLabel3_2;
    QCheckBox *_yNumStepsReadToEnd;
    QCheckBox *_yStartCountFromEnd;
    QLabel *textLabel4_3;
    QSpinBox *_yStart;
    QLabel *textLabel2;
    QGridLayout *QGridLayout4;
    QSpinBox *_skip;
    QCheckBox *_doAverage;
    QCheckBox *_doSkip;
    QGroupBox *_sourceGroup;
    QGridLayout *gridLayout4;
    QRadioButton *_readFromSource;
    QRadioButton *_generateGradient;
    QSpacerItem *spacerItem1;
    QGroupBox *_dataSourceGroup;
    QGridLayout *gridLayout5;
    QLabel *TextLabel4_3;
    Kst::DataSourceSelector *_fileName;
    QLabel *TextLabel6;
    Kst::ComboBox *_field;
    QLabel *_frameLabel;
    QSpinBox *_frame;
    QCheckBox *_lastFrame;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *horizontalSpacer;
    QPushButton *_configure;

    void setupUi(QWidget *MatrixTab)
    {
        if (MatrixTab->objectName().isEmpty())
            MatrixTab->setObjectName("MatrixTab");
        MatrixTab->resize(512, 644);
        gridLayout = new QGridLayout(MatrixTab);
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(9, 9, 9, 9);
        _scalingGroup = new QGroupBox(MatrixTab);
        _scalingGroup->setObjectName("_scalingGroup");
        _scalingGroup->setCheckable(true);
        _scalingGroup->setChecked(false);
        gridLayout1 = new QGridLayout(_scalingGroup);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(-1, 0, -1, 0);
        textLabel7 = new QLabel(_scalingGroup);
        textLabel7->setObjectName("textLabel7");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel7->sizePolicy().hasHeightForWidth());
        textLabel7->setSizePolicy(sizePolicy);
        textLabel7->setWordWrap(false);

        gridLayout1->addWidget(textLabel7, 1, 0, 1, 1);

        textLabel5_2_2 = new QLabel(_scalingGroup);
        textLabel5_2_2->setObjectName("textLabel5_2_2");
        sizePolicy.setHeightForWidth(textLabel5_2_2->sizePolicy().hasHeightForWidth());
        textLabel5_2_2->setSizePolicy(sizePolicy);
        textLabel5_2_2->setWordWrap(false);

        gridLayout1->addWidget(textLabel5_2_2, 0, 0, 1, 1);

        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        _minX = new QLineEdit(_scalingGroup);
        _minX->setObjectName("_minX");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_minX->sizePolicy().hasHeightForWidth());
        _minX->setSizePolicy(sizePolicy1);
        _minX->setMinimumSize(QSize(60, 0));
        _minX->setText(QString::fromUtf8("0"));

        vboxLayout->addWidget(_minX);

        _xStep = new QLineEdit(_scalingGroup);
        _xStep->setObjectName("_xStep");
        sizePolicy1.setHeightForWidth(_xStep->sizePolicy().hasHeightForWidth());
        _xStep->setSizePolicy(sizePolicy1);
        _xStep->setMinimumSize(QSize(60, 0));
        _xStep->setText(QString::fromUtf8("1"));

        vboxLayout->addWidget(_xStep);


        gridLayout1->addLayout(vboxLayout, 0, 1, 2, 1);

        textLabel6 = new QLabel(_scalingGroup);
        textLabel6->setObjectName("textLabel6");
        sizePolicy.setHeightForWidth(textLabel6->sizePolicy().hasHeightForWidth());
        textLabel6->setSizePolicy(sizePolicy);
        textLabel6->setWordWrap(false);

        gridLayout1->addWidget(textLabel6, 0, 2, 1, 1);

        textLabel7_2 = new QLabel(_scalingGroup);
        textLabel7_2->setObjectName("textLabel7_2");
        sizePolicy.setHeightForWidth(textLabel7_2->sizePolicy().hasHeightForWidth());
        textLabel7_2->setSizePolicy(sizePolicy);
        textLabel7_2->setWordWrap(false);

        gridLayout1->addWidget(textLabel7_2, 1, 2, 1, 1);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        _minY = new QLineEdit(_scalingGroup);
        _minY->setObjectName("_minY");
        sizePolicy1.setHeightForWidth(_minY->sizePolicy().hasHeightForWidth());
        _minY->setSizePolicy(sizePolicy1);
        _minY->setMinimumSize(QSize(60, 0));
        _minY->setText(QString::fromUtf8("0"));

        vboxLayout1->addWidget(_minY);

        _yStep = new QLineEdit(_scalingGroup);
        _yStep->setObjectName("_yStep");
        sizePolicy1.setHeightForWidth(_yStep->sizePolicy().hasHeightForWidth());
        _yStep->setSizePolicy(sizePolicy1);
        _yStep->setMinimumSize(QSize(60, 0));
        _yStep->setText(QString::fromUtf8("1"));

        vboxLayout1->addWidget(_yStep);


        gridLayout1->addLayout(vboxLayout1, 0, 3, 2, 1);


        gridLayout->addWidget(_scalingGroup, 4, 0, 1, 1);

        _gradientGroup = new QGroupBox(MatrixTab);
        _gradientGroup->setObjectName("_gradientGroup");
        _gradientGroup->setEnabled(false);
        gridLayout2 = new QGridLayout(_gradientGroup);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(-1, 0, -1, 0);
        QGridLayout1 = new QGridLayout();
        QGridLayout1->setSpacing(6);
        QGridLayout1->setObjectName("QGridLayout1");
        textLabel5 = new QLabel(_gradientGroup);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        QGridLayout1->addWidget(textLabel5, 0, 0, 1, 1);

        _gradientX = new QRadioButton(_gradientGroup);
        _gradientX->setObjectName("_gradientX");
        _gradientX->setChecked(true);

        QGridLayout1->addWidget(_gradientX, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        QGridLayout1->addItem(spacerItem, 0, 3, 1, 1);

        _gradientY = new QRadioButton(_gradientGroup);
        _gradientY->setObjectName("_gradientY");

        QGridLayout1->addWidget(_gradientY, 0, 2, 1, 1);


        gridLayout2->addLayout(QGridLayout1, 0, 0, 1, 1);

        QGridLayout2 = new QGridLayout();
        QGridLayout2->setSpacing(6);
        QGridLayout2->setObjectName("QGridLayout2");
        _nX = new QSpinBox(_gradientGroup);
        _nX->setObjectName("_nX");
        _nX->setMinimum(1);
        _nX->setMaximum(999999999);
        _nX->setValue(100);

        QGridLayout2->addWidget(_nX, 1, 1, 1, 1);

        _nY = new QSpinBox(_gradientGroup);
        _nY->setObjectName("_nY");
        _nY->setMaximum(999999999);
        _nY->setValue(100);

        QGridLayout2->addWidget(_nY, 1, 3, 1, 1);

        _gradientZAtMin = new QLineEdit(_gradientGroup);
        _gradientZAtMin->setObjectName("_gradientZAtMin");
        _gradientZAtMin->setText(QString::fromUtf8(""));

        QGridLayout2->addWidget(_gradientZAtMin, 0, 1, 1, 1);

        textLabel6_2 = new QLabel(_gradientGroup);
        textLabel6_2->setObjectName("textLabel6_2");
        textLabel6_2->setWordWrap(false);

        QGridLayout2->addWidget(textLabel6_2, 0, 0, 1, 1);

        textLabel8 = new QLabel(_gradientGroup);
        textLabel8->setObjectName("textLabel8");
        textLabel8->setWordWrap(false);

        QGridLayout2->addWidget(textLabel8, 1, 0, 1, 1);

        textLabel8_2 = new QLabel(_gradientGroup);
        textLabel8_2->setObjectName("textLabel8_2");
        textLabel8_2->setWordWrap(false);

        QGridLayout2->addWidget(textLabel8_2, 1, 2, 1, 1);

        textLabel7_3 = new QLabel(_gradientGroup);
        textLabel7_3->setObjectName("textLabel7_3");
        textLabel7_3->setWordWrap(false);

        QGridLayout2->addWidget(textLabel7_3, 0, 2, 1, 1);

        _gradientZAtMax = new QLineEdit(_gradientGroup);
        _gradientZAtMax->setObjectName("_gradientZAtMax");
        _gradientZAtMax->setText(QString::fromUtf8("100"));

        QGridLayout2->addWidget(_gradientZAtMax, 0, 3, 1, 1);


        gridLayout2->addLayout(QGridLayout2, 1, 0, 1, 1);


        gridLayout->addWidget(_gradientGroup, 3, 0, 1, 1);

        _partialMatrixGroup = new QGroupBox(MatrixTab);
        _partialMatrixGroup->setObjectName("_partialMatrixGroup");
        _partialMatrixGroup->setEnabled(false);
        gridLayout3 = new QGridLayout(_partialMatrixGroup);
        gridLayout3->setSpacing(6);
        gridLayout3->setContentsMargins(11, 11, 11, 11);
        gridLayout3->setObjectName("gridLayout3");
        gridLayout3->setContentsMargins(-1, 0, -1, 0);
        QGridLayout3 = new QGridLayout();
        QGridLayout3->setSpacing(6);
        QGridLayout3->setObjectName("QGridLayout3");
        textLabel4_4 = new QLabel(_partialMatrixGroup);
        textLabel4_4->setObjectName("textLabel4_4");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(textLabel4_4->sizePolicy().hasHeightForWidth());
        textLabel4_4->setSizePolicy(sizePolicy2);
        textLabel4_4->setWordWrap(false);

        QGridLayout3->addWidget(textLabel4_4, 3, 2, 1, 1);

        _xNumSteps = new QSpinBox(_partialMatrixGroup);
        _xNumSteps->setObjectName("_xNumSteps");
        _xNumSteps->setEnabled(false);
        _xNumSteps->setMinimum(1);
        _xNumSteps->setMaximum(999999999);

        QGridLayout3->addWidget(_xNumSteps, 2, 1, 1, 1);

        textLabel3 = new QLabel(_partialMatrixGroup);
        textLabel3->setObjectName("textLabel3");
        sizePolicy2.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy2);
        textLabel3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3->setWordWrap(false);

        QGridLayout3->addWidget(textLabel3, 1, 0, 1, 1);

        textLabel4 = new QLabel(_partialMatrixGroup);
        textLabel4->setObjectName("textLabel4");
        sizePolicy2.setHeightForWidth(textLabel4->sizePolicy().hasHeightForWidth());
        textLabel4->setSizePolicy(sizePolicy2);
        textLabel4->setWordWrap(false);

        QGridLayout3->addWidget(textLabel4, 0, 2, 1, 1);

        textLabel2_2 = new QLabel(_partialMatrixGroup);
        textLabel2_2->setObjectName("textLabel2_2");
        sizePolicy2.setHeightForWidth(textLabel2_2->sizePolicy().hasHeightForWidth());
        textLabel2_2->setSizePolicy(sizePolicy2);
        textLabel2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2_2->setWordWrap(false);

        QGridLayout3->addWidget(textLabel2_2, 2, 0, 1, 1);

        _xStart = new QSpinBox(_partialMatrixGroup);
        _xStart->setObjectName("_xStart");
        _xStart->setMaximum(999999999);

        QGridLayout3->addWidget(_xStart, 0, 1, 1, 1);

        _yNumSteps = new QSpinBox(_partialMatrixGroup);
        _yNumSteps->setObjectName("_yNumSteps");
        _yNumSteps->setEnabled(false);
        _yNumSteps->setMinimum(1);
        _yNumSteps->setMaximum(999999999);

        QGridLayout3->addWidget(_yNumSteps, 3, 1, 1, 1);

        _xStartCountFromEnd = new QCheckBox(_partialMatrixGroup);
        _xStartCountFromEnd->setObjectName("_xStartCountFromEnd");
        sizePolicy2.setHeightForWidth(_xStartCountFromEnd->sizePolicy().hasHeightForWidth());
        _xStartCountFromEnd->setSizePolicy(sizePolicy2);

        QGridLayout3->addWidget(_xStartCountFromEnd, 0, 3, 1, 1);

        _xNumStepsReadToEnd = new QCheckBox(_partialMatrixGroup);
        _xNumStepsReadToEnd->setObjectName("_xNumStepsReadToEnd");
        sizePolicy2.setHeightForWidth(_xNumStepsReadToEnd->sizePolicy().hasHeightForWidth());
        _xNumStepsReadToEnd->setSizePolicy(sizePolicy2);
        _xNumStepsReadToEnd->setChecked(true);

        QGridLayout3->addWidget(_xNumStepsReadToEnd, 2, 3, 1, 1);

        textLabel4_2 = new QLabel(_partialMatrixGroup);
        textLabel4_2->setObjectName("textLabel4_2");
        sizePolicy2.setHeightForWidth(textLabel4_2->sizePolicy().hasHeightForWidth());
        textLabel4_2->setSizePolicy(sizePolicy2);
        textLabel4_2->setWordWrap(false);

        QGridLayout3->addWidget(textLabel4_2, 1, 2, 1, 1);

        textLabel3_2 = new QLabel(_partialMatrixGroup);
        textLabel3_2->setObjectName("textLabel3_2");
        sizePolicy2.setHeightForWidth(textLabel3_2->sizePolicy().hasHeightForWidth());
        textLabel3_2->setSizePolicy(sizePolicy2);
        textLabel3_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3_2->setWordWrap(false);

        QGridLayout3->addWidget(textLabel3_2, 3, 0, 1, 1);

        _yNumStepsReadToEnd = new QCheckBox(_partialMatrixGroup);
        _yNumStepsReadToEnd->setObjectName("_yNumStepsReadToEnd");
        sizePolicy2.setHeightForWidth(_yNumStepsReadToEnd->sizePolicy().hasHeightForWidth());
        _yNumStepsReadToEnd->setSizePolicy(sizePolicy2);
        _yNumStepsReadToEnd->setChecked(true);

        QGridLayout3->addWidget(_yNumStepsReadToEnd, 3, 3, 1, 1);

        _yStartCountFromEnd = new QCheckBox(_partialMatrixGroup);
        _yStartCountFromEnd->setObjectName("_yStartCountFromEnd");
        sizePolicy2.setHeightForWidth(_yStartCountFromEnd->sizePolicy().hasHeightForWidth());
        _yStartCountFromEnd->setSizePolicy(sizePolicy2);

        QGridLayout3->addWidget(_yStartCountFromEnd, 1, 3, 1, 1);

        textLabel4_3 = new QLabel(_partialMatrixGroup);
        textLabel4_3->setObjectName("textLabel4_3");
        sizePolicy2.setHeightForWidth(textLabel4_3->sizePolicy().hasHeightForWidth());
        textLabel4_3->setSizePolicy(sizePolicy2);
        textLabel4_3->setWordWrap(false);

        QGridLayout3->addWidget(textLabel4_3, 2, 2, 1, 1);

        _yStart = new QSpinBox(_partialMatrixGroup);
        _yStart->setObjectName("_yStart");
        _yStart->setMaximum(999999999);

        QGridLayout3->addWidget(_yStart, 1, 1, 1, 1);

        textLabel2 = new QLabel(_partialMatrixGroup);
        textLabel2->setObjectName("textLabel2");
        sizePolicy2.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy2);
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2->setWordWrap(false);

        QGridLayout3->addWidget(textLabel2, 0, 0, 1, 1);


        gridLayout3->addLayout(QGridLayout3, 0, 0, 1, 1);

        QGridLayout4 = new QGridLayout();
        QGridLayout4->setSpacing(6);
        QGridLayout4->setObjectName("QGridLayout4");
        _skip = new QSpinBox(_partialMatrixGroup);
        _skip->setObjectName("_skip");
        _skip->setEnabled(false);
        _skip->setMinimum(1);
        _skip->setMaximum(999999999);

        QGridLayout4->addWidget(_skip, 0, 1, 1, 1);

        _doAverage = new QCheckBox(_partialMatrixGroup);
        _doAverage->setObjectName("_doAverage");
        _doAverage->setEnabled(false);

        QGridLayout4->addWidget(_doAverage, 0, 2, 1, 1);

        _doSkip = new QCheckBox(_partialMatrixGroup);
        _doSkip->setObjectName("_doSkip");

        QGridLayout4->addWidget(_doSkip, 0, 0, 1, 1);


        gridLayout3->addLayout(QGridLayout4, 1, 0, 1, 1);


        gridLayout->addWidget(_partialMatrixGroup, 2, 0, 1, 1);

        _sourceGroup = new QGroupBox(MatrixTab);
        _sourceGroup->setObjectName("_sourceGroup");
        _sourceGroup->setEnabled(false);
        gridLayout4 = new QGridLayout(_sourceGroup);
        gridLayout4->setSpacing(6);
        gridLayout4->setContentsMargins(11, 11, 11, 11);
        gridLayout4->setObjectName("gridLayout4");
        gridLayout4->setContentsMargins(-1, 0, -1, 0);
        _readFromSource = new QRadioButton(_sourceGroup);
        _readFromSource->setObjectName("_readFromSource");
        _readFromSource->setChecked(true);

        gridLayout4->addWidget(_readFromSource, 0, 0, 1, 1);

        _generateGradient = new QRadioButton(_sourceGroup);
        _generateGradient->setObjectName("_generateGradient");

        gridLayout4->addWidget(_generateGradient, 0, 1, 1, 1);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout4->addItem(spacerItem1, 0, 2, 1, 1);


        gridLayout->addWidget(_sourceGroup, 0, 0, 1, 1);

        _dataSourceGroup = new QGroupBox(MatrixTab);
        _dataSourceGroup->setObjectName("_dataSourceGroup");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_dataSourceGroup->sizePolicy().hasHeightForWidth());
        _dataSourceGroup->setSizePolicy(sizePolicy3);
        gridLayout5 = new QGridLayout(_dataSourceGroup);
        gridLayout5->setSpacing(6);
        gridLayout5->setContentsMargins(11, 11, 11, 11);
        gridLayout5->setObjectName("gridLayout5");
        TextLabel4_3 = new QLabel(_dataSourceGroup);
        TextLabel4_3->setObjectName("TextLabel4_3");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(TextLabel4_3->sizePolicy().hasHeightForWidth());
        TextLabel4_3->setSizePolicy(sizePolicy4);
        TextLabel4_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel4_3->setWordWrap(false);

        gridLayout5->addWidget(TextLabel4_3, 0, 0, 1, 1);

        _fileName = new Kst::DataSourceSelector(_dataSourceGroup);
        _fileName->setObjectName("_fileName");

        gridLayout5->addWidget(_fileName, 0, 1, 1, 2);

        TextLabel6 = new QLabel(_dataSourceGroup);
        TextLabel6->setObjectName("TextLabel6");
        sizePolicy2.setHeightForWidth(TextLabel6->sizePolicy().hasHeightForWidth());
        TextLabel6->setSizePolicy(sizePolicy2);
        TextLabel6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel6->setWordWrap(false);

        gridLayout5->addWidget(TextLabel6, 1, 0, 1, 1);

        _field = new Kst::ComboBox(_dataSourceGroup);
        _field->setObjectName("_field");
        _field->setEnabled(false);
        _field->setEditable(true);
        _field->setDuplicatesEnabled(false);

        gridLayout5->addWidget(_field, 1, 1, 1, 2);

        _frameLabel = new QLabel(_dataSourceGroup);
        _frameLabel->setObjectName("_frameLabel");
        sizePolicy4.setHeightForWidth(_frameLabel->sizePolicy().hasHeightForWidth());
        _frameLabel->setSizePolicy(sizePolicy4);
        _frameLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout5->addWidget(_frameLabel, 2, 0, 1, 1);

        _frame = new QSpinBox(_dataSourceGroup);
        _frame->setObjectName("_frame");
        _frame->setMaximum(10000000);

        gridLayout5->addWidget(_frame, 2, 1, 1, 1);

        _lastFrame = new QCheckBox(_dataSourceGroup);
        _lastFrame->setObjectName("_lastFrame");

        gridLayout5->addWidget(_lastFrame, 2, 2, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(horizontalSpacer);

        _configure = new QPushButton(_dataSourceGroup);
        _configure->setObjectName("_configure");
        _configure->setEnabled(false);

        hboxLayout1->addWidget(_configure);


        gridLayout5->addLayout(hboxLayout1, 3, 0, 1, 3);


        gridLayout->addWidget(_dataSourceGroup, 1, 0, 1, 1);

#if QT_CONFIG(shortcut)
        textLabel7->setBuddy(_xStep);
        textLabel5_2_2->setBuddy(_minX);
        textLabel6->setBuddy(_minY);
        textLabel7_2->setBuddy(_yStep);
        textLabel6_2->setBuddy(_gradientZAtMin);
        textLabel8->setBuddy(_nX);
        textLabel8_2->setBuddy(_nY);
        textLabel7_3->setBuddy(_gradientZAtMax);
        textLabel3->setBuddy(_yStart);
        textLabel2_2->setBuddy(_xNumSteps);
        textLabel3_2->setBuddy(_yNumSteps);
        textLabel2->setBuddy(_xStart);
        TextLabel4_3->setBuddy(_fileName);
        TextLabel6->setBuddy(_field);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_readFromSource, _generateGradient);
        QWidget::setTabOrder(_generateGradient, _field);
        QWidget::setTabOrder(_field, _xStart);
        QWidget::setTabOrder(_xStart, _xStartCountFromEnd);
        QWidget::setTabOrder(_xStartCountFromEnd, _yStart);
        QWidget::setTabOrder(_yStart, _yStartCountFromEnd);
        QWidget::setTabOrder(_yStartCountFromEnd, _xNumSteps);
        QWidget::setTabOrder(_xNumSteps, _xNumStepsReadToEnd);
        QWidget::setTabOrder(_xNumStepsReadToEnd, _yNumSteps);
        QWidget::setTabOrder(_yNumSteps, _yNumStepsReadToEnd);
        QWidget::setTabOrder(_yNumStepsReadToEnd, _doSkip);
        QWidget::setTabOrder(_doSkip, _skip);
        QWidget::setTabOrder(_skip, _doAverage);
        QWidget::setTabOrder(_doAverage, _gradientX);
        QWidget::setTabOrder(_gradientX, _gradientY);
        QWidget::setTabOrder(_gradientY, _gradientZAtMin);
        QWidget::setTabOrder(_gradientZAtMin, _gradientZAtMax);
        QWidget::setTabOrder(_gradientZAtMax, _nX);
        QWidget::setTabOrder(_nX, _nY);
        QWidget::setTabOrder(_nY, _minX);
        QWidget::setTabOrder(_minX, _xStep);
        QWidget::setTabOrder(_xStep, _minY);
        QWidget::setTabOrder(_minY, _yStep);

        retranslateUi(MatrixTab);
        QObject::connect(_lastFrame, &QCheckBox::toggled, _frame, &QSpinBox::setDisabled);
        QObject::connect(_lastFrame, &QCheckBox::toggled, _frameLabel, &QLabel::setDisabled);

        QMetaObject::connectSlotsByName(MatrixTab);
    } // setupUi

    void retranslateUi(QWidget *MatrixTab)
    {
        _scalingGroup->setTitle(QCoreApplication::translate("MatrixTab", "Override Scaling and Translation", nullptr));
        textLabel7->setText(QCoreApplication::translate("MatrixTab", "X step si&ze:", nullptr));
        textLabel5_2_2->setText(QCoreApplication::translate("MatrixTab", "&X minimum:", nullptr));
        textLabel6->setText(QCoreApplication::translate("MatrixTab", "&Y minimum:", nullptr));
        textLabel7_2->setText(QCoreApplication::translate("MatrixTab", "Y ste&p size:", nullptr));
        _gradientGroup->setTitle(QCoreApplication::translate("MatrixTab", "Gradient Parameters", nullptr));
        textLabel5->setText(QCoreApplication::translate("MatrixTab", "Increment/decrement Z values along:", nullptr));
        _gradientX->setText(QCoreApplication::translate("MatrixTab", "X-axis", nullptr));
        _gradientY->setText(QCoreApplication::translate("MatrixTab", "Y-axis", nullptr));
        textLabel6_2->setText(QCoreApplication::translate("MatrixTab", "Z at minimum:", nullptr));
        textLabel8->setText(QCoreApplication::translate("MatrixTab", "X steps:", nullptr));
        textLabel8_2->setText(QCoreApplication::translate("MatrixTab", "Y steps:", nullptr));
        textLabel7_3->setText(QCoreApplication::translate("MatrixTab", "Z at maximum:", nullptr));
        _partialMatrixGroup->setTitle(QCoreApplication::translate("MatrixTab", "Partial Matrix", nullptr));
        textLabel4_4->setText(QCoreApplication::translate("MatrixTab", "or", nullptr));
        textLabel3->setText(QCoreApplication::translate("MatrixTab", "Y s&tarting frame:", nullptr));
        textLabel4->setText(QCoreApplication::translate("MatrixTab", "or", nullptr));
        textLabel2_2->setText(QCoreApplication::translate("MatrixTab", "X n&umber of frames:", nullptr));
        _xStartCountFromEnd->setText(QCoreApplication::translate("MatrixTab", "C&ount from end", nullptr));
        _xNumStepsReadToEnd->setText(QCoreApplication::translate("MatrixTab", "&Read to end", nullptr));
        textLabel4_2->setText(QCoreApplication::translate("MatrixTab", "or", nullptr));
        textLabel3_2->setText(QCoreApplication::translate("MatrixTab", "Y nu&mber of frames:", nullptr));
        _yNumStepsReadToEnd->setText(QCoreApplication::translate("MatrixTab", "R&ead to end", nullptr));
        _yStartCountFromEnd->setText(QCoreApplication::translate("MatrixTab", "Count from en&d", nullptr));
        textLabel4_3->setText(QCoreApplication::translate("MatrixTab", "or", nullptr));
        textLabel2->setText(QCoreApplication::translate("MatrixTab", "X &starting frame:", nullptr));
        _skip->setSuffix(QCoreApplication::translate("MatrixTab", " frames", nullptr));
        _doAverage->setText(QCoreApplication::translate("MatrixTab", "Boxcar &filter first", nullptr));
        _doSkip->setText(QCoreApplication::translate("MatrixTab", "Read &1 sample per", nullptr));
        _sourceGroup->setTitle(QCoreApplication::translate("MatrixTab", "Source", "the source of the item"));
        _readFromSource->setText(QCoreApplication::translate("MatrixTab", "Read from data source", nullptr));
        _generateGradient->setText(QCoreApplication::translate("MatrixTab", "Generate gradient", nullptr));
        _dataSourceGroup->setTitle(QCoreApplication::translate("MatrixTab", "Data Source Parameters", nullptr));
        TextLabel4_3->setText(QCoreApplication::translate("MatrixTab", "File &name:", nullptr));
        TextLabel6->setText(QCoreApplication::translate("MatrixTab", "Field or &column:", nullptr));
        _frameLabel->setText(QCoreApplication::translate("MatrixTab", "Frame:", nullptr));
        _lastFrame->setText(QCoreApplication::translate("MatrixTab", "Last Frame", nullptr));
        _configure->setText(QCoreApplication::translate("MatrixTab", "Co&nfigure...", nullptr));
        (void)MatrixTab;
    } // retranslateUi

};

namespace Ui {
    class MatrixTab: public Ui_MatrixTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MATRIXTAB_H
