/********************************************************************************
** Form generated from reading UI file 'filterfittab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERFITTAB_H
#define UI_FILTERFITTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "curveappearance.h"
#include "curveplacement.h"

QT_BEGIN_NAMESPACE

class Ui_FilterFitTab
{
public:
    QVBoxLayout *verticalLayout_3;
    QGroupBox *GroupBox9_3;
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QComboBox *_pluginCombo;
    QLabel *label;
    QLabel *_pluginLabel;
    QLabel *label_3;
    QLabel *_descriptionLabel;
    QGroupBox *_inputOutputBox;
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout;
    Kst::CurveAppearance *_curveAppearance;
    QCheckBox *_ignoreAutoScale;
    Kst::CurvePlacement *_curvePlacement;

    void setupUi(QWidget *FilterFitTab)
    {
        if (FilterFitTab->objectName().isEmpty())
            FilterFitTab->setObjectName("FilterFitTab");
        FilterFitTab->resize(609, 310);
        verticalLayout_3 = new QVBoxLayout(FilterFitTab);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        GroupBox9_3 = new QGroupBox(FilterFitTab);
        GroupBox9_3->setObjectName("GroupBox9_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(GroupBox9_3->sizePolicy().hasHeightForWidth());
        GroupBox9_3->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(GroupBox9_3);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName("gridLayout1");
        _pluginCombo = new QComboBox(GroupBox9_3);
        _pluginCombo->setObjectName("_pluginCombo");

        gridLayout1->addWidget(_pluginCombo, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 2);

        label = new QLabel(GroupBox9_3);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        _pluginLabel = new QLabel(GroupBox9_3);
        _pluginLabel->setObjectName("_pluginLabel");

        gridLayout->addWidget(_pluginLabel, 1, 1, 1, 1);

        label_3 = new QLabel(GroupBox9_3);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        _descriptionLabel = new QLabel(GroupBox9_3);
        _descriptionLabel->setObjectName("_descriptionLabel");

        gridLayout->addWidget(_descriptionLabel, 2, 1, 1, 1);


        verticalLayout_3->addWidget(GroupBox9_3);

        _inputOutputBox = new QGroupBox(FilterFitTab);
        _inputOutputBox->setObjectName("_inputOutputBox");

        verticalLayout_3->addWidget(_inputOutputBox);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        _curveAppearance = new Kst::CurveAppearance(FilterFitTab);
        _curveAppearance->setObjectName("_curveAppearance");

        verticalLayout->addWidget(_curveAppearance);

        _ignoreAutoScale = new QCheckBox(FilterFitTab);
        _ignoreAutoScale->setObjectName("_ignoreAutoScale");

        verticalLayout->addWidget(_ignoreAutoScale);


        horizontalLayout_4->addLayout(verticalLayout);

        _curvePlacement = new Kst::CurvePlacement(FilterFitTab);
        _curvePlacement->setObjectName("_curvePlacement");

        horizontalLayout_4->addWidget(_curvePlacement);


        verticalLayout_3->addLayout(horizontalLayout_4);


        retranslateUi(FilterFitTab);

        QMetaObject::connectSlotsByName(FilterFitTab);
    } // setupUi

    void retranslateUi(QWidget *FilterFitTab)
    {
        GroupBox9_3->setTitle(QCoreApplication::translate("FilterFitTab", "Plugin Selection", nullptr));
        label->setText(QCoreApplication::translate("FilterFitTab", "Plugin name:", nullptr));
        _pluginLabel->setText(QString());
        label_3->setText(QCoreApplication::translate("FilterFitTab", "Description:", nullptr));
        _descriptionLabel->setText(QString());
        _inputOutputBox->setTitle(QCoreApplication::translate("FilterFitTab", "Inputs and Outputs", nullptr));
        _ignoreAutoScale->setText(QCoreApplication::translate("FilterFitTab", "I&gnore in automatic axes range calculations", nullptr));
        (void)FilterFitTab;
    } // retranslateUi

};

namespace Ui {
    class FilterFitTab: public Ui_FilterFitTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERFITTAB_H
