/********************************************************************************
** Form generated from reading UI file 'fitgradient_unweightedconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FITGRADIENT_UNWEIGHTEDCONFIG_H
#define UI_FITGRADIENT_UNWEIGHTEDCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FitGradient_UnweightedConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_4;
    Kst::VectorSelector *_vectorX;
    QHBoxLayout *horizontalLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vectorY;

    void setupUi(QWidget *FitGradient_UnweightedConfig)
    {
        if (FitGradient_UnweightedConfig->objectName().isEmpty())
            FitGradient_UnweightedConfig->setObjectName("FitGradient_UnweightedConfig");
        FitGradient_UnweightedConfig->resize(420, 89);
        FitGradient_UnweightedConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(FitGradient_UnweightedConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_4 = new QLabel(FitGradient_UnweightedConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setWordWrap(false);

        horizontalLayout_2->addWidget(label_4);

        _vectorX = new Kst::VectorSelector(FitGradient_UnweightedConfig);
        _vectorX->setObjectName("_vectorX");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorX->sizePolicy().hasHeightForWidth());
        _vectorX->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(_vectorX);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_3 = new QLabel(FitGradient_UnweightedConfig);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout->addWidget(label_3);

        _vectorY = new Kst::VectorSelector(FitGradient_UnweightedConfig);
        _vectorY->setObjectName("_vectorY");
        sizePolicy1.setHeightForWidth(_vectorY->sizePolicy().hasHeightForWidth());
        _vectorY->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(_vectorY);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(FitGradient_UnweightedConfig);

        QMetaObject::connectSlotsByName(FitGradient_UnweightedConfig);
    } // setupUi

    void retranslateUi(QWidget *FitGradient_UnweightedConfig)
    {
        label_4->setText(QCoreApplication::translate("FitGradient_UnweightedConfig", "X:", "The X (horizontal) axis of a plot"));
        label_3->setText(QCoreApplication::translate("FitGradient_UnweightedConfig", "Y:", "The Y (vertical) axis of a plot"));
        (void)FitGradient_UnweightedConfig;
    } // retranslateUi

};

namespace Ui {
    class FitGradient_UnweightedConfig: public Ui_FitGradient_UnweightedConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FITGRADIENT_UNWEIGHTEDCONFIG_H
