/********************************************************************************
** Form generated from reading UI file 'fftoptions.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FFTOPTIONS_H
#define UI_FFTOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FFTOptions
{
public:
    QGridLayout *gridLayout_2;
    QGroupBox *GroupBoxFFTOptions;
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QComboBox *_apodizeFunction;
    QLabel *_sigmaLabel;
    QDoubleSpinBox *_sigma;
    QCheckBox *_apodize;
    QLabel *textLabel2;
    QCheckBox *_removeMean;
    QGridLayout *gridLayout2;
    QLabel *_FFTLengthLabel;
    QSpinBox *_FFTLength;
    QCheckBox *_interleavedAverage;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1_9_2;
    QLineEdit *_sampleRate;
    QLabel *TextLabel1_9_3;
    QLineEdit *_vectorUnits;
    QLabel *TextLabel1_9_2_2;
    QLineEdit *_rateUnits;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel1_2;
    QComboBox *_output;

    void setupUi(QWidget *FFTOptions)
    {
        if (FFTOptions->objectName().isEmpty())
            FFTOptions->setObjectName("FFTOptions");
        FFTOptions->resize(737, 194);
        gridLayout_2 = new QGridLayout(FFTOptions);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        GroupBoxFFTOptions = new QGroupBox(FFTOptions);
        GroupBoxFFTOptions->setObjectName("GroupBoxFFTOptions");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(GroupBoxFFTOptions->sizePolicy().hasHeightForWidth());
        GroupBoxFFTOptions->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(GroupBoxFFTOptions);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        _apodizeFunction = new QComboBox(GroupBoxFFTOptions);
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->addItem(QString());
        _apodizeFunction->setObjectName("_apodizeFunction");

        gridLayout1->addWidget(_apodizeFunction, 0, 3, 1, 1);

        _sigmaLabel = new QLabel(GroupBoxFFTOptions);
        _sigmaLabel->setObjectName("_sigmaLabel");
        _sigmaLabel->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_sigmaLabel->sizePolicy().hasHeightForWidth());
        _sigmaLabel->setSizePolicy(sizePolicy1);
        _sigmaLabel->setWordWrap(false);

        gridLayout1->addWidget(_sigmaLabel, 0, 4, 1, 1);

        _sigma = new QDoubleSpinBox(GroupBoxFFTOptions);
        _sigma->setObjectName("_sigma");
        _sigma->setEnabled(false);
        _sigma->setMinimum(0.010000000000000);

        gridLayout1->addWidget(_sigma, 0, 5, 1, 1);

        _apodize = new QCheckBox(GroupBoxFFTOptions);
        _apodize->setObjectName("_apodize");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_apodize->sizePolicy().hasHeightForWidth());
        _apodize->setSizePolicy(sizePolicy2);
        _apodize->setChecked(true);

        gridLayout1->addWidget(_apodize, 0, 1, 1, 1);

        textLabel2 = new QLabel(GroupBoxFFTOptions);
        textLabel2->setObjectName("textLabel2");
        sizePolicy1.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy1);
        textLabel2->setWordWrap(false);

        gridLayout1->addWidget(textLabel2, 0, 2, 1, 1);

        _removeMean = new QCheckBox(GroupBoxFFTOptions);
        _removeMean->setObjectName("_removeMean");
        _removeMean->setChecked(true);

        gridLayout1->addWidget(_removeMean, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);

        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        _FFTLengthLabel = new QLabel(GroupBoxFFTOptions);
        _FFTLengthLabel->setObjectName("_FFTLengthLabel");
        sizePolicy2.setHeightForWidth(_FFTLengthLabel->sizePolicy().hasHeightForWidth());
        _FFTLengthLabel->setSizePolicy(sizePolicy2);
        _FFTLengthLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _FFTLengthLabel->setWordWrap(false);

        gridLayout2->addWidget(_FFTLengthLabel, 0, 2, 1, 1);

        _FFTLength = new QSpinBox(GroupBoxFFTOptions);
        _FFTLength->setObjectName("_FFTLength");
        _FFTLength->setMinimum(4);
        _FFTLength->setMaximum(24);
        _FFTLength->setValue(10);

        gridLayout2->addWidget(_FFTLength, 0, 3, 1, 1);

        _interleavedAverage = new QCheckBox(GroupBoxFFTOptions);
        _interleavedAverage->setObjectName("_interleavedAverage");
        _interleavedAverage->setChecked(true);

        gridLayout2->addWidget(_interleavedAverage, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout2, 1, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        TextLabel1_9_2 = new QLabel(GroupBoxFFTOptions);
        TextLabel1_9_2->setObjectName("TextLabel1_9_2");
        sizePolicy1.setHeightForWidth(TextLabel1_9_2->sizePolicy().hasHeightForWidth());
        TextLabel1_9_2->setSizePolicy(sizePolicy1);
        TextLabel1_9_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel1_9_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_9_2);

        _sampleRate = new QLineEdit(GroupBoxFFTOptions);
        _sampleRate->setObjectName("_sampleRate");
        _sampleRate->setText(QString::fromUtf8("60.0"));

        hboxLayout->addWidget(_sampleRate);

        TextLabel1_9_3 = new QLabel(GroupBoxFFTOptions);
        TextLabel1_9_3->setObjectName("TextLabel1_9_3");
        sizePolicy1.setHeightForWidth(TextLabel1_9_3->sizePolicy().hasHeightForWidth());
        TextLabel1_9_3->setSizePolicy(sizePolicy1);
        TextLabel1_9_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel1_9_3->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_9_3);

        _vectorUnits = new QLineEdit(GroupBoxFFTOptions);
        _vectorUnits->setObjectName("_vectorUnits");

        hboxLayout->addWidget(_vectorUnits);

        TextLabel1_9_2_2 = new QLabel(GroupBoxFFTOptions);
        TextLabel1_9_2_2->setObjectName("TextLabel1_9_2_2");
        sizePolicy2.setHeightForWidth(TextLabel1_9_2_2->sizePolicy().hasHeightForWidth());
        TextLabel1_9_2_2->setSizePolicy(sizePolicy2);
        TextLabel1_9_2_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextLabel1_9_2_2->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1_9_2_2);

        _rateUnits = new QLineEdit(GroupBoxFFTOptions);
        _rateUnits->setObjectName("_rateUnits");

        hboxLayout->addWidget(_rateUnits);


        gridLayout->addLayout(hboxLayout, 2, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        textLabel1_2 = new QLabel(GroupBoxFFTOptions);
        textLabel1_2->setObjectName("textLabel1_2");
        sizePolicy1.setHeightForWidth(textLabel1_2->sizePolicy().hasHeightForWidth());
        textLabel1_2->setSizePolicy(sizePolicy1);
        textLabel1_2->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1_2);

        _output = new QComboBox(GroupBoxFFTOptions);
        _output->addItem(QString());
        _output->addItem(QString());
        _output->addItem(QString());
        _output->addItem(QString());
        _output->setObjectName("_output");

        hboxLayout1->addWidget(_output);


        gridLayout->addLayout(hboxLayout1, 3, 0, 1, 1);


        gridLayout_2->addWidget(GroupBoxFFTOptions, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        _sigmaLabel->setBuddy(_sigma);
        textLabel2->setBuddy(_apodizeFunction);
        _FFTLengthLabel->setBuddy(_FFTLength);
        TextLabel1_9_2->setBuddy(_sampleRate);
        TextLabel1_9_3->setBuddy(_vectorUnits);
        TextLabel1_9_2_2->setBuddy(_rateUnits);
        textLabel1_2->setBuddy(_output);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_removeMean, _apodize);
        QWidget::setTabOrder(_apodize, _apodizeFunction);
        QWidget::setTabOrder(_apodizeFunction, _sigma);
        QWidget::setTabOrder(_sigma, _interleavedAverage);
        QWidget::setTabOrder(_interleavedAverage, _FFTLength);
        QWidget::setTabOrder(_FFTLength, _sampleRate);
        QWidget::setTabOrder(_sampleRate, _vectorUnits);
        QWidget::setTabOrder(_vectorUnits, _rateUnits);
        QWidget::setTabOrder(_rateUnits, _output);

        retranslateUi(FFTOptions);

        QMetaObject::connectSlotsByName(FFTOptions);
    } // setupUi

    void retranslateUi(QWidget *FFTOptions)
    {
        GroupBoxFFTOptions->setTitle(QCoreApplication::translate("FFTOptions", "FFT Options", nullptr));
        _apodizeFunction->setItemText(0, QCoreApplication::translate("FFTOptions", "Default", "Use the application default option"));
        _apodizeFunction->setItemText(1, QCoreApplication::translate("FFTOptions", "Bartlett", nullptr));
        _apodizeFunction->setItemText(2, QCoreApplication::translate("FFTOptions", "Blackman", nullptr));
        _apodizeFunction->setItemText(3, QCoreApplication::translate("FFTOptions", "Connes", nullptr));
        _apodizeFunction->setItemText(4, QCoreApplication::translate("FFTOptions", "cosine", nullptr));
        _apodizeFunction->setItemText(5, QCoreApplication::translate("FFTOptions", "Gaussian (Custom Sigma)", nullptr));
        _apodizeFunction->setItemText(6, QCoreApplication::translate("FFTOptions", "Hamming", nullptr));
        _apodizeFunction->setItemText(7, QCoreApplication::translate("FFTOptions", "Hann", nullptr));
        _apodizeFunction->setItemText(8, QCoreApplication::translate("FFTOptions", "Welch", nullptr));

        _sigmaLabel->setText(QCoreApplication::translate("FFTOptions", "S&igma:", nullptr));
#if QT_CONFIG(whatsthis)
        _apodize->setWhatsThis(QCoreApplication::translate("FFTOptions", "Select from this pulldown the source vector for the Curve's y axis error bars.  If \"<None>\" is selected, there will be no y axis error bars drawn.  Create new vectors with the \"Vectors\" tab.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _apodize->setText(QCoreApplication::translate("FFTOptions", "Apodi&ze", nullptr));
        textLabel2->setText(QCoreApplication::translate("FFTOptions", "Fun&ction:", nullptr));
        _removeMean->setText(QCoreApplication::translate("FFTOptions", "Remo&ve mean", nullptr));
        _FFTLengthLabel->setText(QCoreApplication::translate("FFTOptions", "FFT &length: 2^", nullptr));
#if QT_CONFIG(whatsthis)
        _FFTLength->setWhatsThis(QCoreApplication::translate("FFTOptions", "Sets the length of the FFTs in generating the power spectrum.  2^10 = 1024, etc.  The larger the number, the larger the noise and resolution.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _interleavedAverage->setText(QCoreApplication::translate("FFTOptions", "Interleave&d average", nullptr));
        TextLabel1_9_2->setText(QCoreApplication::translate("FFTOptions", "Sa&mple rate:", nullptr));
#if QT_CONFIG(whatsthis)
        _sampleRate->setWhatsThis(QCoreApplication::translate("FFTOptions", "The sample rate at which the measurements were acquired, for proper normalization.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        TextLabel1_9_3->setWhatsThis(QCoreApplication::translate("FFTOptions", "Apply new data sample specifications to selected vectors.", nullptr));
#endif // QT_CONFIG(whatsthis)
        TextLabel1_9_3->setText(QCoreApplication::translate("FFTOptions", "Vector units:", nullptr));
#if QT_CONFIG(whatsthis)
        _vectorUnits->setWhatsThis(QCoreApplication::translate("FFTOptions", "The units of the Data Vector - for automatic labels.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _vectorUnits->setText(QCoreApplication::translate("FFTOptions", "V", "Abbreviation for Volts"));
        TextLabel1_9_2_2->setText(QCoreApplication::translate("FFTOptions", "Rate units:", nullptr));
#if QT_CONFIG(whatsthis)
        _rateUnits->setWhatsThis(QCoreApplication::translate("FFTOptions", "The units for the sample rate.  Used for automatic labels.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _rateUnits->setText(QCoreApplication::translate("FFTOptions", "Hz", "Abbreviation for Hertz (unit)"));
        textLabel1_2->setText(QCoreApplication::translate("FFTOptions", "Ou&tput:", nullptr));
        _output->setItemText(0, QCoreApplication::translate("FFTOptions", "Amplitude Spectral Density (V/Hz^1/2)", nullptr));
        _output->setItemText(1, QCoreApplication::translate("FFTOptions", "Power Spectral Density (V^2/Hz)", nullptr));
        _output->setItemText(2, QCoreApplication::translate("FFTOptions", "Amplitude Spectrum (V)", nullptr));
        _output->setItemText(3, QCoreApplication::translate("FFTOptions", "Power Spectrum (V^2)", nullptr));

        (void)FFTOptions;
    } // retranslateUi

};

namespace Ui {
    class FFTOptions: public Ui_FFTOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FFTOPTIONS_H
