/********************************************************************************
** Form generated from reading UI file 'viewmatrixdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VIEWMATRIXDIALOG_H
#define UI_VIEWMATRIXDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTableView>
#include "matrixselector.h"

QT_BEGIN_NAMESPACE

class Ui_ViewMatrixDialog
{
public:
    QGridLayout *gridLayout;
    Kst::MatrixSelector *matrixSelector;
    QTableView *_matrices;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ViewMatrixDialog)
    {
        if (ViewMatrixDialog->objectName().isEmpty())
            ViewMatrixDialog->setObjectName("ViewMatrixDialog");
        ViewMatrixDialog->resize(597, 536);
        gridLayout = new QGridLayout(ViewMatrixDialog);
        gridLayout->setObjectName("gridLayout");
        matrixSelector = new Kst::MatrixSelector(ViewMatrixDialog);
        matrixSelector->setObjectName("matrixSelector");

        gridLayout->addWidget(matrixSelector, 0, 0, 1, 1);

        _matrices = new QTableView(ViewMatrixDialog);
        _matrices->setObjectName("_matrices");

        gridLayout->addWidget(_matrices, 1, 0, 1, 1);

        buttonBox = new QDialogButtonBox(ViewMatrixDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        gridLayout->addWidget(buttonBox, 2, 0, 1, 1);


        retranslateUi(ViewMatrixDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ViewMatrixDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ViewMatrixDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ViewMatrixDialog);
    } // setupUi

    void retranslateUi(QDialog *ViewMatrixDialog)
    {
        ViewMatrixDialog->setWindowTitle(QCoreApplication::translate("ViewMatrixDialog", "Matrix View", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ViewMatrixDialog: public Ui_ViewMatrixDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VIEWMATRIXDIALOG_H
