/********************************************************************************
** Form generated from reading UI file 'exportgraphicsdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTGRAPHICSDIALOG_H
#define UI_EXPORTGRAPHICSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include "combobox.h"
#include "filerequester.h"

QT_BEGIN_NAMESPACE

class Ui_ExportGraphicsDialog
{
public:
    QGridLayout *gridLayout;
    Kst::ComboBox *_comboBoxFormats;
    Kst::FileRequester *_saveLocation;
    QGroupBox *groupBox1;
    QGridLayout *gridLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *_widthLabel;
    QSpinBox *_xSize;
    QLabel *_heightLabel;
    QSpinBox *_ySize;
    QSpacerItem *spacerItem;
    QHBoxLayout *hboxLayout1;
    QLabel *textLabel1_2;
    QComboBox *_comboBoxSizeOption;
    QSpacerItem *spacerItem1;
    QCheckBox *_exportAll;
    QLabel *label;
    QCheckBox *_autoExtension;
    QGroupBox *buttonGroup2;
    QHBoxLayout *hboxLayout2;
    QRadioButton *_saveOnce;
    QRadioButton *_autosave;
    QSpinBox *_period;
    QSpacerItem *spacerItem2;
    QLabel *_saveLocationLabel;
    QLabel *textLabel1;
    QCheckBox *_listVectorFormats;
    QCheckBox *_listBitmapFormats;
    QDialogButtonBox *_buttonBox;
    QLabel *_filenameLabel;

    void setupUi(QDialog *ExportGraphicsDialog)
    {
        if (ExportGraphicsDialog->objectName().isEmpty())
            ExportGraphicsDialog->setObjectName("ExportGraphicsDialog");
        ExportGraphicsDialog->resize(365, 350);
        gridLayout = new QGridLayout(ExportGraphicsDialog);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _comboBoxFormats = new Kst::ComboBox(ExportGraphicsDialog);
        _comboBoxFormats->setObjectName("_comboBoxFormats");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_comboBoxFormats->sizePolicy().hasHeightForWidth());
        _comboBoxFormats->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_comboBoxFormats, 4, 1, 1, 2);

        _saveLocation = new Kst::FileRequester(ExportGraphicsDialog);
        _saveLocation->setObjectName("_saveLocation");

        gridLayout->addWidget(_saveLocation, 0, 1, 1, 2);

        groupBox1 = new QGroupBox(ExportGraphicsDialog);
        groupBox1->setObjectName("groupBox1");
        gridLayout1 = new QGridLayout(groupBox1);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        _widthLabel = new QLabel(groupBox1);
        _widthLabel->setObjectName("_widthLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_widthLabel->sizePolicy().hasHeightForWidth());
        _widthLabel->setSizePolicy(sizePolicy1);
        _widthLabel->setWordWrap(false);

        hboxLayout->addWidget(_widthLabel);

        _xSize = new QSpinBox(groupBox1);
        _xSize->setObjectName("_xSize");
        _xSize->setMinimum(5);
        _xSize->setMaximum(10000);
        _xSize->setSingleStep(20);
        _xSize->setValue(640);

        hboxLayout->addWidget(_xSize);

        _heightLabel = new QLabel(groupBox1);
        _heightLabel->setObjectName("_heightLabel");
        sizePolicy1.setHeightForWidth(_heightLabel->sizePolicy().hasHeightForWidth());
        _heightLabel->setSizePolicy(sizePolicy1);
        _heightLabel->setWordWrap(false);

        hboxLayout->addWidget(_heightLabel);

        _ySize = new QSpinBox(groupBox1);
        _ySize->setObjectName("_ySize");
        _ySize->setMinimum(5);
        _ySize->setMaximum(10000);
        _ySize->setSingleStep(20);
        _ySize->setValue(480);

        hboxLayout->addWidget(_ySize);

        spacerItem = new QSpacerItem(120, 31, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        gridLayout1->addLayout(hboxLayout, 1, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        textLabel1_2 = new QLabel(groupBox1);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        textLabel1_2->setWordWrap(false);

        hboxLayout1->addWidget(textLabel1_2);

        _comboBoxSizeOption = new QComboBox(groupBox1);
        _comboBoxSizeOption->addItem(QString());
        _comboBoxSizeOption->addItem(QString());
        _comboBoxSizeOption->addItem(QString());
        _comboBoxSizeOption->addItem(QString());
        _comboBoxSizeOption->setObjectName("_comboBoxSizeOption");

        hboxLayout1->addWidget(_comboBoxSizeOption);

        spacerItem1 = new QSpacerItem(61, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem1);


        gridLayout1->addLayout(hboxLayout1, 0, 0, 1, 1);


        gridLayout->addWidget(groupBox1, 6, 0, 1, 3);

        _exportAll = new QCheckBox(ExportGraphicsDialog);
        _exportAll->setObjectName("_exportAll");

        gridLayout->addWidget(_exportAll, 3, 0, 1, 3);

        label = new QLabel(ExportGraphicsDialog);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 5, 0, 1, 1);

        _autoExtension = new QCheckBox(ExportGraphicsDialog);
        _autoExtension->setObjectName("_autoExtension");
        _autoExtension->setChecked(true);

        gridLayout->addWidget(_autoExtension, 2, 0, 1, 2);

        buttonGroup2 = new QGroupBox(ExportGraphicsDialog);
        buttonGroup2->setObjectName("buttonGroup2");
        hboxLayout2 = new QHBoxLayout(buttonGroup2);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        _saveOnce = new QRadioButton(buttonGroup2);
        _saveOnce->setObjectName("_saveOnce");
        _saveOnce->setChecked(true);

        hboxLayout2->addWidget(_saveOnce);

        _autosave = new QRadioButton(buttonGroup2);
        _autosave->setObjectName("_autosave");

        hboxLayout2->addWidget(_autosave);

        _period = new QSpinBox(buttonGroup2);
        _period->setObjectName("_period");
        _period->setEnabled(false);
        _period->setMinimum(1);
        _period->setMaximum(100000);
        _period->setValue(15);

        hboxLayout2->addWidget(_period);

        spacerItem2 = new QSpacerItem(21, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacerItem2);


        gridLayout->addWidget(buttonGroup2, 7, 0, 1, 3);

        _saveLocationLabel = new QLabel(ExportGraphicsDialog);
        _saveLocationLabel->setObjectName("_saveLocationLabel");
        sizePolicy1.setHeightForWidth(_saveLocationLabel->sizePolicy().hasHeightForWidth());
        _saveLocationLabel->setSizePolicy(sizePolicy1);
        _saveLocationLabel->setWordWrap(false);

        gridLayout->addWidget(_saveLocationLabel, 0, 0, 1, 1);

        textLabel1 = new QLabel(ExportGraphicsDialog);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 4, 0, 1, 1);

        _listVectorFormats = new QCheckBox(ExportGraphicsDialog);
        _listVectorFormats->setObjectName("_listVectorFormats");

        gridLayout->addWidget(_listVectorFormats, 5, 1, 1, 1);

        _listBitmapFormats = new QCheckBox(ExportGraphicsDialog);
        _listBitmapFormats->setObjectName("_listBitmapFormats");

        gridLayout->addWidget(_listBitmapFormats, 5, 2, 1, 1);

        _buttonBox = new QDialogButtonBox(ExportGraphicsDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(_buttonBox, 8, 0, 1, 3);

        _filenameLabel = new QLabel(ExportGraphicsDialog);
        _filenameLabel->setObjectName("_filenameLabel");

        gridLayout->addWidget(_filenameLabel, 1, 0, 1, 3);

#if QT_CONFIG(shortcut)
        _widthLabel->setBuddy(_xSize);
        _heightLabel->setBuddy(_ySize);
        textLabel1_2->setBuddy(_comboBoxSizeOption);
        textLabel1->setBuddy(_comboBoxFormats);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_comboBoxFormats, _listVectorFormats);
        QWidget::setTabOrder(_listVectorFormats, _listBitmapFormats);
        QWidget::setTabOrder(_listBitmapFormats, _comboBoxSizeOption);
        QWidget::setTabOrder(_comboBoxSizeOption, _xSize);
        QWidget::setTabOrder(_xSize, _ySize);
        QWidget::setTabOrder(_ySize, _saveOnce);
        QWidget::setTabOrder(_saveOnce, _autosave);
        QWidget::setTabOrder(_autosave, _period);
        QWidget::setTabOrder(_period, _buttonBox);

        retranslateUi(ExportGraphicsDialog);
        QObject::connect(_autosave, &QRadioButton::toggled, _period, &QSpinBox::setEnabled);

        QMetaObject::connectSlotsByName(ExportGraphicsDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportGraphicsDialog)
    {
        ExportGraphicsDialog->setWindowTitle(QCoreApplication::translate("ExportGraphicsDialog", "Export to Graphics File", nullptr));
        groupBox1->setTitle(QCoreApplication::translate("ExportGraphicsDialog", "Size", nullptr));
        _widthLabel->setText(QCoreApplication::translate("ExportGraphicsDialog", "&Width:", nullptr));
#if QT_CONFIG(whatsthis)
        _xSize->setWhatsThis(QCoreApplication::translate("ExportGraphicsDialog", "Width (in Pixels) of the image.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _heightLabel->setText(QCoreApplication::translate("ExportGraphicsDialog", "&Height:", nullptr));
#if QT_CONFIG(whatsthis)
        _ySize->setWhatsThis(QCoreApplication::translate("ExportGraphicsDialog", "Height of the image (in pixels).", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_2->setText(QCoreApplication::translate("ExportGraphicsDialog", "S&pecify:", nullptr));
        _comboBoxSizeOption->setItemText(0, QCoreApplication::translate("ExportGraphicsDialog", "Width and Maintain Aspect Ratio", nullptr));
        _comboBoxSizeOption->setItemText(1, QCoreApplication::translate("ExportGraphicsDialog", "Height and Maintain Aspect Ratio", nullptr));
        _comboBoxSizeOption->setItemText(2, QCoreApplication::translate("ExportGraphicsDialog", "Width and Height", nullptr));
        _comboBoxSizeOption->setItemText(3, QCoreApplication::translate("ExportGraphicsDialog", "Size of Square", nullptr));

        _exportAll->setText(QCoreApplication::translate("ExportGraphicsDialog", "Export all tabs (<name>_View ?.<ext>)", nullptr));
        label->setText(QCoreApplication::translate("ExportGraphicsDialog", "Include:", nullptr));
        _autoExtension->setText(QCoreApplication::translate("ExportGraphicsDialog", "Automatic &extension", nullptr));
        buttonGroup2->setTitle(QCoreApplication::translate("ExportGraphicsDialog", "Save Options", nullptr));
#if QT_CONFIG(whatsthis)
        _saveOnce->setWhatsThis(QCoreApplication::translate("ExportGraphicsDialog", "Do not autosave.  Instead, on OK or Apply, save once.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _saveOnce->setText(QCoreApplication::translate("ExportGraphicsDialog", "Sa&ve once", nullptr));
#if QT_CONFIG(tooltip)
        _autosave->setToolTip(QCoreApplication::translate("ExportGraphicsDialog", "Save image on timer", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _autosave->setWhatsThis(QCoreApplication::translate("ExportGraphicsDialog", "When checked, the image will be repeatedly saved using the indicated period.  This is useful, for example, for live web page images.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _autosave->setText(QCoreApplication::translate("ExportGraphicsDialog", "&Autosave every", nullptr));
#if QT_CONFIG(tooltip)
        _period->setToolTip(QCoreApplication::translate("ExportGraphicsDialog", "Autosave timer", nullptr));
#endif // QT_CONFIG(tooltip)
        _period->setSuffix(QCoreApplication::translate("ExportGraphicsDialog", "s", nullptr));
        _saveLocationLabel->setText(QCoreApplication::translate("ExportGraphicsDialog", "File name:", nullptr));
        textLabel1->setText(QCoreApplication::translate("ExportGraphicsDialog", "Fi&le format:", nullptr));
        _listVectorFormats->setText(QCoreApplication::translate("ExportGraphicsDialog", "Vector formats", nullptr));
        _listBitmapFormats->setText(QCoreApplication::translate("ExportGraphicsDialog", "Bitmap formats", nullptr));
        _filenameLabel->setText(QCoreApplication::translate("ExportGraphicsDialog", "TextLabel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportGraphicsDialog: public Ui_ExportGraphicsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTGRAPHICSDIALOG_H
