/********************************************************************************
** Form generated from reading UI file 'scalarlistselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCALARLISTSELECTOR_H
#define UI_SCALARLISTSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>

QT_BEGIN_NAMESPACE

class Ui_ScalarListSelector
{
public:
    QDialogButtonBox *buttonBox;
    QLabel *label;
    QLineEdit *_search;
    QListWidget *_scalars;

    void setupUi(QDialog *ScalarListSelector)
    {
        if (ScalarListSelector->objectName().isEmpty())
            ScalarListSelector->setObjectName("ScalarListSelector");
        ScalarListSelector->resize(283, 284);
        buttonBox = new QDialogButtonBox(ScalarListSelector);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setGeometry(QRect(70, 240, 201, 32));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        label = new QLabel(ScalarListSelector);
        label->setObjectName("label");
        label->setGeometry(QRect(20, 10, 41, 21));
        _search = new QLineEdit(ScalarListSelector);
        _search->setObjectName("_search");
        _search->setGeometry(QRect(70, 10, 201, 21));
        _scalars = new QListWidget(ScalarListSelector);
        _scalars->setObjectName("_scalars");
        _scalars->setGeometry(QRect(20, 40, 256, 192));

        retranslateUi(ScalarListSelector);

        QMetaObject::connectSlotsByName(ScalarListSelector);
    } // setupUi

    void retranslateUi(QDialog *ScalarListSelector)
    {
        ScalarListSelector->setWindowTitle(QCoreApplication::translate("ScalarListSelector", "Dialog", nullptr));
        label->setText(QCoreApplication::translate("ScalarListSelector", "Search:", "enter a search term"));
    } // retranslateUi

};

namespace Ui {
    class ScalarListSelector: public Ui_ScalarListSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCALARLISTSELECTOR_H
