/********************************************************************************
** Form generated from reading UI file 'labeldimensionstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LABELDIMENSIONSTAB_H
#define UI_LABELDIMENSIONSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LabelDimensionsTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QCheckBox *_lockPosToData;
    QRadioButton *_right;
    QLabel *label_4;
    QRadioButton *_left;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QDoubleSpinBox *_rotation;
    QLabel *label;
    QLabel *label_2;
    QDoubleSpinBox *_y;
    QDoubleSpinBox *_x;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *LabelDimensionsTab)
    {
        if (LabelDimensionsTab->objectName().isEmpty())
            LabelDimensionsTab->setObjectName("LabelDimensionsTab");
        LabelDimensionsTab->resize(276, 176);
        verticalLayout = new QVBoxLayout(LabelDimensionsTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        _lockPosToData = new QCheckBox(LabelDimensionsTab);
        _lockPosToData->setObjectName("_lockPosToData");

        gridLayout_2->addWidget(_lockPosToData, 0, 0, 1, 3);

        _right = new QRadioButton(LabelDimensionsTab);
        _right->setObjectName("_right");

        gridLayout_2->addWidget(_right, 2, 2, 1, 1);

        label_4 = new QLabel(LabelDimensionsTab);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(label_4, 2, 0, 1, 1);

        _left = new QRadioButton(LabelDimensionsTab);
        _left->setObjectName("_left");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_left->sizePolicy().hasHeightForWidth());
        _left->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(_left, 2, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(LabelDimensionsTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        _rotation = new QDoubleSpinBox(LabelDimensionsTab);
        _rotation->setObjectName("_rotation");
        _rotation->setDecimals(4);
        _rotation->setMinimum(-180.000000000000000);
        _rotation->setMaximum(360.000000000000000);
        _rotation->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_rotation, 2, 1, 1, 1);

        label = new QLabel(LabelDimensionsTab);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        label_2 = new QLabel(LabelDimensionsTab);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_2, 1, 2, 1, 1);

        _y = new QDoubleSpinBox(LabelDimensionsTab);
        _y->setObjectName("_y");
        _y->setDecimals(4);
        _y->setMaximum(1.000000000000000);
        _y->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_y, 1, 3, 1, 1);

        _x = new QDoubleSpinBox(LabelDimensionsTab);
        _x->setObjectName("_x");
        _x->setDecimals(4);
        _x->setMaximum(1.000000000000000);
        _x->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_x, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 37, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(_rotation);
        label->setBuddy(_x);
        label_2->setBuddy(_y);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_lockPosToData, _left);
        QWidget::setTabOrder(_left, _right);
        QWidget::setTabOrder(_right, _x);
        QWidget::setTabOrder(_x, _y);
        QWidget::setTabOrder(_y, _rotation);

        retranslateUi(LabelDimensionsTab);
        QObject::connect(_lockPosToData, SIGNAL(clicked()), LabelDimensionsTab, SLOT(modified()));
        QObject::connect(_left, SIGNAL(toggled(bool)), LabelDimensionsTab, SLOT(modified()));
        QObject::connect(_x, SIGNAL(valueChanged(QString)), LabelDimensionsTab, SLOT(modified()));
        QObject::connect(_y, SIGNAL(valueChanged(QString)), LabelDimensionsTab, SLOT(modified()));
        QObject::connect(_rotation, SIGNAL(valueChanged(QString)), LabelDimensionsTab, SLOT(modified()));

        QMetaObject::connectSlotsByName(LabelDimensionsTab);
    } // setupUi

    void retranslateUi(QWidget *LabelDimensionsTab)
    {
        _lockPosToData->setText(QCoreApplication::translate("LabelDimensionsTab", "Lock Position To Data", nullptr));
        _right->setText(QCoreApplication::translate("LabelDimensionsTab", "Right", "the right side"));
        label_4->setText(QCoreApplication::translate("LabelDimensionsTab", "Label Reference Point:", nullptr));
        _left->setText(QCoreApplication::translate("LabelDimensionsTab", "Left", "the left side"));
        label_3->setText(QCoreApplication::translate("LabelDimensionsTab", "Rotatio&n", nullptr));
        label->setText(QCoreApplication::translate("LabelDimensionsTab", "Position", nullptr));
        label_2->setText(QCoreApplication::translate("LabelDimensionsTab", " , ", nullptr));
        (void)LabelDimensionsTab;
    } // retranslateUi

};

namespace Ui {
    class LabelDimensionsTab: public Ui_LabelDimensionsTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LABELDIMENSIONSTAB_H
