/********************************************************************************
** Form generated from reading UI file 'baddatasourcedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BADDATASOURCEDIALOG_H
#define UI_BADDATASOURCEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_BadDatasourceDialog
{
public:
    QGridLayout *gridLayout_2;
    QLabel *_label;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    Kst::DataSourceSelector *_url;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *_fileType;
    QPushButton *_configureSource;
    QPushButton *_change;
    QPushButton *_skip;

    void setupUi(QDialog *BadDatasourceDialog)
    {
        if (BadDatasourceDialog->objectName().isEmpty())
            BadDatasourceDialog->setObjectName("BadDatasourceDialog");
        BadDatasourceDialog->resize(625, 144);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(BadDatasourceDialog->sizePolicy().hasHeightForWidth());
        BadDatasourceDialog->setSizePolicy(sizePolicy);
        gridLayout_2 = new QGridLayout(BadDatasourceDialog);
        gridLayout_2->setObjectName("gridLayout_2");
        _label = new QLabel(BadDatasourceDialog);
        _label->setObjectName("_label");
        sizePolicy.setHeightForWidth(_label->sizePolicy().hasHeightForWidth());
        _label->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(_label, 0, 0, 1, 1);

        groupBox = new QGroupBox(BadDatasourceDialog);
        groupBox->setObjectName("groupBox");
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        _url = new Kst::DataSourceSelector(groupBox);
        _url->setObjectName("_url");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_url->sizePolicy().hasHeightForWidth());
        _url->setSizePolicy(sizePolicy1);
        _url->setMinimumSize(QSize(30, 24));

        verticalLayout->addWidget(_url);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        _fileType = new QLabel(groupBox);
        _fileType->setObjectName("_fileType");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(10);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_fileType->sizePolicy().hasHeightForWidth());
        _fileType->setSizePolicy(sizePolicy2);
        _fileType->setWordWrap(false);

        horizontalLayout->addWidget(_fileType);

        _configureSource = new QPushButton(groupBox);
        _configureSource->setObjectName("_configureSource");
        _configureSource->setEnabled(false);
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_configureSource->sizePolicy().hasHeightForWidth());
        _configureSource->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(_configureSource);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        gridLayout_2->addWidget(groupBox, 1, 0, 1, 3);

        _change = new QPushButton(BadDatasourceDialog);
        _change->setObjectName("_change");
        _change->setEnabled(false);
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_change->sizePolicy().hasHeightForWidth());
        _change->setSizePolicy(sizePolicy4);

        gridLayout_2->addWidget(_change, 2, 1, 1, 1);

        _skip = new QPushButton(BadDatasourceDialog);
        _skip->setObjectName("_skip");
        sizePolicy4.setHeightForWidth(_skip->sizePolicy().hasHeightForWidth());
        _skip->setSizePolicy(sizePolicy4);

        gridLayout_2->addWidget(_skip, 2, 2, 1, 1);


        retranslateUi(BadDatasourceDialog);

        QMetaObject::connectSlotsByName(BadDatasourceDialog);
    } // setupUi

    void retranslateUi(QDialog *BadDatasourceDialog)
    {
        BadDatasourceDialog->setWindowTitle(QCoreApplication::translate("BadDatasourceDialog", "Dialog", nullptr));
        _label->setText(QCoreApplication::translate("BadDatasourceDialog", "TextLabel", nullptr));
        groupBox->setTitle(QCoreApplication::translate("BadDatasourceDialog", "Select new data file:", nullptr));
        _fileType->setText(QCoreApplication::translate("BadDatasourceDialog", "Unknown file type", nullptr));
        _configureSource->setText(QCoreApplication::translate("BadDatasourceDialog", "Con&figure...", nullptr));
        _change->setText(QCoreApplication::translate("BadDatasourceDialog", "Change", nullptr));
        _skip->setText(QCoreApplication::translate("BadDatasourceDialog", "Skip", nullptr));
    } // retranslateUi

};

namespace Ui {
    class BadDatasourceDialog: public Ui_BadDatasourceDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BADDATASOURCEDIALOG_H
