/********************************************************************************
** Form generated from reading UI file 'arrowpropertiestab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ARROWPROPERTIESTAB_H
#define UI_ARROWPROPERTIESTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ArrowPropertiesTab
{
public:
    QGridLayout *gridLayout;
    QCheckBox *_startArrow;
    QLabel *_startArrowLabel;
    QDoubleSpinBox *_startArrowScale;
    QCheckBox *_endArrow;
    QLabel *_endArrowLabel;
    QDoubleSpinBox *_endArrowScale;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *ArrowPropertiesTab)
    {
        if (ArrowPropertiesTab->objectName().isEmpty())
            ArrowPropertiesTab->setObjectName("ArrowPropertiesTab");
        ArrowPropertiesTab->setWindowModality(Qt::NonModal);
        ArrowPropertiesTab->resize(336, 289);
        gridLayout = new QGridLayout(ArrowPropertiesTab);
        gridLayout->setObjectName("gridLayout");
        _startArrow = new QCheckBox(ArrowPropertiesTab);
        _startArrow->setObjectName("_startArrow");

        gridLayout->addWidget(_startArrow, 0, 0, 1, 2);

        _startArrowLabel = new QLabel(ArrowPropertiesTab);
        _startArrowLabel->setObjectName("_startArrowLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_startArrowLabel->sizePolicy().hasHeightForWidth());
        _startArrowLabel->setSizePolicy(sizePolicy);
        _startArrowLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_startArrowLabel, 1, 0, 1, 1);

        _startArrowScale = new QDoubleSpinBox(ArrowPropertiesTab);
        _startArrowScale->setObjectName("_startArrowScale");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_startArrowScale->sizePolicy().hasHeightForWidth());
        _startArrowScale->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_startArrowScale, 1, 1, 1, 1);

        _endArrow = new QCheckBox(ArrowPropertiesTab);
        _endArrow->setObjectName("_endArrow");

        gridLayout->addWidget(_endArrow, 2, 0, 1, 2);

        _endArrowLabel = new QLabel(ArrowPropertiesTab);
        _endArrowLabel->setObjectName("_endArrowLabel");
        sizePolicy.setHeightForWidth(_endArrowLabel->sizePolicy().hasHeightForWidth());
        _endArrowLabel->setSizePolicy(sizePolicy);
        _endArrowLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_endArrowLabel, 3, 0, 1, 1);

        _endArrowScale = new QDoubleSpinBox(ArrowPropertiesTab);
        _endArrowScale->setObjectName("_endArrowScale");

        gridLayout->addWidget(_endArrowScale, 3, 1, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 4, 0, 1, 2);

#if QT_CONFIG(shortcut)
        _startArrowLabel->setBuddy(_startArrowScale);
        _endArrowLabel->setBuddy(_endArrowScale);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ArrowPropertiesTab);

        QMetaObject::connectSlotsByName(ArrowPropertiesTab);
    } // setupUi

    void retranslateUi(QWidget *ArrowPropertiesTab)
    {
        _startArrow->setText(QCoreApplication::translate("ArrowPropertiesTab", "Arrow at &start", nullptr));
        _startArrowLabel->setText(QCoreApplication::translate("ArrowPropertiesTab", "S&tart arrow scale", nullptr));
#if QT_CONFIG(tooltip)
        _startArrowScale->setToolTip(QCoreApplication::translate("ArrowPropertiesTab", "Arrow Head Size", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _startArrowScale->setWhatsThis(QCoreApplication::translate("ArrowPropertiesTab", "The size of the arrow head is proportional to this value.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _endArrow->setText(QCoreApplication::translate("ArrowPropertiesTab", "Arrow at &end", nullptr));
        _endArrowLabel->setText(QCoreApplication::translate("ArrowPropertiesTab", "E&nd arrow scale", nullptr));
#if QT_CONFIG(tooltip)
        _endArrowScale->setToolTip(QCoreApplication::translate("ArrowPropertiesTab", "Arrow Head Size", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _endArrowScale->setWhatsThis(QCoreApplication::translate("ArrowPropertiesTab", "The size of the arrow head is proportional to this value.", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)ArrowPropertiesTab;
    } // retranslateUi

};

namespace Ui {
    class ArrowPropertiesTab: public Ui_ArrowPropertiesTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ARROWPROPERTIESTAB_H
