/********************************************************************************
** Form generated from reading UI file 'fitsinusoid_unweightedconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FITSINUSOID_UNWEIGHTEDCONFIG_H
#define UI_FITSINUSOID_UNWEIGHTEDCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FitSinusoid_UnweightedConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_4;
    Kst::VectorSelector *_vectorX;
    QLabel *label_3;
    Kst::VectorSelector *_vectorY;
    QLabel *label_6;
    Kst::ScalarSelector *_scalarHarmonics;
    QLabel *label_7;
    Kst::ScalarSelector *_scalarPeriod;

    void setupUi(QWidget *FitSinusoid_UnweightedConfig)
    {
        if (FitSinusoid_UnweightedConfig->objectName().isEmpty())
            FitSinusoid_UnweightedConfig->setObjectName("FitSinusoid_UnweightedConfig");
        FitSinusoid_UnweightedConfig->resize(419, 198);
        FitSinusoid_UnweightedConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(FitSinusoid_UnweightedConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(FitSinusoid_UnweightedConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        _vectorX = new Kst::VectorSelector(FitSinusoid_UnweightedConfig);
        _vectorX->setObjectName("_vectorX");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorX->sizePolicy().hasHeightForWidth());
        _vectorX->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorX, 0, 1, 1, 1);

        label_3 = new QLabel(FitSinusoid_UnweightedConfig);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        _vectorY = new Kst::VectorSelector(FitSinusoid_UnweightedConfig);
        _vectorY->setObjectName("_vectorY");
        sizePolicy1.setHeightForWidth(_vectorY->sizePolicy().hasHeightForWidth());
        _vectorY->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorY, 1, 1, 1, 1);

        label_6 = new QLabel(FitSinusoid_UnweightedConfig);
        label_6->setObjectName("label_6");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy2);
        label_6->setWordWrap(false);

        gridLayout->addWidget(label_6, 2, 0, 1, 1);

        _scalarHarmonics = new Kst::ScalarSelector(FitSinusoid_UnweightedConfig);
        _scalarHarmonics->setObjectName("_scalarHarmonics");

        gridLayout->addWidget(_scalarHarmonics, 2, 1, 1, 1);

        label_7 = new QLabel(FitSinusoid_UnweightedConfig);
        label_7->setObjectName("label_7");
        sizePolicy.setHeightForWidth(label_7->sizePolicy().hasHeightForWidth());
        label_7->setSizePolicy(sizePolicy);
        label_7->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_7->setWordWrap(false);

        gridLayout->addWidget(label_7, 3, 0, 1, 1);

        _scalarPeriod = new Kst::ScalarSelector(FitSinusoid_UnweightedConfig);
        _scalarPeriod->setObjectName("_scalarPeriod");

        gridLayout->addWidget(_scalarPeriod, 3, 1, 1, 1);


        retranslateUi(FitSinusoid_UnweightedConfig);

        QMetaObject::connectSlotsByName(FitSinusoid_UnweightedConfig);
    } // setupUi

    void retranslateUi(QWidget *FitSinusoid_UnweightedConfig)
    {
        label_4->setText(QCoreApplication::translate("FitSinusoid_UnweightedConfig", "X:", "The X axis of a plot"));
        label_3->setText(QCoreApplication::translate("FitSinusoid_UnweightedConfig", "Y:", "The Y axis of a plot"));
        label_6->setText(QCoreApplication::translate("FitSinusoid_UnweightedConfig", "Harmonics:", nullptr));
        label_7->setText(QCoreApplication::translate("FitSinusoid_UnweightedConfig", "Period:", "the period of a wave."));
        (void)FitSinusoid_UnweightedConfig;
    } // retranslateUi

};

namespace Ui {
    class FitSinusoid_UnweightedConfig: public Ui_FitSinusoid_UnweightedConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FITSINUSOID_UNWEIGHTEDCONFIG_H
