/********************************************************************************
** Form generated from reading UI file 'fitkneefrequencyconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FITKNEEFREQUENCYCONFIG_H
#define UI_FITKNEEFREQUENCYCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FitKneeFrequencyConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_4;
    Kst::VectorSelector *_vectorX;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    Kst::VectorSelector *_vectorY;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_11;
    Kst::ScalarSelector *_scalarMax;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_9;
    Kst::ScalarSelector *_scalarMin;
    QHBoxLayout *horizontalLayout;
    QLabel *label_8;
    Kst::ScalarSelector *_scalarWhiteNoise;

    void setupUi(QWidget *FitKneeFrequencyConfig)
    {
        if (FitKneeFrequencyConfig->objectName().isEmpty())
            FitKneeFrequencyConfig->setObjectName("FitKneeFrequencyConfig");
        FitKneeFrequencyConfig->resize(427, 215);
        FitKneeFrequencyConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(FitKneeFrequencyConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(6);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_4 = new QLabel(FitKneeFrequencyConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setWordWrap(false);

        horizontalLayout_5->addWidget(label_4);

        _vectorX = new Kst::VectorSelector(FitKneeFrequencyConfig);
        _vectorX->setObjectName("_vectorX");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorX->sizePolicy().hasHeightForWidth());
        _vectorX->setSizePolicy(sizePolicy1);

        horizontalLayout_5->addWidget(_vectorX);


        verticalLayout->addLayout(horizontalLayout_5);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(FitKneeFrequencyConfig);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout_4->addWidget(label_3);

        _vectorY = new Kst::VectorSelector(FitKneeFrequencyConfig);
        _vectorY->setObjectName("_vectorY");
        sizePolicy1.setHeightForWidth(_vectorY->sizePolicy().hasHeightForWidth());
        _vectorY->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(_vectorY);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_11 = new QLabel(FitKneeFrequencyConfig);
        label_11->setObjectName("label_11");
        sizePolicy.setHeightForWidth(label_11->sizePolicy().hasHeightForWidth());
        label_11->setSizePolicy(sizePolicy);
        label_11->setWordWrap(false);

        horizontalLayout_3->addWidget(label_11);

        _scalarMax = new Kst::ScalarSelector(FitKneeFrequencyConfig);
        _scalarMax->setObjectName("_scalarMax");

        horizontalLayout_3->addWidget(_scalarMax);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_9 = new QLabel(FitKneeFrequencyConfig);
        label_9->setObjectName("label_9");
        sizePolicy.setHeightForWidth(label_9->sizePolicy().hasHeightForWidth());
        label_9->setSizePolicy(sizePolicy);
        label_9->setWordWrap(false);

        horizontalLayout_2->addWidget(label_9);

        _scalarMin = new Kst::ScalarSelector(FitKneeFrequencyConfig);
        _scalarMin->setObjectName("_scalarMin");

        horizontalLayout_2->addWidget(_scalarMin);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_8 = new QLabel(FitKneeFrequencyConfig);
        label_8->setObjectName("label_8");
        sizePolicy.setHeightForWidth(label_8->sizePolicy().hasHeightForWidth());
        label_8->setSizePolicy(sizePolicy);
        label_8->setWordWrap(false);

        horizontalLayout->addWidget(label_8);

        _scalarWhiteNoise = new Kst::ScalarSelector(FitKneeFrequencyConfig);
        _scalarWhiteNoise->setObjectName("_scalarWhiteNoise");

        horizontalLayout->addWidget(_scalarWhiteNoise);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(FitKneeFrequencyConfig);

        QMetaObject::connectSlotsByName(FitKneeFrequencyConfig);
    } // setupUi

    void retranslateUi(QWidget *FitKneeFrequencyConfig)
    {
        label_4->setText(QCoreApplication::translate("FitKneeFrequencyConfig", "Input Vector - X", nullptr));
        label_3->setText(QCoreApplication::translate("FitKneeFrequencyConfig", "Input Vector - Y", nullptr));
        label_11->setText(QCoreApplication::translate("FitKneeFrequencyConfig", "Input Scalar - Max. 1/f^a Freq", nullptr));
        label_9->setText(QCoreApplication::translate("FitKneeFrequencyConfig", "Input Scalar - Min. White Noise Freq", nullptr));
        label_8->setText(QCoreApplication::translate("FitKneeFrequencyConfig", "Input Scalar - White Noise C", nullptr));
        (void)FitKneeFrequencyConfig;
    } // retranslateUi

};

namespace Ui {
    class FitKneeFrequencyConfig: public Ui_FitKneeFrequencyConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FITKNEEFREQUENCYCONFIG_H
