/********************************************************************************
** Form generated from reading UI file 'bugreportwizard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BUGREPORTWIZARD_H
#define UI_BUGREPORTWIZARD_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_BugReportWizard
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *_kstVersion;
    QLabel *label_2;
    QLabel *_OS;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *_reportBugButton;
    QSpacerItem *horizontalSpacer;
    QFrame *line;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *BugReportWizard)
    {
        if (BugReportWizard->objectName().isEmpty())
            BugReportWizard->setObjectName("BugReportWizard");
        BugReportWizard->resize(640, 197);
        verticalLayout = new QVBoxLayout(BugReportWizard);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(BugReportWizard);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        _kstVersion = new QLabel(BugReportWizard);
        _kstVersion->setObjectName("_kstVersion");

        gridLayout->addWidget(_kstVersion, 0, 1, 1, 1);

        label_2 = new QLabel(BugReportWizard);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        _OS = new QLabel(BugReportWizard);
        _OS->setObjectName("_OS");

        gridLayout->addWidget(_OS, 1, 1, 1, 1);


        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(238, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        label_3 = new QLabel(BugReportWizard);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        _reportBugButton = new QPushButton(BugReportWizard);
        _reportBugButton->setObjectName("_reportBugButton");

        horizontalLayout_4->addWidget(_reportBugButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_4);

        line = new QFrame(BugReportWizard);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        buttonBox = new QDialogButtonBox(BugReportWizard);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(BugReportWizard);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, BugReportWizard, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, BugReportWizard, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(BugReportWizard);
    } // setupUi

    void retranslateUi(QDialog *BugReportWizard)
    {
        BugReportWizard->setWindowTitle(QCoreApplication::translate("BugReportWizard", "Bug Report Wizard", nullptr));
        label->setText(QCoreApplication::translate("BugReportWizard", "Kst version:", nullptr));
        _kstVersion->setText(QString());
        label_2->setText(QCoreApplication::translate("BugReportWizard", "OS:", nullptr));
        _OS->setText(QString());
        label_3->setText(QCoreApplication::translate("BugReportWizard", "To submit a bug click on the button below which will open\n"
"http://bugs.kde.org where you can submit the bug.\n"
"You will have to create a KDE Bugs account if you don't have one.", nullptr));
        _reportBugButton->setText(QCoreApplication::translate("BugReportWizard", "Launch Bug Report Wizard", nullptr));
    } // retranslateUi

};

namespace Ui {
    class BugReportWizard: public Ui_BugReportWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BUGREPORTWIZARD_H
