/********************************************************************************
** Form generated from reading UI file 'datasourcedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATASOURCEDIALOG_H
#define UI_DATASOURCEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>

QT_BEGIN_NAMESPACE

class Ui_DataSourceDialog
{
public:
    QGridLayout *gridLayout;
    QComboBox *_updateBox;
    QLabel *_fileType_3;
    QPushButton *_configure;
    QLabel *_fileName;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DataSourceDialog)
    {
        if (DataSourceDialog->objectName().isEmpty())
            DataSourceDialog->setObjectName("DataSourceDialog");
        DataSourceDialog->resize(489, 153);
        gridLayout = new QGridLayout(DataSourceDialog);
        gridLayout->setObjectName("gridLayout");
        _updateBox = new QComboBox(DataSourceDialog);
        _updateBox->setObjectName("_updateBox");
        _updateBox->setMinimumSize(QSize(0, 0));

        gridLayout->addWidget(_updateBox, 1, 3, 1, 1);

        _fileType_3 = new QLabel(DataSourceDialog);
        _fileType_3->setObjectName("_fileType_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(10);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_fileType_3->sizePolicy().hasHeightForWidth());
        _fileType_3->setSizePolicy(sizePolicy);
        _fileType_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _fileType_3->setWordWrap(false);

        gridLayout->addWidget(_fileType_3, 1, 0, 1, 3);

        _configure = new QPushButton(DataSourceDialog);
        _configure->setObjectName("_configure");
        _configure->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_configure->sizePolicy().hasHeightForWidth());
        _configure->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_configure, 0, 3, 1, 1);

        _fileName = new QLabel(DataSourceDialog);
        _fileName->setObjectName("_fileName");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_fileName->sizePolicy().hasHeightForWidth());
        _fileName->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(_fileName, 0, 0, 1, 3);

        buttonBox = new QDialogButtonBox(DataSourceDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        gridLayout->addWidget(buttonBox, 2, 3, 1, 1);


        retranslateUi(DataSourceDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DataSourceDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DataSourceDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DataSourceDialog);
    } // setupUi

    void retranslateUi(QDialog *DataSourceDialog)
    {
        DataSourceDialog->setWindowTitle(QCoreApplication::translate("DataSourceDialog", "Dialog", nullptr));
        _fileType_3->setText(QCoreApplication::translate("DataSourceDialog", "Update type. Use 'Time Interval' for non local data.", nullptr));
        _configure->setText(QCoreApplication::translate("DataSourceDialog", "Co&nfigure...", nullptr));
        _fileName->setText(QCoreApplication::translate("DataSourceDialog", "TextLabel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DataSourceDialog: public Ui_DataSourceDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATASOURCEDIALOG_H
