/********************************************************************************
** Form generated from reading UI file 'choosecolordialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHOOSECOLORDIALOG_H
#define UI_CHOOSECOLORDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>

QT_BEGIN_NAMESPACE

class Ui_ChooseColorDialog
{
public:
    QGridLayout *gridLayout;
    QGroupBox *colorGroup;
    QGridLayout *gridLayout1;
    QFrame *colorFrame;
    QDialogButtonBox *_buttonBox;
    QGroupBox *vectorGroup;
    QGridLayout *gridLayout2;
    QRadioButton *_xVector;
    QRadioButton *_yVector;

    void setupUi(QDialog *ChooseColorDialog)
    {
        if (ChooseColorDialog->objectName().isEmpty())
            ChooseColorDialog->setObjectName("ChooseColorDialog");
        ChooseColorDialog->resize(425, 162);
        gridLayout = new QGridLayout(ChooseColorDialog);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        colorGroup = new QGroupBox(ChooseColorDialog);
        colorGroup->setObjectName("colorGroup");
        gridLayout1 = new QGridLayout(colorGroup);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName("gridLayout1");
        colorFrame = new QFrame(colorGroup);
        colorFrame->setObjectName("colorFrame");
        colorFrame->setFrameShape(QFrame::NoFrame);
        colorFrame->setFrameShadow(QFrame::Plain);
        colorFrame->setLineWidth(0);

        gridLayout1->addWidget(colorFrame, 0, 0, 1, 1);


        gridLayout->addWidget(colorGroup, 0, 0, 1, 1);

        _buttonBox = new QDialogButtonBox(ChooseColorDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(_buttonBox, 2, 0, 1, 1);

        vectorGroup = new QGroupBox(ChooseColorDialog);
        vectorGroup->setObjectName("vectorGroup");
        gridLayout2 = new QGridLayout(vectorGroup);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        gridLayout2->setObjectName("gridLayout2");
        _xVector = new QRadioButton(vectorGroup);
        _xVector->setObjectName("_xVector");
        _xVector->setChecked(true);

        gridLayout2->addWidget(_xVector, 0, 0, 1, 1);

        _yVector = new QRadioButton(vectorGroup);
        _yVector->setObjectName("_yVector");

        gridLayout2->addWidget(_yVector, 0, 1, 1, 1);


        gridLayout->addWidget(vectorGroup, 1, 0, 1, 1);

        QWidget::setTabOrder(_xVector, _yVector);

        retranslateUi(ChooseColorDialog);

        QMetaObject::connectSlotsByName(ChooseColorDialog);
    } // setupUi

    void retranslateUi(QDialog *ChooseColorDialog)
    {
        ChooseColorDialog->setWindowTitle(QCoreApplication::translate("ChooseColorDialog", "Assign Curve Color From File", nullptr));
        colorGroup->setTitle(QCoreApplication::translate("ChooseColorDialog", "Assign Colors", nullptr));
        vectorGroup->setTitle(QCoreApplication::translate("ChooseColorDialog", "Base Curve Color on:", nullptr));
        _xVector->setText(QCoreApplication::translate("ChooseColorDialog", "&X vector", nullptr));
        _yVector->setText(QCoreApplication::translate("ChooseColorDialog", "&Y vector", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ChooseColorDialog: public Ui_ChooseColorDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHOOSECOLORDIALOG_H
