/********************************************************************************
** Form generated from reading UI file 'shortcutdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHORTCUTDIALOG_H
#define UI_SHORTCUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>

QT_BEGIN_NAMESPACE

class Ui_ShortcutDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label;
    QLabel *_toplabel;
    QSpacerItem *spacerItem;
    QTextEdit *text;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ShortcutDialog)
    {
        if (ShortcutDialog->objectName().isEmpty())
            ShortcutDialog->setObjectName("ShortcutDialog");
        ShortcutDialog->resize(476, 491);
        gridLayout = new QGridLayout(ShortcutDialog);
        gridLayout->setObjectName("gridLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        label = new QLabel(ShortcutDialog);
        label->setObjectName("label");
        label->setPixmap(QPixmap(QString::fromUtf8(":/kst.png")));

        hboxLayout->addWidget(label);

        _toplabel = new QLabel(ShortcutDialog);
        _toplabel->setObjectName("_toplabel");
        QFont font;
        font.setPointSize(26);
        _toplabel->setFont(font);

        hboxLayout->addWidget(_toplabel);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 1, 1, 1);

        text = new QTextEdit(ShortcutDialog);
        text->setObjectName("text");

        gridLayout->addWidget(text, 1, 0, 1, 2);

        buttonBox = new QDialogButtonBox(ShortcutDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        gridLayout->addWidget(buttonBox, 2, 1, 1, 1);


        retranslateUi(ShortcutDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::clicked, ShortcutDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(ShortcutDialog);
    } // setupUi

    void retranslateUi(QDialog *ShortcutDialog)
    {
        ShortcutDialog->setWindowTitle(QCoreApplication::translate("ShortcutDialog", "Kst keyboard shortcuts", "kst is the name of the app."));
        label->setText(QString());
        _toplabel->setText(QCoreApplication::translate("ShortcutDialog", "Kst shortcuts", "the name of the app, but no other meaning."));
    } // retranslateUi

};

namespace Ui {
    class ShortcutDialog: public Ui_ShortcutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHORTCUTDIALOG_H
