/********************************************************************************
** Form generated from reading UI file 'filtermultipledialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERMULTIPLEDIALOG_H
#define UI_FILTERMULTIPLEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_FilterMultipleDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QListWidget *_availableVectors;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QLabel *label_3;
    QPushButton *_timestreams;
    QPushButton *_addAll;
    QPushButton *_add;
    QPushButton *_remove;
    QPushButton *_removeAll;
    QSpacerItem *spacerItem;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_2;
    QListWidget *_selectedVectors;
    QFrame *frame;
    QVBoxLayout *verticalLayout_3;
    QComboBox *_pluginCombo;
    QLabel *_descriptionLabel;
    QFrame *_inputOutputBox;
    QRadioButton *_replaceCurves;
    QRadioButton *_copyCurves;
    QRadioButton *_neitherCurves;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *FilterMultipleDialog)
    {
        if (FilterMultipleDialog->objectName().isEmpty())
            FilterMultipleDialog->setObjectName("FilterMultipleDialog");
        gridLayout = new QGridLayout(FilterMultipleDialog);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(FilterMultipleDialog);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        _availableVectors = new QListWidget(FilterMultipleDialog);
        _availableVectors->setObjectName("_availableVectors");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_availableVectors->sizePolicy().hasHeightForWidth());
        _availableVectors->setSizePolicy(sizePolicy);
        _availableVectors->setSelectionMode(QAbstractItemView::ExtendedSelection);

        verticalLayout_2->addWidget(_availableVectors);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        label_3 = new QLabel(FilterMultipleDialog);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        _timestreams = new QPushButton(FilterMultipleDialog);
        _timestreams->setObjectName("_timestreams");

        verticalLayout->addWidget(_timestreams);

        _addAll = new QPushButton(FilterMultipleDialog);
        _addAll->setObjectName("_addAll");

        verticalLayout->addWidget(_addAll);

        _add = new QPushButton(FilterMultipleDialog);
        _add->setObjectName("_add");

        verticalLayout->addWidget(_add);

        _remove = new QPushButton(FilterMultipleDialog);
        _remove->setObjectName("_remove");

        verticalLayout->addWidget(_remove);

        _removeAll = new QPushButton(FilterMultipleDialog);
        _removeAll->setObjectName("_removeAll");

        verticalLayout->addWidget(_removeAll);

        spacerItem = new QSpacerItem(80, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);


        horizontalLayout->addLayout(verticalLayout);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_2 = new QLabel(FilterMultipleDialog);
        label_2->setObjectName("label_2");

        verticalLayout_4->addWidget(label_2);

        _selectedVectors = new QListWidget(FilterMultipleDialog);
        _selectedVectors->setObjectName("_selectedVectors");
        sizePolicy.setHeightForWidth(_selectedVectors->sizePolicy().hasHeightForWidth());
        _selectedVectors->setSizePolicy(sizePolicy);
        _selectedVectors->setSelectionMode(QAbstractItemView::ExtendedSelection);

        verticalLayout_4->addWidget(_selectedVectors);


        horizontalLayout->addLayout(verticalLayout_4);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 2);

        frame = new QFrame(FilterMultipleDialog);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout_3 = new QVBoxLayout(frame);
        verticalLayout_3->setObjectName("verticalLayout_3");
        _pluginCombo = new QComboBox(frame);
        _pluginCombo->setObjectName("_pluginCombo");

        verticalLayout_3->addWidget(_pluginCombo);

        _descriptionLabel = new QLabel(frame);
        _descriptionLabel->setObjectName("_descriptionLabel");

        verticalLayout_3->addWidget(_descriptionLabel);


        gridLayout->addWidget(frame, 1, 0, 1, 2);

        _inputOutputBox = new QFrame(FilterMultipleDialog);
        _inputOutputBox->setObjectName("_inputOutputBox");
        sizePolicy1.setHeightForWidth(_inputOutputBox->sizePolicy().hasHeightForWidth());
        _inputOutputBox->setSizePolicy(sizePolicy1);
        _inputOutputBox->setFrameShape(QFrame::StyledPanel);
        _inputOutputBox->setFrameShadow(QFrame::Raised);

        gridLayout->addWidget(_inputOutputBox, 2, 0, 4, 1);

        _replaceCurves = new QRadioButton(FilterMultipleDialog);
        _replaceCurves->setObjectName("_replaceCurves");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_replaceCurves->sizePolicy().hasHeightForWidth());
        _replaceCurves->setSizePolicy(sizePolicy2);
        _replaceCurves->setChecked(true);

        gridLayout->addWidget(_replaceCurves, 2, 1, 1, 1);

        _copyCurves = new QRadioButton(FilterMultipleDialog);
        _copyCurves->setObjectName("_copyCurves");

        gridLayout->addWidget(_copyCurves, 3, 1, 1, 1);

        _neitherCurves = new QRadioButton(FilterMultipleDialog);
        _neitherCurves->setObjectName("_neitherCurves");

        gridLayout->addWidget(_neitherCurves, 4, 1, 1, 1);

        buttonBox = new QDialogButtonBox(FilterMultipleDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 6, 0, 1, 2);


        retranslateUi(FilterMultipleDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, FilterMultipleDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FilterMultipleDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(FilterMultipleDialog);
    } // setupUi

    void retranslateUi(QDialog *FilterMultipleDialog)
    {
        FilterMultipleDialog->setWindowTitle(QCoreApplication::translate("FilterMultipleDialog", "Dialog", nullptr));
        label->setText(QCoreApplication::translate("FilterMultipleDialog", "Available", nullptr));
#if QT_CONFIG(whatsthis)
        _availableVectors->setWhatsThis(QCoreApplication::translate("FilterMultipleDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_3->setText(QString());
#if QT_CONFIG(statustip)
        _timestreams->setStatusTip(QCoreApplication::translate("FilterMultipleDialog", "Select timestream Y axis vectors", nullptr));
#endif // QT_CONFIG(statustip)
        _timestreams->setText(QCoreApplication::translate("FilterMultipleDialog", "Smart", nullptr));
#if QT_CONFIG(statustip)
        _addAll->setStatusTip(QCoreApplication::translate("FilterMultipleDialog", "Select all in list", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        _addAll->setWhatsThis(QCoreApplication::translate("FilterMultipleDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _addAll->setText(QCoreApplication::translate("FilterMultipleDialog", "All", nullptr));
#if QT_CONFIG(statustip)
        _add->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        _remove->setWhatsThis(QCoreApplication::translate("FilterMultipleDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        _removeAll->setWhatsThis(QCoreApplication::translate("FilterMultipleDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _removeAll->setText(QCoreApplication::translate("FilterMultipleDialog", "All", nullptr));
        label_2->setText(QCoreApplication::translate("FilterMultipleDialog", "Selected", nullptr));
#if QT_CONFIG(whatsthis)
        _selectedVectors->setWhatsThis(QCoreApplication::translate("FilterMultipleDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _descriptionLabel->setText(QString());
        _replaceCurves->setText(QCoreApplication::translate("FilterMultipleDialog", "Rep&lace in Curves", nullptr));
        _copyCurves->setText(QCoreApplication::translate("FilterMultipleDialog", "Copy cur&ves", nullptr));
        _neitherCurves->setText(QCoreApplication::translate("FilterMultipleDialog", "&Neither", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FilterMultipleDialog: public Ui_FilterMultipleDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERMULTIPLEDIALOG_H
