/********************************************************************************
** Form generated from reading UI file 'colorpalette.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLORPALETTE_H
#define UI_COLORPALETTE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ColorPalette
{
public:
    QHBoxLayout *hboxLayout;
    QLabel *_label;
    QComboBox *_palette;
    QLabel *_paletteDisplay;

    void setupUi(QWidget *ColorPalette)
    {
        if (ColorPalette->objectName().isEmpty())
            ColorPalette->setObjectName("ColorPalette");
        ColorPalette->resize(474, 30);
        hboxLayout = new QHBoxLayout(ColorPalette);
        hboxLayout->setSpacing(0);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        _label = new QLabel(ColorPalette);
        _label->setObjectName("_label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_label->sizePolicy().hasHeightForWidth());
        _label->setSizePolicy(sizePolicy);
        _label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _label->setWordWrap(false);

        hboxLayout->addWidget(_label);

        _palette = new QComboBox(ColorPalette);
        _palette->setObjectName("_palette");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_palette->sizePolicy().hasHeightForWidth());
        _palette->setSizePolicy(sizePolicy1);
        _palette->setEditable(false);

        hboxLayout->addWidget(_palette);

        _paletteDisplay = new QLabel(ColorPalette);
        _paletteDisplay->setObjectName("_paletteDisplay");
        sizePolicy.setHeightForWidth(_paletteDisplay->sizePolicy().hasHeightForWidth());
        _paletteDisplay->setSizePolicy(sizePolicy);
        _paletteDisplay->setWordWrap(false);

        hboxLayout->addWidget(_paletteDisplay);

#if QT_CONFIG(shortcut)
        _label->setBuddy(_palette);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ColorPalette);

        QMetaObject::connectSlotsByName(ColorPalette);
    } // setupUi

    void retranslateUi(QWidget *ColorPalette)
    {
        _label->setText(QCoreApplication::translate("ColorPalette", "C&olor palette:", nullptr));
        _paletteDisplay->setText(QString());
        (void)ColorPalette;
    } // retranslateUi

};

namespace Ui {
    class ColorPalette: public Ui_ColorPalette {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COLORPALETTE_H
