/********************************************************************************
** Form generated from reading UI file 'basicplugintab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BASICPLUGINTAB_H
#define UI_BASICPLUGINTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_BasicPluginTab
{
public:
    QGridLayout *gridLayout;
    QGroupBox *_inputOutputBox;

    void setupUi(QWidget *BasicPluginTab)
    {
        if (BasicPluginTab->objectName().isEmpty())
            BasicPluginTab->setObjectName("BasicPluginTab");
        BasicPluginTab->setWindowModality(Qt::NonModal);
        BasicPluginTab->resize(182, 52);
        gridLayout = new QGridLayout(BasicPluginTab);
        gridLayout->setSpacing(-1);
        gridLayout->setContentsMargins(9, 9, 9, 9);
        gridLayout->setObjectName("gridLayout");
        _inputOutputBox = new QGroupBox(BasicPluginTab);
        _inputOutputBox->setObjectName("_inputOutputBox");

        gridLayout->addWidget(_inputOutputBox, 1, 0, 1, 1);


        retranslateUi(BasicPluginTab);

        QMetaObject::connectSlotsByName(BasicPluginTab);
    } // setupUi

    void retranslateUi(QWidget *BasicPluginTab)
    {
        _inputOutputBox->setTitle(QCoreApplication::translate("BasicPluginTab", "Inputs and Outputs", nullptr));
        (void)BasicPluginTab;
    } // retranslateUi

};

namespace Ui {
    class BasicPluginTab: public Ui_BasicPluginTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BASICPLUGINTAB_H
