/********************************************************************************
** Form generated from reading UI file 'datawizardpagevectors.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATAWIZARDPAGEVECTORS_H
#define UI_DATAWIZARDPAGEVECTORS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWizardPage>

QT_BEGIN_NAMESPACE

class Ui_DataWizardPageVectors
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *spacer117;
    QGridLayout *gridLayout1;
    QPushButton *_up;
    QPushButton *_down;
    QPushButton *_add;
    QPushButton *_remove;
    QSpacerItem *spacer118;
    QGridLayout *gridLayout2;
    QListWidget *_vectorsToPlot;
    QLabel *textLabel1;
    QLabel *_selectedLabel;
    QGridLayout *gridLayout3;
    QLineEdit *_vectorReduction;
    QPushButton *_vectorSearch;
    QListWidget *_vectors;
    QLabel *textLabel1_2;
    QLabel *_availableLabel;

    void setupUi(QWizardPage *DataWizardPageVectors)
    {
        if (DataWizardPageVectors->objectName().isEmpty())
            DataWizardPageVectors->setObjectName("DataWizardPageVectors");
        DataWizardPageVectors->resize(779, 313);
        gridLayout = new QGridLayout(DataWizardPageVectors);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        spacer117 = new QSpacerItem(121, 102, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer117, 0, 1, 1, 1);

        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        _up = new QPushButton(DataWizardPageVectors);
        _up->setObjectName("_up");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_up->sizePolicy().hasHeightForWidth());
        _up->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_up, 1, 1, 1, 1);

        _down = new QPushButton(DataWizardPageVectors);
        _down->setObjectName("_down");
        sizePolicy.setHeightForWidth(_down->sizePolicy().hasHeightForWidth());
        _down->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_down, 3, 1, 1, 1);

        _add = new QPushButton(DataWizardPageVectors);
        _add->setObjectName("_add");
        sizePolicy.setHeightForWidth(_add->sizePolicy().hasHeightForWidth());
        _add->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_add, 2, 2, 1, 1);

        _remove = new QPushButton(DataWizardPageVectors);
        _remove->setObjectName("_remove");
        sizePolicy.setHeightForWidth(_remove->sizePolicy().hasHeightForWidth());
        _remove->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_remove, 2, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 1, 1, 1, 1);

        spacer118 = new QSpacerItem(138, 97, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer118, 2, 1, 1, 1);

        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        _vectorsToPlot = new QListWidget(DataWizardPageVectors);
        _vectorsToPlot->setObjectName("_vectorsToPlot");
        _vectorsToPlot->setSelectionMode(QAbstractItemView::ExtendedSelection);

        gridLayout2->addWidget(_vectorsToPlot, 1, 0, 1, 2);

        textLabel1 = new QLabel(DataWizardPageVectors);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout2->addWidget(textLabel1, 0, 0, 1, 2);

        _selectedLabel = new QLabel(DataWizardPageVectors);
        _selectedLabel->setObjectName("_selectedLabel");

        gridLayout2->addWidget(_selectedLabel, 2, 0, 1, 1);


        gridLayout->addLayout(gridLayout2, 0, 2, 3, 1);

        gridLayout3 = new QGridLayout();
        gridLayout3->setSpacing(6);
        gridLayout3->setObjectName("gridLayout3");
        gridLayout3->setContentsMargins(0, 0, 0, 0);
        _vectorReduction = new QLineEdit(DataWizardPageVectors);
        _vectorReduction->setObjectName("_vectorReduction");

        gridLayout3->addWidget(_vectorReduction, 1, 0, 1, 1);

        _vectorSearch = new QPushButton(DataWizardPageVectors);
        _vectorSearch->setObjectName("_vectorSearch");
        sizePolicy.setHeightForWidth(_vectorSearch->sizePolicy().hasHeightForWidth());
        _vectorSearch->setSizePolicy(sizePolicy);

        gridLayout3->addWidget(_vectorSearch, 1, 1, 1, 1);

        _vectors = new QListWidget(DataWizardPageVectors);
        _vectors->setObjectName("_vectors");
        _vectors->setSelectionMode(QAbstractItemView::ExtendedSelection);

        gridLayout3->addWidget(_vectors, 2, 0, 1, 2);

        textLabel1_2 = new QLabel(DataWizardPageVectors);
        textLabel1_2->setObjectName("textLabel1_2");
        textLabel1_2->setWordWrap(false);

        gridLayout3->addWidget(textLabel1_2, 0, 0, 1, 2);

        _availableLabel = new QLabel(DataWizardPageVectors);
        _availableLabel->setObjectName("_availableLabel");

        gridLayout3->addWidget(_availableLabel, 3, 0, 1, 1);


        gridLayout->addLayout(gridLayout3, 0, 0, 3, 1);

        QWidget::setTabOrder(_vectorReduction, _vectorSearch);
        QWidget::setTabOrder(_vectorSearch, _vectors);
        QWidget::setTabOrder(_vectors, _up);
        QWidget::setTabOrder(_up, _remove);
        QWidget::setTabOrder(_remove, _add);
        QWidget::setTabOrder(_add, _down);
        QWidget::setTabOrder(_down, _vectorsToPlot);

        retranslateUi(DataWizardPageVectors);
        QObject::connect(_add, &QPushButton::clicked, _vectorReduction, qOverload<>(&QLineEdit::setFocus));
        QObject::connect(_add, &QPushButton::clicked, _vectorReduction, qOverload<>(&QLineEdit::selectAll));
        QObject::connect(_vectors, &QListWidget::clicked, _vectorReduction, qOverload<>(&QLineEdit::clear));

        QMetaObject::connectSlotsByName(DataWizardPageVectors);
    } // setupUi

    void retranslateUi(QWizardPage *DataWizardPageVectors)
    {
#if QT_CONFIG(whatsthis)
        _up->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Raise in plot order (Alt-up)", nullptr));
#endif // QT_CONFIG(whatsthis)
        _up->setText(QString());
#if QT_CONFIG(shortcut)
        _up->setShortcut(QCoreApplication::translate("DataWizardPageVectors", "Alt+Up", nullptr));
#endif // QT_CONFIG(shortcut)
        _down->setText(QString());
#if QT_CONFIG(shortcut)
        _down->setShortcut(QCoreApplication::translate("DataWizardPageVectors", "Alt+Down", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        _add->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Select (Alt-S): Move highlighted fields from the Available data list to the Selected data list for plotting.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _add->setText(QString());
#if QT_CONFIG(shortcut)
        _add->setShortcut(QCoreApplication::translate("DataWizardPageVectors", "Alt+S", "Select for list."));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        _remove->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Remove (Alt-R): Move highlighted fields from the Selected data list to the Available data list so that they will not be plotted.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _remove->setText(QString());
#if QT_CONFIG(shortcut)
        _remove->setShortcut(QCoreApplication::translate("DataWizardPageVectors", "Alt+R", "Remove from list"));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        _vectorsToPlot->setToolTip(QCoreApplication::translate("DataWizardPageVectors", "Selected data: All fields in this list are plotted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _vectorsToPlot->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Selected data: All fields in this list are plotted.  To remove fields from this list, drag them to the Available data list, double click on them, or select them and use the Remove button (right arrow icon or Alt-R).  The order in which fields appear in this list is the order in which they will be plotted.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(QCoreApplication::translate("DataWizardPageVectors", "Selected data:", nullptr));
        _selectedLabel->setText(QCoreApplication::translate("DataWizardPageVectors", "0 vector(s) selected", nullptr));
#if QT_CONFIG(tooltip)
        _vectorReduction->setToolTip(QCoreApplication::translate("DataWizardPageVectors", "Search Filter", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _vectorReduction->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Search Filter: Fields matching this filter will be highlighted and moved to the top of the Available data list.  '*' and '?' wild cards are supported.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        _vectorSearch->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Appends the '*' wild card to the beginning and end of the search filter to match everything containing the search filter text.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _vectorSearch->setText(QCoreApplication::translate("DataWizardPageVectors", "S&earch", nullptr));
#if QT_CONFIG(tooltip)
        _vectors->setToolTip(QCoreApplication::translate("DataWizardPageVectors", "Field selection: Checked fields are plotted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _vectors->setWhatsThis(QCoreApplication::translate("DataWizardPageVectors", "Available data: This box lists all fields which have not been selected for plotting.  To select a field for plotting, move it to the Selected data list, by dragging it with the mouse, by double clicking, or by highlighting it, and clicking on the select button (Right Arrow - accelerator is Alt-S).", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1_2->setText(QCoreApplication::translate("DataWizardPageVectors", "Available data:", nullptr));
        _availableLabel->setText(QCoreApplication::translate("DataWizardPageVectors", "- vector(s) available", nullptr));
        (void)DataWizardPageVectors;
    } // retranslateUi

};

namespace Ui {
    class DataWizardPageVectors: public Ui_DataWizardPageVectors {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATAWIZARDPAGEVECTORS_H
