/********************************************************************************
** Form generated from reading UI file 'scalarselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCALARSELECTOR_H
#define UI_SCALARSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "combobox.h"

QT_BEGIN_NAMESPACE

class Ui_ScalarSelector
{
public:
    QGridLayout *gridLayout;
    Kst::ComboBox *_scalar;
    QToolButton *_newScalar;
    QToolButton *_editScalar;
    QToolButton *_selectScalar;
    QLabel *_cutoffLabel;
    QLabel *_SRLabel;
    QLineEdit *_cutoff;
    QLineEdit *_SR;

    void setupUi(QWidget *ScalarSelector)
    {
        if (ScalarSelector->objectName().isEmpty())
            ScalarSelector->setObjectName("ScalarSelector");
        ScalarSelector->resize(570, 38);
        ScalarSelector->setProperty("isFOverSR", QVariant(false));
        gridLayout = new QGridLayout(ScalarSelector);
        gridLayout->setSpacing(0);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        _scalar = new Kst::ComboBox(ScalarSelector);
        _scalar->setObjectName("_scalar");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_scalar->sizePolicy().hasHeightForWidth());
        _scalar->setSizePolicy(sizePolicy);
        _scalar->setEditable(true);
        _scalar->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        gridLayout->addWidget(_scalar, 0, 0, 1, 1);

        _newScalar = new QToolButton(ScalarSelector);
        _newScalar->setObjectName("_newScalar");

        gridLayout->addWidget(_newScalar, 0, 2, 1, 1);

        _editScalar = new QToolButton(ScalarSelector);
        _editScalar->setObjectName("_editScalar");
        _editScalar->setEnabled(false);

        gridLayout->addWidget(_editScalar, 0, 3, 1, 1);

        _selectScalar = new QToolButton(ScalarSelector);
        _selectScalar->setObjectName("_selectScalar");

        gridLayout->addWidget(_selectScalar, 0, 1, 1, 1);

        _cutoffLabel = new QLabel(ScalarSelector);
        _cutoffLabel->setObjectName("_cutoffLabel");
        _cutoffLabel->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_cutoffLabel->sizePolicy().hasHeightForWidth());
        _cutoffLabel->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_cutoffLabel, 0, 4, 1, 1);

        _SRLabel = new QLabel(ScalarSelector);
        _SRLabel->setObjectName("_SRLabel");
        sizePolicy1.setHeightForWidth(_SRLabel->sizePolicy().hasHeightForWidth());
        _SRLabel->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_SRLabel, 0, 6, 1, 1);

        _cutoff = new QLineEdit(ScalarSelector);
        _cutoff->setObjectName("_cutoff");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_cutoff->sizePolicy().hasHeightForWidth());
        _cutoff->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(_cutoff, 0, 5, 1, 1);

        _SR = new QLineEdit(ScalarSelector);
        _SR->setObjectName("_SR");
        sizePolicy2.setHeightForWidth(_SR->sizePolicy().hasHeightForWidth());
        _SR->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(_SR, 0, 7, 1, 1);

        QWidget::setTabOrder(_scalar, _selectScalar);
        QWidget::setTabOrder(_selectScalar, _newScalar);
        QWidget::setTabOrder(_newScalar, _editScalar);

        retranslateUi(ScalarSelector);

        QMetaObject::connectSlotsByName(ScalarSelector);
    } // setupUi

    void retranslateUi(QWidget *ScalarSelector)
    {
#if QT_CONFIG(whatsthis)
        ScalarSelector->setWhatsThis(QCoreApplication::translate("ScalarSelector", "Select a scalar to utilize.  If you wish to create a new scalar, select the icon to the right.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        _newScalar->setToolTip(QCoreApplication::translate("ScalarSelector", "Create a new scalar.", nullptr));
#endif // QT_CONFIG(tooltip)
        _newScalar->setText(QString());
#if QT_CONFIG(tooltip)
        _editScalar->setToolTip(QCoreApplication::translate("ScalarSelector", "Edit selected scalar.", nullptr));
#endif // QT_CONFIG(tooltip)
        _editScalar->setText(QString());
        _selectScalar->setText(QCoreApplication::translate("ScalarSelector", "...", nullptr));
        _cutoffLabel->setText(QCoreApplication::translate("ScalarSelector", "  f:", nullptr));
        _SRLabel->setText(QCoreApplication::translate("ScalarSelector", " SR:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ScalarSelector: public Ui_ScalarSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCALARSELECTOR_H
