/********************************************************************************
** Form generated from reading UI file 'overridelabeltab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OVERRIDELABELTAB_H
#define UI_OVERRIDELABELTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_OverrideLabelTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    QPushButton *_italic;
    Kst::ColorButton *_labelColor;
    QLabel *_Label_11;
    QLabel *_Label_12;
    QDoubleSpinBox *_fontSize;
    QPushButton *_bold;
    QSpacerItem *horizontalSpacer;
    QFontComboBox *_family;
    QCheckBox *_useDefault;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *OverrideLabelTab)
    {
        if (OverrideLabelTab->objectName().isEmpty())
            OverrideLabelTab->setObjectName("OverrideLabelTab");
        OverrideLabelTab->resize(1425, 858);
        verticalLayout = new QVBoxLayout(OverrideLabelTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(OverrideLabelTab);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label, 1, 2, 1, 1);

        _italic = new QPushButton(OverrideLabelTab);
        _italic->setObjectName("_italic");
        _italic->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_italic->sizePolicy().hasHeightForWidth());
        _italic->setSizePolicy(sizePolicy1);
        _italic->setCheckable(true);

        gridLayout->addWidget(_italic, 1, 4, 1, 1);

        _labelColor = new Kst::ColorButton(OverrideLabelTab);
        _labelColor->setObjectName("_labelColor");
        _labelColor->setEnabled(false);

        gridLayout->addWidget(_labelColor, 1, 5, 1, 1);

        _Label_11 = new QLabel(OverrideLabelTab);
        _Label_11->setObjectName("_Label_11");
        _Label_11->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_Label_11->sizePolicy().hasHeightForWidth());
        _Label_11->setSizePolicy(sizePolicy2);
        _Label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_11, 1, 0, 1, 1);

        _Label_12 = new QLabel(OverrideLabelTab);
        _Label_12->setObjectName("_Label_12");
        _Label_12->setEnabled(false);
        sizePolicy1.setHeightForWidth(_Label_12->sizePolicy().hasHeightForWidth());
        _Label_12->setSizePolicy(sizePolicy1);
        _Label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_12, 2, 0, 1, 1);

        _fontSize = new QDoubleSpinBox(OverrideLabelTab);
        _fontSize->setObjectName("_fontSize");
        _fontSize->setEnabled(false);
        _fontSize->setDecimals(1);
        _fontSize->setMinimum(-25.000000000000000);
        _fontSize->setSingleStep(1.000000000000000);

        gridLayout->addWidget(_fontSize, 1, 1, 1, 1);

        _bold = new QPushButton(OverrideLabelTab);
        _bold->setObjectName("_bold");
        _bold->setEnabled(false);
        sizePolicy1.setHeightForWidth(_bold->sizePolicy().hasHeightForWidth());
        _bold->setSizePolicy(sizePolicy1);
        _bold->setCheckable(true);

        gridLayout->addWidget(_bold, 1, 3, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 6, 1, 1);

        _family = new QFontComboBox(OverrideLabelTab);
        _family->setObjectName("_family");
        _family->setEnabled(false);

        gridLayout->addWidget(_family, 2, 1, 1, 6);

        _useDefault = new QCheckBox(OverrideLabelTab);
        _useDefault->setObjectName("_useDefault");
        _useDefault->setChecked(true);

        gridLayout->addWidget(_useDefault, 0, 0, 1, 7);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 659, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        _Label_11->setBuddy(_fontSize);
        _Label_12->setBuddy(_family);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_useDefault, _fontSize);
        QWidget::setTabOrder(_fontSize, _bold);
        QWidget::setTabOrder(_bold, _italic);
        QWidget::setTabOrder(_italic, _labelColor);
        QWidget::setTabOrder(_labelColor, _family);

        retranslateUi(OverrideLabelTab);
        QObject::connect(_useDefault, &QCheckBox::toggled, _Label_11, &QLabel::setDisabled);
        QObject::connect(_useDefault, &QCheckBox::toggled, _Label_12, &QLabel::setDisabled);
        QObject::connect(_useDefault, SIGNAL(toggled(bool)), _labelColor, SLOT(setDisabled(bool)));
        QObject::connect(_useDefault, &QCheckBox::toggled, _family, &QFontComboBox::setDisabled);
        QObject::connect(_useDefault, &QCheckBox::toggled, _fontSize, &QDoubleSpinBox::setDisabled);
        QObject::connect(_useDefault, &QCheckBox::toggled, _italic, &QPushButton::setDisabled);
        QObject::connect(_useDefault, &QCheckBox::toggled, _bold, &QPushButton::setDisabled);

        QMetaObject::connectSlotsByName(OverrideLabelTab);
    } // setupUi

    void retranslateUi(QWidget *OverrideLabelTab)
    {
        label->setText(QCoreApplication::translate("OverrideLabelTab", " points", nullptr));
        _Label_11->setText(QCoreApplication::translate("OverrideLabelTab", "Font si&ze:", nullptr));
        _Label_12->setText(QCoreApplication::translate("OverrideLabelTab", "Font fa&mily:", nullptr));
        _useDefault->setText(QCoreApplication::translate("OverrideLabelTab", "&Use plot default font", nullptr));
        (void)OverrideLabelTab;
    } // retranslateUi

};

namespace Ui {
    class OverrideLabelTab: public Ui_OverrideLabelTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OVERRIDELABELTAB_H
