/********************************************************************************
** Form generated from reading UI file 'contenttab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONTENTTAB_H
#define UI_CONTENTTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ContentTab
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel4_4_2;
    QHBoxLayout *horizontalLayout_5;
    QLineEdit *_curveSearchEntry;
    QPushButton *_curveSearch;
    QListWidget *_availableRelationList;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *_deselectAllAvailable;
    QPushButton *_selectAllAvailable;
    QPushButton *_editSelectedAvailable;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *spacerItem;
    QGridLayout *gridLayout1;
    QPushButton *_up;
    QPushButton *_down;
    QPushButton *_add;
    QPushButton *_remove;
    QSpacerItem *spacerItem1;
    QVBoxLayout *verticalLayout;
    QLabel *TextLabel3_3_2;
    QListWidget *_displayedRelationList;
    QHBoxLayout *horizontalLayout;
    QPushButton *_deselectAllDisplayed;
    QPushButton *_selectAllDisplayed;
    QPushButton *_editSelectedDisplayed;
    QFrame *line;
    QButtonGroup *buttonGroup;

    void setupUi(QWidget *ContentTab)
    {
        if (ContentTab->objectName().isEmpty())
            ContentTab->setObjectName("ContentTab");
        ContentTab->resize(731, 419);
        gridLayout = new QGridLayout(ContentTab);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        TextLabel4_4_2 = new QLabel(ContentTab);
        TextLabel4_4_2->setObjectName("TextLabel4_4_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel4_4_2->sizePolicy().hasHeightForWidth());
        TextLabel4_4_2->setSizePolicy(sizePolicy);
        TextLabel4_4_2->setWordWrap(false);

        vboxLayout->addWidget(TextLabel4_4_2);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        _curveSearchEntry = new QLineEdit(ContentTab);
        _curveSearchEntry->setObjectName("_curveSearchEntry");

        horizontalLayout_5->addWidget(_curveSearchEntry);

        _curveSearch = new QPushButton(ContentTab);
        _curveSearch->setObjectName("_curveSearch");

        horizontalLayout_5->addWidget(_curveSearch);


        vboxLayout->addLayout(horizontalLayout_5);

        _availableRelationList = new QListWidget(ContentTab);
        _availableRelationList->setObjectName("_availableRelationList");
        _availableRelationList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout->addWidget(_availableRelationList);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _deselectAllAvailable = new QPushButton(ContentTab);
        _deselectAllAvailable->setObjectName("_deselectAllAvailable");

        horizontalLayout_2->addWidget(_deselectAllAvailable);

        _selectAllAvailable = new QPushButton(ContentTab);
        _selectAllAvailable->setObjectName("_selectAllAvailable");

        horizontalLayout_2->addWidget(_selectAllAvailable);

        _editSelectedAvailable = new QPushButton(ContentTab);
        _editSelectedAvailable->setObjectName("_editSelectedAvailable");

        horizontalLayout_2->addWidget(_editSelectedAvailable);


        vboxLayout->addLayout(horizontalLayout_2);


        horizontalLayout_3->addLayout(vboxLayout);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        spacerItem = new QSpacerItem(31, 50, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacerItem);

        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        _up = new QPushButton(ContentTab);
        _up->setObjectName("_up");
        _up->setEnabled(false);
        sizePolicy.setHeightForWidth(_up->sizePolicy().hasHeightForWidth());
        _up->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_up, 1, 1, 1, 1);

        _down = new QPushButton(ContentTab);
        _down->setObjectName("_down");
        _down->setEnabled(false);
        sizePolicy.setHeightForWidth(_down->sizePolicy().hasHeightForWidth());
        _down->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_down, 3, 1, 1, 1);

        _add = new QPushButton(ContentTab);
        _add->setObjectName("_add");
        _add->setEnabled(false);
        sizePolicy.setHeightForWidth(_add->sizePolicy().hasHeightForWidth());
        _add->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_add, 2, 2, 1, 1);

        _remove = new QPushButton(ContentTab);
        _remove->setObjectName("_remove");
        _remove->setEnabled(false);
        sizePolicy.setHeightForWidth(_remove->sizePolicy().hasHeightForWidth());
        _remove->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(_remove, 2, 0, 1, 1);


        verticalLayout_2->addLayout(gridLayout1);

        spacerItem1 = new QSpacerItem(31, 110, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacerItem1);


        horizontalLayout_3->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        TextLabel3_3_2 = new QLabel(ContentTab);
        TextLabel3_3_2->setObjectName("TextLabel3_3_2");
        sizePolicy.setHeightForWidth(TextLabel3_3_2->sizePolicy().hasHeightForWidth());
        TextLabel3_3_2->setSizePolicy(sizePolicy);
        TextLabel3_3_2->setWordWrap(false);

        verticalLayout->addWidget(TextLabel3_3_2);

        _displayedRelationList = new QListWidget(ContentTab);
        _displayedRelationList->setObjectName("_displayedRelationList");
        _displayedRelationList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        verticalLayout->addWidget(_displayedRelationList);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        _deselectAllDisplayed = new QPushButton(ContentTab);
        buttonGroup = new QButtonGroup(ContentTab);
        buttonGroup->setObjectName("buttonGroup");
        buttonGroup->addButton(_deselectAllDisplayed);
        _deselectAllDisplayed->setObjectName("_deselectAllDisplayed");

        horizontalLayout->addWidget(_deselectAllDisplayed);

        _selectAllDisplayed = new QPushButton(ContentTab);
        buttonGroup->addButton(_selectAllDisplayed);
        _selectAllDisplayed->setObjectName("_selectAllDisplayed");

        horizontalLayout->addWidget(_selectAllDisplayed);

        _editSelectedDisplayed = new QPushButton(ContentTab);
        buttonGroup->addButton(_editSelectedDisplayed);
        _editSelectedDisplayed->setObjectName("_editSelectedDisplayed");

        horizontalLayout->addWidget(_editSelectedDisplayed);


        verticalLayout->addLayout(horizontalLayout);


        horizontalLayout_3->addLayout(verticalLayout);


        gridLayout->addLayout(horizontalLayout_3, 0, 0, 1, 1);

        line = new QFrame(ContentTab);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 1, 0, 1, 1);

#if QT_CONFIG(shortcut)
        TextLabel4_4_2->setBuddy(_availableRelationList);
        TextLabel3_3_2->setBuddy(_displayedRelationList);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_availableRelationList, _up);
        QWidget::setTabOrder(_up, _remove);
        QWidget::setTabOrder(_remove, _add);
        QWidget::setTabOrder(_add, _down);
        QWidget::setTabOrder(_down, _displayedRelationList);
        QWidget::setTabOrder(_displayedRelationList, _deselectAllAvailable);
        QWidget::setTabOrder(_deselectAllAvailable, _selectAllAvailable);
        QWidget::setTabOrder(_selectAllAvailable, _editSelectedAvailable);
        QWidget::setTabOrder(_editSelectedAvailable, _deselectAllDisplayed);
        QWidget::setTabOrder(_deselectAllDisplayed, _selectAllDisplayed);
        QWidget::setTabOrder(_selectAllDisplayed, _editSelectedDisplayed);

        retranslateUi(ContentTab);
        QObject::connect(_deselectAllAvailable, &QPushButton::clicked, _availableRelationList, qOverload<>(&QListWidget::clearSelection));
        QObject::connect(_selectAllAvailable, &QPushButton::clicked, _availableRelationList, qOverload<>(&QListWidget::selectAll));
        QObject::connect(_deselectAllDisplayed, &QPushButton::clicked, _displayedRelationList, qOverload<>(&QListWidget::clearSelection));
        QObject::connect(_selectAllDisplayed, &QPushButton::clicked, _displayedRelationList, qOverload<>(&QListWidget::selectAll));

        QMetaObject::connectSlotsByName(ContentTab);
    } // setupUi

    void retranslateUi(QWidget *ContentTab)
    {
        TextLabel4_4_2->setText(QCoreApplication::translate("ContentTab", "A&vailable objects:", nullptr));
        _curveSearch->setText(QCoreApplication::translate("ContentTab", "Search", nullptr));
        _deselectAllAvailable->setText(QCoreApplication::translate("ContentTab", "Deselect  All", nullptr));
        _selectAllAvailable->setText(QCoreApplication::translate("ContentTab", "Select All", nullptr));
        _editSelectedAvailable->setText(QCoreApplication::translate("ContentTab", "Edit", nullptr));
#if QT_CONFIG(tooltip)
        _up->setToolTip(QCoreApplication::translate("ContentTab", "Raise in plot order: Alt+up", nullptr));
#endif // QT_CONFIG(tooltip)
        _up->setText(QString());
#if QT_CONFIG(shortcut)
        _up->setShortcut(QCoreApplication::translate("ContentTab", "Alt+Up", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        _down->setToolTip(QCoreApplication::translate("ContentTab", "Lower in plot order: Alt+down", nullptr));
#endif // QT_CONFIG(tooltip)
        _down->setText(QString());
#if QT_CONFIG(shortcut)
        _down->setShortcut(QCoreApplication::translate("ContentTab", "Alt+Down", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        _add->setToolTip(QCoreApplication::translate("ContentTab", "Select: Alt+s", nullptr));
#endif // QT_CONFIG(tooltip)
        _add->setText(QString());
#if QT_CONFIG(tooltip)
        _remove->setToolTip(QCoreApplication::translate("ContentTab", "Remove: Alt+r", nullptr));
#endif // QT_CONFIG(tooltip)
        _remove->setText(QString());
        TextLabel3_3_2->setText(QCoreApplication::translate("ContentTab", "Displayed obje&cts:", nullptr));
        _deselectAllDisplayed->setText(QCoreApplication::translate("ContentTab", "Deselect  All", nullptr));
        _selectAllDisplayed->setText(QCoreApplication::translate("ContentTab", "Select All", nullptr));
        _editSelectedDisplayed->setText(QCoreApplication::translate("ContentTab", "Edit", nullptr));
        (void)ContentTab;
    } // retranslateUi

};

namespace Ui {
    class ContentTab: public Ui_ContentTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONTENTTAB_H
