/********************************************************************************
** Form generated from reading UI file 'dimensionstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIMENSIONSTAB_H
#define UI_DIMENSIONSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DimensionsTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_5;
    QDoubleSpinBox *_rotation;
    QLabel *label_3;
    QDoubleSpinBox *_width;
    QLabel *_dimXlabel;
    QLabel *label_2;
    QDoubleSpinBox *_height;
    QLabel *label;
    QDoubleSpinBox *_y;
    QCheckBox *_fixAspectRatio;
    QDoubleSpinBox *_x;
    QCheckBox *_lockPosToData;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *DimensionsTab)
    {
        if (DimensionsTab->objectName().isEmpty())
            DimensionsTab->setObjectName("DimensionsTab");
        DimensionsTab->resize(361, 189);
        verticalLayout = new QVBoxLayout(DimensionsTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_5 = new QLabel(DimensionsTab);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_5, 4, 0, 1, 1);

        _rotation = new QDoubleSpinBox(DimensionsTab);
        _rotation->setObjectName("_rotation");
        _rotation->setMinimum(-180.000000000000000);
        _rotation->setMaximum(360.000000000000000);

        gridLayout->addWidget(_rotation, 4, 1, 1, 1);

        label_3 = new QLabel(DimensionsTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 3, 0, 1, 1);

        _width = new QDoubleSpinBox(DimensionsTab);
        _width->setObjectName("_width");
        _width->setDecimals(4);
        _width->setMaximum(1.000000000000000);
        _width->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_width, 3, 1, 1, 1);

        _dimXlabel = new QLabel(DimensionsTab);
        _dimXlabel->setObjectName("_dimXlabel");
        sizePolicy.setHeightForWidth(_dimXlabel->sizePolicy().hasHeightForWidth());
        _dimXlabel->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_dimXlabel, 3, 2, 1, 1);

        label_2 = new QLabel(DimensionsTab);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_2, 2, 2, 1, 1);

        _height = new QDoubleSpinBox(DimensionsTab);
        _height->setObjectName("_height");
        _height->setDecimals(4);
        _height->setMaximum(1.000000000000000);
        _height->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_height, 3, 3, 1, 1);

        label = new QLabel(DimensionsTab);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 2, 0, 1, 1);

        _y = new QDoubleSpinBox(DimensionsTab);
        _y->setObjectName("_y");
        _y->setDecimals(4);
        _y->setMaximum(1.000000000000000);
        _y->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_y, 2, 3, 1, 1);

        _fixAspectRatio = new QCheckBox(DimensionsTab);
        _fixAspectRatio->setObjectName("_fixAspectRatio");

        gridLayout->addWidget(_fixAspectRatio, 0, 1, 1, 3);

        _x = new QDoubleSpinBox(DimensionsTab);
        _x->setObjectName("_x");
        _x->setDecimals(4);
        _x->setMaximum(1.000000000000000);
        _x->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_x, 2, 1, 1, 1);

        _lockPosToData = new QCheckBox(DimensionsTab);
        _lockPosToData->setObjectName("_lockPosToData");

        gridLayout->addWidget(_lockPosToData, 1, 1, 1, 3);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 23, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_5->setBuddy(_rotation);
        label_3->setBuddy(_width);
        _dimXlabel->setBuddy(_height);
        label_2->setBuddy(_y);
        label->setBuddy(_x);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_fixAspectRatio, _lockPosToData);
        QWidget::setTabOrder(_lockPosToData, _x);
        QWidget::setTabOrder(_x, _y);
        QWidget::setTabOrder(_y, _width);
        QWidget::setTabOrder(_width, _height);
        QWidget::setTabOrder(_height, _rotation);

        retranslateUi(DimensionsTab);
        QObject::connect(_fixAspectRatio, &QCheckBox::toggled, _height, &QDoubleSpinBox::setHidden);
        QObject::connect(_fixAspectRatio, &QCheckBox::toggled, _dimXlabel, &QLabel::setHidden);
        QObject::connect(_x, SIGNAL(valueChanged(double)), DimensionsTab, SLOT(modified()));
        QObject::connect(_y, SIGNAL(valueChanged(double)), DimensionsTab, SLOT(modified()));
        QObject::connect(_width, SIGNAL(valueChanged(double)), DimensionsTab, SLOT(modified()));
        QObject::connect(_height, SIGNAL(valueChanged(double)), DimensionsTab, SLOT(modified()));
        QObject::connect(_rotation, SIGNAL(valueChanged(double)), DimensionsTab, SLOT(modified()));
        QObject::connect(_fixAspectRatio, SIGNAL(toggled(bool)), DimensionsTab, SLOT(modified()));
        QObject::connect(_lockPosToData, SIGNAL(stateChanged(int)), DimensionsTab, SLOT(modified()));

        QMetaObject::connectSlotsByName(DimensionsTab);
    } // setupUi

    void retranslateUi(QWidget *DimensionsTab)
    {
        label_5->setText(QCoreApplication::translate("DimensionsTab", "&Rotation:", nullptr));
        label_3->setText(QCoreApplication::translate("DimensionsTab", "Si&ze:", nullptr));
        _dimXlabel->setText(QCoreApplication::translate("DimensionsTab", "&x", nullptr));
        label_2->setText(QCoreApplication::translate("DimensionsTab", "&,", nullptr));
        label->setText(QCoreApplication::translate("DimensionsTab", "&Position:", nullptr));
        _fixAspectRatio->setText(QCoreApplication::translate("DimensionsTab", "&Fix aspect ratio", nullptr));
        _lockPosToData->setText(QCoreApplication::translate("DimensionsTab", "Lock Position To Data", nullptr));
        (void)DimensionsTab;
    } // retranslateUi

};

namespace Ui {
    class DimensionsTab: public Ui_DimensionsTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIMENSIONSTAB_H
