/********************************************************************************
** Form generated from reading UI file 'filltab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILLTAB_H
#define UI_FILLTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"
#include "gradienteditor.h"

QT_BEGIN_NAMESPACE

class Ui_FillTab
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    Kst::ColorButton *_color;
    QLabel *label_2;
    QComboBox *_style;
    QCheckBox *_useGradient;
    QPushButton *_gradientReset;
    Kst::GradientEditor *_gradientEditor;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *FillTab)
    {
        if (FillTab->objectName().isEmpty())
            FillTab->setObjectName("FillTab");
        FillTab->resize(338, 141);
        FillTab->setMinimumSize(QSize(100, 0));
        vboxLayout = new QVBoxLayout(FillTab);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(FillTab);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        _color = new Kst::ColorButton(FillTab);
        _color->setObjectName("_color");

        gridLayout->addWidget(_color, 0, 1, 1, 2);

        label_2 = new QLabel(FillTab);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        _style = new QComboBox(FillTab);
        _style->setObjectName("_style");

        gridLayout->addWidget(_style, 1, 1, 1, 2);

        _useGradient = new QCheckBox(FillTab);
        _useGradient->setObjectName("_useGradient");

        gridLayout->addWidget(_useGradient, 2, 0, 1, 2);

        _gradientReset = new QPushButton(FillTab);
        _gradientReset->setObjectName("_gradientReset");
        _gradientReset->setEnabled(false);

        gridLayout->addWidget(_gradientReset, 2, 2, 1, 1);

        _gradientEditor = new Kst::GradientEditor(FillTab);
        _gradientEditor->setObjectName("_gradientEditor");
        _gradientEditor->setEnabled(false);
        _gradientEditor->setMinimumSize(QSize(0, 22));

        gridLayout->addWidget(_gradientEditor, 3, 0, 1, 3);


        vboxLayout->addLayout(gridLayout);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);

#if QT_CONFIG(shortcut)
        label->setBuddy(_color);
        label_2->setBuddy(_style);
#endif // QT_CONFIG(shortcut)

        retranslateUi(FillTab);

        QMetaObject::connectSlotsByName(FillTab);
    } // setupUi

    void retranslateUi(QWidget *FillTab)
    {
        label->setText(QCoreApplication::translate("FillTab", "&Color:", nullptr));
        label_2->setText(QCoreApplication::translate("FillTab", "St&yle:", nullptr));
        _useGradient->setText(QCoreApplication::translate("FillTab", "Use &linear gradient", nullptr));
        _gradientReset->setText(QCoreApplication::translate("FillTab", "&Reset Gradient", nullptr));
        (void)FillTab;
    } // retranslateUi

};

namespace Ui {
    class FillTab: public Ui_FillTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILLTAB_H
