/********************************************************************************
** Form generated from reading UI file 'curvetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CURVETAB_H
#define UI_CURVETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "curveappearance.h"
#include "curveplacement.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_CurveTab
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *_legendLabel;
    QLineEdit *_manualLegendName;
    QCheckBox *_autoLegend;
    QGroupBox *GroupBox;
    QGridLayout *_2;
    QLabel *_xVectorLabel;
    Kst::VectorSelector *_xVector;
    QLabel *_yVectorLabel;
    Kst::VectorSelector *_yVector;
    QLabel *_xErrorLabel;
    Kst::VectorSelector *_xError;
    QLabel *_yErrorLabel;
    Kst::VectorSelector *_yError;
    QLabel *_xMinusErrorLabel;
    Kst::VectorSelector *_xMinusError;
    QLabel *_yMinusErrorLabel;
    Kst::VectorSelector *_yMinusError;
    QHBoxLayout *_3;
    QCheckBox *_xMinusSameAsPlus;
    QCheckBox *_yMinusSameAsPlus;
    QSpacerItem *spacerItem;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *verticalLayout;
    Kst::CurveAppearance *_curveAppearance;
    QCheckBox *_ignoreAutoScale;
    Kst::CurvePlacement *_curvePlacement;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *CurveTab)
    {
        if (CurveTab->objectName().isEmpty())
            CurveTab->setObjectName("CurveTab");
        CurveTab->resize(443, 250);
        verticalLayout_2 = new QVBoxLayout(CurveTab);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        _legendLabel = new QLabel(CurveTab);
        _legendLabel->setObjectName("_legendLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_legendLabel->sizePolicy().hasHeightForWidth());
        _legendLabel->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(_legendLabel);

        _manualLegendName = new QLineEdit(CurveTab);
        _manualLegendName->setObjectName("_manualLegendName");
        _manualLegendName->setEnabled(false);

        horizontalLayout->addWidget(_manualLegendName);

        _autoLegend = new QCheckBox(CurveTab);
        _autoLegend->setObjectName("_autoLegend");
        _autoLegend->setChecked(true);

        horizontalLayout->addWidget(_autoLegend);


        verticalLayout_2->addLayout(horizontalLayout);

        GroupBox = new QGroupBox(CurveTab);
        GroupBox->setObjectName("GroupBox");
        _2 = new QGridLayout(GroupBox);
        _2->setSpacing(6);
        _2->setContentsMargins(11, 11, 11, 11);
        _2->setObjectName("_2");
        _2->setVerticalSpacing(0);
        _2->setContentsMargins(-1, 0, -1, 0);
        _xVectorLabel = new QLabel(GroupBox);
        _xVectorLabel->setObjectName("_xVectorLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_xVectorLabel->sizePolicy().hasHeightForWidth());
        _xVectorLabel->setSizePolicy(sizePolicy1);
        _xVectorLabel->setWordWrap(false);

        _2->addWidget(_xVectorLabel, 0, 0, 1, 1);

        _xVector = new Kst::VectorSelector(GroupBox);
        _xVector->setObjectName("_xVector");

        _2->addWidget(_xVector, 0, 1, 1, 1);

        _yVectorLabel = new QLabel(GroupBox);
        _yVectorLabel->setObjectName("_yVectorLabel");
        sizePolicy1.setHeightForWidth(_yVectorLabel->sizePolicy().hasHeightForWidth());
        _yVectorLabel->setSizePolicy(sizePolicy1);
        _yVectorLabel->setWordWrap(false);

        _2->addWidget(_yVectorLabel, 0, 2, 1, 1);

        _yVector = new Kst::VectorSelector(GroupBox);
        _yVector->setObjectName("_yVector");

        _2->addWidget(_yVector, 0, 3, 1, 1);

        _xErrorLabel = new QLabel(GroupBox);
        _xErrorLabel->setObjectName("_xErrorLabel");
        sizePolicy1.setHeightForWidth(_xErrorLabel->sizePolicy().hasHeightForWidth());
        _xErrorLabel->setSizePolicy(sizePolicy1);
        _xErrorLabel->setWordWrap(false);

        _2->addWidget(_xErrorLabel, 1, 0, 1, 1);

        _xError = new Kst::VectorSelector(GroupBox);
        _xError->setObjectName("_xError");

        _2->addWidget(_xError, 1, 1, 1, 1);

        _yErrorLabel = new QLabel(GroupBox);
        _yErrorLabel->setObjectName("_yErrorLabel");
        sizePolicy1.setHeightForWidth(_yErrorLabel->sizePolicy().hasHeightForWidth());
        _yErrorLabel->setSizePolicy(sizePolicy1);
        _yErrorLabel->setWordWrap(false);

        _2->addWidget(_yErrorLabel, 1, 2, 1, 1);

        _yError = new Kst::VectorSelector(GroupBox);
        _yError->setObjectName("_yError");

        _2->addWidget(_yError, 1, 3, 1, 1);

        _xMinusErrorLabel = new QLabel(GroupBox);
        _xMinusErrorLabel->setObjectName("_xMinusErrorLabel");
        sizePolicy1.setHeightForWidth(_xMinusErrorLabel->sizePolicy().hasHeightForWidth());
        _xMinusErrorLabel->setSizePolicy(sizePolicy1);
        _xMinusErrorLabel->setWordWrap(false);

        _2->addWidget(_xMinusErrorLabel, 2, 0, 1, 1);

        _xMinusError = new Kst::VectorSelector(GroupBox);
        _xMinusError->setObjectName("_xMinusError");

        _2->addWidget(_xMinusError, 2, 1, 1, 1);

        _yMinusErrorLabel = new QLabel(GroupBox);
        _yMinusErrorLabel->setObjectName("_yMinusErrorLabel");
        sizePolicy1.setHeightForWidth(_yMinusErrorLabel->sizePolicy().hasHeightForWidth());
        _yMinusErrorLabel->setSizePolicy(sizePolicy1);
        _yMinusErrorLabel->setWordWrap(false);

        _2->addWidget(_yMinusErrorLabel, 2, 2, 1, 1);

        _yMinusError = new Kst::VectorSelector(GroupBox);
        _yMinusError->setObjectName("_yMinusError");

        _2->addWidget(_yMinusError, 2, 3, 1, 1);

        _3 = new QHBoxLayout();
        _3->setSpacing(6);
        _3->setObjectName("_3");
        _xMinusSameAsPlus = new QCheckBox(GroupBox);
        _xMinusSameAsPlus->setObjectName("_xMinusSameAsPlus");

        _3->addWidget(_xMinusSameAsPlus);

        _yMinusSameAsPlus = new QCheckBox(GroupBox);
        _yMinusSameAsPlus->setObjectName("_yMinusSameAsPlus");

        _3->addWidget(_yMinusSameAsPlus);

        spacerItem = new QSpacerItem(71, 31, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        _3->addItem(spacerItem);


        _2->addLayout(_3, 3, 0, 1, 4);


        verticalLayout_2->addWidget(GroupBox);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        _curveAppearance = new Kst::CurveAppearance(CurveTab);
        _curveAppearance->setObjectName("_curveAppearance");
        sizePolicy1.setHeightForWidth(_curveAppearance->sizePolicy().hasHeightForWidth());
        _curveAppearance->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(_curveAppearance);

        _ignoreAutoScale = new QCheckBox(CurveTab);
        _ignoreAutoScale->setObjectName("_ignoreAutoScale");

        verticalLayout->addWidget(_ignoreAutoScale);


        horizontalLayout_3->addLayout(verticalLayout);

        _curvePlacement = new Kst::CurvePlacement(CurveTab);
        _curvePlacement->setObjectName("_curvePlacement");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_curvePlacement->sizePolicy().hasHeightForWidth());
        _curvePlacement->setSizePolicy(sizePolicy2);

        horizontalLayout_3->addWidget(_curvePlacement);


        verticalLayout_2->addLayout(horizontalLayout_3);

        verticalSpacer = new QSpacerItem(20, 87, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        QWidget::setTabOrder(_xVector, _yVector);
        QWidget::setTabOrder(_yVector, _xError);
        QWidget::setTabOrder(_xError, _yError);
        QWidget::setTabOrder(_yError, _xMinusError);
        QWidget::setTabOrder(_xMinusError, _yMinusError);
        QWidget::setTabOrder(_yMinusError, _xMinusSameAsPlus);
        QWidget::setTabOrder(_xMinusSameAsPlus, _yMinusSameAsPlus);
        QWidget::setTabOrder(_yMinusSameAsPlus, _ignoreAutoScale);
        QWidget::setTabOrder(_ignoreAutoScale, _curveAppearance);
        QWidget::setTabOrder(_curveAppearance, _curvePlacement);

        retranslateUi(CurveTab);
        QObject::connect(_autoLegend, &QCheckBox::toggled, _manualLegendName, &QLineEdit::setDisabled);

        QMetaObject::connectSlotsByName(CurveTab);
    } // setupUi

    void retranslateUi(QWidget *CurveTab)
    {
        _legendLabel->setText(QCoreApplication::translate("CurveTab", "Legend Name:", nullptr));
        _autoLegend->setText(QCoreApplication::translate("CurveTab", "Auto", nullptr));
        GroupBox->setTitle(QCoreApplication::translate("CurveTab", "Curve Contents", nullptr));
        _xVectorLabel->setText(QCoreApplication::translate("CurveTab", "&X-axis vector:", nullptr));
        _yVectorLabel->setText(QCoreApplication::translate("CurveTab", "&Y-axis vector:", nullptr));
        _xErrorLabel->setText(QCoreApplication::translate("CurveTab", "+X e&rror bar:", nullptr));
        _yErrorLabel->setText(QCoreApplication::translate("CurveTab", "&+Y error bar:", nullptr));
        _xMinusErrorLabel->setText(QCoreApplication::translate("CurveTab", "-X error bar:", nullptr));
        _yMinusErrorLabel->setText(QCoreApplication::translate("CurveTab", "&-Y error bar:", nullptr));
        _xMinusSameAsPlus->setText(QCoreApplication::translate("CurveTab", "&Use +X error bar for -X", nullptr));
        _yMinusSameAsPlus->setText(QCoreApplication::translate("CurveTab", "U&se +Y error bar for -Y", nullptr));
        _ignoreAutoScale->setText(QCoreApplication::translate("CurveTab", "I&gnore in automatic axes range calculations", nullptr));
        (void)CurveTab;
    } // retranslateUi

};

namespace Ui {
    class CurveTab: public Ui_CurveTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CURVETAB_H
