/********************************************************************************
** Form generated from reading UI file 'defaultlabelpropertiestab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DEFAULTLABELPROPERTIESTAB_H
#define UI_DEFAULTLABELPROPERTIESTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DefaultLabelPropertiesTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QSpinBox *_minFontSize;
    QLabel *label_5;
    QComboBox *_referenceViewSizeCombo;
    QLabel *label;
    QDoubleSpinBox *_refViewWidth;
    QLabel *label_2;
    QDoubleSpinBox *_refViewHeight;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *DefaultLabelPropertiesTab)
    {
        if (DefaultLabelPropertiesTab->objectName().isEmpty())
            DefaultLabelPropertiesTab->setObjectName("DefaultLabelPropertiesTab");
        DefaultLabelPropertiesTab->resize(479, 113);
        verticalLayout = new QVBoxLayout(DefaultLabelPropertiesTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(DefaultLabelPropertiesTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 3, 0, 1, 1);

        _minFontSize = new QSpinBox(DefaultLabelPropertiesTab);
        _minFontSize->setObjectName("_minFontSize");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_minFontSize->sizePolicy().hasHeightForWidth());
        _minFontSize->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_minFontSize, 3, 1, 1, 5);

        label_5 = new QLabel(DefaultLabelPropertiesTab);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 4, 0, 1, 1);

        _referenceViewSizeCombo = new QComboBox(DefaultLabelPropertiesTab);
        _referenceViewSizeCombo->setObjectName("_referenceViewSizeCombo");

        gridLayout->addWidget(_referenceViewSizeCombo, 4, 1, 1, 5);

        label = new QLabel(DefaultLabelPropertiesTab);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 5, 0, 1, 1);

        _refViewWidth = new QDoubleSpinBox(DefaultLabelPropertiesTab);
        _refViewWidth->setObjectName("_refViewWidth");
        sizePolicy.setHeightForWidth(_refViewWidth->sizePolicy().hasHeightForWidth());
        _refViewWidth->setSizePolicy(sizePolicy);
        _refViewWidth->setMaximum(2000.000000000000000);

        gridLayout->addWidget(_refViewWidth, 5, 1, 1, 2);

        label_2 = new QLabel(DefaultLabelPropertiesTab);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);
        label_2->setText(QString::fromUtf8("x"));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 5, 3, 1, 1);

        _refViewHeight = new QDoubleSpinBox(DefaultLabelPropertiesTab);
        _refViewHeight->setObjectName("_refViewHeight");
        sizePolicy.setHeightForWidth(_refViewHeight->sizePolicy().hasHeightForWidth());
        _refViewHeight->setSizePolicy(sizePolicy);
        _refViewHeight->setMaximum(2000.000000000000000);

        gridLayout->addWidget(_refViewHeight, 5, 4, 1, 2);


        verticalLayout->addLayout(gridLayout);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(_minFontSize);
        label->setBuddy(_refViewWidth);
        label_2->setBuddy(_refViewHeight);
#endif // QT_CONFIG(shortcut)

        retranslateUi(DefaultLabelPropertiesTab);

        QMetaObject::connectSlotsByName(DefaultLabelPropertiesTab);
    } // setupUi

    void retranslateUi(QWidget *DefaultLabelPropertiesTab)
    {
        label_3->setText(QCoreApplication::translate("DefaultLabelPropertiesTab", "&Minimum font size (points):", nullptr));
#if QT_CONFIG(tooltip)
        _minFontSize->setToolTip(QCoreApplication::translate("DefaultLabelPropertiesTab", "On window resize, fonts will be displayed at least this large.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_5->setText(QCoreApplication::translate("DefaultLabelPropertiesTab", "Reference view size:", nullptr));
#if QT_CONFIG(tooltip)
        _referenceViewSizeCombo->setToolTip(QCoreApplication::translate("DefaultLabelPropertiesTab", "When the window is this large, fonts will have stated size.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _referenceViewSizeCombo->setWhatsThis(QCoreApplication::translate("DefaultLabelPropertiesTab", "Fonts are scaled with the window or print size.  When the window/print is this large, the font sizes stated will be accurate.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label->setText(QCoreApplication::translate("DefaultLabelPropertiesTab", "Custom reference size (cm):", nullptr));
        (void)DefaultLabelPropertiesTab;
    } // retranslateUi

};

namespace Ui {
    class DefaultLabelPropertiesTab: public Ui_DefaultLabelPropertiesTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DEFAULTLABELPROPERTIESTAB_H
