/********************************************************************************
** Form generated from reading UI file 'filterbutterworthbandstopconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERBUTTERWORTHBANDSTOPCONFIG_H
#define UI_FILTERBUTTERWORTHBANDSTOPCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FilterButterworthBandStopConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vector;
    QLabel *label_4;
    Kst::ScalarSelector *_scalarOrder;
    QLabel *label_5;
    Kst::ScalarSelector *_scalarRate;
    QLabel *label_6;
    Kst::ScalarSelector *_scalarBandwidth;

    void setupUi(QWidget *FilterButterworthBandStopConfig)
    {
        if (FilterButterworthBandStopConfig->objectName().isEmpty())
            FilterButterworthBandStopConfig->setObjectName("FilterButterworthBandStopConfig");
        FilterButterworthBandStopConfig->resize(400, 103);
        FilterButterworthBandStopConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(FilterButterworthBandStopConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(FilterButterworthBandStopConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _vector = new Kst::VectorSelector(FilterButterworthBandStopConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vector, 0, 1, 1, 1);

        label_4 = new QLabel(FilterButterworthBandStopConfig);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        _scalarOrder = new Kst::ScalarSelector(FilterButterworthBandStopConfig);
        _scalarOrder->setObjectName("_scalarOrder");
        sizePolicy1.setHeightForWidth(_scalarOrder->sizePolicy().hasHeightForWidth());
        _scalarOrder->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_scalarOrder, 1, 1, 1, 1);

        label_5 = new QLabel(FilterButterworthBandStopConfig);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy2);
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_5->setWordWrap(false);

        gridLayout->addWidget(label_5, 2, 0, 1, 1);

        _scalarRate = new Kst::ScalarSelector(FilterButterworthBandStopConfig);
        _scalarRate->setObjectName("_scalarRate");
        sizePolicy1.setHeightForWidth(_scalarRate->sizePolicy().hasHeightForWidth());
        _scalarRate->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_scalarRate, 2, 1, 1, 1);

        label_6 = new QLabel(FilterButterworthBandStopConfig);
        label_6->setObjectName("label_6");
        sizePolicy.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy);
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_6->setWordWrap(false);

        gridLayout->addWidget(label_6, 3, 0, 1, 1);

        _scalarBandwidth = new Kst::ScalarSelector(FilterButterworthBandStopConfig);
        _scalarBandwidth->setObjectName("_scalarBandwidth");
        sizePolicy1.setHeightForWidth(_scalarBandwidth->sizePolicy().hasHeightForWidth());
        _scalarBandwidth->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_scalarBandwidth, 3, 1, 1, 1);


        retranslateUi(FilterButterworthBandStopConfig);

        QMetaObject::connectSlotsByName(FilterButterworthBandStopConfig);
    } // setupUi

    void retranslateUi(QWidget *FilterButterworthBandStopConfig)
    {
        label_3->setText(QCoreApplication::translate("FilterButterworthBandStopConfig", "Input Vector:", nullptr));
        label_4->setText(QCoreApplication::translate("FilterButterworthBandStopConfig", "Filter Order:", nullptr));
        label_5->setText(QCoreApplication::translate("FilterButterworthBandStopConfig", "Central Frequency / Sample Rate:", nullptr));
        label_6->setText(QCoreApplication::translate("FilterButterworthBandStopConfig", "Band width:", nullptr));
        (void)FilterButterworthBandStopConfig;
    } // retranslateUi

};

namespace Ui {
    class FilterButterworthBandStopConfig: public Ui_FilterButterworthBandStopConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERBUTTERWORTHBANDSTOPCONFIG_H
