/********************************************************************************
** Form generated from reading UI file 'exportvectorsdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTVECTORSDIALOG_H
#define UI_EXPORTVECTORSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "filerequester.h"

QT_BEGIN_NAMESPACE

class Ui_ExportVectorsDialog
{
public:
    QGridLayout *gridLayout;
    QGroupBox *buttonGroup1;
    QHBoxLayout *horizontalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *_changeVectorList;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *_addAll;
    QPushButton *_add;
    QPushButton *_remove;
    QPushButton *_removeAll;
    QSpacerItem *spacerItem;
    QListWidget *_selectedVectorList;
    QLabel *_saveLocationLabel;
    Kst::FileRequester *_saveLocation;
    QDialogButtonBox *_buttonBox;

    void setupUi(QDialog *ExportVectorsDialog)
    {
        if (ExportVectorsDialog->objectName().isEmpty())
            ExportVectorsDialog->setObjectName("ExportVectorsDialog");
        ExportVectorsDialog->resize(578, 251);
        gridLayout = new QGridLayout(ExportVectorsDialog);
        gridLayout->setObjectName("gridLayout");
        buttonGroup1 = new QGroupBox(ExportVectorsDialog);
        buttonGroup1->setObjectName("buttonGroup1");
        horizontalLayout_3 = new QHBoxLayout(buttonGroup1);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _changeVectorList = new QListWidget(buttonGroup1);
        _changeVectorList->setObjectName("_changeVectorList");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_changeVectorList->sizePolicy().hasHeightForWidth());
        _changeVectorList->setSizePolicy(sizePolicy);
        _changeVectorList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout_2->addWidget(_changeVectorList);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        _addAll = new QPushButton(buttonGroup1);
        _addAll->setObjectName("_addAll");

        verticalLayout->addWidget(_addAll);

        _add = new QPushButton(buttonGroup1);
        _add->setObjectName("_add");

        verticalLayout->addWidget(_add);

        _remove = new QPushButton(buttonGroup1);
        _remove->setObjectName("_remove");

        verticalLayout->addWidget(_remove);

        _removeAll = new QPushButton(buttonGroup1);
        _removeAll->setObjectName("_removeAll");

        verticalLayout->addWidget(_removeAll);

        spacerItem = new QSpacerItem(80, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);


        horizontalLayout_2->addLayout(verticalLayout);

        _selectedVectorList = new QListWidget(buttonGroup1);
        _selectedVectorList->setObjectName("_selectedVectorList");
        sizePolicy.setHeightForWidth(_selectedVectorList->sizePolicy().hasHeightForWidth());
        _selectedVectorList->setSizePolicy(sizePolicy);
        _selectedVectorList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout_2->addWidget(_selectedVectorList);


        horizontalLayout_3->addLayout(horizontalLayout_2);


        gridLayout->addWidget(buttonGroup1, 0, 0, 1, 2);

        _saveLocationLabel = new QLabel(ExportVectorsDialog);
        _saveLocationLabel->setObjectName("_saveLocationLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_saveLocationLabel->sizePolicy().hasHeightForWidth());
        _saveLocationLabel->setSizePolicy(sizePolicy1);
        _saveLocationLabel->setWordWrap(false);

        gridLayout->addWidget(_saveLocationLabel, 1, 0, 1, 1);

        _saveLocation = new Kst::FileRequester(ExportVectorsDialog);
        _saveLocation->setObjectName("_saveLocation");

        gridLayout->addWidget(_saveLocation, 1, 1, 1, 1);

        _buttonBox = new QDialogButtonBox(ExportVectorsDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(_buttonBox, 2, 1, 1, 1);


        retranslateUi(ExportVectorsDialog);
        QObject::connect(_buttonBox, &QDialogButtonBox::accepted, ExportVectorsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(_buttonBox, &QDialogButtonBox::rejected, ExportVectorsDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ExportVectorsDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportVectorsDialog)
    {
        ExportVectorsDialog->setWindowTitle(QCoreApplication::translate("ExportVectorsDialog", "Export Vectors", nullptr));
        buttonGroup1->setTitle(QCoreApplication::translate("ExportVectorsDialog", "Vector Selection", nullptr));
#if QT_CONFIG(whatsthis)
        _changeVectorList->setWhatsThis(QCoreApplication::translate("ExportVectorsDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        _addAll->setWhatsThis(QCoreApplication::translate("ExportVectorsDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _addAll->setText(QCoreApplication::translate("ExportVectorsDialog", "Add A&ll", nullptr));
#if QT_CONFIG(whatsthis)
        _add->setWhatsThis(QCoreApplication::translate("ExportVectorsDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _add->setText(QCoreApplication::translate("ExportVectorsDialog", "&Add", nullptr));
#if QT_CONFIG(whatsthis)
        _remove->setWhatsThis(QCoreApplication::translate("ExportVectorsDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _remove->setText(QCoreApplication::translate("ExportVectorsDialog", "&Remove", nullptr));
#if QT_CONFIG(whatsthis)
        _removeAll->setWhatsThis(QCoreApplication::translate("ExportVectorsDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _removeAll->setText(QCoreApplication::translate("ExportVectorsDialog", "R&emove All", nullptr));
#if QT_CONFIG(whatsthis)
        _selectedVectorList->setWhatsThis(QCoreApplication::translate("ExportVectorsDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _saveLocationLabel->setText(QCoreApplication::translate("ExportVectorsDialog", "&Save location:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportVectorsDialog: public Ui_ExportVectorsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTVECTORSDIALOG_H
