/****************************************************************************
** Meta object code from reading C++ file 'sharedaxisboxitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstapp/sharedaxisboxitem.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sharedaxisboxitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::SharedAxisBoxItem::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::SharedAxisBoxItem",
        "breakShareSignal",
        "",
        "breakShare",
        "acceptItems",
        "lockItems",
        "shareXAxis",
        "shareYAxis",
        "zoomFixedExpression",
        "QRectF",
        "projection",
        "PlotItem*",
        "originPlotItem",
        "zoomXRange",
        "zoomYRange",
        "zoomMaximum",
        "zoomMaxSpikeInsensitive",
        "zoomMeanCentered",
        "zoomXMeanCentered",
        "zoomXMaximum",
        "zoomXNoSpike",
        "zoomXAutoBorder",
        "zoomXRight",
        "far",
        "zoomXLeft",
        "zoomXOut",
        "zoomXIn",
        "zoomNormalizeXtoY",
        "zoomLogX",
        "autoEnable",
        "enable",
        "zoomYMeanCentered",
        "dY",
        "zoomYLocalMaximum",
        "zoomYMaximum",
        "zoomYNoSpike",
        "zoomYAutoBorder",
        "zoomYUp",
        "zoomYDown",
        "zoomYOut",
        "zoomYIn",
        "zoomNormalizeYtoX",
        "zoomLogY",
        "getSharedPlots",
        "QList<PlotItem*>",
        "creationPolygonChanged",
        "View::CreationEvent",
        "event"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'breakShareSignal'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'breakShare'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'acceptItems'
        QtMocHelpers::SlotData<bool()>(4, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'lockItems'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shareXAxis'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shareYAxis'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomFixedExpression'
        QtMocHelpers::SlotData<void(const QRectF &, PlotItem *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXRange'
        QtMocHelpers::SlotData<void(const QRectF &, PlotItem *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYRange'
        QtMocHelpers::SlotData<void(const QRectF &, PlotItem *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomMaximum'
        QtMocHelpers::SlotData<void(PlotItem *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomMaxSpikeInsensitive'
        QtMocHelpers::SlotData<void(PlotItem *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomMeanCentered'
        QtMocHelpers::SlotData<void(PlotItem *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXMeanCentered'
        QtMocHelpers::SlotData<void(PlotItem *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXMaximum'
        QtMocHelpers::SlotData<void(PlotItem *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXNoSpike'
        QtMocHelpers::SlotData<void(PlotItem *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXAutoBorder'
        QtMocHelpers::SlotData<void(PlotItem *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXRight'
        QtMocHelpers::SlotData<void(PlotItem *, bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 23 },
        }}),
        // Slot 'zoomXRight'
        QtMocHelpers::SlotData<void(PlotItem *)>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXLeft'
        QtMocHelpers::SlotData<void(PlotItem *, bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 23 },
        }}),
        // Slot 'zoomXLeft'
        QtMocHelpers::SlotData<void(PlotItem *)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXOut'
        QtMocHelpers::SlotData<void(PlotItem *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomXIn'
        QtMocHelpers::SlotData<void(PlotItem *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomNormalizeXtoY'
        QtMocHelpers::SlotData<void(PlotItem *)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void(PlotItem *, bool, bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 29 }, { QMetaType::Bool, 30 },
        }}),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void(PlotItem *, bool)>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 29 },
        }}),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void(PlotItem *)>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYMeanCentered'
        QtMocHelpers::SlotData<void(qreal, PlotItem *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 32 }, { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYLocalMaximum'
        QtMocHelpers::SlotData<void(PlotItem *)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYMaximum'
        QtMocHelpers::SlotData<void(PlotItem *)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYNoSpike'
        QtMocHelpers::SlotData<void(PlotItem *)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYAutoBorder'
        QtMocHelpers::SlotData<void(PlotItem *)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYUp'
        QtMocHelpers::SlotData<void(PlotItem *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYDown'
        QtMocHelpers::SlotData<void(PlotItem *)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYOut'
        QtMocHelpers::SlotData<void(PlotItem *)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomYIn'
        QtMocHelpers::SlotData<void(PlotItem *)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomNormalizeYtoX'
        QtMocHelpers::SlotData<void(PlotItem *)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void(PlotItem *, bool, bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 29 }, { QMetaType::Bool, 30 },
        }}),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void(PlotItem *, bool)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { QMetaType::Bool, 29 },
        }}),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void(PlotItem *)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'getSharedPlots'
        QtMocHelpers::SlotData<QList<PlotItem*>()>(43, 2, QMC::AccessPublic, 0x80000000 | 44),
        // Slot 'creationPolygonChanged'
        QtMocHelpers::SlotData<void(View::CreationEvent)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 46, 47 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SharedAxisBoxItem, qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::SharedAxisBoxItem::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t>.metaTypes,
    nullptr
} };

void Kst::SharedAxisBoxItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SharedAxisBoxItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->breakShareSignal(); break;
        case 1: _t->breakShare(); break;
        case 2: { bool _r = _t->acceptItems();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->lockItems(); break;
        case 4: _t->shareXAxis(); break;
        case 5: _t->shareYAxis(); break;
        case 6: _t->zoomFixedExpression((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[2]))); break;
        case 7: _t->zoomXRange((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[2]))); break;
        case 8: _t->zoomYRange((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[2]))); break;
        case 9: _t->zoomMaximum((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 10: _t->zoomMaxSpikeInsensitive((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 11: _t->zoomMeanCentered((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 12: _t->zoomXMeanCentered((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 13: _t->zoomXMaximum((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 14: _t->zoomXNoSpike((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 15: _t->zoomXAutoBorder((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 16: _t->zoomXRight((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->zoomXRight((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 18: _t->zoomXLeft((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->zoomXLeft((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 20: _t->zoomXOut((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 21: _t->zoomXIn((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 22: _t->zoomNormalizeXtoY((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 23: _t->zoomLogX((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 24: _t->zoomLogX((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 25: _t->zoomLogX((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 26: _t->zoomYMeanCentered((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[2]))); break;
        case 27: _t->zoomYLocalMaximum((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 28: _t->zoomYMaximum((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 29: _t->zoomYNoSpike((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 30: _t->zoomYAutoBorder((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 31: _t->zoomYUp((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 32: _t->zoomYDown((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 33: _t->zoomYOut((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 34: _t->zoomYIn((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 35: _t->zoomNormalizeYtoX((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 36: _t->zoomLogY((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 37: _t->zoomLogY((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 38: _t->zoomLogY((*reinterpret_cast<std::add_pointer_t<PlotItem*>>(_a[1]))); break;
        case 39: { QList<PlotItem*> _r = _t->getSharedPlots();
            if (_a[0]) *reinterpret_cast<QList<PlotItem*>*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->creationPolygonChanged((*reinterpret_cast<std::add_pointer_t<View::CreationEvent>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PlotItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SharedAxisBoxItem::*)()>(_a, &SharedAxisBoxItem::breakShareSignal, 0))
            return;
    }
}

const QMetaObject *Kst::SharedAxisBoxItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::SharedAxisBoxItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst17SharedAxisBoxItemE_t>.strings))
        return static_cast<void*>(this);
    return ViewItem::qt_metacast(_clname);
}

int Kst::SharedAxisBoxItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    return _id;
}

// SIGNAL 0
void Kst::SharedAxisBoxItem::breakShareSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::CreateSharedAxisBoxCommand::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::CreateSharedAxisBoxCommand",
        "creationComplete",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'creationComplete'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CreateSharedAxisBoxCommand, qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::CreateSharedAxisBoxCommand::staticMetaObject = { {
    QMetaObject::SuperData::link<CreateCommand::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t>.metaTypes,
    nullptr
} };

void Kst::CreateSharedAxisBoxCommand::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CreateSharedAxisBoxCommand *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->creationComplete(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *Kst::CreateSharedAxisBoxCommand::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::CreateSharedAxisBoxCommand::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst26CreateSharedAxisBoxCommandE_t>.strings))
        return static_cast<void*>(this);
    return CreateCommand::qt_metacast(_clname);
}

int Kst::CreateSharedAxisBoxCommand::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CreateCommand::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
