/****************************************************************************
** Meta object code from reading C++ file 'dialoglauncher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstmath/dialoglauncher.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dialoglauncher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst14DialogLauncherE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DialogLauncher::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst14DialogLauncherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DialogLauncher",
        "showDataSourceDialog",
        "",
        "QString&",
        "datasourcename",
        "ObjectPtr",
        "objectPtr",
        "modal",
        "showVectorDialog",
        "vectorname",
        "showMatrixDialog",
        "matrixName",
        "showScalarDialog",
        "scalarname",
        "showStringDialog",
        "showCurveDialog",
        "VectorPtr",
        "vector",
        "PlotItemInterface*",
        "plotItem",
        "showMultiCurveDialog",
        "QList<ObjectPtr>",
        "curves",
        "showImageDialog",
        "MatrixPtr",
        "matrix",
        "showMultiImageDialog",
        "images",
        "showEquationDialog",
        "showHistogramDialog",
        "showPowerSpectrumDialog",
        "showCSDDialog",
        "showEventMonitorDialog",
        "showBasicPluginDialog",
        "pluginName",
        "vectorX",
        "vectorY",
        "showObjectDialog",
        "showMultiObjectDialog",
        "names"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'showDataSourceDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'showDataSourceDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showDataSourceDialog'
        QtMocHelpers::SlotData<void(QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'showVectorDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'showVectorDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr)>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 9 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showVectorDialog'
        QtMocHelpers::SlotData<void(QString &)>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 9 },
        }}),
        // Slot 'showMatrixDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'showMatrixDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showMatrixDialog'
        QtMocHelpers::SlotData<void(QString &)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 11 },
        }}),
        // Slot 'showScalarDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr, bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'showScalarDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showScalarDialog'
        QtMocHelpers::SlotData<void(QString &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 13 },
        }}),
        // Slot 'showStringDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'showStringDialog'
        QtMocHelpers::SlotData<void(QString &, ObjectPtr)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 13 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showStringDialog'
        QtMocHelpers::SlotData<void(QString &)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 13 },
        }}),
        // Slot 'showCurveDialog'
        QtMocHelpers::SlotData<void(ObjectPtr, VectorPtr, PlotItemInterface *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'showCurveDialog'
        QtMocHelpers::SlotData<void(ObjectPtr, VectorPtr)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'showCurveDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showCurveDialog'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showMultiCurveDialog'
        QtMocHelpers::SlotData<void(QList<ObjectPtr>)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'showImageDialog'
        QtMocHelpers::SlotData<void(ObjectPtr, MatrixPtr)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 24, 25 },
        }}),
        // Slot 'showImageDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showImageDialog'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showMultiImageDialog'
        QtMocHelpers::SlotData<void(QList<ObjectPtr>)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 27 },
        }}),
        // Slot 'showEquationDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showEquationDialog'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showHistogramDialog'
        QtMocHelpers::SlotData<void(ObjectPtr, VectorPtr)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'showHistogramDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showHistogramDialog'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showPowerSpectrumDialog'
        QtMocHelpers::SlotData<void(ObjectPtr, VectorPtr)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'showPowerSpectrumDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(30, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showPowerSpectrumDialog'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showCSDDialog'
        QtMocHelpers::SlotData<void(ObjectPtr, VectorPtr)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'showCSDDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showCSDDialog'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showEventMonitorDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showEventMonitorDialog'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showBasicPluginDialog'
        QtMocHelpers::SlotData<void(QString, ObjectPtr, VectorPtr, VectorPtr, PlotItemInterface *)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 34 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 35 }, { 0x80000000 | 16, 36 },
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'showBasicPluginDialog'
        QtMocHelpers::SlotData<void(QString, ObjectPtr, VectorPtr, VectorPtr)>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 34 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 35 }, { 0x80000000 | 16, 36 },
        }}),
        // Slot 'showBasicPluginDialog'
        QtMocHelpers::SlotData<void(QString, ObjectPtr, VectorPtr)>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 34 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 16, 35 },
        }}),
        // Slot 'showBasicPluginDialog'
        QtMocHelpers::SlotData<void(QString, ObjectPtr)>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 34 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showBasicPluginDialog'
        QtMocHelpers::SlotData<void(QString)>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Slot 'showObjectDialog'
        QtMocHelpers::SlotData<void(ObjectPtr)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showObjectDialog'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'showMultiObjectDialog'
        QtMocHelpers::SlotData<void(QList<ObjectPtr>)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DialogLauncher, qt_meta_tag_ZN3Kst14DialogLauncherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DialogLauncher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst14DialogLauncherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst14DialogLauncherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst14DialogLauncherE_t>.metaTypes,
    nullptr
} };

void Kst::DialogLauncher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DialogLauncher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showDataSourceDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->showDataSourceDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2]))); break;
        case 2: _t->showDataSourceDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 3: _t->showVectorDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 4: _t->showVectorDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2]))); break;
        case 5: _t->showVectorDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 6: _t->showMatrixDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 7: _t->showMatrixDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2]))); break;
        case 8: _t->showMatrixDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 9: _t->showScalarDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 10: _t->showScalarDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2]))); break;
        case 11: _t->showScalarDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 12: _t->showStringDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 13: _t->showStringDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2]))); break;
        case 14: _t->showStringDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 15: _t->showCurveDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<PlotItemInterface*>>(_a[3]))); break;
        case 16: _t->showCurveDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[2]))); break;
        case 17: _t->showCurveDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 18: _t->showCurveDialog(); break;
        case 19: _t->showMultiCurveDialog((*reinterpret_cast<std::add_pointer_t<QList<ObjectPtr>>>(_a[1]))); break;
        case 20: _t->showImageDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MatrixPtr>>(_a[2]))); break;
        case 21: _t->showImageDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 22: _t->showImageDialog(); break;
        case 23: _t->showMultiImageDialog((*reinterpret_cast<std::add_pointer_t<QList<ObjectPtr>>>(_a[1]))); break;
        case 24: _t->showEquationDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 25: _t->showEquationDialog(); break;
        case 26: _t->showHistogramDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[2]))); break;
        case 27: _t->showHistogramDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 28: _t->showHistogramDialog(); break;
        case 29: _t->showPowerSpectrumDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[2]))); break;
        case 30: _t->showPowerSpectrumDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 31: _t->showPowerSpectrumDialog(); break;
        case 32: _t->showCSDDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[2]))); break;
        case 33: _t->showCSDDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 34: _t->showCSDDialog(); break;
        case 35: _t->showEventMonitorDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 36: _t->showEventMonitorDialog(); break;
        case 37: _t->showBasicPluginDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<PlotItemInterface*>>(_a[5]))); break;
        case 38: _t->showBasicPluginDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[4]))); break;
        case 39: _t->showBasicPluginDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<VectorPtr>>(_a[3]))); break;
        case 40: _t->showBasicPluginDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[2]))); break;
        case 41: _t->showBasicPluginDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->showObjectDialog((*reinterpret_cast<std::add_pointer_t<ObjectPtr>>(_a[1]))); break;
        case 43: _t->showObjectDialog(); break;
        case 44: _t->showMultiObjectDialog((*reinterpret_cast<std::add_pointer_t<QList<ObjectPtr>>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Kst::DialogLauncher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DialogLauncher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst14DialogLauncherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Kst::DialogLauncher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 45;
    }
    return _id;
}
QT_WARNING_POP
