/****************************************************************************
** Meta object code from reading C++ file 'asciiconfigwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/datasources/ascii/asciiconfigwidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'asciiconfigwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t {};
} // unnamed namespace

template <> constexpr inline auto AsciiConfigWidgetInternal::qt_create_metaobjectdata<qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AsciiConfigWidgetInternal",
        "showBeginning",
        "",
        "showPreviewWindow",
        "interpretationChanged",
        "enabled",
        "testAsciiFormatString",
        "format"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'showBeginning'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showPreviewWindow'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'interpretationChanged'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'testAsciiFormatString'
        QtMocHelpers::SlotData<void(QString)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AsciiConfigWidgetInternal, qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AsciiConfigWidgetInternal::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t>.metaTypes,
    nullptr
} };

void AsciiConfigWidgetInternal::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AsciiConfigWidgetInternal *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showBeginning(); break;
        case 1: _t->showPreviewWindow(); break;
        case 2: _t->interpretationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->testAsciiFormatString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *AsciiConfigWidgetInternal::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AsciiConfigWidgetInternal::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25AsciiConfigWidgetInternalE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui_AsciiConfig"))
        return static_cast< Ui_AsciiConfig*>(this);
    return QWidget::qt_metacast(_clname);
}

int AsciiConfigWidgetInternal::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN17AsciiConfigWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto AsciiConfigWidget::qt_create_metaobjectdata<qt_meta_tag_ZN17AsciiConfigWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AsciiConfigWidget",
        "updateIndexVector",
        "",
        "cancel"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'updateIndexVector'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AsciiConfigWidget, qt_meta_tag_ZN17AsciiConfigWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AsciiConfigWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<Kst::DataSourceConfigWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AsciiConfigWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AsciiConfigWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17AsciiConfigWidgetE_t>.metaTypes,
    nullptr
} };

void AsciiConfigWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AsciiConfigWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateIndexVector(); break;
        case 1: _t->cancel(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *AsciiConfigWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AsciiConfigWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17AsciiConfigWidgetE_t>.strings))
        return static_cast<void*>(this);
    return Kst::DataSourceConfigWidget::qt_metacast(_clname);
}

int AsciiConfigWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Kst::DataSourceConfigWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
