/****************************************************************************
** Meta object code from reading C++ file 'periodogram.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/plugins/dataobject/periodogram/periodogram.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'periodogram.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17PeriodogramSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto PeriodogramSource::qt_create_metaobjectdata<qt_meta_tag_ZN17PeriodogramSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PeriodogramSource"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PeriodogramSource, qt_meta_tag_ZN17PeriodogramSourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PeriodogramSource::staticMetaObject = { {
    QMetaObject::SuperData::link<Kst::BasicPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PeriodogramSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PeriodogramSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17PeriodogramSourceE_t>.metaTypes,
    nullptr
} };

void PeriodogramSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PeriodogramSource *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PeriodogramSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PeriodogramSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PeriodogramSourceE_t>.strings))
        return static_cast<void*>(this);
    return Kst::BasicPlugin::qt_metacast(_clname);
}

int PeriodogramSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Kst::BasicPlugin::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17PeriodogramPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto PeriodogramPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN17PeriodogramPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PeriodogramPlugin"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PeriodogramPlugin, qt_meta_tag_ZN17PeriodogramPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PeriodogramPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PeriodogramPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PeriodogramPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17PeriodogramPluginE_t>.metaTypes,
    nullptr
} };

void PeriodogramPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PeriodogramPlugin *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PeriodogramPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PeriodogramPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17PeriodogramPluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Kst::DataObjectPluginInterface"))
        return static_cast< Kst::DataObjectPluginInterface*>(this);
    if (!strcmp(_clname, "com.kst.DataObjectPluginInterface/2.0"))
        return static_cast< Kst::DataObjectPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int PeriodogramPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_PeriodogramPlugin[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x25,  'c',  'o',  'm',  '.',  'k', 
    's',  't',  '.',  'D',  'a',  't',  'a',  'O', 
    'b',  'j',  'e',  'c',  't',  'P',  'l',  'u', 
    'g',  'i',  'n',  'I',  'n',  't',  'e',  'r', 
    'f',  'a',  'c',  'e',  '/',  '2',  '.',  '0', 
    // "className"
    0x03,  0x71,  'P',  'e',  'r',  'i',  'o',  'd', 
    'o',  'g',  'r',  'a',  'm',  'P',  'l',  'u', 
    'g',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(PeriodogramPlugin, PeriodogramPlugin, qt_pluginMetaDataV2_PeriodogramPlugin)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_PeriodogramPlugin[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x25,  'c',  'o',  'm',  '.',  'k', 
    's',  't',  '.',  'D',  'a',  't',  'a',  'O', 
    'b',  'j',  'e',  'c',  't',  'P',  'l',  'u', 
    'g',  'i',  'n',  'I',  'n',  't',  'e',  'r', 
    'f',  'a',  'c',  'e',  '/',  '2',  '.',  '0', 
    // "className"
    0x03,  0x71,  'P',  'e',  'r',  'i',  'o',  'd', 
    'o',  'g',  'r',  'a',  'm',  'P',  'l',  'u', 
    'g',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(PeriodogramPlugin, PeriodogramPlugin)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
