/********************************************************************************
** Form generated from reading UI file 'imagetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGETAB_H
#define UI_IMAGETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"
#include "colorpalette.h"
#include "curveplacement.h"
#include "matrixselector.h"

QT_BEGIN_NAMESPACE

class Ui_ImageTab
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *_matrixLabel;
    Kst::MatrixSelector *_matrix;
    QGroupBox *_imageTypeGroup;
    QGridLayout *_6;
    QRadioButton *_colorOnly;
    QRadioButton *_contourOnly;
    QRadioButton *_colorAndContour;
    QGroupBox *_colorMapGroup;
    QGridLayout *_7;
    QHBoxLayout *_8;
    QLabel *textLabel2;
    QLineEdit *_upperThreshold;
    QHBoxLayout *_9;
    QLabel *textLabel2_4;
    QLineEdit *_lowerThreshold;
    QHBoxLayout *_10;
    QPushButton *_smartThreshold;
    QLabel *textLabel1_4;
    QDoubleSpinBox *_smartThresholdValue;
    QSpacerItem *spacerItem;
    Kst::ColorPalette *_colorPalette;
    QPushButton *_autoThreshold;
    QCheckBox *_realTimeAutoThreshold;
    QGroupBox *_contourMapGroup;
    QGridLayout *gridLayout;
    QHBoxLayout *_2;
    QLabel *textLabel2_3;
    Kst::ColorButton *_contourColor;
    QHBoxLayout *_3;
    QLabel *textLabel1_3;
    QSpinBox *_numContourLines;
    QHBoxLayout *_4;
    QLabel *textLabel1;
    QSpinBox *_contourWeight;
    QHBoxLayout *_5;
    QCheckBox *_useVariableWeight;
    QSpacerItem *spacerItem1;
    Kst::CurvePlacement *_curvePlacement;

    void setupUi(QWidget *ImageTab)
    {
        if (ImageTab->objectName().isEmpty())
            ImageTab->setObjectName("ImageTab");
        ImageTab->resize(748, 613);
        verticalLayout_2 = new QVBoxLayout(ImageTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(ImageTab);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        _matrixLabel = new QLabel(groupBox);
        _matrixLabel->setObjectName("_matrixLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_matrixLabel->sizePolicy().hasHeightForWidth());
        _matrixLabel->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(_matrixLabel);

        _matrix = new Kst::MatrixSelector(groupBox);
        _matrix->setObjectName("_matrix");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_matrix->sizePolicy().hasHeightForWidth());
        _matrix->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(_matrix);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(groupBox);

        _imageTypeGroup = new QGroupBox(ImageTab);
        _imageTypeGroup->setObjectName("_imageTypeGroup");
        _6 = new QGridLayout(_imageTypeGroup);
        _6->setObjectName("_6");
        _6->setContentsMargins(-1, 0, -1, 0);
        _colorOnly = new QRadioButton(_imageTypeGroup);
        _colorOnly->setObjectName("_colorOnly");
        _colorOnly->setChecked(true);

        _6->addWidget(_colorOnly, 0, 0, 1, 1);

        _contourOnly = new QRadioButton(_imageTypeGroup);
        _contourOnly->setObjectName("_contourOnly");

        _6->addWidget(_contourOnly, 0, 1, 1, 1);

        _colorAndContour = new QRadioButton(_imageTypeGroup);
        _colorAndContour->setObjectName("_colorAndContour");

        _6->addWidget(_colorAndContour, 0, 2, 1, 1);


        verticalLayout_2->addWidget(_imageTypeGroup);

        _colorMapGroup = new QGroupBox(ImageTab);
        _colorMapGroup->setObjectName("_colorMapGroup");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_colorMapGroup->sizePolicy().hasHeightForWidth());
        _colorMapGroup->setSizePolicy(sizePolicy2);
        _7 = new QGridLayout(_colorMapGroup);
        _7->setObjectName("_7");
        _7->setContentsMargins(-1, 0, -1, -1);
        _8 = new QHBoxLayout();
        _8->setObjectName("_8");
        _8->setContentsMargins(0, 0, 0, 0);
        textLabel2 = new QLabel(_colorMapGroup);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        _8->addWidget(textLabel2);

        _upperThreshold = new QLineEdit(_colorMapGroup);
        _upperThreshold->setObjectName("_upperThreshold");
        _upperThreshold->setEnabled(false);
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_upperThreshold->sizePolicy().hasHeightForWidth());
        _upperThreshold->setSizePolicy(sizePolicy3);

        _8->addWidget(_upperThreshold);


        _7->addLayout(_8, 1, 2, 1, 2);

        _9 = new QHBoxLayout();
        _9->setObjectName("_9");
        _9->setContentsMargins(0, 0, 0, 0);
        textLabel2_4 = new QLabel(_colorMapGroup);
        textLabel2_4->setObjectName("textLabel2_4");
        textLabel2_4->setWordWrap(false);

        _9->addWidget(textLabel2_4);

        _lowerThreshold = new QLineEdit(_colorMapGroup);
        _lowerThreshold->setObjectName("_lowerThreshold");
        _lowerThreshold->setEnabled(false);
        sizePolicy3.setHeightForWidth(_lowerThreshold->sizePolicy().hasHeightForWidth());
        _lowerThreshold->setSizePolicy(sizePolicy3);

        _9->addWidget(_lowerThreshold);


        _7->addLayout(_9, 1, 0, 1, 2);

        _10 = new QHBoxLayout();
        _10->setObjectName("_10");
        _10->setContentsMargins(0, 0, 0, 0);
        _smartThreshold = new QPushButton(_colorMapGroup);
        _smartThreshold->setObjectName("_smartThreshold");
        _smartThreshold->setEnabled(false);
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_smartThreshold->sizePolicy().hasHeightForWidth());
        _smartThreshold->setSizePolicy(sizePolicy4);

        _10->addWidget(_smartThreshold);

        textLabel1_4 = new QLabel(_colorMapGroup);
        textLabel1_4->setObjectName("textLabel1_4");
        sizePolicy.setHeightForWidth(textLabel1_4->sizePolicy().hasHeightForWidth());
        textLabel1_4->setSizePolicy(sizePolicy);
        textLabel1_4->setWordWrap(false);

        _10->addWidget(textLabel1_4);

        _smartThresholdValue = new QDoubleSpinBox(_colorMapGroup);
        _smartThresholdValue->setObjectName("_smartThresholdValue");
        _smartThresholdValue->setEnabled(false);
        _smartThresholdValue->setDecimals(1);
        _smartThresholdValue->setMaximum(45.000000000000000);
        _smartThresholdValue->setValue(0.500000000000000);

        _10->addWidget(_smartThresholdValue);


        _7->addLayout(_10, 2, 1, 1, 1);

        spacerItem = new QSpacerItem(41, 21, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _7->addItem(spacerItem, 2, 2, 1, 1);

        _colorPalette = new Kst::ColorPalette(_colorMapGroup);
        _colorPalette->setObjectName("_colorPalette");
        _colorPalette->setMinimumSize(QSize(0, 0));

        _7->addWidget(_colorPalette, 0, 0, 1, 4);

        _autoThreshold = new QPushButton(_colorMapGroup);
        _autoThreshold->setObjectName("_autoThreshold");
        _autoThreshold->setEnabled(false);
        sizePolicy4.setHeightForWidth(_autoThreshold->sizePolicy().hasHeightForWidth());
        _autoThreshold->setSizePolicy(sizePolicy4);

        _7->addWidget(_autoThreshold, 2, 0, 1, 1);

        _realTimeAutoThreshold = new QCheckBox(_colorMapGroup);
        _realTimeAutoThreshold->setObjectName("_realTimeAutoThreshold");
        _realTimeAutoThreshold->setChecked(true);

        _7->addWidget(_realTimeAutoThreshold, 2, 3, 1, 1);


        verticalLayout_2->addWidget(_colorMapGroup);

        _contourMapGroup = new QGroupBox(ImageTab);
        _contourMapGroup->setObjectName("_contourMapGroup");
        _contourMapGroup->setEnabled(false);
        gridLayout = new QGridLayout(_contourMapGroup);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(-1, 0, -1, -1);
        _2 = new QHBoxLayout();
        _2->setObjectName("_2");
        _2->setContentsMargins(0, 0, 0, 0);
        textLabel2_3 = new QLabel(_contourMapGroup);
        textLabel2_3->setObjectName("textLabel2_3");
        sizePolicy4.setHeightForWidth(textLabel2_3->sizePolicy().hasHeightForWidth());
        textLabel2_3->setSizePolicy(sizePolicy4);
        textLabel2_3->setWordWrap(false);

        _2->addWidget(textLabel2_3);

        _contourColor = new Kst::ColorButton(_contourMapGroup);
        _contourColor->setObjectName("_contourColor");
        _contourColor->setProperty("color", QVariant(QColor(0, 0, 0)));

        _2->addWidget(_contourColor);


        gridLayout->addLayout(_2, 0, 2, 1, 1);

        _3 = new QHBoxLayout();
        _3->setObjectName("_3");
        _3->setContentsMargins(0, 0, 0, 0);
        textLabel1_3 = new QLabel(_contourMapGroup);
        textLabel1_3->setObjectName("textLabel1_3");
        sizePolicy4.setHeightForWidth(textLabel1_3->sizePolicy().hasHeightForWidth());
        textLabel1_3->setSizePolicy(sizePolicy4);
        textLabel1_3->setWordWrap(false);

        _3->addWidget(textLabel1_3);

        _numContourLines = new QSpinBox(_contourMapGroup);
        _numContourLines->setObjectName("_numContourLines");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(_numContourLines->sizePolicy().hasHeightForWidth());
        _numContourLines->setSizePolicy(sizePolicy5);
        _numContourLines->setMinimum(1);
        _numContourLines->setMaximum(10);

        _3->addWidget(_numContourLines);


        gridLayout->addLayout(_3, 0, 0, 1, 2);

        _4 = new QHBoxLayout();
        _4->setObjectName("_4");
        _4->setContentsMargins(0, 0, 0, 0);
        textLabel1 = new QLabel(_contourMapGroup);
        textLabel1->setObjectName("textLabel1");
        sizePolicy4.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy4);
        textLabel1->setWordWrap(false);

        _4->addWidget(textLabel1);

        _contourWeight = new QSpinBox(_contourMapGroup);
        _contourWeight->setObjectName("_contourWeight");
        _contourWeight->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        _contourWeight->setMaximum(5);

        _4->addWidget(_contourWeight);


        gridLayout->addLayout(_4, 1, 0, 1, 1);

        _5 = new QHBoxLayout();
        _5->setObjectName("_5");
        _5->setContentsMargins(0, 0, 0, 0);
        _useVariableWeight = new QCheckBox(_contourMapGroup);
        _useVariableWeight->setObjectName("_useVariableWeight");

        _5->addWidget(_useVariableWeight);

        spacerItem1 = new QSpacerItem(76, 25, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _5->addItem(spacerItem1);


        gridLayout->addLayout(_5, 1, 1, 1, 2);


        verticalLayout_2->addWidget(_contourMapGroup);

        _curvePlacement = new Kst::CurvePlacement(ImageTab);
        _curvePlacement->setObjectName("_curvePlacement");

        verticalLayout_2->addWidget(_curvePlacement);

#if QT_CONFIG(shortcut)
        textLabel2->setBuddy(_upperThreshold);
        textLabel2_4->setBuddy(_lowerThreshold);
        textLabel1_4->setBuddy(_smartThresholdValue);
        textLabel2_3->setBuddy(_contourColor);
        textLabel1_3->setBuddy(_numContourLines);
        textLabel1->setBuddy(_contourWeight);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_colorOnly, _contourOnly);
        QWidget::setTabOrder(_contourOnly, _colorAndContour);
        QWidget::setTabOrder(_colorAndContour, _realTimeAutoThreshold);
        QWidget::setTabOrder(_realTimeAutoThreshold, _lowerThreshold);
        QWidget::setTabOrder(_lowerThreshold, _upperThreshold);
        QWidget::setTabOrder(_upperThreshold, _autoThreshold);
        QWidget::setTabOrder(_autoThreshold, _smartThreshold);
        QWidget::setTabOrder(_smartThreshold, _smartThresholdValue);
        QWidget::setTabOrder(_smartThresholdValue, _numContourLines);
        QWidget::setTabOrder(_numContourLines, _contourColor);
        QWidget::setTabOrder(_contourColor, _contourWeight);
        QWidget::setTabOrder(_contourWeight, _useVariableWeight);

        retranslateUi(ImageTab);

        QMetaObject::connectSlotsByName(ImageTab);
    } // setupUi

    void retranslateUi(QWidget *ImageTab)
    {
        groupBox->setTitle(QCoreApplication::translate("ImageTab", "Image Contents", nullptr));
        _matrixLabel->setText(QCoreApplication::translate("ImageTab", "&Matrix:", nullptr));
        _imageTypeGroup->setTitle(QCoreApplication::translate("ImageTab", "Image Type", nullptr));
        _colorOnly->setText(QCoreApplication::translate("ImageTab", "Color &map", nullptr));
        _contourOnly->setText(QCoreApplication::translate("ImageTab", "Con&tour map", nullptr));
        _colorAndContour->setText(QCoreApplication::translate("ImageTab", "&Color map and contour map", nullptr));
        _colorMapGroup->setTitle(QCoreApplication::translate("ImageTab", "Color Map Parameters", nullptr));
        textLabel2->setText(QCoreApplication::translate("ImageTab", "&Upper:", nullptr));
        textLabel2_4->setText(QCoreApplication::translate("ImageTab", "Threshold - &Lower:", nullptr));
        _smartThreshold->setText(QCoreApplication::translate("ImageTab", "&Smart", nullptr));
        textLabel1_4->setText(QCoreApplication::translate("ImageTab", "Percent&ile:", nullptr));
        _autoThreshold->setText(QCoreApplication::translate("ImageTab", "Max&/Min", nullptr));
        _realTimeAutoThreshold->setText(QCoreApplication::translate("ImageTab", "&Real-time auto threshold", nullptr));
        _contourMapGroup->setTitle(QCoreApplication::translate("ImageTab", "Contour Map Parameters", nullptr));
        textLabel2_3->setText(QCoreApplication::translate("ImageTab", "Co&lor:", nullptr));
        _contourColor->setText(QString());
        textLabel1_3->setText(QCoreApplication::translate("ImageTab", "&Number of contour levels:", nullptr));
        textLabel1->setText(QCoreApplication::translate("ImageTab", "&Weight:", nullptr));
        _useVariableWeight->setText(QCoreApplication::translate("ImageTab", "Use &variable line weight", nullptr));
        (void)ImageTab;
    } // retranslateUi

};

namespace Ui {
    class ImageTab: public Ui_ImageTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGETAB_H
