/********************************************************************************
** Form generated from reading UI file 'vectorselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VECTORSELECTOR_H
#define UI_VECTORSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "combobox.h"

QT_BEGIN_NAMESPACE

class Ui_VectorSelector
{
public:
    QHBoxLayout *hboxLayout;
    Kst::ComboBox *_vector;
    QToolButton *_newVector;
    QToolButton *_editVector;

    void setupUi(QWidget *VectorSelector)
    {
        if (VectorSelector->objectName().isEmpty())
            VectorSelector->setObjectName("VectorSelector");
        hboxLayout = new QHBoxLayout(VectorSelector);
        hboxLayout->setSpacing(0);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        _vector = new Kst::ComboBox(VectorSelector);
        _vector->setObjectName("_vector");

        hboxLayout->addWidget(_vector);

        _newVector = new QToolButton(VectorSelector);
        _newVector->setObjectName("_newVector");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_newVector->sizePolicy().hasHeightForWidth());
        _newVector->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(_newVector);

        _editVector = new QToolButton(VectorSelector);
        _editVector->setObjectName("_editVector");
        _editVector->setEnabled(false);
        sizePolicy.setHeightForWidth(_editVector->sizePolicy().hasHeightForWidth());
        _editVector->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(_editVector);


        retranslateUi(VectorSelector);

        QMetaObject::connectSlotsByName(VectorSelector);
    } // setupUi

    void retranslateUi(QWidget *VectorSelector)
    {
#if QT_CONFIG(whatsthis)
        VectorSelector->setWhatsThis(QCoreApplication::translate("VectorSelector", "Select a vector to utilize.  If you wish to create a new vector, click on the icon to the right.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        _newVector->setToolTip(QCoreApplication::translate("VectorSelector", "Create a new vector", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _newVector->setWhatsThis(QCoreApplication::translate("VectorSelector", "Opens a vector dialog to create a new vector, which will become the selected vector.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _newVector->setText(QString());
#if QT_CONFIG(tooltip)
        _editVector->setToolTip(QCoreApplication::translate("VectorSelector", "Edit selected vector.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _editVector->setWhatsThis(QCoreApplication::translate("VectorSelector", "Opens a vector dialog to edit selected vector.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _editVector->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class VectorSelector: public Ui_VectorSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VECTORSELECTOR_H
