/********************************************************************************
** Form generated from reading UI file 'stroketab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STROKETAB_H
#define UI_STROKETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_StrokeTab
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QComboBox *_style;
    QLabel *label_2;
    QDoubleSpinBox *_width;
    QLabel *label_3;
    Kst::ColorButton *_brushColor;
    QLabel *label_4;
    QComboBox *_brushStyle;
    QLabel *label_5;
    QComboBox *_joinStyle;
    QLabel *label_6;
    QComboBox *_capStyle;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *StrokeTab)
    {
        if (StrokeTab->objectName().isEmpty())
            StrokeTab->setObjectName("StrokeTab");
        StrokeTab->resize(264, 208);
        gridLayout = new QGridLayout(StrokeTab);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(StrokeTab);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        _style = new QComboBox(StrokeTab);
        _style->setObjectName("_style");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_style->sizePolicy().hasHeightForWidth());
        _style->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_style, 0, 1, 1, 1);

        label_2 = new QLabel(StrokeTab);
        label_2->setObjectName("label_2");
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        _width = new QDoubleSpinBox(StrokeTab);
        _width->setObjectName("_width");

        gridLayout->addWidget(_width, 1, 1, 1, 1);

        label_3 = new QLabel(StrokeTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        _brushColor = new Kst::ColorButton(StrokeTab);
        _brushColor->setObjectName("_brushColor");

        gridLayout->addWidget(_brushColor, 2, 1, 1, 1);

        label_4 = new QLabel(StrokeTab);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        _brushStyle = new QComboBox(StrokeTab);
        _brushStyle->setObjectName("_brushStyle");

        gridLayout->addWidget(_brushStyle, 3, 1, 1, 1);

        label_5 = new QLabel(StrokeTab);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 4, 0, 1, 1);

        _joinStyle = new QComboBox(StrokeTab);
        _joinStyle->setObjectName("_joinStyle");

        gridLayout->addWidget(_joinStyle, 4, 1, 1, 1);

        label_6 = new QLabel(StrokeTab);
        label_6->setObjectName("label_6");
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_6, 5, 0, 1, 1);

        _capStyle = new QComboBox(StrokeTab);
        _capStyle->setObjectName("_capStyle");

        gridLayout->addWidget(_capStyle, 5, 1, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 6, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label->setBuddy(_style);
        label_2->setBuddy(_width);
        label_3->setBuddy(_brushColor);
        label_4->setBuddy(_brushStyle);
        label_5->setBuddy(_joinStyle);
        label_6->setBuddy(_capStyle);
#endif // QT_CONFIG(shortcut)

        retranslateUi(StrokeTab);

        QMetaObject::connectSlotsByName(StrokeTab);
    } // setupUi

    void retranslateUi(QWidget *StrokeTab)
    {
        label->setText(QCoreApplication::translate("StrokeTab", "&Style:", nullptr));
        label_2->setText(QCoreApplication::translate("StrokeTab", "&Width:", nullptr));
        label_3->setText(QCoreApplication::translate("StrokeTab", "Brush &color:", nullptr));
        label_4->setText(QCoreApplication::translate("StrokeTab", "&Brush style:", nullptr));
        label_5->setText(QCoreApplication::translate("StrokeTab", "&Join style:", nullptr));
        label_6->setText(QCoreApplication::translate("StrokeTab", "Ca&p style:", nullptr));
        (void)StrokeTab;
    } // retranslateUi

};

namespace Ui {
    class StrokeTab: public Ui_StrokeTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STROKETAB_H
