/****************************************************************************
** Meta object code from reading C++ file 'plotitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstapp/plotitem.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plotitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst9PlotLabelE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::PlotLabel::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst9PlotLabelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::PlotLabel",
        "labelChanged",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'labelChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlotLabel, qt_meta_tag_ZN3Kst9PlotLabelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::PlotLabel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst9PlotLabelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst9PlotLabelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst9PlotLabelE_t>.metaTypes,
    nullptr
} };

void Kst::PlotLabel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlotLabel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->labelChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlotLabel::*)()>(_a, &PlotLabel::labelChanged, 0))
            return;
    }
}

const QMetaObject *Kst::PlotLabel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::PlotLabel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst9PlotLabelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Kst::PlotLabel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Kst::PlotLabel::labelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN3Kst8PlotItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::PlotItem::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst8PlotItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::PlotItem",
        "updatePlotRect",
        "",
        "updateAxes",
        "triggerRedraw",
        "breakShareTriggered",
        "shareXAxisTriggered",
        "shareYAxisTriggered",
        "zoomFixedExpression",
        "QRectF",
        "projection",
        "force",
        "zoomXRange",
        "zoomYRange",
        "zoomMaximum",
        "zoomMaxSpikeInsensitive",
        "zoomPrevious",
        "adjustImageColorScale",
        "zoomTied",
        "zoomXTied",
        "zoomYTied",
        "zoomXMaximum",
        "zoomXNoSpike",
        "zoomXAutoBorder",
        "zoomXRight",
        "scroll_far",
        "zoomXFarRight",
        "zoomXLeft",
        "zoomXFarLeft",
        "zoomXOut",
        "zoomXIn",
        "zoomNormalizeXtoY",
        "zoomLogX",
        "autoLog",
        "enableLog",
        "zoomMeanCentered",
        "zoomYMeanCentered",
        "dY",
        "zoomXMeanCentered",
        "dX",
        "zoomYLocalMaximum",
        "zoomYMaximum",
        "zoomYNoSpike",
        "zoomYAutoBorder",
        "zoomYUp",
        "zoomYDown",
        "zoomYOut",
        "zoomYIn",
        "zoomNormalizeYtoX",
        "zoomLogY",
        "zoomOut",
        "zoomIn",
        "copyStatus",
        "copyXCoord",
        "copyYCoord",
        "setPlotBordersDirty",
        "dirty",
        "edit",
        "PlotClickEditRegion",
        "region",
        "plotMaximize",
        "redrawPlot",
        "setPlotRectsDirty",
        "showFilterDialog",
        "QAction*",
        "showFitDialog",
        "showPSDDialog",
        "showCurveDialog",
        "showHistogramDialog",
        "showEditDialog",
        "action",
        "setLeftLabelDirty",
        "setRightLabelDirty",
        "setTopLabelDirty",
        "setBottomLabelDirty",
        "setLabelsDirty",
        "setPlotPixmapDirty",
        "setAxisLabelsDirty"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updatePlotRect'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateAxes'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'triggerRedraw'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'breakShareTriggered'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareXAxisTriggered'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shareYAxisTriggered'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomFixedExpression'
        QtMocHelpers::SlotData<void(const QRectF &, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomFixedExpression'
        QtMocHelpers::SlotData<void(const QRectF &)>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'zoomXRange'
        QtMocHelpers::SlotData<void(const QRectF &, bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXRange'
        QtMocHelpers::SlotData<void(const QRectF &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'zoomYRange'
        QtMocHelpers::SlotData<void(const QRectF &, bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYRange'
        QtMocHelpers::SlotData<void(const QRectF &)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'zoomMaximum'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomMaximum'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomMaxSpikeInsensitive'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomMaxSpikeInsensitive'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomPrevious'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'adjustImageColorScale'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomTied'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomXTied'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomYTied'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomXMaximum'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXMaximum'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXNoSpike'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXNoSpike'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXAutoBorder'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXAutoBorder'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXRight'
        QtMocHelpers::SlotData<void(bool, bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'zoomXRight'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXRight'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXFarRight'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXFarRight'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXLeft'
        QtMocHelpers::SlotData<void(bool, bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Bool, 25 },
        }}),
        // Slot 'zoomXLeft'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXLeft'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXFarLeft'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXFarLeft'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXOut'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXOut'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomXIn'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXIn'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomNormalizeXtoY'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomNormalizeXtoY'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void(bool, bool, bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Bool, 33 }, { QMetaType::Bool, 34 },
        }}),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void(bool, bool)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Bool, 33 },
        }}),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomLogX'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomMeanCentered'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomMeanCentered'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYMeanCentered'
        QtMocHelpers::SlotData<void(qreal, bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 37 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYMeanCentered'
        QtMocHelpers::SlotData<void(qreal)>(36, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QReal, 37 },
        }}),
        // Slot 'zoomXMeanCentered'
        QtMocHelpers::SlotData<void(qreal, bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 39 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomXMeanCentered'
        QtMocHelpers::SlotData<void(qreal)>(38, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QReal, 39 },
        }}),
        // Slot 'zoomYLocalMaximum'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYLocalMaximum'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYMaximum'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYMaximum'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYNoSpike'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYNoSpike'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYAutoBorder'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYAutoBorder'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYUp'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYUp'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYDown'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYDown'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYOut'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYOut'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomYIn'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomYIn'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomNormalizeYtoX'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomNormalizeYtoX'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void(bool, bool, bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Bool, 33 }, { QMetaType::Bool, 34 },
        }}),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void(bool, bool)>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Bool, 33 },
        }}),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomLogY'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'copyStatus'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyXCoord'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyYCoord'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPlotBordersDirty'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'setPlotBordersDirty'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'edit'
        QtMocHelpers::SlotData<void(enum PlotClickEditRegion)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 58, 59 },
        }}),
        // Slot 'edit'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'plotMaximize'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redrawPlot'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPlotRectsDirty'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showFilterDialog'
        QtMocHelpers::SlotData<void(QAction *)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 2 },
        }}),
        // Slot 'showFitDialog'
        QtMocHelpers::SlotData<void(QAction *)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 2 },
        }}),
        // Slot 'showPSDDialog'
        QtMocHelpers::SlotData<void(QAction *)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 2 },
        }}),
        // Slot 'showCurveDialog'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showHistogramDialog'
        QtMocHelpers::SlotData<void(QAction *)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 2 },
        }}),
        // Slot 'showEditDialog'
        QtMocHelpers::SlotData<void(QAction *)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 70 },
        }}),
        // Slot 'setLeftLabelDirty'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRightLabelDirty'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setTopLabelDirty'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setBottomLabelDirty'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLabelsDirty'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPlotPixmapDirty'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'setPlotPixmapDirty'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'setAxisLabelsDirty'
        QtMocHelpers::SlotData<void(bool)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'setAxisLabelsDirty'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlotItem, qt_meta_tag_ZN3Kst8PlotItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::PlotItem::staticMetaObject = { {
    QMetaObject::SuperData::link<ViewItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst8PlotItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst8PlotItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst8PlotItemE_t>.metaTypes,
    nullptr
} };

void Kst::PlotItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlotItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updatePlotRect(); break;
        case 1: _t->updateAxes(); break;
        case 2: _t->triggerRedraw(); break;
        case 3: _t->breakShareTriggered(); break;
        case 4: _t->shareXAxisTriggered(); break;
        case 5: _t->shareYAxisTriggered(); break;
        case 6: _t->zoomFixedExpression((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->zoomFixedExpression((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 8: _t->zoomXRange((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->zoomXRange((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 10: _t->zoomYRange((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->zoomYRange((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 12: _t->zoomMaximum((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->zoomMaximum(); break;
        case 14: _t->zoomMaxSpikeInsensitive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->zoomMaxSpikeInsensitive(); break;
        case 16: _t->zoomPrevious(); break;
        case 17: _t->adjustImageColorScale(); break;
        case 18: _t->zoomTied(); break;
        case 19: _t->zoomXTied(); break;
        case 20: _t->zoomYTied(); break;
        case 21: _t->zoomXMaximum((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->zoomXMaximum(); break;
        case 23: _t->zoomXNoSpike((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->zoomXNoSpike(); break;
        case 25: _t->zoomXAutoBorder((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->zoomXAutoBorder(); break;
        case 27: _t->zoomXRight((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 28: _t->zoomXRight((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->zoomXRight(); break;
        case 30: _t->zoomXFarRight((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->zoomXFarRight(); break;
        case 32: _t->zoomXLeft((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 33: _t->zoomXLeft((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->zoomXLeft(); break;
        case 35: _t->zoomXFarLeft((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->zoomXFarLeft(); break;
        case 37: _t->zoomXOut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->zoomXOut(); break;
        case 39: _t->zoomXIn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->zoomXIn(); break;
        case 41: _t->zoomNormalizeXtoY((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->zoomNormalizeXtoY(); break;
        case 43: _t->zoomLogX((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 44: _t->zoomLogX((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 45: _t->zoomLogX((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->zoomLogX(); break;
        case 47: _t->zoomMeanCentered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->zoomMeanCentered(); break;
        case 49: _t->zoomYMeanCentered((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 50: _t->zoomYMeanCentered((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 51: _t->zoomXMeanCentered((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 52: _t->zoomXMeanCentered((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 53: _t->zoomYLocalMaximum((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->zoomYLocalMaximum(); break;
        case 55: _t->zoomYMaximum((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 56: _t->zoomYMaximum(); break;
        case 57: _t->zoomYNoSpike((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 58: _t->zoomYNoSpike(); break;
        case 59: _t->zoomYAutoBorder((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 60: _t->zoomYAutoBorder(); break;
        case 61: _t->zoomYUp((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 62: _t->zoomYUp(); break;
        case 63: _t->zoomYDown((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 64: _t->zoomYDown(); break;
        case 65: _t->zoomYOut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 66: _t->zoomYOut(); break;
        case 67: _t->zoomYIn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 68: _t->zoomYIn(); break;
        case 69: _t->zoomNormalizeYtoX((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 70: _t->zoomNormalizeYtoX(); break;
        case 71: _t->zoomLogY((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 72: _t->zoomLogY((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 73: _t->zoomLogY((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 74: _t->zoomLogY(); break;
        case 75: _t->zoomOut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 76: _t->zoomOut(); break;
        case 77: _t->zoomIn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->zoomIn(); break;
        case 79: _t->copyStatus(); break;
        case 80: _t->copyXCoord(); break;
        case 81: _t->copyYCoord(); break;
        case 82: _t->setPlotBordersDirty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 83: _t->setPlotBordersDirty(); break;
        case 84: _t->edit((*reinterpret_cast<std::add_pointer_t<enum PlotClickEditRegion>>(_a[1]))); break;
        case 85: _t->edit(); break;
        case 86: _t->plotMaximize(); break;
        case 87: _t->redrawPlot(); break;
        case 88: _t->setPlotRectsDirty(); break;
        case 89: _t->showFilterDialog((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 90: _t->showFitDialog((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 91: _t->showPSDDialog((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 92: _t->showCurveDialog(); break;
        case 93: _t->showHistogramDialog((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 94: _t->showEditDialog((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 95: _t->setLeftLabelDirty(); break;
        case 96: _t->setRightLabelDirty(); break;
        case 97: _t->setTopLabelDirty(); break;
        case 98: _t->setBottomLabelDirty(); break;
        case 99: _t->setLabelsDirty(); break;
        case 100: _t->setPlotPixmapDirty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 101: _t->setPlotPixmapDirty(); break;
        case 102: _t->setAxisLabelsDirty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 103: _t->setAxisLabelsDirty(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 89:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 90:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 91:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 93:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 94:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlotItem::*)()>(_a, &PlotItem::updatePlotRect, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotItem::*)()>(_a, &PlotItem::updateAxes, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotItem::*)()>(_a, &PlotItem::triggerRedraw, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotItem::*)()>(_a, &PlotItem::breakShareTriggered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotItem::*)()>(_a, &PlotItem::shareXAxisTriggered, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotItem::*)()>(_a, &PlotItem::shareYAxisTriggered, 5))
            return;
    }
}

const QMetaObject *Kst::PlotItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::PlotItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst8PlotItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PlotItemInterface"))
        return static_cast< PlotItemInterface*>(this);
    return ViewItem::qt_metacast(_clname);
}

int Kst::PlotItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ViewItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 104)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 104;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 104)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 104;
    }
    return _id;
}

// SIGNAL 0
void Kst::PlotItem::updatePlotRect()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Kst::PlotItem::updateAxes()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Kst::PlotItem::triggerRedraw()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Kst::PlotItem::breakShareTriggered()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Kst::PlotItem::shareXAxisTriggered()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Kst::PlotItem::shareYAxisTriggered()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
