/********************************************************************************
** Form generated from reading UI file 'generaltab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GENERALTAB_H
#define UI_GENERALTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_GeneralTab
{
public:
    QGridLayout *gridLayout;
    QCheckBox *_useRaster;
    QCheckBox *_transparentDrag;
    QCheckBox *_antialiasPlots;
    QLabel *label_5;
    QSpinBox *_maxUpdate;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *GeneralTab)
    {
        if (GeneralTab->objectName().isEmpty())
            GeneralTab->setObjectName("GeneralTab");
        GeneralTab->resize(325, 246);
        gridLayout = new QGridLayout(GeneralTab);
        gridLayout->setObjectName("gridLayout");
        _useRaster = new QCheckBox(GeneralTab);
        _useRaster->setObjectName("_useRaster");

        gridLayout->addWidget(_useRaster, 0, 0, 1, 2);

        _transparentDrag = new QCheckBox(GeneralTab);
        _transparentDrag->setObjectName("_transparentDrag");

        gridLayout->addWidget(_transparentDrag, 1, 0, 1, 2);

        _antialiasPlots = new QCheckBox(GeneralTab);
        _antialiasPlots->setObjectName("_antialiasPlots");

        gridLayout->addWidget(_antialiasPlots, 2, 0, 1, 1);

        label_5 = new QLabel(GeneralTab);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 3, 0, 1, 1);

        _maxUpdate = new QSpinBox(GeneralTab);
        _maxUpdate->setObjectName("_maxUpdate");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_maxUpdate->sizePolicy().hasHeightForWidth());
        _maxUpdate->setSizePolicy(sizePolicy);
        _maxUpdate->setMinimum(10);
        _maxUpdate->setMaximum(60000);
        _maxUpdate->setSingleStep(500);
        _maxUpdate->setValue(2000);

        gridLayout->addWidget(_maxUpdate, 3, 1, 1, 1);

        spacerItem = new QSpacerItem(44, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 4, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_5->setBuddy(_maxUpdate);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_useRaster, _transparentDrag);
        QWidget::setTabOrder(_transparentDrag, _maxUpdate);

        retranslateUi(GeneralTab);

        QMetaObject::connectSlotsByName(GeneralTab);
    } // setupUi

    void retranslateUi(QWidget *GeneralTab)
    {
#if QT_CONFIG(tooltip)
        _useRaster->setToolTip(QCoreApplication::translate("GeneralTab", "Render using \"raster\".  This is faster, except for remote X.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _useRaster->setWhatsThis(QCoreApplication::translate("GeneralTab", "Rendering using \"raster\" is faster unless you are running kst over a remote X connection.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _useRaster->setText(QCoreApplication::translate("GeneralTab", "Use \"raster\" engine on (Kst restart needed)", nullptr));
        _transparentDrag->setText(QCoreApplication::translate("GeneralTab", "Use transparency when dragging", nullptr));
        _antialiasPlots->setText(QCoreApplication::translate("GeneralTab", "&Antialias Plots", nullptr));
        label_5->setText(QCoreApplication::translate("GeneralTab", "&Period between updates (ms):", nullptr));
#if QT_CONFIG(tooltip)
        _maxUpdate->setToolTip(QCoreApplication::translate("GeneralTab", "Dynamic files update this fast or slower.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _maxUpdate->setWhatsThis(QCoreApplication::translate("GeneralTab", "Minimum time between updates when looking at dynamic files.  If there is a lot of processing, it might take longer.", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)GeneralTab;
    } // retranslateUi

};

namespace Ui {
    class GeneralTab: public Ui_GeneralTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GENERALTAB_H
