/********************************************************************************
** Form generated from reading UI file 'datawizardpagedatasource.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATAWIZARDPAGEDATASOURCE_H
#define UI_DATAWIZARDPAGEDATASOURCE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_DataWizardPageDataSource
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *textLabel2;
    QVBoxLayout *verticalLayout;
    Kst::DataSourceSelector *_url;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *_fileType;
    QPushButton *_configureSource;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_2;
    QLabel *_fileType_2;
    QComboBox *_updateBox;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QPushButton *_cleanupRecentFiles;
    QSpacerItem *horizontalSpacer_3;
    QListWidget *_recentFiles;

    void setupUi(QWizardPage *DataWizardPageDataSource)
    {
        if (DataWizardPageDataSource->objectName().isEmpty())
            DataWizardPageDataSource->setObjectName("DataWizardPageDataSource");
        DataWizardPageDataSource->resize(850, 569);
        DataWizardPageDataSource->setMinimumSize(QSize(660, 350));
        verticalLayout_2 = new QVBoxLayout(DataWizardPageDataSource);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        textLabel2 = new QLabel(DataWizardPageDataSource);
        textLabel2->setObjectName("textLabel2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabel2->sizePolicy().hasHeightForWidth());
        textLabel2->setSizePolicy(sizePolicy);
        textLabel2->setAlignment(Qt::AlignTop);
        textLabel2->setWordWrap(true);

        verticalLayout_2->addWidget(textLabel2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        _url = new Kst::DataSourceSelector(DataWizardPageDataSource);
        _url->setObjectName("_url");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_url->sizePolicy().hasHeightForWidth());
        _url->setSizePolicy(sizePolicy1);
        _url->setMinimumSize(QSize(30, 24));

        verticalLayout->addWidget(_url);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        _fileType = new QLabel(DataWizardPageDataSource);
        _fileType->setObjectName("_fileType");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(10);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_fileType->sizePolicy().hasHeightForWidth());
        _fileType->setSizePolicy(sizePolicy2);
        _fileType->setWordWrap(false);

        horizontalLayout->addWidget(_fileType);

        _configureSource = new QPushButton(DataWizardPageDataSource);
        _configureSource->setObjectName("_configureSource");
        _configureSource->setEnabled(false);
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_configureSource->sizePolicy().hasHeightForWidth());
        _configureSource->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(_configureSource);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        _fileType_2 = new QLabel(DataWizardPageDataSource);
        _fileType_2->setObjectName("_fileType_2");
        sizePolicy2.setHeightForWidth(_fileType_2->sizePolicy().hasHeightForWidth());
        _fileType_2->setSizePolicy(sizePolicy2);
        _fileType_2->setWordWrap(false);

        horizontalLayout_2->addWidget(_fileType_2);

        _updateBox = new QComboBox(DataWizardPageDataSource);
        _updateBox->setObjectName("_updateBox");
        _updateBox->setMinimumSize(QSize(0, 0));

        horizontalLayout_2->addWidget(_updateBox);


        verticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_2->addLayout(verticalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label = new QLabel(DataWizardPageDataSource);
        label->setObjectName("label");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy4);

        horizontalLayout_3->addWidget(label);

        _cleanupRecentFiles = new QPushButton(DataWizardPageDataSource);
        _cleanupRecentFiles->setObjectName("_cleanupRecentFiles");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(_cleanupRecentFiles->sizePolicy().hasHeightForWidth());
        _cleanupRecentFiles->setSizePolicy(sizePolicy5);

        horizontalLayout_3->addWidget(_cleanupRecentFiles);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_2->addLayout(horizontalLayout_3);

        _recentFiles = new QListWidget(DataWizardPageDataSource);
        _recentFiles->setObjectName("_recentFiles");
        _recentFiles->setEditTriggers(QAbstractItemView::NoEditTriggers);

        verticalLayout_2->addWidget(_recentFiles);


        retranslateUi(DataWizardPageDataSource);

        QMetaObject::connectSlotsByName(DataWizardPageDataSource);
    } // setupUi

    void retranslateUi(QWizardPage *DataWizardPageDataSource)
    {
        textLabel2->setText(QCoreApplication::translate("DataWizardPageDataSource", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:8pt;\">Please select a </span><span style=\" font-size:8pt; font-style:italic;\">data source</span><span style=\" font-size:8pt;\"> to work from. This will typically be your file containing data points or vectors.</span></p></body></html>", nullptr));
        _fileType->setText(QCoreApplication::translate("DataWizardPageDataSource", "Unknown file type", nullptr));
        _configureSource->setText(QCoreApplication::translate("DataWizardPageDataSource", "Con&figure...", nullptr));
        _fileType_2->setText(QCoreApplication::translate("DataWizardPageDataSource", "Update type; use 'Time Interval' for non-local data", nullptr));
        label->setText(QCoreApplication::translate("DataWizardPageDataSource", "Recent files", nullptr));
#if QT_CONFIG(tooltip)
        _cleanupRecentFiles->setToolTip(QCoreApplication::translate("DataWizardPageDataSource", "Cleanup nonexistent files", nullptr));
#endif // QT_CONFIG(tooltip)
        _cleanupRecentFiles->setText(QCoreApplication::translate("DataWizardPageDataSource", "Cleanup", nullptr));
        (void)DataWizardPageDataSource;
    } // retranslateUi

};

namespace Ui {
    class DataWizardPageDataSource: public Ui_DataWizardPageDataSource {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATAWIZARDPAGEDATASOURCE_H
