/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstapp/mainwindow.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::MainWindow",
        "copyTab",
        "",
        "toggleStatusBar",
        "showDataManager",
        "showDebugDialog",
        "showExportGraphicsDialog",
        "showExportVectorsDialog",
        "showLogDialog",
        "showVectorEditor",
        "showScalarEditor",
        "showMatrixEditor",
        "showStringEditor",
        "showSettingsDialog",
        "showThemeDialog",
        "clearDefaults",
        "clearDataSourceSettings",
        "showDifferentiateCurvesDialog",
        "filterMultipleDialog",
        "showChooseColorDialog",
        "showChangeDataSampleDialog",
        "showChangeFileDialog",
        "showDataWizard",
        "dataFile",
        "showBugReportWizard",
        "docBookStarter",
        "showPluginDialog",
        "QString&",
        "pluginName",
        "readFromEnd",
        "readToEnd",
        "pause",
        "back",
        "scale",
        "forward",
        "backSmall",
        "forwardSmall",
        "reload",
        "updateViewItems",
        "serial",
        "updateProgress",
        "percent",
        "message",
        "save",
        "saveAs",
        "open",
        "newDoc",
        "force",
        "openFile",
        "file",
        "cleanUpDataSourceList",
        "print",
        "setPrinterDefaults",
        "QPrinter*",
        "printer",
        "savePrinterDefaults",
        "printToPrinter",
        "printFromCommandLine",
        "printFileName",
        "exportGraphicsFile",
        "filename",
        "format",
        "w",
        "h",
        "display",
        "export_all",
        "autosave_period",
        "exportLog",
        "imagename",
        "msgfilename",
        "_format",
        "x_size",
        "y_size",
        "size_option_index",
        "clearDrawingMarker",
        "tiedZoomRemoved",
        "allPlotsTiedZoom",
        "updateRecentDataFiles",
        "newfilename",
        "videoTutorial",
        "recentDataFiles",
        "cleanupRecentDataFilesList",
        "aboutToQuit",
        "about",
        "showShortcutDialog",
        "currentViewChanged",
        "currentViewModeChanged",
        "createBox",
        "createEllipse",
        "createLabel",
        "createCircle",
        "createLine",
        "createArrow",
        "createPicture",
        "createPlot",
        "insertPlot",
        "createSvg",
        "createSharedAxisBox",
        "createScalar",
        "createVector",
        "createMatrix",
        "createString",
        "createCurve",
        "createPSD",
        "createEquation",
        "createHistogram",
        "createImage",
        "createSpectogram",
        "createEventMonitor",
        "performHeavyStartupActions",
        "cleanup",
        "setLayoutMode",
        "layoutMode",
        "toggleTiedZoom",
        "setHighlightPoint",
        "changeZoomOnlyMode",
        "QAction*",
        "openRecentKstFile",
        "openRecentDataFile",
        "checkRecentFilesOnExistence",
        "autoExportImage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'copyTab'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleStatusBar'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showDataManager'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showDebugDialog'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showExportGraphicsDialog'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showExportVectorsDialog'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showLogDialog'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showVectorEditor'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showScalarEditor'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showMatrixEditor'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showStringEditor'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSettingsDialog'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showThemeDialog'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearDefaults'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearDataSourceSettings'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showDifferentiateCurvesDialog'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filterMultipleDialog'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showChooseColorDialog'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showChangeDataSampleDialog'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showChangeFileDialog'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showDataWizard'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showDataWizard'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'showBugReportWizard'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'docBookStarter'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showPluginDialog'
        QtMocHelpers::SlotData<void(QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'readFromEnd'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'readToEnd'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pause'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'back'
        QtMocHelpers::SlotData<void(double)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 33 },
        }}),
        // Slot 'back'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'forward'
        QtMocHelpers::SlotData<void(double)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 33 },
        }}),
        // Slot 'forward'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'backSmall'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forwardSmall'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reload'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateViewItems'
        QtMocHelpers::SlotData<void(qint64)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 39 },
        }}),
        // Slot 'updateProgress'
        QtMocHelpers::SlotData<void(int, const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 41 }, { QMetaType::QString, 42 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAs'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'open'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newDoc'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 47 },
        }}),
        // Slot 'newDoc'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'openFile'
        QtMocHelpers::SlotData<void(const QString &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'cleanUpDataSourceList'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPrinterDefaults'
        QtMocHelpers::SlotData<void(QPrinter *)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'savePrinterDefaults'
        QtMocHelpers::SlotData<void(QPrinter *)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'printToPrinter'
        QtMocHelpers::SlotData<void(QPrinter *)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'printFromCommandLine'
        QtMocHelpers::SlotData<void(const QString &)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 58 },
        }}),
        // Slot 'exportGraphicsFile'
        QtMocHelpers::SlotData<void(const QString &, const QString &, int, int, int, bool, int)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 }, { QMetaType::QString, 61 }, { QMetaType::Int, 62 }, { QMetaType::Int, 63 },
            { QMetaType::Int, 64 }, { QMetaType::Bool, 65 }, { QMetaType::Int, 66 },
        }}),
        // Slot 'exportLog'
        QtMocHelpers::SlotData<void(const QString &, QString &, const QString &, int, int, int, const QString &)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 }, { 0x80000000 | 27, 69 }, { QMetaType::QString, 70 }, { QMetaType::Int, 71 },
            { QMetaType::Int, 72 }, { QMetaType::Int, 73 }, { QMetaType::QString, 42 },
        }}),
        // Slot 'clearDrawingMarker'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tiedZoomRemoved'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'allPlotsTiedZoom'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateRecentDataFiles'
        QtMocHelpers::SlotData<void(const QString &)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 78 },
        }}),
        // Slot 'updateRecentDataFiles'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'videoTutorial'
        QtMocHelpers::SlotData<void(int)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'recentDataFiles'
        QtMocHelpers::SlotData<QStringList()>(80, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'cleanupRecentDataFilesList'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'aboutToQuit'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'about'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showShortcutDialog'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'currentViewChanged'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'currentViewModeChanged'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createBox'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEllipse'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createLabel'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createCircle'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createLine'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createArrow'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createPicture'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createPlot'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'insertPlot'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createSvg'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createSharedAxisBox'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createScalar'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createVector'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createMatrix'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createString'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createCurve'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createPSD'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEquation'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createHistogram'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createImage'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createSpectogram'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEventMonitor'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'performHeavyStartupActions'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanup'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setLayoutMode'
        QtMocHelpers::SlotData<void(bool)>(111, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 112 },
        }}),
        // Slot 'toggleTiedZoom'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setHighlightPoint'
        QtMocHelpers::SlotData<void(bool)>(114, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'changeZoomOnlyMode'
        QtMocHelpers::SlotData<void(QAction *)>(115, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 116, 2 },
        }}),
        // Slot 'openRecentKstFile'
        QtMocHelpers::SlotData<void()>(117, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openRecentDataFile'
        QtMocHelpers::SlotData<void()>(118, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkRecentFilesOnExistence'
        QtMocHelpers::SlotData<void()>(119, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'autoExportImage'
        QtMocHelpers::SlotData<void()>(120, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN3Kst10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst10MainWindowE_t>.metaTypes,
    nullptr
} };

void Kst::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->copyTab(); break;
        case 1: _t->toggleStatusBar(); break;
        case 2: _t->showDataManager(); break;
        case 3: _t->showDebugDialog(); break;
        case 4: _t->showExportGraphicsDialog(); break;
        case 5: _t->showExportVectorsDialog(); break;
        case 6: _t->showLogDialog(); break;
        case 7: _t->showVectorEditor(); break;
        case 8: _t->showScalarEditor(); break;
        case 9: _t->showMatrixEditor(); break;
        case 10: _t->showStringEditor(); break;
        case 11: _t->showSettingsDialog(); break;
        case 12: _t->showThemeDialog(); break;
        case 13: _t->clearDefaults(); break;
        case 14: _t->clearDataSourceSettings(); break;
        case 15: _t->showDifferentiateCurvesDialog(); break;
        case 16: _t->filterMultipleDialog(); break;
        case 17: _t->showChooseColorDialog(); break;
        case 18: _t->showChangeDataSampleDialog(); break;
        case 19: _t->showChangeFileDialog(); break;
        case 20: _t->showDataWizard(); break;
        case 21: _t->showDataWizard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->showBugReportWizard(); break;
        case 23: _t->docBookStarter(); break;
        case 24: _t->showPluginDialog((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 25: _t->readFromEnd(); break;
        case 26: _t->readToEnd(); break;
        case 27: _t->pause((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->back((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->back(); break;
        case 30: _t->forward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 31: _t->forward(); break;
        case 32: _t->backSmall(); break;
        case 33: _t->forwardSmall(); break;
        case 34: _t->reload(); break;
        case 35: _t->updateViewItems((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 36: _t->updateProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 37: _t->save(); break;
        case 38: _t->saveAs(); break;
        case 39: _t->open(); break;
        case 40: _t->newDoc((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->newDoc(); break;
        case 42: _t->openFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->cleanUpDataSourceList(); break;
        case 44: _t->print(); break;
        case 45: _t->setPrinterDefaults((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 46: _t->savePrinterDefaults((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 47: _t->printToPrinter((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 48: _t->printFromCommandLine((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 49: _t->exportGraphicsFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[7]))); break;
        case 50: _t->exportLog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7]))); break;
        case 51: _t->clearDrawingMarker(); break;
        case 52: _t->tiedZoomRemoved(); break;
        case 53: _t->allPlotsTiedZoom(); break;
        case 54: _t->updateRecentDataFiles((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 55: _t->updateRecentDataFiles(); break;
        case 56: _t->videoTutorial((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 57: { QStringList _r = _t->recentDataFiles();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 58: _t->cleanupRecentDataFilesList(); break;
        case 59: _t->aboutToQuit(); break;
        case 60: _t->about(); break;
        case 61: _t->showShortcutDialog(); break;
        case 62: _t->currentViewChanged(); break;
        case 63: _t->currentViewModeChanged(); break;
        case 64: _t->createBox(); break;
        case 65: _t->createEllipse(); break;
        case 66: _t->createLabel(); break;
        case 67: _t->createCircle(); break;
        case 68: _t->createLine(); break;
        case 69: _t->createArrow(); break;
        case 70: _t->createPicture(); break;
        case 71: _t->createPlot(); break;
        case 72: _t->insertPlot(); break;
        case 73: _t->createSvg(); break;
        case 74: _t->createSharedAxisBox(); break;
        case 75: _t->createScalar(); break;
        case 76: _t->createVector(); break;
        case 77: _t->createMatrix(); break;
        case 78: _t->createString(); break;
        case 79: _t->createCurve(); break;
        case 80: _t->createPSD(); break;
        case 81: _t->createEquation(); break;
        case 82: _t->createHistogram(); break;
        case 83: _t->createImage(); break;
        case 84: _t->createSpectogram(); break;
        case 85: _t->createEventMonitor(); break;
        case 86: _t->performHeavyStartupActions(); break;
        case 87: _t->cleanup(); break;
        case 88: _t->setLayoutMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 89: _t->toggleTiedZoom(); break;
        case 90: _t->setHighlightPoint((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 91: _t->changeZoomOnlyMode((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 92: _t->openRecentKstFile(); break;
        case 93: _t->openRecentDataFile(); break;
        case 94: _t->checkRecentFilesOnExistence(); break;
        case 95: _t->autoExportImage(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 91:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *Kst::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Kst::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 96)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 96;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 96)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 96;
    }
    return _id;
}
QT_WARNING_POP
