/********************************************************************************
** Form generated from reading UI file 'fitsinusoid_weightedconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FITSINUSOID_WEIGHTEDCONFIG_H
#define UI_FITSINUSOID_WEIGHTEDCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FitSinusoid_WeightedConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer;
    Kst::VectorSelector *_vectorX;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    QSpacerItem *horizontalSpacer_2;
    Kst::VectorSelector *_vectorY;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_5;
    QSpacerItem *horizontalSpacer_3;
    Kst::VectorSelector *_vectorWeights;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_6;
    QSpacerItem *horizontalSpacer_4;
    Kst::ScalarSelector *_scalarHarmonics;
    QHBoxLayout *horizontalLayout;
    QLabel *label_7;
    QSpacerItem *horizontalSpacer_5;
    Kst::ScalarSelector *_scalarPeriod;

    void setupUi(QWidget *FitSinusoid_WeightedConfig)
    {
        if (FitSinusoid_WeightedConfig->objectName().isEmpty())
            FitSinusoid_WeightedConfig->setObjectName("FitSinusoid_WeightedConfig");
        FitSinusoid_WeightedConfig->resize(426, 193);
        FitSinusoid_WeightedConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(FitSinusoid_WeightedConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(6);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_4 = new QLabel(FitSinusoid_WeightedConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setWordWrap(false);

        horizontalLayout_5->addWidget(label_4);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer);

        _vectorX = new Kst::VectorSelector(FitSinusoid_WeightedConfig);
        _vectorX->setObjectName("_vectorX");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorX->sizePolicy().hasHeightForWidth());
        _vectorX->setSizePolicy(sizePolicy1);

        horizontalLayout_5->addWidget(_vectorX);


        verticalLayout->addLayout(horizontalLayout_5);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(FitSinusoid_WeightedConfig);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout_4->addWidget(label_3);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);

        _vectorY = new Kst::VectorSelector(FitSinusoid_WeightedConfig);
        _vectorY->setObjectName("_vectorY");
        sizePolicy1.setHeightForWidth(_vectorY->sizePolicy().hasHeightForWidth());
        _vectorY->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(_vectorY);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_5 = new QLabel(FitSinusoid_WeightedConfig);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setWordWrap(false);

        horizontalLayout_3->addWidget(label_5);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        _vectorWeights = new Kst::VectorSelector(FitSinusoid_WeightedConfig);
        _vectorWeights->setObjectName("_vectorWeights");
        sizePolicy1.setHeightForWidth(_vectorWeights->sizePolicy().hasHeightForWidth());
        _vectorWeights->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(_vectorWeights);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_6 = new QLabel(FitSinusoid_WeightedConfig);
        label_6->setObjectName("label_6");
        sizePolicy.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy);
        label_6->setWordWrap(false);

        horizontalLayout_2->addWidget(label_6);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_4);

        _scalarHarmonics = new Kst::ScalarSelector(FitSinusoid_WeightedConfig);
        _scalarHarmonics->setObjectName("_scalarHarmonics");

        horizontalLayout_2->addWidget(_scalarHarmonics);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_7 = new QLabel(FitSinusoid_WeightedConfig);
        label_7->setObjectName("label_7");
        sizePolicy.setHeightForWidth(label_7->sizePolicy().hasHeightForWidth());
        label_7->setSizePolicy(sizePolicy);
        label_7->setWordWrap(false);

        horizontalLayout->addWidget(label_7);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_5);

        _scalarPeriod = new Kst::ScalarSelector(FitSinusoid_WeightedConfig);
        _scalarPeriod->setObjectName("_scalarPeriod");

        horizontalLayout->addWidget(_scalarPeriod);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(FitSinusoid_WeightedConfig);

        QMetaObject::connectSlotsByName(FitSinusoid_WeightedConfig);
    } // setupUi

    void retranslateUi(QWidget *FitSinusoid_WeightedConfig)
    {
        label_4->setText(QCoreApplication::translate("FitSinusoid_WeightedConfig", "Input Vector - X", nullptr));
        label_3->setText(QCoreApplication::translate("FitSinusoid_WeightedConfig", "Input Vector - Y", nullptr));
        label_5->setText(QCoreApplication::translate("FitSinusoid_WeightedConfig", "Input Vector - Weights", nullptr));
        label_6->setText(QCoreApplication::translate("FitSinusoid_WeightedConfig", "Input Scalar - Harmonics", nullptr));
        label_7->setText(QCoreApplication::translate("FitSinusoid_WeightedConfig", "Input Scalar - Period", nullptr));
        (void)FitSinusoid_WeightedConfig;
    } // retranslateUi

};

namespace Ui {
    class FitSinusoid_WeightedConfig: public Ui_FitSinusoid_WeightedConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FITSINUSOID_WEIGHTEDCONFIG_H
