/********************************************************************************
** Form generated from reading UI file 'labelpropertiestab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LABELPROPERTIESTAB_H
#define UI_LABELPROPERTIESTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"
#include "labelbuilder.h"

QT_BEGIN_NAMESPACE

class Ui_LabelPropertiesTab
{
public:
    QVBoxLayout *verticalLayout;
    Kst::LabelBuilder *_labelText;
    QGridLayout *gridLayout;
    QLabel *_Label_11;
    QDoubleSpinBox *_labelFontScale;
    QLabel *_Label_12;
    QFontComboBox *_family;
    QPushButton *_italic;
    Kst::ColorButton *_labelColor;
    QPushButton *_bold;
    QLabel *label;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *LabelPropertiesTab)
    {
        if (LabelPropertiesTab->objectName().isEmpty())
            LabelPropertiesTab->setObjectName("LabelPropertiesTab");
        LabelPropertiesTab->setWindowModality(Qt::NonModal);
        LabelPropertiesTab->resize(643, 227);
        verticalLayout = new QVBoxLayout(LabelPropertiesTab);
        verticalLayout->setObjectName("verticalLayout");
        _labelText = new Kst::LabelBuilder(LabelPropertiesTab);
        _labelText->setObjectName("_labelText");

        verticalLayout->addWidget(_labelText);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        _Label_11 = new QLabel(LabelPropertiesTab);
        _Label_11->setObjectName("_Label_11");
        _Label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_11, 0, 0, 1, 1);

        _labelFontScale = new QDoubleSpinBox(LabelPropertiesTab);
        _labelFontScale->setObjectName("_labelFontScale");
        _labelFontScale->setDecimals(1);
        _labelFontScale->setMinimum(-25.000000000000000);
        _labelFontScale->setSingleStep(1.000000000000000);

        gridLayout->addWidget(_labelFontScale, 0, 1, 1, 2);

        _Label_12 = new QLabel(LabelPropertiesTab);
        _Label_12->setObjectName("_Label_12");
        _Label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_12, 1, 0, 1, 1);

        _family = new QFontComboBox(LabelPropertiesTab);
        _family->setObjectName("_family");

        gridLayout->addWidget(_family, 1, 1, 1, 6);

        _italic = new QPushButton(LabelPropertiesTab);
        _italic->setObjectName("_italic");
        _italic->setCheckable(true);

        gridLayout->addWidget(_italic, 0, 5, 1, 1);

        _labelColor = new Kst::ColorButton(LabelPropertiesTab);
        _labelColor->setObjectName("_labelColor");

        gridLayout->addWidget(_labelColor, 0, 6, 1, 1);

        _bold = new QPushButton(LabelPropertiesTab);
        _bold->setObjectName("_bold");
        _bold->setCheckable(true);

        gridLayout->addWidget(_bold, 0, 4, 1, 1);

        label = new QLabel(LabelPropertiesTab);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 3, 1, 1);


        verticalLayout->addLayout(gridLayout);

        spacerItem = new QSpacerItem(20, 131, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);

#if QT_CONFIG(shortcut)
        _Label_11->setBuddy(_labelFontScale);
        _Label_12->setBuddy(_family);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_labelFontScale, _bold);
        QWidget::setTabOrder(_bold, _italic);
        QWidget::setTabOrder(_italic, _labelColor);
        QWidget::setTabOrder(_labelColor, _family);

        retranslateUi(LabelPropertiesTab);

        QMetaObject::connectSlotsByName(LabelPropertiesTab);
    } // setupUi

    void retranslateUi(QWidget *LabelPropertiesTab)
    {
        _Label_11->setText(QCoreApplication::translate("LabelPropertiesTab", "Font si&ze:", nullptr));
        _Label_12->setText(QCoreApplication::translate("LabelPropertiesTab", "Fo&nt family:", nullptr));
#if QT_CONFIG(tooltip)
        _italic->setToolTip(QCoreApplication::translate("LabelPropertiesTab", "Italic", "italic font"));
#endif // QT_CONFIG(tooltip)
        _italic->setText(QString());
#if QT_CONFIG(tooltip)
        _bold->setToolTip(QCoreApplication::translate("LabelPropertiesTab", "Bold", "bold font"));
#endif // QT_CONFIG(tooltip)
        _bold->setText(QString());
        label->setText(QCoreApplication::translate("LabelPropertiesTab", " points", nullptr));
        (void)LabelPropertiesTab;
    } // retranslateUi

};

namespace Ui {
    class LabelPropertiesTab: public Ui_LabelPropertiesTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LABELPROPERTIESTAB_H
