/********************************************************************************
** Form generated from reading UI file 'viewprimitivedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VIEWPRIMITIVEDIALOG_H
#define UI_VIEWPRIMITIVEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ViewPrimitiveDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *_tree;
    QHBoxLayout *horizontalLayout;
    QPushButton *updateButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ViewPrimitiveDialog)
    {
        if (ViewPrimitiveDialog->objectName().isEmpty())
            ViewPrimitiveDialog->setObjectName("ViewPrimitiveDialog");
        ViewPrimitiveDialog->resize(595, 534);
        verticalLayout = new QVBoxLayout(ViewPrimitiveDialog);
        verticalLayout->setObjectName("verticalLayout");
        _tree = new QTreeView(ViewPrimitiveDialog);
        _tree->setObjectName("_tree");

        verticalLayout->addWidget(_tree);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        updateButton = new QPushButton(ViewPrimitiveDialog);
        updateButton->setObjectName("updateButton");

        horizontalLayout->addWidget(updateButton);

        buttonBox = new QDialogButtonBox(ViewPrimitiveDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(ViewPrimitiveDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ViewPrimitiveDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ViewPrimitiveDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ViewPrimitiveDialog);
    } // setupUi

    void retranslateUi(QDialog *ViewPrimitiveDialog)
    {
        ViewPrimitiveDialog->setWindowTitle(QCoreApplication::translate("ViewPrimitiveDialog", "Primitive View", nullptr));
#if QT_CONFIG(tooltip)
        updateButton->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        updateButton->setText(QCoreApplication::translate("ViewPrimitiveDialog", "Update", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ViewPrimitiveDialog: public Ui_ViewPrimitiveDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VIEWPRIMITIVEDIALOG_H
