/********************************************************************************
** Form generated from reading UI file 'editmultiplewidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITMULTIPLEWIDGET_H
#define UI_EDITMULTIPLEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_EditMultipleWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *_layout;
    QLabel *_header;
    QHBoxLayout *horizontalLayout_2;
    QLabel *textLabel1;
    QLineEdit *_filterEdit;
    QHBoxLayout *horizontalLayout;
    QPushButton *_selectNoneBut;
    QPushButton *_selectAllBut;
    QListWidget *_objectList;

    void setupUi(QWidget *EditMultipleWidget)
    {
        if (EditMultipleWidget->objectName().isEmpty())
            EditMultipleWidget->setObjectName("EditMultipleWidget");
        EditMultipleWidget->resize(268, 405);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(EditMultipleWidget->sizePolicy().hasHeightForWidth());
        EditMultipleWidget->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(EditMultipleWidget);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        _layout = new QVBoxLayout();
        _layout->setSpacing(6);
        _layout->setObjectName("_layout");
        _header = new QLabel(EditMultipleWidget);
        _header->setObjectName("_header");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_header->sizePolicy().hasHeightForWidth());
        _header->setSizePolicy(sizePolicy1);
        _header->setAlignment(Qt::AlignCenter);

        _layout->addWidget(_header);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        textLabel1 = new QLabel(EditMultipleWidget);
        textLabel1->setObjectName("textLabel1");
        sizePolicy1.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy1);
        textLabel1->setWordWrap(false);

        horizontalLayout_2->addWidget(textLabel1);

        _filterEdit = new QLineEdit(EditMultipleWidget);
        _filterEdit->setObjectName("_filterEdit");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_filterEdit->sizePolicy().hasHeightForWidth());
        _filterEdit->setSizePolicy(sizePolicy2);

        horizontalLayout_2->addWidget(_filterEdit);


        _layout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        _selectNoneBut = new QPushButton(EditMultipleWidget);
        _selectNoneBut->setObjectName("_selectNoneBut");

        horizontalLayout->addWidget(_selectNoneBut);

        _selectAllBut = new QPushButton(EditMultipleWidget);
        _selectAllBut->setObjectName("_selectAllBut");

        horizontalLayout->addWidget(_selectAllBut);


        _layout->addLayout(horizontalLayout);

        _objectList = new QListWidget(EditMultipleWidget);
        _objectList->setObjectName("_objectList");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_objectList->sizePolicy().hasHeightForWidth());
        _objectList->setSizePolicy(sizePolicy3);
        _objectList->setMinimumSize(QSize(150, 0));
        _objectList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        _layout->addWidget(_objectList);


        verticalLayout_2->addLayout(_layout);


        retranslateUi(EditMultipleWidget);

        QMetaObject::connectSlotsByName(EditMultipleWidget);
    } // setupUi

    void retranslateUi(QWidget *EditMultipleWidget)
    {
        _header->setText(QCoreApplication::translate("EditMultipleWidget", "Select Objects to Edit", nullptr));
        textLabel1->setText(QCoreApplication::translate("EditMultipleWidget", "Filter:", nullptr));
        _selectNoneBut->setText(QCoreApplication::translate("EditMultipleWidget", "Clear", "clear the list"));
        _selectAllBut->setText(QCoreApplication::translate("EditMultipleWidget", "Select &All", nullptr));
        (void)EditMultipleWidget;
    } // retranslateUi

};

namespace Ui {
    class EditMultipleWidget: public Ui_EditMultipleWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITMULTIPLEWIDGET_H
