/********************************************************************************
** Form generated from reading UI file 'labelbuilder.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LABELBUILDER_H
#define UI_LABELBUILDER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>
#include "cclineedit.h"

QT_BEGIN_NAMESPACE

class Ui_LabelBuilder
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *_Label_13;
    QPushButton *_help;
    Kst::SVCCTextEdit *_label;

    void setupUi(QWidget *LabelBuilder)
    {
        if (LabelBuilder->objectName().isEmpty())
            LabelBuilder->setObjectName("LabelBuilder");
        LabelBuilder->resize(610, 149);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LabelBuilder->sizePolicy().hasHeightForWidth());
        LabelBuilder->setSizePolicy(sizePolicy);
        LabelBuilder->setMaximumSize(QSize(16777215, 16777215));
        gridLayout = new QGridLayout(LabelBuilder);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(LabelBuilder);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        QFont font;
        font.setItalic(true);
        label->setFont(font);

        gridLayout->addWidget(label, 1, 1, 1, 1);

        _Label_13 = new QLabel(LabelBuilder);
        _Label_13->setObjectName("_Label_13");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_Label_13->sizePolicy().hasHeightForWidth());
        _Label_13->setSizePolicy(sizePolicy2);
        _Label_13->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_13, 0, 0, 1, 1);

        _help = new QPushButton(LabelBuilder);
        _help->setObjectName("_help");
        sizePolicy1.setHeightForWidth(_help->sizePolicy().hasHeightForWidth());
        _help->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_help, 1, 2, 1, 1);

        _label = new Kst::SVCCTextEdit(LabelBuilder);
        _label->setObjectName("_label");

        gridLayout->addWidget(_label, 0, 1, 1, 2);

#if QT_CONFIG(shortcut)
        _Label_13->setBuddy(_label);
#endif // QT_CONFIG(shortcut)

        retranslateUi(LabelBuilder);

        QMetaObject::connectSlotsByName(LabelBuilder);
    } // setupUi

    void retranslateUi(QWidget *LabelBuilder)
    {
        label->setText(QCoreApplication::translate("LabelBuilder", "Supports LaTeX symbols, live strings & scalars, and embedded equations.", nullptr));
        _Label_13->setText(QCoreApplication::translate("LabelBuilder", "&Label:", nullptr));
        _help->setText(QCoreApplication::translate("LabelBuilder", "Help", nullptr));
        (void)LabelBuilder;
    } // retranslateUi

};

namespace Ui {
    class LabelBuilder: public Ui_LabelBuilder {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LABELBUILDER_H
