/********************************************************************************
** Form generated from reading UI file 'filterflagconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERFLAGCONFIG_H
#define UI_FILTERFLAGCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QWidget>
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_FilterFlagConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vector;
    QLabel *label_4;
    Kst::VectorSelector *_flag;
    QLabel *label;
    QLineEdit *_mask;
    QCheckBox *_validIsZero;

    void setupUi(QWidget *FilterFlagConfig)
    {
        if (FilterFlagConfig->objectName().isEmpty())
            FilterFlagConfig->setObjectName("FilterFlagConfig");
        FilterFlagConfig->resize(395, 89);
        gridLayout = new QGridLayout(FilterFlagConfig);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(FilterFlagConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _vector = new Kst::VectorSelector(FilterFlagConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vector, 0, 1, 1, 2);

        label_4 = new QLabel(FilterFlagConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 1, 0, 2, 1);

        _flag = new Kst::VectorSelector(FilterFlagConfig);
        _flag->setObjectName("_flag");
        sizePolicy1.setHeightForWidth(_flag->sizePolicy().hasHeightForWidth());
        _flag->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_flag, 1, 1, 2, 2);

        label = new QLabel(FilterFlagConfig);
        label->setObjectName("label");
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 3, 0, 1, 1);

        _mask = new QLineEdit(FilterFlagConfig);
        _mask->setObjectName("_mask");

        gridLayout->addWidget(_mask, 3, 1, 1, 1);

        _validIsZero = new QCheckBox(FilterFlagConfig);
        _validIsZero->setObjectName("_validIsZero");

        gridLayout->addWidget(_validIsZero, 3, 2, 1, 1);


        retranslateUi(FilterFlagConfig);

        QMetaObject::connectSlotsByName(FilterFlagConfig);
    } // setupUi

    void retranslateUi(QWidget *FilterFlagConfig)
    {
        label_3->setText(QCoreApplication::translate("FilterFlagConfig", "Input Vector:", nullptr));
        label_4->setText(QCoreApplication::translate("FilterFlagConfig", "Flag:", nullptr));
        label->setText(QCoreApplication::translate("FilterFlagConfig", "Mask:", nullptr));
        _mask->setText(QString());
        _validIsZero->setText(QCoreApplication::translate("FilterFlagConfig", "Masked Flag == 0 is valid", nullptr));
        (void)FilterFlagConfig;
    } // retranslateUi

};

namespace Ui {
    class FilterFlagConfig: public Ui_FilterFlagConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERFLAGCONFIG_H
