/********************************************************************************
** Form generated from reading UI file 'scalartab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCALARTAB_H
#define UI_SCALARTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "combobox.h"
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_ScalarTab
{
public:
    QGridLayout *gridLayout_2;
    QRadioButton *_readFromSource;
    QRadioButton *_readFromRVector;
    QRadioButton *_generateX;
    QGroupBox *_dataScalarGroup;
    QGridLayout *gridLayout;
    QLabel *label_4;
    Kst::DataSourceSelector *_fileName;
    QLabel *label_5;
    Kst::ComboBox *_field;
    QLabel *label_8;
    Kst::ComboBox *_fieldRV;
    QHBoxLayout *_3;
    QSpacerItem *spacerItem;
    QPushButton *_connect;
    QPushButton *_configure;
    QHBoxLayout *_rVectorFrames;
    QLabel *label_6;
    QSpinBox *_start;
    QLabel *label_7;
    QCheckBox *_countFromEnd;
    QGroupBox *_generatedScalarGroup;
    QGridLayout *_4;
    QHBoxLayout *_5;
    QLabel *label_3;
    QLineEdit *_scalarValue;

    void setupUi(QWidget *ScalarTab)
    {
        if (ScalarTab->objectName().isEmpty())
            ScalarTab->setObjectName("ScalarTab");
        ScalarTab->resize(629, 339);
        gridLayout_2 = new QGridLayout(ScalarTab);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        _readFromSource = new QRadioButton(ScalarTab);
        _readFromSource->setObjectName("_readFromSource");
        _readFromSource->setChecked(true);

        gridLayout_2->addWidget(_readFromSource, 0, 0, 1, 1);

        _readFromRVector = new QRadioButton(ScalarTab);
        _readFromRVector->setObjectName("_readFromRVector");

        gridLayout_2->addWidget(_readFromRVector, 0, 1, 1, 1);

        _generateX = new QRadioButton(ScalarTab);
        _generateX->setObjectName("_generateX");

        gridLayout_2->addWidget(_generateX, 0, 2, 1, 1);

        _dataScalarGroup = new QGroupBox(ScalarTab);
        _dataScalarGroup->setObjectName("_dataScalarGroup");
        gridLayout = new QGridLayout(_dataScalarGroup);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(_dataScalarGroup);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        _fileName = new Kst::DataSourceSelector(_dataScalarGroup);
        _fileName->setObjectName("_fileName");

        gridLayout->addWidget(_fileName, 0, 1, 1, 1);

        label_5 = new QLabel(_dataScalarGroup);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        label_5->setWordWrap(false);

        gridLayout->addWidget(label_5, 1, 0, 1, 1);

        _field = new Kst::ComboBox(_dataScalarGroup);
        _field->setObjectName("_field");
        _field->setEnabled(false);

        gridLayout->addWidget(_field, 1, 1, 1, 1);

        label_8 = new QLabel(_dataScalarGroup);
        label_8->setObjectName("label_8");
        sizePolicy.setHeightForWidth(label_8->sizePolicy().hasHeightForWidth());
        label_8->setSizePolicy(sizePolicy);
        label_8->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        label_8->setWordWrap(false);

        gridLayout->addWidget(label_8, 2, 0, 1, 1);

        _fieldRV = new Kst::ComboBox(_dataScalarGroup);
        _fieldRV->setObjectName("_fieldRV");
        _fieldRV->setEnabled(false);

        gridLayout->addWidget(_fieldRV, 2, 1, 1, 1);

        _3 = new QHBoxLayout();
        _3->setSpacing(6);
        _3->setObjectName("_3");
        _3->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(21, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _3->addItem(spacerItem);

        _connect = new QPushButton(_dataScalarGroup);
        _connect->setObjectName("_connect");
        _connect->setEnabled(false);

        _3->addWidget(_connect);

        _configure = new QPushButton(_dataScalarGroup);
        _configure->setObjectName("_configure");
        _configure->setEnabled(false);

        _3->addWidget(_configure);


        gridLayout->addLayout(_3, 3, 0, 1, 2);

        _rVectorFrames = new QHBoxLayout();
        _rVectorFrames->setSpacing(6);
        _rVectorFrames->setObjectName("_rVectorFrames");
        label_6 = new QLabel(_dataScalarGroup);
        label_6->setObjectName("label_6");
        label_6->setEnabled(false);
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_6->setWordWrap(false);

        _rVectorFrames->addWidget(label_6);

        _start = new QSpinBox(_dataScalarGroup);
        _start->setObjectName("_start");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_start->sizePolicy().hasHeightForWidth());
        _start->setSizePolicy(sizePolicy1);
        _start->setMinimum(0);
        _start->setMaximum(1000000000);

        _rVectorFrames->addWidget(_start);

        label_7 = new QLabel(_dataScalarGroup);
        label_7->setObjectName("label_7");
        label_7->setEnabled(false);
        label_7->setWordWrap(false);

        _rVectorFrames->addWidget(label_7);

        _countFromEnd = new QCheckBox(_dataScalarGroup);
        _countFromEnd->setObjectName("_countFromEnd");
        _countFromEnd->setEnabled(false);

        _rVectorFrames->addWidget(_countFromEnd);


        gridLayout->addLayout(_rVectorFrames, 4, 0, 1, 2);


        gridLayout_2->addWidget(_dataScalarGroup, 1, 0, 1, 3);

        _generatedScalarGroup = new QGroupBox(ScalarTab);
        _generatedScalarGroup->setObjectName("_generatedScalarGroup");
        _generatedScalarGroup->setEnabled(false);
        _4 = new QGridLayout(_generatedScalarGroup);
        _4->setSpacing(6);
        _4->setContentsMargins(11, 11, 11, 11);
        _4->setObjectName("_4");
        _4->setContentsMargins(-1, 0, -1, -1);
        _5 = new QHBoxLayout();
        _5->setSpacing(6);
        _5->setObjectName("_5");
        _5->setContentsMargins(0, 0, 0, 0);
        label_3 = new QLabel(_generatedScalarGroup);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setWordWrap(false);

        _5->addWidget(label_3);

        _scalarValue = new QLineEdit(_generatedScalarGroup);
        _scalarValue->setObjectName("_scalarValue");

        _5->addWidget(_scalarValue);


        _4->addLayout(_5, 0, 0, 1, 2);


        gridLayout_2->addWidget(_generatedScalarGroup, 2, 0, 1, 3);

#if QT_CONFIG(shortcut)
        label_5->setBuddy(_field);
        label_8->setBuddy(_fieldRV);
        label_6->setBuddy(_start);
        label_3->setBuddy(_scalarValue);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ScalarTab);

        QMetaObject::connectSlotsByName(ScalarTab);
    } // setupUi

    void retranslateUi(QWidget *ScalarTab)
    {
        _readFromSource->setText(QCoreApplication::translate("ScalarTab", "Read from data &source", nullptr));
#if QT_CONFIG(whatsthis)
        _readFromRVector->setWhatsThis(QCoreApplication::translate("ScalarTab", "Generate a new vector with the properties given here as the independent variable.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _readFromRVector->setText(QCoreApplication::translate("ScalarTab", "Read from data &vector", nullptr));
#if QT_CONFIG(whatsthis)
        _generateX->setWhatsThis(QCoreApplication::translate("ScalarTab", "Generate a new vector with the properties given here as the independent variable.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _generateX->setText(QCoreApplication::translate("ScalarTab", "&Generate", nullptr));
        _dataScalarGroup->setTitle(QCoreApplication::translate("ScalarTab", "Read from data source", nullptr));
        label_4->setText(QCoreApplication::translate("ScalarTab", "File name:", nullptr));
        label_5->setText(QCoreApplication::translate("ScalarTab", "&Field:", nullptr));
        label_8->setText(QCoreApplication::translate("ScalarTab", "&Field:", nullptr));
        _connect->setText(QCoreApplication::translate("ScalarTab", "&Connect", nullptr));
        _configure->setText(QCoreApplication::translate("ScalarTab", "Co&nfigure...", nullptr));
        label_6->setText(QCoreApplication::translate("ScalarTab", "Fra&me:", nullptr));
        label_7->setText(QCoreApplication::translate("ScalarTab", "or", nullptr));
#if QT_CONFIG(whatsthis)
        _countFromEnd->setWhatsThis(QCoreApplication::translate("ScalarTab", "Select this to read data from the end of the file.  This is useful for scrolling displays when the file is being appended by another application.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _countFromEnd->setText(QCoreApplication::translate("ScalarTab", "&last frame", nullptr));
        _generatedScalarGroup->setTitle(QCoreApplication::translate("ScalarTab", "Generate", nullptr));
        label_3->setText(QCoreApplication::translate("ScalarTab", "Va&lue: ", nullptr));
#if QT_CONFIG(whatsthis)
        _scalarValue->setWhatsThis(QCoreApplication::translate("ScalarTab", "The maximum value of the independent variable.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _scalarValue->setText(QString());
        (void)ScalarTab;
    } // retranslateUi

};

namespace Ui {
    class ScalarTab: public Ui_ScalarTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCALARTAB_H
