/********************************************************************************
** Form generated from reading UI file 'logdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOGDIALOG_H
#define UI_LOGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "filerequester.h"

QT_BEGIN_NAMESPACE

class Ui_LogDialog
{
public:
    QGridLayout *gridLayout_3;
    QTabWidget *_tab;
    QWidget *tab;
    QGridLayout *gridLayout_4;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QPushButton *pushButton;
    QPlainTextEdit *_message;
    QLabel *label_4;
    QPlainTextEdit *_scriptOutput;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox1;
    QGridLayout *gridLayout;
    QLabel *_widthLabel;
    QComboBox *_formats;
    QSpacerItem *horizontalSpacer;
    QComboBox *_sizeOption;
    QLabel *_heightLabel;
    QSpinBox *_ySize;
    QLabel *textLabel1;
    QLabel *textLabel1_2;
    QSpinBox *_xSize;
    QGridLayout *gridLayout_2;
    QLabel *_saveLocationLabel;
    Kst::FileRequester *_saveLocation;
    QLabel *label_3;
    QLineEdit *_script;
    QLabel *label;
    QLineEdit *_user;
    QSpacerItem *verticalSpacer;
    QPushButton *_rerunScript;
    QLabel *_scriptRunning;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *_ok;
    QPushButton *_apply;
    QPushButton *_close;

    void setupUi(QDialog *LogDialog)
    {
        if (LogDialog->objectName().isEmpty())
            LogDialog->setObjectName("LogDialog");
        LogDialog->resize(644, 421);
        gridLayout_3 = new QGridLayout(LogDialog);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        _tab = new QTabWidget(LogDialog);
        _tab->setObjectName("_tab");
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout_4 = new QGridLayout(tab);
        gridLayout_4->setSpacing(6);
        gridLayout_4->setContentsMargins(11, 11, 11, 11);
        gridLayout_4->setObjectName("gridLayout_4");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(tab);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        pushButton = new QPushButton(tab);
        pushButton->setObjectName("pushButton");

        horizontalLayout->addWidget(pushButton);


        gridLayout_4->addLayout(horizontalLayout, 0, 0, 1, 1);

        _message = new QPlainTextEdit(tab);
        _message->setObjectName("_message");

        gridLayout_4->addWidget(_message, 1, 0, 1, 1);

        label_4 = new QLabel(tab);
        label_4->setObjectName("label_4");

        gridLayout_4->addWidget(label_4, 2, 0, 1, 1);

        _scriptOutput = new QPlainTextEdit(tab);
        _scriptOutput->setObjectName("_scriptOutput");

        gridLayout_4->addWidget(_scriptOutput, 3, 0, 1, 1);

        _tab->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout = new QVBoxLayout(tab_2);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        groupBox1 = new QGroupBox(tab_2);
        groupBox1->setObjectName("groupBox1");
        gridLayout = new QGridLayout(groupBox1);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _widthLabel = new QLabel(groupBox1);
        _widthLabel->setObjectName("_widthLabel");
        _widthLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _widthLabel->setWordWrap(false);

        gridLayout->addWidget(_widthLabel, 1, 0, 1, 1);

        _formats = new QComboBox(groupBox1);
        _formats->setObjectName("_formats");

        gridLayout->addWidget(_formats, 0, 5, 1, 2);

        horizontalSpacer = new QSpacerItem(250, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 4, 1, 3);

        _sizeOption = new QComboBox(groupBox1);
        _sizeOption->addItem(QString());
        _sizeOption->addItem(QString());
        _sizeOption->addItem(QString());
        _sizeOption->addItem(QString());
        _sizeOption->setObjectName("_sizeOption");

        gridLayout->addWidget(_sizeOption, 0, 1, 1, 3);

        _heightLabel = new QLabel(groupBox1);
        _heightLabel->setObjectName("_heightLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_heightLabel->sizePolicy().hasHeightForWidth());
        _heightLabel->setSizePolicy(sizePolicy);
        _heightLabel->setWordWrap(false);

        gridLayout->addWidget(_heightLabel, 1, 2, 1, 1);

        _ySize = new QSpinBox(groupBox1);
        _ySize->setObjectName("_ySize");
        _ySize->setMinimum(5);
        _ySize->setMaximum(10000);
        _ySize->setSingleStep(20);
        _ySize->setValue(768);

        gridLayout->addWidget(_ySize, 1, 3, 1, 1);

        textLabel1 = new QLabel(groupBox1);
        textLabel1->setObjectName("textLabel1");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy1);
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 4, 1, 1);

        textLabel1_2 = new QLabel(groupBox1);
        textLabel1_2->setObjectName("textLabel1_2");
        sizePolicy1.setHeightForWidth(textLabel1_2->sizePolicy().hasHeightForWidth());
        textLabel1_2->setSizePolicy(sizePolicy1);
        textLabel1_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 0, 0, 1, 1);

        _xSize = new QSpinBox(groupBox1);
        _xSize->setObjectName("_xSize");
        _xSize->setMinimum(5);
        _xSize->setMaximum(10000);
        _xSize->setSingleStep(20);
        _xSize->setValue(1024);

        gridLayout->addWidget(_xSize, 1, 1, 1, 1);


        verticalLayout->addWidget(groupBox1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(6);
        gridLayout_2->setObjectName("gridLayout_2");
        _saveLocationLabel = new QLabel(tab_2);
        _saveLocationLabel->setObjectName("_saveLocationLabel");
        sizePolicy1.setHeightForWidth(_saveLocationLabel->sizePolicy().hasHeightForWidth());
        _saveLocationLabel->setSizePolicy(sizePolicy1);
        _saveLocationLabel->setWordWrap(false);

        gridLayout_2->addWidget(_saveLocationLabel, 0, 0, 1, 1);

        _saveLocation = new Kst::FileRequester(tab_2);
        _saveLocation->setObjectName("_saveLocation");

        gridLayout_2->addWidget(_saveLocation, 0, 1, 1, 1);

        label_3 = new QLabel(tab_2);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_3, 1, 0, 1, 1);

        _script = new QLineEdit(tab_2);
        _script->setObjectName("_script");

        gridLayout_2->addWidget(_script, 1, 1, 1, 1);

        label = new QLabel(tab_2);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label, 2, 0, 1, 1);

        _user = new QLineEdit(tab_2);
        _user->setObjectName("_user");

        gridLayout_2->addWidget(_user, 2, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        _tab->addTab(tab_2, QString());

        gridLayout_3->addWidget(_tab, 0, 0, 1, 6);

        _rerunScript = new QPushButton(LogDialog);
        _rerunScript->setObjectName("_rerunScript");
        _rerunScript->setEnabled(false);
        sizePolicy.setHeightForWidth(_rerunScript->sizePolicy().hasHeightForWidth());
        _rerunScript->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(_rerunScript, 1, 0, 1, 1);

        _scriptRunning = new QLabel(LogDialog);
        _scriptRunning->setObjectName("_scriptRunning");

        gridLayout_3->addWidget(_scriptRunning, 1, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(321, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 1, 2, 1, 1);

        _ok = new QPushButton(LogDialog);
        _ok->setObjectName("_ok");
        sizePolicy.setHeightForWidth(_ok->sizePolicy().hasHeightForWidth());
        _ok->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(_ok, 1, 3, 1, 1);

        _apply = new QPushButton(LogDialog);
        _apply->setObjectName("_apply");
        sizePolicy.setHeightForWidth(_apply->sizePolicy().hasHeightForWidth());
        _apply->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(_apply, 1, 4, 1, 1);

        _close = new QPushButton(LogDialog);
        _close->setObjectName("_close");
        sizePolicy.setHeightForWidth(_close->sizePolicy().hasHeightForWidth());
        _close->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(_close, 1, 5, 1, 1);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(_message);
        _widthLabel->setBuddy(_xSize);
        _heightLabel->setBuddy(_ySize);
        textLabel1->setBuddy(_formats);
        textLabel1_2->setBuddy(_sizeOption);
        label_3->setBuddy(_script);
        label->setBuddy(_user);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_message, pushButton);
        QWidget::setTabOrder(pushButton, _tab);
        QWidget::setTabOrder(_tab, _apply);
        QWidget::setTabOrder(_apply, _close);
        QWidget::setTabOrder(_close, _rerunScript);
        QWidget::setTabOrder(_rerunScript, _scriptOutput);
        QWidget::setTabOrder(_scriptOutput, _sizeOption);
        QWidget::setTabOrder(_sizeOption, _formats);
        QWidget::setTabOrder(_formats, _xSize);
        QWidget::setTabOrder(_xSize, _ySize);
        QWidget::setTabOrder(_ySize, _script);
        QWidget::setTabOrder(_script, _user);

        retranslateUi(LogDialog);
        QObject::connect(_close, &QPushButton::clicked, LogDialog, qOverload<>(&QDialog::close));
        QObject::connect(pushButton, &QPushButton::clicked, _message, qOverload<>(&QPlainTextEdit::clear));

        _tab->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(LogDialog);
    } // setupUi

    void retranslateUi(QDialog *LogDialog)
    {
        LogDialog->setWindowTitle(QCoreApplication::translate("LogDialog", "Make a log entry", nullptr));
        label_2->setText(QCoreApplication::translate("LogDialog", "&Message:", nullptr));
        pushButton->setText(QCoreApplication::translate("LogDialog", "&Clear", nullptr));
        label_4->setText(QCoreApplication::translate("LogDialog", "Script output:", nullptr));
        _tab->setTabText(_tab->indexOf(tab), QCoreApplication::translate("LogDialog", "Message", nullptr));
        groupBox1->setTitle(QCoreApplication::translate("LogDialog", "Plot Properties", nullptr));
        _widthLabel->setText(QCoreApplication::translate("LogDialog", "&Width:", nullptr));
        _sizeOption->setItemText(0, QCoreApplication::translate("LogDialog", "Width and Maintain Aspect Ratio", nullptr));
        _sizeOption->setItemText(1, QCoreApplication::translate("LogDialog", "Height and Maintain Aspect Ratio", nullptr));
        _sizeOption->setItemText(2, QCoreApplication::translate("LogDialog", "Width and Height", nullptr));
        _sizeOption->setItemText(3, QCoreApplication::translate("LogDialog", "Size of Square", nullptr));

        _heightLabel->setText(QCoreApplication::translate("LogDialog", "&Height:", nullptr));
#if QT_CONFIG(whatsthis)
        _ySize->setWhatsThis(QCoreApplication::translate("LogDialog", "Height of the image (in pixels).", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(QCoreApplication::translate("LogDialog", "&File format:", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("LogDialog", "S&pecify:", nullptr));
#if QT_CONFIG(whatsthis)
        _xSize->setWhatsThis(QCoreApplication::translate("LogDialog", "Width (in Pixels) of the image.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _saveLocationLabel->setText(QCoreApplication::translate("LogDialog", "Local log directory:", nullptr));
        label_3->setText(QCoreApplication::translate("LogDialog", "&Script:", nullptr));
#if QT_CONFIG(tooltip)
        _script->setToolTip(QCoreApplication::translate("LogDialog", "The variables $user, $imagefile and $messagefile can be used in the argument list.", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("LogDialog", "&User name:", nullptr));
        _tab->setTabText(_tab->indexOf(tab_2), QCoreApplication::translate("LogDialog", "Configuration", nullptr));
        _rerunScript->setText(QCoreApplication::translate("LogDialog", "&Rerun Script", nullptr));
        _scriptRunning->setText(QCoreApplication::translate("LogDialog", "Script:", nullptr));
        _ok->setText(QCoreApplication::translate("LogDialog", "&Ok", nullptr));
        _apply->setText(QCoreApplication::translate("LogDialog", "&Apply", nullptr));
        _close->setText(QCoreApplication::translate("LogDialog", "&Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LogDialog: public Ui_LogDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LOGDIALOG_H
