/********************************************************************************
** Form generated from reading UI file 'noiseadditionconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NOISEADDITIONCONFIG_H
#define UI_NOISEADDITIONCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_NoiseAdditionConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    Kst::VectorSelector *_vector;
    QHBoxLayout *horizontalLayout;
    QLabel *label_4;
    Kst::ScalarSelector *_scalarSigma;

    void setupUi(QWidget *NoiseAdditionConfig)
    {
        if (NoiseAdditionConfig->objectName().isEmpty())
            NoiseAdditionConfig->setObjectName("NoiseAdditionConfig");
        NoiseAdditionConfig->resize(419, 97);
        NoiseAdditionConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(NoiseAdditionConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_3 = new QLabel(NoiseAdditionConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout_2->addWidget(label_3);

        _vector = new Kst::VectorSelector(NoiseAdditionConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(_vector);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_4 = new QLabel(NoiseAdditionConfig);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setWordWrap(false);

        horizontalLayout->addWidget(label_4);

        _scalarSigma = new Kst::ScalarSelector(NoiseAdditionConfig);
        _scalarSigma->setObjectName("_scalarSigma");

        horizontalLayout->addWidget(_scalarSigma);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(NoiseAdditionConfig);

        QMetaObject::connectSlotsByName(NoiseAdditionConfig);
    } // setupUi

    void retranslateUi(QWidget *NoiseAdditionConfig)
    {
        label_3->setText(QCoreApplication::translate("NoiseAdditionConfig", "Input Vector", nullptr));
        label_4->setText(QCoreApplication::translate("NoiseAdditionConfig", "Sigma", nullptr));
        (void)NoiseAdditionConfig;
    } // retranslateUi

};

namespace Ui {
    class NoiseAdditionConfig: public Ui_NoiseAdditionConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NOISEADDITIONCONFIG_H
