/********************************************************************************
** Form generated from reading UI file 'modelessinfobox.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MODELESSINFOBOX_H
#define UI_MODELESSINFOBOX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>

QT_BEGIN_NAMESPACE

class Ui_ModelessInfoBox
{
public:
    QGridLayout *gridLayout;
    QTextEdit *_text;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButton;

    void setupUi(QDialog *ModelessInfoBox)
    {
        if (ModelessInfoBox->objectName().isEmpty())
            ModelessInfoBox->setObjectName("ModelessInfoBox");
        ModelessInfoBox->resize(400, 300);
        gridLayout = new QGridLayout(ModelessInfoBox);
        gridLayout->setObjectName("gridLayout");
        _text = new QTextEdit(ModelessInfoBox);
        _text->setObjectName("_text");
        _text->setReadOnly(true);

        gridLayout->addWidget(_text, 0, 0, 1, 2);

        horizontalSpacer = new QSpacerItem(325, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 0, 1, 1);

        pushButton = new QPushButton(ModelessInfoBox);
        pushButton->setObjectName("pushButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pushButton->sizePolicy().hasHeightForWidth());
        pushButton->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pushButton, 1, 1, 1, 1);


        retranslateUi(ModelessInfoBox);
        QObject::connect(pushButton, &QPushButton::clicked, ModelessInfoBox, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(ModelessInfoBox);
    } // setupUi

    void retranslateUi(QDialog *ModelessInfoBox)
    {
        ModelessInfoBox->setWindowTitle(QCoreApplication::translate("ModelessInfoBox", "Dialog", nullptr));
        pushButton->setText(QCoreApplication::translate("ModelessInfoBox", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ModelessInfoBox: public Ui_ModelessInfoBox {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MODELESSINFOBOX_H
