/********************************************************************************
** Form generated from reading UI file 'dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOG_H
#define UI_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Dialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QWidget *_topCustom;
    QWidget *_extensionWidget;
    QWidget *_leftCustom;
    QWidget *_rightCustom;
    QWidget *_bottomCustom;
    QSplitter *_splitter;
    QListWidget *_listWidget;
    QStackedWidget *_stackedWidget;
    QDialogButtonBox *_buttonBox;
    QCheckBox *_saveAsDefault;
    QCheckBox *_applyToExisting;

    void setupUi(QDialog *Dialog)
    {
        if (Dialog->objectName().isEmpty())
            Dialog->setObjectName("Dialog");
        Dialog->setWindowModality(Qt::NonModal);
        Dialog->resize(692, 295);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Dialog->sizePolicy().hasHeightForWidth());
        Dialog->setSizePolicy(sizePolicy);
        Dialog->setModal(false);
        gridLayout = new QGridLayout(Dialog);
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(0);
        gridLayout1->setObjectName("gridLayout1");
        _topCustom = new QWidget(Dialog);
        _topCustom->setObjectName("_topCustom");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_topCustom->sizePolicy().hasHeightForWidth());
        _topCustom->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(_topCustom, 0, 0, 1, 3);

        _extensionWidget = new QWidget(Dialog);
        _extensionWidget->setObjectName("_extensionWidget");
        _extensionWidget->setEnabled(true);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_extensionWidget->sizePolicy().hasHeightForWidth());
        _extensionWidget->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(_extensionWidget, 0, 3, 3, 1);

        _leftCustom = new QWidget(Dialog);
        _leftCustom->setObjectName("_leftCustom");
        sizePolicy1.setHeightForWidth(_leftCustom->sizePolicy().hasHeightForWidth());
        _leftCustom->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(_leftCustom, 1, 0, 1, 1);

        _rightCustom = new QWidget(Dialog);
        _rightCustom->setObjectName("_rightCustom");
        sizePolicy1.setHeightForWidth(_rightCustom->sizePolicy().hasHeightForWidth());
        _rightCustom->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(_rightCustom, 1, 2, 1, 1);

        _bottomCustom = new QWidget(Dialog);
        _bottomCustom->setObjectName("_bottomCustom");
        sizePolicy1.setHeightForWidth(_bottomCustom->sizePolicy().hasHeightForWidth());
        _bottomCustom->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(_bottomCustom, 2, 0, 1, 3);

        _splitter = new QSplitter(Dialog);
        _splitter->setObjectName("_splitter");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(_splitter->sizePolicy().hasHeightForWidth());
        _splitter->setSizePolicy(sizePolicy3);
        _splitter->setLayoutDirection(Qt::LeftToRight);
        _splitter->setOrientation(Qt::Horizontal);
        _splitter->setChildrenCollapsible(false);
        _listWidget = new QListWidget(_splitter);
        _listWidget->setObjectName("_listWidget");
        sizePolicy2.setHeightForWidth(_listWidget->sizePolicy().hasHeightForWidth());
        _listWidget->setSizePolicy(sizePolicy2);
        _listWidget->setResizeMode(QListView::Adjust);
        _listWidget->setUniformItemSizes(true);
        _splitter->addWidget(_listWidget);
        _stackedWidget = new QStackedWidget(_splitter);
        _stackedWidget->setObjectName("_stackedWidget");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy4.setHorizontalStretch(1);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_stackedWidget->sizePolicy().hasHeightForWidth());
        _stackedWidget->setSizePolicy(sizePolicy4);
        _splitter->addWidget(_stackedWidget);

        gridLayout1->addWidget(_splitter, 1, 1, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 4);

        _buttonBox = new QDialogButtonBox(Dialog);
        _buttonBox->setObjectName("_buttonBox");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(_buttonBox->sizePolicy().hasHeightForWidth());
        _buttonBox->setSizePolicy(sizePolicy5);
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(_buttonBox, 1, 3, 1, 1);

        _saveAsDefault = new QCheckBox(Dialog);
        _saveAsDefault->setObjectName("_saveAsDefault");

        gridLayout->addWidget(_saveAsDefault, 1, 0, 1, 1);

        _applyToExisting = new QCheckBox(Dialog);
        _applyToExisting->setObjectName("_applyToExisting");

        gridLayout->addWidget(_applyToExisting, 1, 1, 1, 2);

        QWidget::setTabOrder(_listWidget, _buttonBox);

        retranslateUi(Dialog);
        QObject::connect(_buttonBox, &QDialogButtonBox::accepted, Dialog, qOverload<>(&QDialog::accept));
        QObject::connect(_buttonBox, &QDialogButtonBox::rejected, Dialog, qOverload<>(&QDialog::reject));

        _stackedWidget->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(Dialog);
    } // setupUi

    void retranslateUi(QDialog *Dialog)
    {
        _saveAsDefault->setText(QCoreApplication::translate("Dialog", "Save as default", nullptr));
        _applyToExisting->setText(QCoreApplication::translate("Dialog", "Apply to existing objects", nullptr));
        (void)Dialog;
    } // retranslateUi

};

namespace Ui {
    class Dialog: public Ui_Dialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOG_H
