/********************************************************************************
** Form generated from reading UI file 'themedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_THEMEDIALOG_H
#define UI_THEMEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_ThemeDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QGroupBox *_fillBox;
    QGroupBox *_strokeBox;
    QHBoxLayout *horizontalLayout_3;
    QGroupBox *_curveBox;
    QVBoxLayout *verticalLayout;
    QLabel *label_2;
    QComboBox *_curveColor;
    QSpacerItem *verticalSpacer;
    QGroupBox *_fontBox;
    QGridLayout *gridLayout_3;
    QGridLayout *gridLayout_2;
    QLabel *_Label_11;
    QDoubleSpinBox *_labelFontScale;
    QLabel *_Label_12;
    QFontComboBox *_family;
    QPushButton *_italic;
    Kst::ColorButton *_labelColor;
    QPushButton *_bold;
    QLabel *label;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *_saveAsDefault;
    QCheckBox *_applyToExisting;
    QDialogButtonBox *_buttonBox;

    void setupUi(QWidget *ThemeDialog)
    {
        if (ThemeDialog->objectName().isEmpty())
            ThemeDialog->setObjectName("ThemeDialog");
        ThemeDialog->resize(879, 268);
        verticalLayout_2 = new QVBoxLayout(ThemeDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        _fillBox = new QGroupBox(ThemeDialog);
        _fillBox->setObjectName("_fillBox");
        _fillBox->setCheckable(true);

        horizontalLayout->addWidget(_fillBox);

        _strokeBox = new QGroupBox(ThemeDialog);
        _strokeBox->setObjectName("_strokeBox");
        _strokeBox->setCheckable(true);

        horizontalLayout->addWidget(_strokeBox);


        verticalLayout_2->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        _curveBox = new QGroupBox(ThemeDialog);
        _curveBox->setObjectName("_curveBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_curveBox->sizePolicy().hasHeightForWidth());
        _curveBox->setSizePolicy(sizePolicy);
        _curveBox->setLayoutDirection(Qt::LeftToRight);
        _curveBox->setCheckable(true);
        verticalLayout = new QVBoxLayout(_curveBox);
        verticalLayout->setObjectName("verticalLayout");
        label_2 = new QLabel(_curveBox);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        _curveColor = new QComboBox(_curveBox);
        _curveColor->setObjectName("_curveColor");
        sizePolicy.setHeightForWidth(_curveColor->sizePolicy().hasHeightForWidth());
        _curveColor->setSizePolicy(sizePolicy);
        _curveColor->setLayoutDirection(Qt::LeftToRight);
        _curveColor->setMaxVisibleItems(13);

        verticalLayout->addWidget(_curveColor);

        verticalSpacer = new QSpacerItem(20, 9, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout_3->addWidget(_curveBox);

        _fontBox = new QGroupBox(ThemeDialog);
        _fontBox->setObjectName("_fontBox");
        _fontBox->setMinimumSize(QSize(0, 0));
        _fontBox->setCheckable(true);
        gridLayout_3 = new QGridLayout(_fontBox);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        _Label_11 = new QLabel(_fontBox);
        _Label_11->setObjectName("_Label_11");
        _Label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_Label_11, 0, 0, 1, 1);

        _labelFontScale = new QDoubleSpinBox(_fontBox);
        _labelFontScale->setObjectName("_labelFontScale");
        _labelFontScale->setDecimals(1);
        _labelFontScale->setMinimum(-25.000000000000000);
        _labelFontScale->setSingleStep(1.000000000000000);

        gridLayout_2->addWidget(_labelFontScale, 0, 1, 1, 2);

        _Label_12 = new QLabel(_fontBox);
        _Label_12->setObjectName("_Label_12");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_Label_12->sizePolicy().hasHeightForWidth());
        _Label_12->setSizePolicy(sizePolicy1);
        _Label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_Label_12, 1, 0, 1, 1);

        _family = new QFontComboBox(_fontBox);
        _family->setObjectName("_family");

        gridLayout_2->addWidget(_family, 1, 1, 1, 6);

        _italic = new QPushButton(_fontBox);
        _italic->setObjectName("_italic");
        _italic->setCheckable(true);

        gridLayout_2->addWidget(_italic, 0, 5, 1, 1);

        _labelColor = new Kst::ColorButton(_fontBox);
        _labelColor->setObjectName("_labelColor");

        gridLayout_2->addWidget(_labelColor, 0, 6, 1, 1);

        _bold = new QPushButton(_fontBox);
        _bold->setObjectName("_bold");
        _bold->setCheckable(true);

        gridLayout_2->addWidget(_bold, 0, 4, 1, 1);

        label = new QLabel(_fontBox);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 0, 3, 1, 1);


        gridLayout_3->addLayout(gridLayout_2, 0, 0, 1, 1);


        horizontalLayout_3->addWidget(_fontBox);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _saveAsDefault = new QCheckBox(ThemeDialog);
        _saveAsDefault->setObjectName("_saveAsDefault");

        horizontalLayout_2->addWidget(_saveAsDefault);

        _applyToExisting = new QCheckBox(ThemeDialog);
        _applyToExisting->setObjectName("_applyToExisting");

        horizontalLayout_2->addWidget(_applyToExisting);

        _buttonBox = new QDialogButtonBox(ThemeDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(_buttonBox);


        verticalLayout_2->addLayout(horizontalLayout_2);

#if QT_CONFIG(shortcut)
        _Label_11->setBuddy(_labelFontScale);
        _Label_12->setBuddy(_family);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_fillBox, _strokeBox);
        QWidget::setTabOrder(_strokeBox, _fontBox);
        QWidget::setTabOrder(_fontBox, _labelFontScale);
        QWidget::setTabOrder(_labelFontScale, _bold);
        QWidget::setTabOrder(_bold, _italic);
        QWidget::setTabOrder(_italic, _labelColor);
        QWidget::setTabOrder(_labelColor, _family);
        QWidget::setTabOrder(_family, _saveAsDefault);
        QWidget::setTabOrder(_saveAsDefault, _applyToExisting);
        QWidget::setTabOrder(_applyToExisting, _buttonBox);

        retranslateUi(ThemeDialog);
        QObject::connect(_buttonBox, &QDialogButtonBox::rejected, ThemeDialog, qOverload<>(&QWidget::close));

        QMetaObject::connectSlotsByName(ThemeDialog);
    } // setupUi

    void retranslateUi(QWidget *ThemeDialog)
    {
        _fillBox->setTitle(QCoreApplication::translate("ThemeDialog", "Fill", nullptr));
        _strokeBox->setTitle(QCoreApplication::translate("ThemeDialog", "Stroke", nullptr));
        _curveBox->setTitle(QCoreApplication::translate("ThemeDialog", "Curve", nullptr));
        label_2->setText(QCoreApplication::translate("ThemeDialog", "Default Color:", nullptr));
#if QT_CONFIG(tooltip)
        _curveColor->setToolTip(QCoreApplication::translate("ThemeDialog", "Default Curve Color", nullptr));
#endif // QT_CONFIG(tooltip)
        _fontBox->setTitle(QCoreApplication::translate("ThemeDialog", "Fonts", nullptr));
        _Label_11->setText(QCoreApplication::translate("ThemeDialog", "Fo&nt size:", nullptr));
        _Label_12->setText(QCoreApplication::translate("ThemeDialog", "Font fa&mily:", nullptr));
#if QT_CONFIG(tooltip)
        _italic->setToolTip(QCoreApplication::translate("ThemeDialog", "Italic", "italic font"));
#endif // QT_CONFIG(tooltip)
        _italic->setText(QString());
#if QT_CONFIG(tooltip)
        _bold->setToolTip(QCoreApplication::translate("ThemeDialog", "Bold", "bold font"));
#endif // QT_CONFIG(tooltip)
        _bold->setText(QString());
        label->setText(QCoreApplication::translate("ThemeDialog", " points", nullptr));
        _saveAsDefault->setText(QCoreApplication::translate("ThemeDialog", "Save as default", nullptr));
        _applyToExisting->setText(QCoreApplication::translate("ThemeDialog", "Apply to existing objects", nullptr));
        (void)ThemeDialog;
    } // retranslateUi

};

namespace Ui {
    class ThemeDialog: public Ui_ThemeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_THEMEDIALOG_H
