/********************************************************************************
** Form generated from reading UI file 'curveselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CURVESELECTOR_H
#define UI_CURVESELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QWidget>
#include "combobox.h"

QT_BEGIN_NAMESPACE

class Ui_CurveSelector
{
public:
    QHBoxLayout *hboxLayout;
    Kst::ComboBox *_curve;

    void setupUi(QWidget *CurveSelector)
    {
        if (CurveSelector->objectName().isEmpty())
            CurveSelector->setObjectName("CurveSelector");
        CurveSelector->resize(318, 25);
        hboxLayout = new QHBoxLayout(CurveSelector);
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        _curve = new Kst::ComboBox(CurveSelector);
        _curve->setObjectName("_curve");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_curve->sizePolicy().hasHeightForWidth());
        _curve->setSizePolicy(sizePolicy);
        _curve->setMaximumSize(QSize(300, 72));

        hboxLayout->addWidget(_curve);


        retranslateUi(CurveSelector);

        QMetaObject::connectSlotsByName(CurveSelector);
    } // setupUi

    void retranslateUi(QWidget *CurveSelector)
    {
#if QT_CONFIG(whatsthis)
        CurveSelector->setWhatsThis(QCoreApplication::translate("CurveSelector", "Select a curve to utilize.", nullptr));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class CurveSelector: public Ui_CurveSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CURVESELECTOR_H
