/********************************************************************************
** Form generated from reading UI file 'curveappearance.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CURVEAPPEARANCE_H
#define UI_CURVEAPPEARANCE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include <kstwidgets_export.h>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_CurveAppearance
{
public:
    QGridLayout *gridLayout_2;
    QGroupBox *GroupBox9_2_2_2;
    QGridLayout *gridLayout;
    QLabel *_textLabelWeight;
    QComboBox *_comboPointSymbol;
    QComboBox *_comboLineStyle;
    QComboBox *_comboHeadSymbol;
    QLabel *_label;
    QCheckBox *_showLines;
    QCheckBox *_showPoints;
    QCheckBox *_showHead;
    QCheckBox *_showBars;
    Kst::ColorButton *_color;
    Kst::ColorButton *_barFillColor;
    QLabel *_textLabelPointSize;
    QLabel *_textLabelPointDensity;
    QDoubleSpinBox *_spinBoxPointSize;
    QComboBox *_comboPointDensity;
    Kst::ColorButton *_headColor;
    QSpinBox *_spinBoxLineWidth;

    void setupUi(QWidget *CurveAppearance)
    {
        if (CurveAppearance->objectName().isEmpty())
            CurveAppearance->setObjectName("CurveAppearance");
        CurveAppearance->resize(498, 204);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CurveAppearance->sizePolicy().hasHeightForWidth());
        CurveAppearance->setSizePolicy(sizePolicy);
        gridLayout_2 = new QGridLayout(CurveAppearance);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        GroupBox9_2_2_2 = new QGroupBox(CurveAppearance);
        GroupBox9_2_2_2->setObjectName("GroupBox9_2_2_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(GroupBox9_2_2_2->sizePolicy().hasHeightForWidth());
        GroupBox9_2_2_2->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(false);
        GroupBox9_2_2_2->setFont(font);
        gridLayout = new QGridLayout(GroupBox9_2_2_2);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _textLabelWeight = new QLabel(GroupBox9_2_2_2);
        _textLabelWeight->setObjectName("_textLabelWeight");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_textLabelWeight->sizePolicy().hasHeightForWidth());
        _textLabelWeight->setSizePolicy(sizePolicy2);
        _textLabelWeight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _textLabelWeight->setWordWrap(false);

        gridLayout->addWidget(_textLabelWeight, 2, 5, 1, 1);

        _comboPointSymbol = new QComboBox(GroupBox9_2_2_2);
        _comboPointSymbol->setObjectName("_comboPointSymbol");
        _comboPointSymbol->setEnabled(false);

        gridLayout->addWidget(_comboPointSymbol, 4, 2, 1, 3);

        _comboLineStyle = new QComboBox(GroupBox9_2_2_2);
        _comboLineStyle->setObjectName("_comboLineStyle");

        gridLayout->addWidget(_comboLineStyle, 2, 2, 1, 3);

        _comboHeadSymbol = new QComboBox(GroupBox9_2_2_2);
        _comboHeadSymbol->setObjectName("_comboHeadSymbol");

        gridLayout->addWidget(_comboHeadSymbol, 7, 2, 1, 3);

        _label = new QLabel(GroupBox9_2_2_2);
        _label->setObjectName("_label");
        sizePolicy1.setHeightForWidth(_label->sizePolicy().hasHeightForWidth());
        _label->setSizePolicy(sizePolicy1);
        _label->setMinimumSize(QSize(100, 16));
        _label->setFrameShape(QFrame::NoFrame);
        _label->setFrameShadow(QFrame::Plain);
        _label->setScaledContents(true);
        _label->setWordWrap(false);

        gridLayout->addWidget(_label, 1, 0, 1, 1);

        _showLines = new QCheckBox(GroupBox9_2_2_2);
        _showLines->setObjectName("_showLines");
        sizePolicy1.setHeightForWidth(_showLines->sizePolicy().hasHeightForWidth());
        _showLines->setSizePolicy(sizePolicy1);
        _showLines->setChecked(true);

        gridLayout->addWidget(_showLines, 2, 0, 1, 1);

        _showPoints = new QCheckBox(GroupBox9_2_2_2);
        _showPoints->setObjectName("_showPoints");
        sizePolicy1.setHeightForWidth(_showPoints->sizePolicy().hasHeightForWidth());
        _showPoints->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_showPoints, 4, 0, 1, 2);

        _showHead = new QCheckBox(GroupBox9_2_2_2);
        _showHead->setObjectName("_showHead");
        sizePolicy1.setHeightForWidth(_showHead->sizePolicy().hasHeightForWidth());
        _showHead->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_showHead, 7, 0, 1, 2);

        _showBars = new QCheckBox(GroupBox9_2_2_2);
        _showBars->setObjectName("_showBars");
        sizePolicy1.setHeightForWidth(_showBars->sizePolicy().hasHeightForWidth());
        _showBars->setSizePolicy(sizePolicy1);
        _showBars->setChecked(false);

        gridLayout->addWidget(_showBars, 8, 0, 1, 2);

        _color = new Kst::ColorButton(GroupBox9_2_2_2);
        _color->setObjectName("_color");
        sizePolicy1.setHeightForWidth(_color->sizePolicy().hasHeightForWidth());
        _color->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_color, 1, 2, 1, 1);

        _barFillColor = new Kst::ColorButton(GroupBox9_2_2_2);
        _barFillColor->setObjectName("_barFillColor");
        sizePolicy1.setHeightForWidth(_barFillColor->sizePolicy().hasHeightForWidth());
        _barFillColor->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_barFillColor, 8, 2, 1, 1);

        _textLabelPointSize = new QLabel(GroupBox9_2_2_2);
        _textLabelPointSize->setObjectName("_textLabelPointSize");
        _textLabelPointSize->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_textLabelPointSize, 4, 5, 1, 1);

        _textLabelPointDensity = new QLabel(GroupBox9_2_2_2);
        _textLabelPointDensity->setObjectName("_textLabelPointDensity");
        _textLabelPointDensity->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        _textLabelPointDensity->setWordWrap(false);

        gridLayout->addWidget(_textLabelPointDensity, 1, 3, 1, 3);

        _spinBoxPointSize = new QDoubleSpinBox(GroupBox9_2_2_2);
        _spinBoxPointSize->setObjectName("_spinBoxPointSize");

        gridLayout->addWidget(_spinBoxPointSize, 4, 6, 1, 1);

        _comboPointDensity = new QComboBox(GroupBox9_2_2_2);
        _comboPointDensity->addItem(QString());
        _comboPointDensity->addItem(QString());
        _comboPointDensity->addItem(QString());
        _comboPointDensity->addItem(QString());
        _comboPointDensity->setObjectName("_comboPointDensity");
        _comboPointDensity->setEnabled(false);

        gridLayout->addWidget(_comboPointDensity, 1, 6, 1, 1);

        _headColor = new Kst::ColorButton(GroupBox9_2_2_2);
        _headColor->setObjectName("_headColor");
        sizePolicy1.setHeightForWidth(_headColor->sizePolicy().hasHeightForWidth());
        _headColor->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_headColor, 7, 5, 1, 1);

        _spinBoxLineWidth = new QSpinBox(GroupBox9_2_2_2);
        _spinBoxLineWidth->setObjectName("_spinBoxLineWidth");
        _spinBoxLineWidth->setMaximum(50);

        gridLayout->addWidget(_spinBoxLineWidth, 2, 6, 1, 1);


        gridLayout_2->addWidget(GroupBox9_2_2_2, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        _textLabelWeight->setBuddy(_spinBoxLineWidth);
        _textLabelPointSize->setBuddy(_spinBoxPointSize);
        _textLabelPointDensity->setBuddy(_comboPointDensity);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_showLines, _comboLineStyle);
        QWidget::setTabOrder(_comboLineStyle, _spinBoxLineWidth);
        QWidget::setTabOrder(_spinBoxLineWidth, _color);
        QWidget::setTabOrder(_color, _showPoints);
        QWidget::setTabOrder(_showPoints, _comboPointSymbol);
        QWidget::setTabOrder(_comboPointSymbol, _comboPointDensity);
        QWidget::setTabOrder(_comboPointDensity, _showHead);
        QWidget::setTabOrder(_showHead, _comboHeadSymbol);
        QWidget::setTabOrder(_comboHeadSymbol, _headColor);
        QWidget::setTabOrder(_headColor, _showBars);
        QWidget::setTabOrder(_showBars, _barFillColor);

        retranslateUi(CurveAppearance);

        QMetaObject::connectSlotsByName(CurveAppearance);
    } // setupUi

    void retranslateUi(QWidget *CurveAppearance)
    {
        GroupBox9_2_2_2->setTitle(QCoreApplication::translate("CurveAppearance", "Appearance", nullptr));
        _textLabelWeight->setText(QCoreApplication::translate("CurveAppearance", "Wei&ght:", nullptr));
#if QT_CONFIG(tooltip)
        _comboPointSymbol->setToolTip(QCoreApplication::translate("CurveAppearance", "Point Type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _comboPointSymbol->setWhatsThis(QCoreApplication::translate("CurveAppearance", "If \"Show Points\" is checked, the selected point type will be drawn at each data point.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        _comboLineStyle->setToolTip(QCoreApplication::translate("CurveAppearance", "Line Type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _comboLineStyle->setWhatsThis(QCoreApplication::translate("CurveAppearance", "If \"Show Lines\" is checked, the selected line style.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        _showLines->setWhatsThis(QCoreApplication::translate("CurveAppearance", "Check this to cause lines to be drawn between data points.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _showLines->setText(QCoreApplication::translate("CurveAppearance", "&Lines", nullptr));
#if QT_CONFIG(whatsthis)
        _showPoints->setWhatsThis(QCoreApplication::translate("CurveAppearance", "Check this to cause the selected points to be drawn at each data point.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _showPoints->setText(QCoreApplication::translate("CurveAppearance", "Po&ints", nullptr));
        _showHead->setText(QCoreApplication::translate("CurveAppearance", "&Head", nullptr));
#if QT_CONFIG(whatsthis)
        _showBars->setWhatsThis(QCoreApplication::translate("CurveAppearance", "Check this to cause lines to be drawn between data points.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _showBars->setText(QCoreApplication::translate("CurveAppearance", "&Bargraph", nullptr));
        _textLabelPointSize->setText(QCoreApplication::translate("CurveAppearance", "Si&ze:", nullptr));
        _textLabelPointDensity->setText(QCoreApplication::translate("CurveAppearance", "Poi&nt Density:", nullptr));
        _comboPointDensity->setItemText(0, QCoreApplication::translate("CurveAppearance", "All", "show every point"));
        _comboPointDensity->setItemText(1, QCoreApplication::translate("CurveAppearance", "High", "high density of points"));
        _comboPointDensity->setItemText(2, QCoreApplication::translate("CurveAppearance", "Medium", "medium density of points"));
        _comboPointDensity->setItemText(3, QCoreApplication::translate("CurveAppearance", "Low", "low density of points"));

#if QT_CONFIG(tooltip)
        _comboPointDensity->setToolTip(QCoreApplication::translate("CurveAppearance", "Point Type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _comboPointDensity->setWhatsThis(QCoreApplication::translate("CurveAppearance", "If \"Show Points\" is checked, the selected point type will be drawn at each data point.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        _spinBoxLineWidth->setToolTip(QCoreApplication::translate("CurveAppearance", "Line Width", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _spinBoxLineWidth->setWhatsThis(QCoreApplication::translate("CurveAppearance", "If \"Show Lines\" is checked, the selected line width.", nullptr));
#endif // QT_CONFIG(whatsthis)
        (void)CurveAppearance;
    } // retranslateUi

};

namespace Ui {
    class CurveAppearance: public Ui_CurveAppearance {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CURVEAPPEARANCE_H
