/********************************************************************************
** Form generated from reading UI file 'linedimensionstab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LINEDIMENSIONSTAB_H
#define UI_LINEDIMENSIONSTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LineDimensionsTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    QDoubleSpinBox *_p2Y;
    QLabel *label_3;
    QDoubleSpinBox *_p1X;
    QDoubleSpinBox *_p2X;
    QCheckBox *_lockPosToData;
    QLabel *_dimXlabel;
    QLabel *label_2;
    QDoubleSpinBox *_p1Y;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *LineDimensionsTab)
    {
        if (LineDimensionsTab->objectName().isEmpty())
            LineDimensionsTab->setObjectName("LineDimensionsTab");
        LineDimensionsTab->resize(213, 116);
        verticalLayout = new QVBoxLayout(LineDimensionsTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(LineDimensionsTab);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        _p2Y = new QDoubleSpinBox(LineDimensionsTab);
        _p2Y->setObjectName("_p2Y");
        _p2Y->setDecimals(4);
        _p2Y->setMaximum(1.000000000000000);
        _p2Y->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_p2Y, 2, 3, 1, 1);

        label_3 = new QLabel(LineDimensionsTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        _p1X = new QDoubleSpinBox(LineDimensionsTab);
        _p1X->setObjectName("_p1X");
        _p1X->setDecimals(4);
        _p1X->setMaximum(1.000000000000000);
        _p1X->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_p1X, 1, 1, 1, 1);

        _p2X = new QDoubleSpinBox(LineDimensionsTab);
        _p2X->setObjectName("_p2X");
        _p2X->setDecimals(4);
        _p2X->setMaximum(1.000000000000000);
        _p2X->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_p2X, 2, 1, 1, 1);

        _lockPosToData = new QCheckBox(LineDimensionsTab);
        _lockPosToData->setObjectName("_lockPosToData");

        gridLayout->addWidget(_lockPosToData, 0, 1, 1, 3);

        _dimXlabel = new QLabel(LineDimensionsTab);
        _dimXlabel->setObjectName("_dimXlabel");
        sizePolicy.setHeightForWidth(_dimXlabel->sizePolicy().hasHeightForWidth());
        _dimXlabel->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_dimXlabel, 2, 2, 1, 1);

        label_2 = new QLabel(LineDimensionsTab);
        label_2->setObjectName("label_2");
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_2, 1, 2, 1, 1);

        _p1Y = new QDoubleSpinBox(LineDimensionsTab);
        _p1Y->setObjectName("_p1Y");
        _p1Y->setDecimals(4);
        _p1Y->setMaximum(1.000000000000000);
        _p1Y->setSingleStep(0.010000000000000);

        gridLayout->addWidget(_p1Y, 1, 3, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 13, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label->setBuddy(_p1X);
        label_3->setBuddy(_p2X);
        _dimXlabel->setBuddy(_p2Y);
        label_2->setBuddy(_p1Y);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_lockPosToData, _p1X);
        QWidget::setTabOrder(_p1X, _p1Y);
        QWidget::setTabOrder(_p1Y, _p2X);
        QWidget::setTabOrder(_p2X, _p2Y);

        retranslateUi(LineDimensionsTab);
        QObject::connect(_lockPosToData, SIGNAL(stateChanged(int)), LineDimensionsTab, SLOT(modified()));
        QObject::connect(_p1X, SIGNAL(valueChanged(double)), LineDimensionsTab, SLOT(modified()));
        QObject::connect(_p2X, SIGNAL(valueChanged(double)), LineDimensionsTab, SLOT(modified()));
        QObject::connect(_p1Y, SIGNAL(valueChanged(double)), LineDimensionsTab, SLOT(modified()));
        QObject::connect(_p2Y, SIGNAL(valueChanged(double)), LineDimensionsTab, SLOT(modified()));

        QMetaObject::connectSlotsByName(LineDimensionsTab);
    } // setupUi

    void retranslateUi(QWidget *LineDimensionsTab)
    {
        label->setText(QCoreApplication::translate("LineDimensionsTab", "P&1", nullptr));
        label_3->setText(QCoreApplication::translate("LineDimensionsTab", "P&2", nullptr));
        _lockPosToData->setText(QCoreApplication::translate("LineDimensionsTab", "Lock Position To Data", nullptr));
        _dimXlabel->setText(QCoreApplication::translate("LineDimensionsTab", " , ", nullptr));
        label_2->setText(QCoreApplication::translate("LineDimensionsTab", " , ", nullptr));
        (void)LineDimensionsTab;
    } // retranslateUi

};

namespace Ui {
    class LineDimensionsTab: public Ui_LineDimensionsTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LINEDIMENSIONSTAB_H
