/********************************************************************************
** Form generated from reading UI file 'layouttab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAYOUTTAB_H
#define UI_LAYOUTTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LayoutTab
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QDoubleSpinBox *_horMargin;
    QLabel *label_4;
    QDoubleSpinBox *_verMargin;
    QLabel *label_5;
    QDoubleSpinBox *_horSpacing;
    QLabel *label_6;
    QDoubleSpinBox *_verSpacing;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *LayoutTab)
    {
        if (LayoutTab->objectName().isEmpty())
            LayoutTab->setObjectName("LayoutTab");
        LayoutTab->resize(419, 327);
        vboxLayout = new QVBoxLayout(LayoutTab);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(LayoutTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _horMargin = new QDoubleSpinBox(LayoutTab);
        _horMargin->setObjectName("_horMargin");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_horMargin->sizePolicy().hasHeightForWidth());
        _horMargin->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_horMargin, 0, 1, 1, 1);

        label_4 = new QLabel(LayoutTab);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        _verMargin = new QDoubleSpinBox(LayoutTab);
        _verMargin->setObjectName("_verMargin");
        sizePolicy.setHeightForWidth(_verMargin->sizePolicy().hasHeightForWidth());
        _verMargin->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_verMargin, 1, 1, 1, 1);

        label_5 = new QLabel(LayoutTab);
        label_5->setObjectName("label_5");
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 2, 0, 1, 1);

        _horSpacing = new QDoubleSpinBox(LayoutTab);
        _horSpacing->setObjectName("_horSpacing");
        sizePolicy.setHeightForWidth(_horSpacing->sizePolicy().hasHeightForWidth());
        _horSpacing->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_horSpacing, 2, 1, 1, 1);

        label_6 = new QLabel(LayoutTab);
        label_6->setObjectName("label_6");
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_6, 3, 0, 1, 1);

        _verSpacing = new QDoubleSpinBox(LayoutTab);
        _verSpacing->setObjectName("_verSpacing");
        sizePolicy.setHeightForWidth(_verSpacing->sizePolicy().hasHeightForWidth());
        _verSpacing->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_verSpacing, 3, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        spacerItem = new QSpacerItem(401, 121, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(_horMargin);
        label_4->setBuddy(_verMargin);
        label_5->setBuddy(_horSpacing);
        label_6->setBuddy(_verSpacing);
#endif // QT_CONFIG(shortcut)

        retranslateUi(LayoutTab);

        QMetaObject::connectSlotsByName(LayoutTab);
    } // setupUi

    void retranslateUi(QWidget *LayoutTab)
    {
        label_3->setText(QCoreApplication::translate("LayoutTab", "Horizontal &margin:", nullptr));
        label_4->setText(QCoreApplication::translate("LayoutTab", "&Vertical margin:", nullptr));
        label_5->setText(QCoreApplication::translate("LayoutTab", "Hori&zontal spacing:", nullptr));
        label_6->setText(QCoreApplication::translate("LayoutTab", "Verti&cal spacing:", nullptr));
        (void)LayoutTab;
    } // retranslateUi

};

namespace Ui {
    class LayoutTab: public Ui_LayoutTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LAYOUTTAB_H
