/********************************************************************************
** Form generated from reading UI file 'shiftconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHIFTCONFIG_H
#define UI_SHIFTCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_ShiftConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vector;
    QLabel *label_4;
    Kst::ScalarSelector *_scalarShift;

    void setupUi(QWidget *ShiftConfig)
    {
        if (ShiftConfig->objectName().isEmpty())
            ShiftConfig->setObjectName("ShiftConfig");
        ShiftConfig->resize(490, 62);
        ShiftConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(ShiftConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(ShiftConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _vector = new Kst::VectorSelector(ShiftConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vector, 0, 1, 1, 1);

        label_4 = new QLabel(ShiftConfig);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        _scalarShift = new Kst::ScalarSelector(ShiftConfig);
        _scalarShift->setObjectName("_scalarShift");

        gridLayout->addWidget(_scalarShift, 1, 1, 1, 1);


        retranslateUi(ShiftConfig);

        QMetaObject::connectSlotsByName(ShiftConfig);
    } // setupUi

    void retranslateUi(QWidget *ShiftConfig)
    {
        label_3->setText(QCoreApplication::translate("ShiftConfig", "Input Vector", nullptr));
        label_4->setText(QCoreApplication::translate("ShiftConfig", "Shift value (# points, negative allowed)", nullptr));
        (void)ShiftConfig;
    } // retranslateUi

};

namespace Ui {
    class ShiftConfig: public Ui_ShiftConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHIFTCONFIG_H
