/********************************************************************************
** Form generated from reading UI file 'debugdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DEBUGDIALOG_H
#define UI_DEBUGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DebugDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *_tabs;
    QWidget *_buildInfoTab;
    QGridLayout *gridLayout;
    QLabel *_buildInfo;
    QSpacerItem *spacerItem;
    QPushButton *_email;
    QSpacerItem *spacerItem1;
    QWidget *_logTab;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *_logTabLayout;
    QHBoxLayout *_logTabButtonsLayout;
    QLabel *label;
    QCheckBox *_showError;
    QCheckBox *_showWarning;
    QCheckBox *_showNotice;
    QCheckBox *_showTrace;
    QSpacerItem *spacerItem2;
    QPushButton *_clear;
    QWidget *_dataSourcesTab;
    QGridLayout *gridLayout1;
    QTreeWidget *_dataSources;
    QHBoxLayout *horizontalLayout;
    QPushButton *_clearDSSettings;
    QSpacerItem *spacerItem3;
    QPushButton *_close;

    void setupUi(QDialog *DebugDialog)
    {
        if (DebugDialog->objectName().isEmpty())
            DebugDialog->setObjectName("DebugDialog");
        DebugDialog->resize(666, 422);
        verticalLayout = new QVBoxLayout(DebugDialog);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        _tabs = new QTabWidget(DebugDialog);
        _tabs->setObjectName("_tabs");
        _buildInfoTab = new QWidget();
        _buildInfoTab->setObjectName("_buildInfoTab");
        gridLayout = new QGridLayout(_buildInfoTab);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        _buildInfo = new QLabel(_buildInfoTab);
        _buildInfo->setObjectName("_buildInfo");
        _buildInfo->setTextFormat(Qt::RichText);
        _buildInfo->setWordWrap(false);

        gridLayout->addWidget(_buildInfo, 0, 0, 1, 2);

        spacerItem = new QSpacerItem(530, 250, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 1, 0, 1, 2);

        _email = new QPushButton(_buildInfoTab);
        _email->setObjectName("_email");

        gridLayout->addWidget(_email, 2, 1, 1, 1);

        spacerItem1 = new QSpacerItem(301, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 2, 0, 1, 1);

        _tabs->addTab(_buildInfoTab, QString());
        _logTab = new QWidget();
        _logTab->setObjectName("_logTab");
        verticalLayout_2 = new QVBoxLayout(_logTab);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        _logTabLayout = new QGridLayout();
        _logTabLayout->setSpacing(6);
        _logTabLayout->setObjectName("_logTabLayout");

        verticalLayout_2->addLayout(_logTabLayout);

        _logTabButtonsLayout = new QHBoxLayout();
        _logTabButtonsLayout->setSpacing(6);
        _logTabButtonsLayout->setObjectName("_logTabButtonsLayout");
        label = new QLabel(_logTab);
        label->setObjectName("label");

        _logTabButtonsLayout->addWidget(label);

        _showError = new QCheckBox(_logTab);
        _showError->setObjectName("_showError");
        _showError->setChecked(true);

        _logTabButtonsLayout->addWidget(_showError);

        _showWarning = new QCheckBox(_logTab);
        _showWarning->setObjectName("_showWarning");
        _showWarning->setChecked(true);

        _logTabButtonsLayout->addWidget(_showWarning);

        _showNotice = new QCheckBox(_logTab);
        _showNotice->setObjectName("_showNotice");
        _showNotice->setChecked(true);

        _logTabButtonsLayout->addWidget(_showNotice);

        _showTrace = new QCheckBox(_logTab);
        _showTrace->setObjectName("_showTrace");
        _showTrace->setEnabled(true);
        _showTrace->setChecked(false);

        _logTabButtonsLayout->addWidget(_showTrace);

        spacerItem2 = new QSpacerItem(221, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _logTabButtonsLayout->addItem(spacerItem2);

        _clear = new QPushButton(_logTab);
        _clear->setObjectName("_clear");

        _logTabButtonsLayout->addWidget(_clear);


        verticalLayout_2->addLayout(_logTabButtonsLayout);

        _tabs->addTab(_logTab, QString());
        _dataSourcesTab = new QWidget();
        _dataSourcesTab->setObjectName("_dataSourcesTab");
        gridLayout1 = new QGridLayout(_dataSourcesTab);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        _dataSources = new QTreeWidget(_dataSourcesTab);
        _dataSources->setObjectName("_dataSources");
        _dataSources->setAllColumnsShowFocus(true);

        gridLayout1->addWidget(_dataSources, 0, 0, 1, 1);

        _tabs->addTab(_dataSourcesTab, QString());

        verticalLayout->addWidget(_tabs);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        _clearDSSettings = new QPushButton(DebugDialog);
        _clearDSSettings->setObjectName("_clearDSSettings");

        horizontalLayout->addWidget(_clearDSSettings);

        spacerItem3 = new QSpacerItem(421, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem3);

        _close = new QPushButton(DebugDialog);
        _close->setObjectName("_close");

        horizontalLayout->addWidget(_close);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(_email, _close);

        retranslateUi(DebugDialog);
        QObject::connect(_close, &QPushButton::clicked, DebugDialog, qOverload<>(&QDialog::close));

        _tabs->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(DebugDialog);
    } // setupUi

    void retranslateUi(QDialog *DebugDialog)
    {
        DebugDialog->setWindowTitle(QCoreApplication::translate("DebugDialog", "Kst Debug Dialog", nullptr));
        _buildInfo->setText(QString());
        _email->setText(QCoreApplication::translate("DebugDialog", "&Email Debugging Information", nullptr));
        _tabs->setTabText(_tabs->indexOf(_buildInfoTab), QCoreApplication::translate("DebugDialog", "Build Information", nullptr));
        label->setText(QCoreApplication::translate("DebugDialog", "Log level: ", nullptr));
        _showError->setText(QCoreApplication::translate("DebugDialog", "&Error", nullptr));
        _showWarning->setText(QCoreApplication::translate("DebugDialog", "&Warning", nullptr));
        _showNotice->setText(QCoreApplication::translate("DebugDialog", "&Notice", nullptr));
        _showTrace->setText(QCoreApplication::translate("DebugDialog", "&Trace", nullptr));
        _clear->setText(QCoreApplication::translate("DebugDialog", "Clea&r", nullptr));
        _tabs->setTabText(_tabs->indexOf(_logTab), QCoreApplication::translate("DebugDialog", "Log", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = _dataSources->headerItem();
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("DebugDialog", "Loaded Instance", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("DebugDialog", "Plugin", nullptr));
        _tabs->setTabText(_tabs->indexOf(_dataSourcesTab), QCoreApplication::translate("DebugDialog", "Data Sources", nullptr));
        _clearDSSettings->setText(QCoreApplication::translate("DebugDialog", "Clear Datasource Settings", nullptr));
        _close->setText(QCoreApplication::translate("DebugDialog", "&Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DebugDialog: public Ui_DebugDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DEBUGDIALOG_H
