/********************************************************************************
** Form generated from reading UI file 'exponentialconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPONENTIALCONFIG_H
#define UI_EXPONENTIALCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_ExponentialConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    Kst::VectorSelector *_vector;
    QLabel *label_2;
    Kst::ScalarSelector *_exp;

    void setupUi(QWidget *ExponentialConfig)
    {
        if (ExponentialConfig->objectName().isEmpty())
            ExponentialConfig->setObjectName("ExponentialConfig");
        ExponentialConfig->resize(428, 62);
        gridLayout = new QGridLayout(ExponentialConfig);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(ExponentialConfig);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        _vector = new Kst::VectorSelector(ExponentialConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_vector, 0, 1, 1, 1);

        label_2 = new QLabel(ExponentialConfig);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        _exp = new Kst::ScalarSelector(ExponentialConfig);
        _exp->setObjectName("_exp");

        gridLayout->addWidget(_exp, 1, 1, 1, 1);


        retranslateUi(ExponentialConfig);

        QMetaObject::connectSlotsByName(ExponentialConfig);
    } // setupUi

    void retranslateUi(QWidget *ExponentialConfig)
    {
        ExponentialConfig->setWindowTitle(QCoreApplication::translate("ExponentialConfig", "Form", nullptr));
        label->setText(QCoreApplication::translate("ExponentialConfig", "Y:", nullptr));
        label_2->setText(QCoreApplication::translate("ExponentialConfig", "Exp:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExponentialConfig: public Ui_ExponentialConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPONENTIALCONFIG_H
