/****************************************************************************
** Meta object code from reading C++ file 'view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstapp/view.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst4ViewE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::View::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst4ViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::View",
        "viewModeChanged",
        "",
        "View::ViewMode",
        "oldMode",
        "mouseModeChanged",
        "View::MouseMode",
        "creationPolygonChanged",
        "View::CreationEvent",
        "event",
        "createLayout",
        "preserve",
        "columns",
        "createUnprotectedLayout",
        "appendToLayout",
        "CurvePlacement::Layout",
        "layout",
        "ViewItem*",
        "item",
        "createCustomLayout",
        "viewChanged",
        "forceChildResize",
        "QRectF",
        "oldRect",
        "newRect",
        "processResize",
        "QSize",
        "size",
        "setZoomOnly",
        "ZoomOnlyMode",
        "updateSettings",
        "loadSettings",
        "edit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'viewModeChanged'
        QtMocHelpers::SignalData<void(View::ViewMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'mouseModeChanged'
        QtMocHelpers::SignalData<void(View::MouseMode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Signal 'creationPolygonChanged'
        QtMocHelpers::SignalData<void(View::CreationEvent)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'createLayout'
        QtMocHelpers::SlotData<void(bool, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'createLayout'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'createLayout'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'createUnprotectedLayout'
        QtMocHelpers::SlotData<void(bool, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'createUnprotectedLayout'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'createUnprotectedLayout'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'appendToLayout'
        QtMocHelpers::SlotData<void(CurvePlacement::Layout, ViewItem *, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'appendToLayout'
        QtMocHelpers::SlotData<void(CurvePlacement::Layout, ViewItem *)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Slot 'createCustomLayout'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'forceChildResize'
        QtMocHelpers::SlotData<void(QRectF, QRectF)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { 0x80000000 | 22, 24 },
        }}),
        // Slot 'processResize'
        QtMocHelpers::SlotData<void(QSize)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'setZoomOnly'
        QtMocHelpers::SlotData<void(enum ZoomOnlyMode)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 2 },
        }}),
        // Slot 'updateSettings'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadSettings'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'edit'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<View, qt_meta_tag_ZN3Kst4ViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::View::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst4ViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst4ViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst4ViewE_t>.metaTypes,
    nullptr
} };

void Kst::View::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<View *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->viewModeChanged((*reinterpret_cast<std::add_pointer_t<View::ViewMode>>(_a[1]))); break;
        case 1: _t->mouseModeChanged((*reinterpret_cast<std::add_pointer_t<View::MouseMode>>(_a[1]))); break;
        case 2: _t->creationPolygonChanged((*reinterpret_cast<std::add_pointer_t<View::CreationEvent>>(_a[1]))); break;
        case 3: _t->createLayout((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->createLayout((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->createLayout(); break;
        case 6: _t->createUnprotectedLayout((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->createUnprotectedLayout((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->createUnprotectedLayout(); break;
        case 9: _t->appendToLayout((*reinterpret_cast<std::add_pointer_t<CurvePlacement::Layout>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ViewItem*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->appendToLayout((*reinterpret_cast<std::add_pointer_t<CurvePlacement::Layout>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ViewItem*>>(_a[2]))); break;
        case 11: _t->createCustomLayout(); break;
        case 12: _t->viewChanged(); break;
        case 13: _t->forceChildResize((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[2]))); break;
        case 14: _t->processResize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 15: _t->setZoomOnly((*reinterpret_cast<std::add_pointer_t<enum ZoomOnlyMode>>(_a[1]))); break;
        case 16: _t->updateSettings(); break;
        case 17: _t->loadSettings(); break;
        case 18: _t->edit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (View::*)(View::ViewMode )>(_a, &View::viewModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(View::MouseMode )>(_a, &View::mouseModeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (View::*)(View::CreationEvent )>(_a, &View::creationPolygonChanged, 2))
            return;
    }
}

const QMetaObject *Kst::View::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::View::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst4ViewE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int Kst::View::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void Kst::View::viewModeChanged(View::ViewMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Kst::View::mouseModeChanged(View::MouseMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Kst::View::creationPolygonChanged(View::CreationEvent _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
