/********************************************************************************
** Form generated from reading UI file 'gridtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRIDTAB_H
#define UI_GRIDTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_GridTab
{
public:
    QGridLayout *gridLayout;
    QCheckBox *_showGrid;
    QCheckBox *_snapToGrid;
    QLabel *label_5;
    QDoubleSpinBox *_gridHorSpacing;
    QLabel *label_6;
    QDoubleSpinBox *_gridVerSpacing;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *GridTab)
    {
        if (GridTab->objectName().isEmpty())
            GridTab->setObjectName("GridTab");
        GridTab->resize(243, 131);
        gridLayout = new QGridLayout(GridTab);
        gridLayout->setObjectName("gridLayout");
        _showGrid = new QCheckBox(GridTab);
        _showGrid->setObjectName("_showGrid");

        gridLayout->addWidget(_showGrid, 0, 0, 1, 1);

        _snapToGrid = new QCheckBox(GridTab);
        _snapToGrid->setObjectName("_snapToGrid");

        gridLayout->addWidget(_snapToGrid, 0, 1, 1, 2);

        label_5 = new QLabel(GridTab);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 1, 0, 1, 2);

        _gridHorSpacing = new QDoubleSpinBox(GridTab);
        _gridHorSpacing->setObjectName("_gridHorSpacing");

        gridLayout->addWidget(_gridHorSpacing, 1, 2, 1, 1);

        label_6 = new QLabel(GridTab);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 2, 0, 1, 2);

        _gridVerSpacing = new QDoubleSpinBox(GridTab);
        _gridVerSpacing->setObjectName("_gridVerSpacing");

        gridLayout->addWidget(_gridVerSpacing, 2, 2, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 3, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_5->setBuddy(_gridHorSpacing);
        label_6->setBuddy(_gridVerSpacing);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GridTab);

        QMetaObject::connectSlotsByName(GridTab);
    } // setupUi

    void retranslateUi(QWidget *GridTab)
    {
        _showGrid->setText(QCoreApplication::translate("GridTab", "Show &grid", nullptr));
        _snapToGrid->setText(QCoreApplication::translate("GridTab", "&Snap to grid", nullptr));
        label_5->setText(QCoreApplication::translate("GridTab", "Grid &horizontal spacing:", nullptr));
        label_6->setText(QCoreApplication::translate("GridTab", "Grid &vertical spacing:", nullptr));
        (void)GridTab;
    } // retranslateUi

};

namespace Ui {
    class GridTab: public Ui_GridTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GRIDTAB_H
