/********************************************************************************
** Form generated from reading UI file 'equationtab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EQUATIONTAB_H
#define UI_EQUATIONTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "cclineedit.h"
#include "curveappearance.h"
#include "curveplacement.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_EquationTab
{
public:
    QGridLayout *gridLayout;
    Kst::CurveAppearance *_curveAppearance;
    QGroupBox *GroupBox14;
    QGridLayout *gridLayout_2;
    QLabel *TextLabel1_11;
    Kst::SVCCLineEdit *_equation;
    QLabel *_xVectorLabel;
    Kst::VectorSelector *_xVectors;
    QCheckBox *_doInterpolation;
    QSpacerItem *verticalSpacer;
    Kst::CurvePlacement *_curvePlacement;

    void setupUi(QWidget *EquationTab)
    {
        if (EquationTab->objectName().isEmpty())
            EquationTab->setObjectName("EquationTab");
        EquationTab->resize(765, 360);
        gridLayout = new QGridLayout(EquationTab);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _curveAppearance = new Kst::CurveAppearance(EquationTab);
        _curveAppearance->setObjectName("_curveAppearance");

        gridLayout->addWidget(_curveAppearance, 1, 0, 1, 1);

        GroupBox14 = new QGroupBox(EquationTab);
        GroupBox14->setObjectName("GroupBox14");
        gridLayout_2 = new QGridLayout(GroupBox14);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        TextLabel1_11 = new QLabel(GroupBox14);
        TextLabel1_11->setObjectName("TextLabel1_11");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TextLabel1_11->sizePolicy().hasHeightForWidth());
        TextLabel1_11->setSizePolicy(sizePolicy);
        TextLabel1_11->setWordWrap(false);

        gridLayout_2->addWidget(TextLabel1_11, 0, 0, 1, 1);

        _equation = new Kst::SVCCLineEdit(GroupBox14);
        _equation->setObjectName("_equation");
        _equation->setMinimumSize(QSize(550, 0));

        gridLayout_2->addWidget(_equation, 0, 1, 1, 3);

        _xVectorLabel = new QLabel(GroupBox14);
        _xVectorLabel->setObjectName("_xVectorLabel");
        _xVectorLabel->setWordWrap(false);

        gridLayout_2->addWidget(_xVectorLabel, 1, 0, 1, 1);

        _xVectors = new Kst::VectorSelector(GroupBox14);
        _xVectors->setObjectName("_xVectors");
        _xVectors->setEnabled(true);

        gridLayout_2->addWidget(_xVectors, 1, 1, 1, 3);

        _doInterpolation = new QCheckBox(GroupBox14);
        _doInterpolation->setObjectName("_doInterpolation");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_doInterpolation->sizePolicy().hasHeightForWidth());
        _doInterpolation->setSizePolicy(sizePolicy1);
        _doInterpolation->setChecked(true);

        gridLayout_2->addWidget(_doInterpolation, 2, 0, 1, 3);


        gridLayout->addWidget(GroupBox14, 0, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);

        _curvePlacement = new Kst::CurvePlacement(EquationTab);
        _curvePlacement->setObjectName("_curvePlacement");

        gridLayout->addWidget(_curvePlacement, 1, 1, 2, 1);

#if QT_CONFIG(shortcut)
        TextLabel1_11->setBuddy(_equation);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_equation, _xVectors);
        QWidget::setTabOrder(_xVectors, _doInterpolation);
        QWidget::setTabOrder(_doInterpolation, _curveAppearance);
        QWidget::setTabOrder(_curveAppearance, _curvePlacement);

        retranslateUi(EquationTab);

        QMetaObject::connectSlotsByName(EquationTab);
    } // setupUi

    void retranslateUi(QWidget *EquationTab)
    {
        GroupBox14->setTitle(QCoreApplication::translate("EquationTab", "Equation", nullptr));
        TextLabel1_11->setText(QCoreApplication::translate("EquationTab", "E&quation:", nullptr));
#if QT_CONFIG(whatsthis)
        _equation->setWhatsThis(QCoreApplication::translate("EquationTab", "Enter the equation to be evaluated here.  Use 'x' as the independent variable.  Vectors and operators are selected from the above lists.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _xVectorLabel->setText(QCoreApplication::translate("EquationTab", "&X vector:", nullptr));
        _doInterpolation->setText(QCoreApplication::translate("EquationTab", "Inte&rpolate to highest resolution vector", nullptr));
        (void)EquationTab;
    } // retranslateUi

};

namespace Ui {
    class EquationTab: public Ui_EquationTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EQUATIONTAB_H
