/****************************************************************************
** Meta object code from reading C++ file 'datawizard.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/libkstapp/datawizard.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'datawizard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DataWizardPageDataSource::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DataWizardPageDataSource",
        "dataSourceChanged",
        "",
        "progress",
        "sourceChanged",
        "configureSource",
        "sourceValid",
        "filename",
        "requestID",
        "updateTypeActivated",
        "recentFileClicked",
        "QListWidgetItem*",
        "cleanupRecentDataFilesClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataSourceChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(int, const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot 'sourceChanged'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'configureSource'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sourceValid'
        QtMocHelpers::SlotData<void(QString, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'updateTypeActivated'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'recentFileClicked'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Slot 'cleanupRecentDataFilesClicked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataWizardPageDataSource, qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DataWizardPageDataSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t>.metaTypes,
    nullptr
} };

void Kst::DataWizardPageDataSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataWizardPageDataSource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataSourceChanged(); break;
        case 1: _t->progress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->sourceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->configureSource(); break;
        case 4: _t->sourceValid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->updateTypeActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->recentFileClicked((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 7: _t->cleanupRecentDataFilesClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataWizardPageDataSource::*)()>(_a, &DataWizardPageDataSource::dataSourceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DataWizardPageDataSource::*)(int , const QString & )>(_a, &DataWizardPageDataSource::progress, 1))
            return;
    }
}

const QMetaObject *Kst::DataWizardPageDataSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DataWizardPageDataSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst24DataWizardPageDataSourceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DataWizardPageDataSource"))
        return static_cast< Ui::DataWizardPageDataSource*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int Kst::DataWizardPageDataSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Kst::DataWizardPageDataSource::dataSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Kst::DataWizardPageDataSource::progress(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DataWizardPageVectors::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DataWizardPageVectors",
        "add",
        "",
        "remove",
        "up",
        "down",
        "filterVectors",
        "searchVectors",
        "updateVectors"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'add'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'remove'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'up'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'down'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filterVectors'
        QtMocHelpers::SlotData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'searchVectors'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateVectors'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataWizardPageVectors, qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DataWizardPageVectors::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t>.metaTypes,
    nullptr
} };

void Kst::DataWizardPageVectors::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataWizardPageVectors *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->add(); break;
        case 1: _t->remove(); break;
        case 2: _t->up(); break;
        case 3: _t->down(); break;
        case 4: _t->filterVectors((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->searchVectors(); break;
        case 6: _t->updateVectors(); break;
        default: ;
        }
    }
}

const QMetaObject *Kst::DataWizardPageVectors::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DataWizardPageVectors::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst21DataWizardPageVectorsE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DataWizardPageVectors"))
        return static_cast< Ui::DataWizardPageVectors*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int Kst::DataWizardPageVectors::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DataWizardPageFilters::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DataWizardPageFilters"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataWizardPageFilters, qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DataWizardPageFilters::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t>.metaTypes,
    nullptr
} };

void Kst::DataWizardPageFilters::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataWizardPageFilters *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Kst::DataWizardPageFilters::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DataWizardPageFilters::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst21DataWizardPageFiltersE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DataWizardPageFilters"))
        return static_cast< Ui::DataWizardPageFilters*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int Kst::DataWizardPageFilters::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DataWizardPagePlot::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DataWizardPagePlot",
        "updatePlotBox",
        "",
        "updateButtons"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'updatePlotBox'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateButtons'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataWizardPagePlot, qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DataWizardPagePlot::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t>.metaTypes,
    nullptr
} };

void Kst::DataWizardPagePlot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataWizardPagePlot *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updatePlotBox(); break;
        case 1: _t->updateButtons(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *Kst::DataWizardPagePlot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DataWizardPagePlot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst18DataWizardPagePlotE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DataWizardPagePlot"))
        return static_cast< Ui::DataWizardPagePlot*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int Kst::DataWizardPagePlot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DataWizardPageDataPresentation::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DataWizardPageDataPresentation",
        "filterApplied",
        "",
        "applyFilter",
        "updateVectors",
        "optionsUpdated",
        "checkWarningLabel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'filterApplied'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'applyFilter'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateVectors'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'optionsUpdated'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkWarningLabel'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataWizardPageDataPresentation, qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DataWizardPageDataPresentation::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizardPage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t>.metaTypes,
    nullptr
} };

void Kst::DataWizardPageDataPresentation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataWizardPageDataPresentation *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->filterApplied((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->applyFilter((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->updateVectors(); break;
        case 3: _t->optionsUpdated(); break;
        case 4: _t->checkWarningLabel(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataWizardPageDataPresentation::*)(bool )>(_a, &DataWizardPageDataPresentation::filterApplied, 0))
            return;
    }
}

const QMetaObject *Kst::DataWizardPageDataPresentation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DataWizardPageDataPresentation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst30DataWizardPageDataPresentationE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DataWizardPageDataPresentation"))
        return static_cast< Ui::DataWizardPageDataPresentation*>(this);
    return QWizardPage::qt_metacast(_clname);
}

int Kst::DataWizardPageDataPresentation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizardPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Kst::DataWizardPageDataPresentation::filterApplied(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3Kst10DataWizardE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::DataWizard::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst10DataWizardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::DataWizard",
        "dataSourceLoaded",
        "",
        "finished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataSourceLoaded'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'finished'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DataWizard, qt_meta_tag_ZN3Kst10DataWizardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::DataWizard::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizard::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10DataWizardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10DataWizardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst10DataWizardE_t>.metaTypes,
    nullptr
} };

void Kst::DataWizard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DataWizard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataSourceLoaded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->finished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DataWizard::*)(const QString & )>(_a, &DataWizard::dataSourceLoaded, 0))
            return;
    }
}

const QMetaObject *Kst::DataWizard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::DataWizard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10DataWizardE_t>.strings))
        return static_cast<void*>(this);
    return QWizard::qt_metacast(_clname);
}

int Kst::DataWizard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizard::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Kst::DataWizard::dataSourceLoaded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
