/********************************************************************************
** Form generated from reading UI file 'periodogramconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PERIODOGRAMCONFIG_H
#define UI_PERIODOGRAMCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_PeriodogramConfig
{
public:
    QGridLayout *gridLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vectorTime;
    QLabel *label_5;
    Kst::VectorSelector *_vectorData;
    QLabel *label_4;
    Kst::ScalarSelector *_scalarOversampling;
    QLabel *label_6;
    Kst::ScalarSelector *_scalarANFF;

    void setupUi(QWidget *PeriodogramConfig)
    {
        if (PeriodogramConfig->objectName().isEmpty())
            PeriodogramConfig->setObjectName("PeriodogramConfig");
        PeriodogramConfig->resize(424, 76);
        PeriodogramConfig->setMinimumSize(QSize(400, 0));
        gridLayout = new QGridLayout(PeriodogramConfig);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(PeriodogramConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_3->setWordWrap(false);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        _vectorTime = new Kst::VectorSelector(PeriodogramConfig);
        _vectorTime->setObjectName("_vectorTime");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorTime->sizePolicy().hasHeightForWidth());
        _vectorTime->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorTime, 0, 1, 1, 1);

        label_5 = new QLabel(PeriodogramConfig);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_5->setWordWrap(false);

        gridLayout->addWidget(label_5, 1, 0, 1, 1);

        _vectorData = new Kst::VectorSelector(PeriodogramConfig);
        _vectorData->setObjectName("_vectorData");
        sizePolicy1.setHeightForWidth(_vectorData->sizePolicy().hasHeightForWidth());
        _vectorData->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_vectorData, 1, 1, 1, 1);

        label_4 = new QLabel(PeriodogramConfig);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        gridLayout->addWidget(label_4, 2, 0, 1, 1);

        _scalarOversampling = new Kst::ScalarSelector(PeriodogramConfig);
        _scalarOversampling->setObjectName("_scalarOversampling");

        gridLayout->addWidget(_scalarOversampling, 2, 1, 1, 1);

        label_6 = new QLabel(PeriodogramConfig);
        label_6->setObjectName("label_6");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy2);
        label_6->setWordWrap(false);

        gridLayout->addWidget(label_6, 3, 0, 1, 1);

        _scalarANFF = new Kst::ScalarSelector(PeriodogramConfig);
        _scalarANFF->setObjectName("_scalarANFF");

        gridLayout->addWidget(_scalarANFF, 3, 1, 1, 1);


        retranslateUi(PeriodogramConfig);

        QMetaObject::connectSlotsByName(PeriodogramConfig);
    } // setupUi

    void retranslateUi(QWidget *PeriodogramConfig)
    {
        label_3->setText(QCoreApplication::translate("PeriodogramConfig", "Input Vector Time", nullptr));
        label_5->setText(QCoreApplication::translate("PeriodogramConfig", "Input Vector Data", nullptr));
        label_4->setText(QCoreApplication::translate("PeriodogramConfig", "Oversampling factor", nullptr));
        label_6->setText(QCoreApplication::translate("PeriodogramConfig", "Average Nyquist frequency factor", nullptr));
        (void)PeriodogramConfig;
    } // retranslateUi

};

namespace Ui {
    class PeriodogramConfig: public Ui_PeriodogramConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PERIODOGRAMCONFIG_H
