/********************************************************************************
** Form generated from reading UI file 'axistab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_AXISTAB_H
#define UI_AXISTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_AxisTab
{
public:
    QGridLayout *gridLayout_5;
    QGridLayout *gridLayout_4;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *_hideTopRight;
    QCheckBox *_hideBottomLeft;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    QSpinBox *_rotation;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_5;
    QCheckBox *_scaleAutoBaseOffset;
    QCheckBox *_scaleBaseOffset;
    QCheckBox *_baseOffsetOff;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_6;
    QLabel *_significantDigitsLabel;
    QSpinBox *_significantDigits;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_7;
    QCheckBox *_forceOffsetMin;
    QGroupBox *_scaleGroup;
    QGridLayout *gridLayout_3;
    QCheckBox *_scaleLog;
    QCheckBox *_scaleReverse;
    QSpacerItem *horizontalSpacer_8;
    QCheckBox *_scaleInterpret;
    QComboBox *_scaleInterpretType;
    QSpacerItem *horizontalSpacer_10;
    QLabel *_Label_6;
    QComboBox *_scaleDisplayType;
    QLabel *label_7;
    QComboBox *_timeZone;
    QSpacerItem *horizontalSpacer_5;
    QLabel *_timeFormatLabel;
    QLineEdit *_scaleDisplayFormatString;
    QLabel *_timeFormatHelp;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_4;
    QGroupBox *_axisMajorGridGroup;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout;
    QCheckBox *_drawAxisMajorTicks;
    QCheckBox *_drawAxisMajorGridLines;
    QLabel *label;
    QComboBox *_axisMajorTickSpacing;
    QLabel *_Label_3;
    QComboBox *_axisMajorLineStyle;
    QLabel *label_10;
    Kst::ColorButton *_axisMajorLineColor;
    QLabel *label_5;
    QDoubleSpinBox *_axisMajorLineWidth;
    QGroupBox *_axisMinorGridGroup;
    QGridLayout *gridLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *_drawAxisMinorTicks;
    QCheckBox *_drawAxisMinorGridLines;
    QLabel *label_3;
    QSpinBox *_axisMinorTickCount;
    QCheckBox *_autoMinorTicks;
    QLabel *_axisMinorLineStyleLabel;
    QComboBox *_axisMinorLineStyle;
    QLabel *label_12;
    Kst::ColorButton *_axisMinorLineColor;
    QLabel *label_6;
    QDoubleSpinBox *_axisMinorLineWidth;

    void setupUi(QWidget *AxisTab)
    {
        if (AxisTab->objectName().isEmpty())
            AxisTab->setObjectName("AxisTab");
        AxisTab->resize(650, 431);
        gridLayout_5 = new QGridLayout(AxisTab);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        groupBox_3 = new QGroupBox(AxisTab);
        groupBox_3->setObjectName("groupBox_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox_3->sizePolicy().hasHeightForWidth());
        groupBox_3->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        _hideTopRight = new QCheckBox(groupBox_3);
        _hideTopRight->setObjectName("_hideTopRight");

        verticalLayout_2->addWidget(_hideTopRight);

        _hideBottomLeft = new QCheckBox(groupBox_3);
        _hideBottomLeft->setObjectName("_hideBottomLeft");

        verticalLayout_2->addWidget(_hideBottomLeft);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        gridLayout_4->addWidget(groupBox_3, 0, 1, 1, 1);

        groupBox_2 = new QGroupBox(AxisTab);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");

        horizontalLayout_3->addWidget(label_4);

        _rotation = new QSpinBox(groupBox_2);
        _rotation->setObjectName("_rotation");
        _rotation->setMinimum(-90);
        _rotation->setMaximum(90);

        horizontalLayout_3->addWidget(_rotation);


        verticalLayout->addLayout(horizontalLayout_3);

        verticalSpacer_2 = new QSpacerItem(20, 52, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);


        gridLayout_4->addWidget(groupBox_2, 1, 1, 1, 1);

        groupBox = new QGroupBox(AxisTab);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        _scaleAutoBaseOffset = new QCheckBox(groupBox);
        _scaleAutoBaseOffset->setObjectName("_scaleAutoBaseOffset");

        horizontalLayout_5->addWidget(_scaleAutoBaseOffset);

        _scaleBaseOffset = new QCheckBox(groupBox);
        _scaleBaseOffset->setObjectName("_scaleBaseOffset");

        horizontalLayout_5->addWidget(_scaleBaseOffset);

        _baseOffsetOff = new QCheckBox(groupBox);
        _baseOffsetOff->setObjectName("_baseOffsetOff");

        horizontalLayout_5->addWidget(_baseOffsetOff);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer);


        verticalLayout_4->addLayout(horizontalLayout_5);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        _significantDigitsLabel = new QLabel(groupBox);
        _significantDigitsLabel->setObjectName("_significantDigitsLabel");
        _significantDigitsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_6->addWidget(_significantDigitsLabel);

        _significantDigits = new QSpinBox(groupBox);
        _significantDigits->setObjectName("_significantDigits");
        _significantDigits->setMinimum(2);
        _significantDigits->setMaximum(15);
        _significantDigits->setValue(9);

        horizontalLayout_6->addWidget(_significantDigits);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);


        verticalLayout_4->addLayout(horizontalLayout_6);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        _forceOffsetMin = new QCheckBox(groupBox);
        _forceOffsetMin->setObjectName("_forceOffsetMin");

        horizontalLayout_7->addWidget(_forceOffsetMin);


        verticalLayout_4->addLayout(horizontalLayout_7);


        gridLayout_4->addWidget(groupBox, 1, 0, 1, 1);

        _scaleGroup = new QGroupBox(AxisTab);
        _scaleGroup->setObjectName("_scaleGroup");
        gridLayout_3 = new QGridLayout(_scaleGroup);
        gridLayout_3->setObjectName("gridLayout_3");
        _scaleLog = new QCheckBox(_scaleGroup);
        _scaleLog->setObjectName("_scaleLog");

        gridLayout_3->addWidget(_scaleLog, 0, 0, 1, 2);

        _scaleReverse = new QCheckBox(_scaleGroup);
        _scaleReverse->setObjectName("_scaleReverse");

        gridLayout_3->addWidget(_scaleReverse, 0, 2, 1, 1);

        horizontalSpacer_8 = new QSpacerItem(0, 25, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_8, 0, 5, 1, 1);

        _scaleInterpret = new QCheckBox(_scaleGroup);
        _scaleInterpret->setObjectName("_scaleInterpret");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_scaleInterpret->sizePolicy().hasHeightForWidth());
        _scaleInterpret->setSizePolicy(sizePolicy1);

        gridLayout_3->addWidget(_scaleInterpret, 1, 0, 1, 2);

        _scaleInterpretType = new QComboBox(_scaleGroup);
        _scaleInterpretType->setObjectName("_scaleInterpretType");
        _scaleInterpretType->setEnabled(false);

        gridLayout_3->addWidget(_scaleInterpretType, 1, 2, 1, 1);

        horizontalSpacer_10 = new QSpacerItem(0, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_10, 1, 5, 1, 1);

        _Label_6 = new QLabel(_scaleGroup);
        _Label_6->setObjectName("_Label_6");
        _Label_6->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_Label_6->sizePolicy().hasHeightForWidth());
        _Label_6->setSizePolicy(sizePolicy2);
        _Label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(_Label_6, 2, 1, 1, 1);

        _scaleDisplayType = new QComboBox(_scaleGroup);
        _scaleDisplayType->setObjectName("_scaleDisplayType");
        _scaleDisplayType->setEnabled(false);

        gridLayout_3->addWidget(_scaleDisplayType, 2, 2, 1, 1);

        label_7 = new QLabel(_scaleGroup);
        label_7->setObjectName("label_7");
        label_7->setEnabled(false);
        sizePolicy.setHeightForWidth(label_7->sizePolicy().hasHeightForWidth());
        label_7->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_7, 2, 3, 1, 1);

        _timeZone = new QComboBox(_scaleGroup);
        _timeZone->setObjectName("_timeZone");
        _timeZone->setEnabled(false);

        gridLayout_3->addWidget(_timeZone, 2, 4, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(0, 18, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_5, 2, 5, 1, 1);

        _timeFormatLabel = new QLabel(_scaleGroup);
        _timeFormatLabel->setObjectName("_timeFormatLabel");
        _timeFormatLabel->setEnabled(true);
        sizePolicy2.setHeightForWidth(_timeFormatLabel->sizePolicy().hasHeightForWidth());
        _timeFormatLabel->setSizePolicy(sizePolicy2);
        _timeFormatLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(_timeFormatLabel, 3, 0, 1, 2);

        _scaleDisplayFormatString = new QLineEdit(_scaleGroup);
        _scaleDisplayFormatString->setObjectName("_scaleDisplayFormatString");
        _scaleDisplayFormatString->setMinimumSize(QSize(150, 0));

        gridLayout_3->addWidget(_scaleDisplayFormatString, 3, 2, 1, 1);

        _timeFormatHelp = new QLabel(_scaleGroup);
        _timeFormatHelp->setObjectName("_timeFormatHelp");

        gridLayout_3->addWidget(_timeFormatHelp, 3, 3, 1, 2);

        horizontalSpacer_6 = new QSpacerItem(0, 17, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_6, 3, 5, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_3, 4, 0, 1, 1);


        gridLayout_4->addWidget(_scaleGroup, 0, 0, 1, 1);


        gridLayout_5->addLayout(gridLayout_4, 0, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        _axisMajorGridGroup = new QGroupBox(AxisTab);
        _axisMajorGridGroup->setObjectName("_axisMajorGridGroup");
        gridLayout = new QGridLayout(_axisMajorGridGroup);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        _drawAxisMajorTicks = new QCheckBox(_axisMajorGridGroup);
        _drawAxisMajorTicks->setObjectName("_drawAxisMajorTicks");
        sizePolicy1.setHeightForWidth(_drawAxisMajorTicks->sizePolicy().hasHeightForWidth());
        _drawAxisMajorTicks->setSizePolicy(sizePolicy1);
        _drawAxisMajorTicks->setChecked(true);

        horizontalLayout->addWidget(_drawAxisMajorTicks);

        _drawAxisMajorGridLines = new QCheckBox(_axisMajorGridGroup);
        _drawAxisMajorGridLines->setObjectName("_drawAxisMajorGridLines");
        sizePolicy1.setHeightForWidth(_drawAxisMajorGridLines->sizePolicy().hasHeightForWidth());
        _drawAxisMajorGridLines->setSizePolicy(sizePolicy1);
        _drawAxisMajorGridLines->setChecked(true);

        horizontalLayout->addWidget(_drawAxisMajorGridLines);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 4);

        label = new QLabel(_axisMajorGridGroup);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        _axisMajorTickSpacing = new QComboBox(_axisMajorGridGroup);
        _axisMajorTickSpacing->setObjectName("_axisMajorTickSpacing");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_axisMajorTickSpacing->sizePolicy().hasHeightForWidth());
        _axisMajorTickSpacing->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(_axisMajorTickSpacing, 1, 1, 1, 3);

        _Label_3 = new QLabel(_axisMajorGridGroup);
        _Label_3->setObjectName("_Label_3");
        sizePolicy.setHeightForWidth(_Label_3->sizePolicy().hasHeightForWidth());
        _Label_3->setSizePolicy(sizePolicy);
        _Label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_3, 2, 0, 1, 1);

        _axisMajorLineStyle = new QComboBox(_axisMajorGridGroup);
        _axisMajorLineStyle->setObjectName("_axisMajorLineStyle");
        sizePolicy3.setHeightForWidth(_axisMajorLineStyle->sizePolicy().hasHeightForWidth());
        _axisMajorLineStyle->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(_axisMajorLineStyle, 2, 1, 1, 3);

        label_10 = new QLabel(_axisMajorGridGroup);
        label_10->setObjectName("label_10");
        sizePolicy.setHeightForWidth(label_10->sizePolicy().hasHeightForWidth());
        label_10->setSizePolicy(sizePolicy);
        label_10->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_10, 3, 0, 1, 1);

        _axisMajorLineColor = new Kst::ColorButton(_axisMajorGridGroup);
        _axisMajorLineColor->setObjectName("_axisMajorLineColor");

        gridLayout->addWidget(_axisMajorLineColor, 3, 1, 1, 1);

        label_5 = new QLabel(_axisMajorGridGroup);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_5, 3, 2, 1, 1);

        _axisMajorLineWidth = new QDoubleSpinBox(_axisMajorGridGroup);
        _axisMajorLineWidth->setObjectName("_axisMajorLineWidth");

        gridLayout->addWidget(_axisMajorLineWidth, 3, 3, 1, 1);


        horizontalLayout_4->addWidget(_axisMajorGridGroup);

        _axisMinorGridGroup = new QGroupBox(AxisTab);
        _axisMinorGridGroup->setObjectName("_axisMinorGridGroup");
        gridLayout_2 = new QGridLayout(_axisMinorGridGroup);
        gridLayout_2->setObjectName("gridLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _drawAxisMinorTicks = new QCheckBox(_axisMinorGridGroup);
        _drawAxisMinorTicks->setObjectName("_drawAxisMinorTicks");
        sizePolicy1.setHeightForWidth(_drawAxisMinorTicks->sizePolicy().hasHeightForWidth());
        _drawAxisMinorTicks->setSizePolicy(sizePolicy1);
        _drawAxisMinorTicks->setChecked(true);

        horizontalLayout_2->addWidget(_drawAxisMinorTicks);

        _drawAxisMinorGridLines = new QCheckBox(_axisMinorGridGroup);
        _drawAxisMinorGridLines->setObjectName("_drawAxisMinorGridLines");
        sizePolicy1.setHeightForWidth(_drawAxisMinorGridLines->sizePolicy().hasHeightForWidth());
        _drawAxisMinorGridLines->setSizePolicy(sizePolicy1);
        _drawAxisMinorGridLines->setChecked(true);

        horizontalLayout_2->addWidget(_drawAxisMinorGridLines);


        gridLayout_2->addLayout(horizontalLayout_2, 0, 0, 1, 4);

        label_3 = new QLabel(_axisMinorGridGroup);
        label_3->setObjectName("label_3");
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_3, 1, 0, 1, 1);

        _axisMinorTickCount = new QSpinBox(_axisMinorGridGroup);
        _axisMinorTickCount->setObjectName("_axisMinorTickCount");
        sizePolicy3.setHeightForWidth(_axisMinorTickCount->sizePolicy().hasHeightForWidth());
        _axisMinorTickCount->setSizePolicy(sizePolicy3);
        _axisMinorTickCount->setMinimum(1);
        _axisMinorTickCount->setMaximum(10);
        _axisMinorTickCount->setValue(4);

        gridLayout_2->addWidget(_axisMinorTickCount, 1, 1, 1, 2);

        _autoMinorTicks = new QCheckBox(_axisMinorGridGroup);
        _autoMinorTicks->setObjectName("_autoMinorTicks");

        gridLayout_2->addWidget(_autoMinorTicks, 1, 3, 1, 1);

        _axisMinorLineStyleLabel = new QLabel(_axisMinorGridGroup);
        _axisMinorLineStyleLabel->setObjectName("_axisMinorLineStyleLabel");
        _axisMinorLineStyleLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_axisMinorLineStyleLabel, 2, 0, 1, 1);

        _axisMinorLineStyle = new QComboBox(_axisMinorGridGroup);
        _axisMinorLineStyle->setObjectName("_axisMinorLineStyle");
        sizePolicy3.setHeightForWidth(_axisMinorLineStyle->sizePolicy().hasHeightForWidth());
        _axisMinorLineStyle->setSizePolicy(sizePolicy3);

        gridLayout_2->addWidget(_axisMinorLineStyle, 2, 1, 1, 3);

        label_12 = new QLabel(_axisMinorGridGroup);
        label_12->setObjectName("label_12");
        label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_12, 3, 0, 1, 1);

        _axisMinorLineColor = new Kst::ColorButton(_axisMinorGridGroup);
        _axisMinorLineColor->setObjectName("_axisMinorLineColor");

        gridLayout_2->addWidget(_axisMinorLineColor, 3, 1, 1, 1);

        label_6 = new QLabel(_axisMinorGridGroup);
        label_6->setObjectName("label_6");
        sizePolicy.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(label_6, 3, 2, 1, 1);

        _axisMinorLineWidth = new QDoubleSpinBox(_axisMinorGridGroup);
        _axisMinorLineWidth->setObjectName("_axisMinorLineWidth");

        gridLayout_2->addWidget(_axisMinorLineWidth, 3, 3, 1, 1);


        horizontalLayout_4->addWidget(_axisMinorGridGroup);


        gridLayout_5->addLayout(horizontalLayout_4, 1, 0, 1, 1);

#if QT_CONFIG(shortcut)
        _significantDigitsLabel->setBuddy(_significantDigits);
        _Label_6->setBuddy(_scaleDisplayType);
        _timeFormatLabel->setBuddy(_scaleDisplayType);
        label->setBuddy(_axisMajorTickSpacing);
        _Label_3->setBuddy(_axisMajorLineStyle);
        label_10->setBuddy(_axisMajorLineColor);
        label_3->setBuddy(_axisMinorTickCount);
        _axisMinorLineStyleLabel->setBuddy(_axisMinorLineStyle);
        label_12->setBuddy(_axisMinorLineColor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(AxisTab);
        QObject::connect(_drawAxisMajorGridLines, &QCheckBox::toggled, _Label_3, &QLabel::setEnabled);
        QObject::connect(_drawAxisMajorGridLines, SIGNAL(toggled(bool)), _axisMajorLineColor, SLOT(setEnabled(bool)));
        QObject::connect(_drawAxisMajorGridLines, &QCheckBox::toggled, _axisMajorLineStyle, &QComboBox::setEnabled);
        QObject::connect(_drawAxisMinorGridLines, &QCheckBox::toggled, _axisMinorLineStyleLabel, &QLabel::setEnabled);
        QObject::connect(_drawAxisMinorGridLines, SIGNAL(toggled(bool)), _axisMinorLineColor, SLOT(setEnabled(bool)));
        QObject::connect(_drawAxisMinorGridLines, &QCheckBox::toggled, _axisMinorLineStyle, &QComboBox::setEnabled);
        QObject::connect(_scaleInterpret, &QCheckBox::toggled, _Label_6, &QLabel::setEnabled);
        QObject::connect(_drawAxisMajorGridLines, &QCheckBox::toggled, label_5, &QLabel::setEnabled);
        QObject::connect(_drawAxisMajorGridLines, &QCheckBox::toggled, _axisMajorLineWidth, &QDoubleSpinBox::setEnabled);
        QObject::connect(_drawAxisMinorGridLines, &QCheckBox::toggled, label_6, &QLabel::setEnabled);
        QObject::connect(_drawAxisMinorGridLines, &QCheckBox::toggled, _axisMinorLineWidth, &QDoubleSpinBox::setEnabled);
        QObject::connect(_scaleInterpret, &QCheckBox::toggled, label_7, &QLabel::setEnabled);
        QObject::connect(_scaleInterpret, &QCheckBox::toggled, _scaleInterpretType, &QComboBox::setEnabled);
        QObject::connect(_scaleInterpret, &QCheckBox::toggled, _scaleDisplayType, &QComboBox::setEnabled);
        QObject::connect(_scaleInterpret, &QCheckBox::toggled, _timeZone, &QComboBox::setEnabled);

        QMetaObject::connectSlotsByName(AxisTab);
    } // setupUi

    void retranslateUi(QWidget *AxisTab)
    {
        groupBox_3->setTitle(QCoreApplication::translate("AxisTab", "Hide Margins and Labels", nullptr));
        _hideTopRight->setText(QCoreApplication::translate("AxisTab", "Hide top", nullptr));
        _hideBottomLeft->setText(QCoreApplication::translate("AxisTab", "Hide bottom", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("AxisTab", "Label Rotation", nullptr));
        label_4->setText(QCoreApplication::translate("AxisTab", "Rotation :", nullptr));
        groupBox->setTitle(QCoreApplication::translate("AxisTab", "Base / Offset Mode", nullptr));
        _scaleAutoBaseOffset->setText(QCoreApplication::translate("AxisTab", "Auto", "Automatically enable or disable an option"));
        _scaleBaseOffset->setText(QCoreApplication::translate("AxisTab", "On", "Enable an option"));
        _baseOffsetOff->setText(QCoreApplication::translate("AxisTab", "Off", "Disable an option"));
        _significantDigitsLabel->setText(QCoreApplication::translate("AxisTab", "Digits before automatic base/offset mode:", nullptr));
#if QT_CONFIG(whatsthis)
        _significantDigits->setWhatsThis(QCoreApplication::translate("AxisTab", "If this many digits are required to display the tick numbers, then the axis will revert to Base/Offset mode.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _forceOffsetMin->setText(QCoreApplication::translate("AxisTab", "Force offset to axis minimum", nullptr));
        _scaleGroup->setTitle(QCoreApplication::translate("AxisTab", "Scale", nullptr));
#if QT_CONFIG(tooltip)
        _scaleLog->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        _scaleLog->setText(QCoreApplication::translate("AxisTab", "&Logarithmic", nullptr));
#if QT_CONFIG(whatsthis)
        _scaleReverse->setWhatsThis(QCoreApplication::translate("AxisTab", "Reverse the direction of the axis, so the numbers are decreasing, rather than increasing.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _scaleReverse->setText(QCoreApplication::translate("AxisTab", "&Reverse", nullptr));
#if QT_CONFIG(tooltip)
        _scaleInterpret->setToolTip(QCoreApplication::translate("AxisTab", "Axis is time", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _scaleInterpret->setWhatsThis(QCoreApplication::translate("AxisTab", "Interpret the axis as time.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _scaleInterpret->setText(QCoreApplication::translate("AxisTab", "&Interpret as:", nullptr));
#if QT_CONFIG(tooltip)
        _scaleInterpretType->setToolTip(QCoreApplication::translate("AxisTab", "Time encoding type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _scaleInterpretType->setWhatsThis(QCoreApplication::translate("AxisTab", "The axis represents time, whose encoding is selected here.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _Label_6->setText(QCoreApplication::translate("AxisTab", "&Display as:", nullptr));
#if QT_CONFIG(tooltip)
        _scaleDisplayType->setToolTip(QCoreApplication::translate("AxisTab", "Time display format", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _scaleDisplayType->setWhatsThis(QCoreApplication::translate("AxisTab", "If the axis is to be interpreted as time, then display the time on the axis with this format.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_7->setText(QCoreApplication::translate("AxisTab", "Timezone:", nullptr));
#if QT_CONFIG(tooltip)
        _timeZone->setToolTip(QCoreApplication::translate("AxisTab", "Time encoding type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _timeZone->setWhatsThis(QCoreApplication::translate("AxisTab", "The axis represents time, whose encoding is selected here.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _timeFormatLabel->setText(QCoreApplication::translate("AxisTab", "Time/Date format:", nullptr));
        _scaleDisplayFormatString->setText(QCoreApplication::translate("AxisTab", "hh:mm.ss.zzz", nullptr));
        _timeFormatHelp->setText(QCoreApplication::translate("AxisTab", "Date: y/M/d - Time: h/m/s/z", nullptr));
        _axisMajorGridGroup->setTitle(QCoreApplication::translate("AxisTab", "Major Ticks and Lines", nullptr));
        _drawAxisMajorTicks->setText(QCoreApplication::translate("AxisTab", "Dra&w ticks", nullptr));
        _drawAxisMajorGridLines->setText(QCoreApplication::translate("AxisTab", "Draw &grid lines", nullptr));
        label->setText(QCoreApplication::translate("AxisTab", "Major s&pacing:", nullptr));
#if QT_CONFIG(tooltip)
        _axisMajorTickSpacing->setToolTip(QCoreApplication::translate("AxisTab", "Density of major ticks", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _axisMajorTickSpacing->setWhatsThis(QCoreApplication::translate("AxisTab", "Specify the density of major ticks.  The major tick interval will automatically be chosen as a power of 10 times 1, 2, or 5.  This changes the target tick spacing within this constraint.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _Label_3->setText(QCoreApplication::translate("AxisTab", "Grid line &style:", nullptr));
        label_10->setText(QCoreApplication::translate("AxisTab", "Grid line &color:", nullptr));
        label_5->setText(QCoreApplication::translate("AxisTab", "Width:", nullptr));
        _axisMinorGridGroup->setTitle(QCoreApplication::translate("AxisTab", "Minor Ticks and Lines", nullptr));
        _drawAxisMinorTicks->setText(QCoreApplication::translate("AxisTab", "Dr&aw ticks", nullptr));
        _drawAxisMinorGridLines->setText(QCoreApplication::translate("AxisTab", "Draw grid lines", nullptr));
        label_3->setText(QCoreApplication::translate("AxisTab", "Minor tic&k count:", nullptr));
        _autoMinorTicks->setText(QCoreApplication::translate("AxisTab", "Auto", nullptr));
        _axisMinorLineStyleLabel->setText(QCoreApplication::translate("AxisTab", "Line st&yle:", nullptr));
        label_12->setText(QCoreApplication::translate("AxisTab", "Lin&e color:", nullptr));
        label_6->setText(QCoreApplication::translate("AxisTab", "Width:", nullptr));
        (void)AxisTab;
    } // retranslateUi

};

namespace Ui {
    class AxisTab: public Ui_AxisTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_AXISTAB_H
