/********************************************************************************
** Form generated from reading UI file 'datawizardpagefilters.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATAWIZARDPAGEFILTERS_H
#define UI_DATAWIZARDPAGEFILTERS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWizardPage>

QT_BEGIN_NAMESPACE

class Ui_DataWizardPageFilters
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QPushButton *_newFilter;
    QSpacerItem *spacer6;
    QListWidget *_filterList;
    QLabel *textLabel4;
    QSpacerItem *spacer7;

    void setupUi(QWizardPage *DataWizardPageFilters)
    {
        if (DataWizardPageFilters->objectName().isEmpty())
            DataWizardPageFilters->setObjectName("DataWizardPageFilters");
        DataWizardPageFilters->resize(593, 221);
        gridLayout = new QGridLayout(DataWizardPageFilters);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName("gridLayout1");
        _newFilter = new QPushButton(DataWizardPageFilters);
        _newFilter->setObjectName("_newFilter");

        gridLayout1->addWidget(_newFilter, 1, 1, 1, 1);

        spacer6 = new QSpacerItem(121, 21, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacer6, 1, 0, 1, 1);

        _filterList = new QListWidget(DataWizardPageFilters);
        _filterList->setObjectName("_filterList");

        gridLayout1->addWidget(_filterList, 0, 0, 1, 2);


        gridLayout->addLayout(gridLayout1, 0, 1, 2, 1);

        textLabel4 = new QLabel(DataWizardPageFilters);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout->addWidget(textLabel4, 0, 0, 1, 1);

        spacer7 = new QSpacerItem(51, 151, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer7, 1, 0, 1, 1);


        retranslateUi(DataWizardPageFilters);

        QMetaObject::connectSlotsByName(DataWizardPageFilters);
    } // setupUi

    void retranslateUi(QWizardPage *DataWizardPageFilters)
    {
        DataWizardPageFilters->setTitle(QCoreApplication::translate("DataWizardPageFilters", "Filters", nullptr));
        _newFilter->setText(QCoreApplication::translate("DataWizardPageFilters", "&New Filter...", nullptr));
        textLabel4->setText(QCoreApplication::translate("DataWizardPageFilters", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">If you wish, select a filter to apply to the fields that are read in:</p></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DataWizardPageFilters: public Ui_DataWizardPageFilters {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATAWIZARDPAGEFILTERS_H
