/********************************************************************************
** Form generated from reading UI file 'aboutdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label;
    QLabel *_toplabel;
    QSpacerItem *spacerItem;
    QTextBrowser *text;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName("AboutDialog");
        AboutDialog->resize(386, 283);
        gridLayout = new QGridLayout(AboutDialog);
        gridLayout->setObjectName("gridLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        label = new QLabel(AboutDialog);
        label->setObjectName("label");
        label->setPixmap(QPixmap(QString::fromUtf8(":/kst.png")));

        hboxLayout->addWidget(label);

        _toplabel = new QLabel(AboutDialog);
        _toplabel->setObjectName("_toplabel");
        QFont font;
        font.setPointSize(26);
        _toplabel->setFont(font);

        hboxLayout->addWidget(_toplabel);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 1, 1, 1);

        text = new QTextBrowser(AboutDialog);
        text->setObjectName("text");
        text->setOpenExternalLinks(true);
        text->setOpenLinks(false);

        gridLayout->addWidget(text, 1, 0, 1, 2);

        buttonBox = new QDialogButtonBox(AboutDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        gridLayout->addWidget(buttonBox, 2, 0, 1, 2);


        retranslateUi(AboutDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::clicked, AboutDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About Kst", "kst is the name of the app."));
        label->setText(QString());
        _toplabel->setText(QCoreApplication::translate("AboutDialog", "Kst", "the name of the app, but no other meaning."));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
