/********************************************************************************
** Form generated from reading UI file 'matrixselector.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MATRIXSELECTOR_H
#define UI_MATRIXSELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "combobox.h"

QT_BEGIN_NAMESPACE

class Ui_MatrixSelector
{
public:
    QHBoxLayout *hboxLayout;
    Kst::ComboBox *_matrix;
    QToolButton *_newMatrix;
    QToolButton *_editMatrix;

    void setupUi(QWidget *MatrixSelector)
    {
        if (MatrixSelector->objectName().isEmpty())
            MatrixSelector->setObjectName("MatrixSelector");
        MatrixSelector->resize(271, 38);
        hboxLayout = new QHBoxLayout(MatrixSelector);
        hboxLayout->setSpacing(0);
        hboxLayout->setContentsMargins(11, 11, 11, 11);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        _matrix = new Kst::ComboBox(MatrixSelector);
        _matrix->setObjectName("_matrix");
        _matrix->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        hboxLayout->addWidget(_matrix);

        _newMatrix = new QToolButton(MatrixSelector);
        _newMatrix->setObjectName("_newMatrix");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_newMatrix->sizePolicy().hasHeightForWidth());
        _newMatrix->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(_newMatrix);

        _editMatrix = new QToolButton(MatrixSelector);
        _editMatrix->setObjectName("_editMatrix");
        _editMatrix->setEnabled(false);
        sizePolicy.setHeightForWidth(_editMatrix->sizePolicy().hasHeightForWidth());
        _editMatrix->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(_editMatrix);


        retranslateUi(MatrixSelector);

        QMetaObject::connectSlotsByName(MatrixSelector);
    } // setupUi

    void retranslateUi(QWidget *MatrixSelector)
    {
#if QT_CONFIG(whatsthis)
        MatrixSelector->setWhatsThis(QCoreApplication::translate("MatrixSelector", "Select a vector to utilize.  If you wish to create a new vector, click on the icon to the right.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        _newMatrix->setToolTip(QCoreApplication::translate("MatrixSelector", "Create a new vector.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _newMatrix->setWhatsThis(QCoreApplication::translate("MatrixSelector", "Opens a vector dialog to create a new vector, which will become the selected vector.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _newMatrix->setText(QString());
#if QT_CONFIG(tooltip)
        _editMatrix->setToolTip(QCoreApplication::translate("MatrixSelector", "Edit selected vector.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        _editMatrix->setWhatsThis(QCoreApplication::translate("MatrixSelector", "Opens a vector dialog to edit selected vector.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _editMatrix->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class MatrixSelector: public Ui_MatrixSelector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MATRIXSELECTOR_H
