/********************************************************************************
** Form generated from reading UI file 'viewvectordialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VIEWVECTORDIALOG_H
#define UI_VIEWVECTORDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ViewVectorDialog
{
public:
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *_showHideLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *_showVectorList;
    QPushButton *_hideVectorList;
    QSpacerItem *verticalSpacer_2;
    QSplitter *_splitter;
    QWidget *layoutWidget;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *_vectorListLayout;
    QVBoxLayout *_addRemoveLayout;
    QWidget *layoutWidget_2;
    QVBoxLayout *_viewLayout;
    QTableView *_vectors;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *_resetButton;
    QPushButton *_closeButton;

    void setupUi(QDialog *ViewVectorDialog)
    {
        if (ViewVectorDialog->objectName().isEmpty())
            ViewVectorDialog->setObjectName("ViewVectorDialog");
        ViewVectorDialog->resize(517, 536);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ViewVectorDialog->sizePolicy().hasHeightForWidth());
        ViewVectorDialog->setSizePolicy(sizePolicy);
        horizontalLayout_3 = new QHBoxLayout(ViewVectorDialog);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        _showHideLayout = new QVBoxLayout();
        _showHideLayout->setObjectName("_showHideLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _showHideLayout->addItem(verticalSpacer);

        _showVectorList = new QPushButton(ViewVectorDialog);
        _showVectorList->setObjectName("_showVectorList");
        _showVectorList->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_showVectorList->sizePolicy().hasHeightForWidth());
        _showVectorList->setSizePolicy(sizePolicy1);
        _showVectorList->setText(QString::fromUtf8("+"));

        _showHideLayout->addWidget(_showVectorList);

        _hideVectorList = new QPushButton(ViewVectorDialog);
        _hideVectorList->setObjectName("_hideVectorList");
        sizePolicy1.setHeightForWidth(_hideVectorList->sizePolicy().hasHeightForWidth());
        _hideVectorList->setSizePolicy(sizePolicy1);
        _hideVectorList->setText(QString::fromUtf8("-"));

        _showHideLayout->addWidget(_hideVectorList);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        _showHideLayout->addItem(verticalSpacer_2);


        horizontalLayout_3->addLayout(_showHideLayout);

        _splitter = new QSplitter(ViewVectorDialog);
        _splitter->setObjectName("_splitter");
        _splitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(_splitter);
        layoutWidget->setObjectName("layoutWidget");
        horizontalLayout = new QHBoxLayout(layoutWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        _vectorListLayout = new QVBoxLayout();
        _vectorListLayout->setObjectName("_vectorListLayout");

        horizontalLayout->addLayout(_vectorListLayout);

        _addRemoveLayout = new QVBoxLayout();
        _addRemoveLayout->setObjectName("_addRemoveLayout");
        _addRemoveLayout->setContentsMargins(-1, -1, 3, -1);

        horizontalLayout->addLayout(_addRemoveLayout);

        _splitter->addWidget(layoutWidget);
        layoutWidget_2 = new QWidget(_splitter);
        layoutWidget_2->setObjectName("layoutWidget_2");
        _viewLayout = new QVBoxLayout(layoutWidget_2);
        _viewLayout->setObjectName("_viewLayout");
        _viewLayout->setContentsMargins(0, 0, 0, 0);
        _vectors = new QTableView(layoutWidget_2);
        _vectors->setObjectName("_vectors");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_vectors->sizePolicy().hasHeightForWidth());
        _vectors->setSizePolicy(sizePolicy2);
        _vectors->setMinimumSize(QSize(200, 200));

        _viewLayout->addWidget(_vectors);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        _resetButton = new QPushButton(layoutWidget_2);
        _resetButton->setObjectName("_resetButton");

        horizontalLayout_2->addWidget(_resetButton);

        _closeButton = new QPushButton(layoutWidget_2);
        _closeButton->setObjectName("_closeButton");

        horizontalLayout_2->addWidget(_closeButton);


        _viewLayout->addLayout(horizontalLayout_2);

        _splitter->addWidget(layoutWidget_2);

        horizontalLayout_3->addWidget(_splitter);


        retranslateUi(ViewVectorDialog);
        QObject::connect(_closeButton, &QPushButton::clicked, ViewVectorDialog, qOverload<>(&QDialog::close));

        QMetaObject::connectSlotsByName(ViewVectorDialog);
    } // setupUi

    void retranslateUi(QDialog *ViewVectorDialog)
    {
        ViewVectorDialog->setWindowTitle(QCoreApplication::translate("ViewVectorDialog", "View Vector Values", nullptr));
#if QT_CONFIG(tooltip)
        _showVectorList->setToolTip(QCoreApplication::translate("ViewVectorDialog", "Show vector list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        _hideVectorList->setToolTip(QCoreApplication::translate("ViewVectorDialog", "Hide vector list", nullptr));
#endif // QT_CONFIG(tooltip)
        _resetButton->setText(QCoreApplication::translate("ViewVectorDialog", "Reset View", nullptr));
        _closeButton->setText(QCoreApplication::translate("ViewVectorDialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ViewVectorDialog: public Ui_ViewVectorDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VIEWVECTORDIALOG_H
