/********************************************************************************
** Form generated from reading UI file 'genericfilterconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GENERICFILTERCONFIG_H
#define UI_GENERICFILTERCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "scalarselector.h"
#include "stringselector.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_GenericFilterConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    QSpacerItem *horizontalSpacer;
    Kst::VectorSelector *_vector;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer_2;
    Kst::ScalarSelector *_scalarInterval;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_5;
    QSpacerItem *horizontalSpacer_3;
    Kst::StringSelector *_stringNumerator;
    QHBoxLayout *horizontalLayout;
    QLabel *label_6;
    QSpacerItem *horizontalSpacer_4;
    Kst::StringSelector *_stringDenominator;

    void setupUi(QWidget *GenericFilterConfig)
    {
        if (GenericFilterConfig->objectName().isEmpty())
            GenericFilterConfig->setObjectName("GenericFilterConfig");
        GenericFilterConfig->resize(426, 170);
        GenericFilterConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(GenericFilterConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(GenericFilterConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout_4->addWidget(label_3);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        _vector = new Kst::VectorSelector(GenericFilterConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(_vector);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_4 = new QLabel(GenericFilterConfig);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setWordWrap(false);

        horizontalLayout_3->addWidget(label_4);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        _scalarInterval = new Kst::ScalarSelector(GenericFilterConfig);
        _scalarInterval->setObjectName("_scalarInterval");

        horizontalLayout_3->addWidget(_scalarInterval);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_5 = new QLabel(GenericFilterConfig);
        label_5->setObjectName("label_5");
        sizePolicy.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy);
        label_5->setWordWrap(false);

        horizontalLayout_2->addWidget(label_5);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        _stringNumerator = new Kst::StringSelector(GenericFilterConfig);
        _stringNumerator->setObjectName("_stringNumerator");

        horizontalLayout_2->addWidget(_stringNumerator);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_6 = new QLabel(GenericFilterConfig);
        label_6->setObjectName("label_6");
        sizePolicy.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy);
        label_6->setWordWrap(false);

        horizontalLayout->addWidget(label_6);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);

        _stringDenominator = new Kst::StringSelector(GenericFilterConfig);
        _stringDenominator->setObjectName("_stringDenominator");

        horizontalLayout->addWidget(_stringDenominator);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(GenericFilterConfig);

        QMetaObject::connectSlotsByName(GenericFilterConfig);
    } // setupUi

    void retranslateUi(QWidget *GenericFilterConfig)
    {
        label_3->setText(QCoreApplication::translate("GenericFilterConfig", "Input Vector", nullptr));
        label_4->setText(QCoreApplication::translate("GenericFilterConfig", "Sampling interval (s)", nullptr));
        label_5->setText(QCoreApplication::translate("GenericFilterConfig", "Numerator (increasing order)", nullptr));
        label_6->setText(QCoreApplication::translate("GenericFilterConfig", "Denominator (increasing order)", nullptr));
        (void)GenericFilterConfig;
    } // retranslateUi

};

namespace Ui {
    class GenericFilterConfig: public Ui_GenericFilterConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GENERICFILTERCONFIG_H
