/********************************************************************************
** Form generated from reading UI file 'convolveconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONVOLVECONFIG_H
#define UI_CONVOLVECONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_ConvolveConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vectorOne;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_4;
    Kst::VectorSelector *_vectorTwo;

    void setupUi(QWidget *ConvolveConfig)
    {
        if (ConvolveConfig->objectName().isEmpty())
            ConvolveConfig->setObjectName("ConvolveConfig");
        ConvolveConfig->resize(429, 70);
        ConvolveConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(ConvolveConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_3 = new QLabel(ConvolveConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout->addWidget(label_3);

        _vectorOne = new Kst::VectorSelector(ConvolveConfig);
        _vectorOne->setObjectName("_vectorOne");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vectorOne->sizePolicy().hasHeightForWidth());
        _vectorOne->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(_vectorOne);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_4 = new QLabel(ConvolveConfig);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setWordWrap(false);

        horizontalLayout_2->addWidget(label_4);

        _vectorTwo = new Kst::VectorSelector(ConvolveConfig);
        _vectorTwo->setObjectName("_vectorTwo");
        sizePolicy1.setHeightForWidth(_vectorTwo->sizePolicy().hasHeightForWidth());
        _vectorTwo->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(_vectorTwo);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(ConvolveConfig);

        QMetaObject::connectSlotsByName(ConvolveConfig);
    } // setupUi

    void retranslateUi(QWidget *ConvolveConfig)
    {
        label_3->setText(QCoreApplication::translate("ConvolveConfig", "Input Vector One", nullptr));
        label_4->setText(QCoreApplication::translate("ConvolveConfig", "Input Vector Two", nullptr));
        (void)ConvolveConfig;
    } // retranslateUi

};

namespace Ui {
    class ConvolveConfig: public Ui_ConvolveConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONVOLVECONFIG_H
