/********************************************************************************
** Form generated from reading UI file 'datawizardpagedatapresentation.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATAWIZARDPAGEDATAPRESENTATION_H
#define UI_DATAWIZARDPAGEDATAPRESENTATION_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QWizardPage>
#include "combobox.h"
#include "datarange.h"
#include "fftoptions.h"
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_DataWizardPageDataPresentation
{
public:
    QGridLayout *gridLayout_2;
    Kst::FFTOptions *_FFTOptions;
    Kst::DataRange *_DataRange;
    QGroupBox *_xAxisGroup;
    QGridLayout *gridLayout;
    Kst::ComboBox *_xVector;
    QRadioButton *_xAxisUseExisting;
    QRadioButton *_xAxisCreateFromField;
    Kst::VectorSelector *_xVectorExisting;
    QLabel *_xAxisWarningLabel;
    QLabel *_XAxisWarningLabel2;

    void setupUi(QWizardPage *DataWizardPageDataPresentation)
    {
        if (DataWizardPageDataPresentation->objectName().isEmpty())
            DataWizardPageDataPresentation->setObjectName("DataWizardPageDataPresentation");
        DataWizardPageDataPresentation->resize(852, 355);
        gridLayout_2 = new QGridLayout(DataWizardPageDataPresentation);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        _FFTOptions = new Kst::FFTOptions(DataWizardPageDataPresentation);
        _FFTOptions->setObjectName("_FFTOptions");

        gridLayout_2->addWidget(_FFTOptions, 2, 0, 1, 2);

        _DataRange = new Kst::DataRange(DataWizardPageDataPresentation);
        _DataRange->setObjectName("_DataRange");

        gridLayout_2->addWidget(_DataRange, 0, 0, 1, 2);

        _xAxisGroup = new QGroupBox(DataWizardPageDataPresentation);
        _xAxisGroup->setObjectName("_xAxisGroup");
        _xAxisGroup->setCheckable(true);
        gridLayout = new QGridLayout(_xAxisGroup);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _xVector = new Kst::ComboBox(_xAxisGroup);
        _xVector->setObjectName("_xVector");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_xVector->sizePolicy().hasHeightForWidth());
        _xVector->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_xVector, 0, 1, 1, 1);

        _xAxisUseExisting = new QRadioButton(_xAxisGroup);
        _xAxisUseExisting->setObjectName("_xAxisUseExisting");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_xAxisUseExisting->sizePolicy().hasHeightForWidth());
        _xAxisUseExisting->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_xAxisUseExisting, 1, 0, 1, 1);

        _xAxisCreateFromField = new QRadioButton(_xAxisGroup);
        _xAxisCreateFromField->setObjectName("_xAxisCreateFromField");
        sizePolicy1.setHeightForWidth(_xAxisCreateFromField->sizePolicy().hasHeightForWidth());
        _xAxisCreateFromField->setSizePolicy(sizePolicy1);
        _xAxisCreateFromField->setChecked(true);

        gridLayout->addWidget(_xAxisCreateFromField, 0, 0, 1, 1);

        _xVectorExisting = new Kst::VectorSelector(_xAxisGroup);
        _xVectorExisting->setObjectName("_xVectorExisting");

        gridLayout->addWidget(_xVectorExisting, 1, 1, 1, 1);

        _xAxisWarningLabel = new QLabel(_xAxisGroup);
        _xAxisWarningLabel->setObjectName("_xAxisWarningLabel");
        QFont font;
        font.setBold(true);
        font.setItalic(true);
        _xAxisWarningLabel->setFont(font);

        gridLayout->addWidget(_xAxisWarningLabel, 2, 0, 1, 2);

        _XAxisWarningLabel2 = new QLabel(_xAxisGroup);
        _XAxisWarningLabel2->setObjectName("_XAxisWarningLabel2");
        _XAxisWarningLabel2->setFont(font);

        gridLayout->addWidget(_XAxisWarningLabel2, 3, 0, 1, 2);


        gridLayout_2->addWidget(_xAxisGroup, 1, 0, 1, 2);

        QWidget::setTabOrder(_xAxisGroup, _xAxisCreateFromField);
        QWidget::setTabOrder(_xAxisCreateFromField, _xVector);
        QWidget::setTabOrder(_xVector, _xAxisUseExisting);

        retranslateUi(DataWizardPageDataPresentation);

        QMetaObject::connectSlotsByName(DataWizardPageDataPresentation);
    } // setupUi

    void retranslateUi(QWizardPage *DataWizardPageDataPresentation)
    {
        _xAxisGroup->setTitle(QCoreApplication::translate("DataWizardPageDataPresentation", "Create &XY plots.  Set X-axis vector properties below:", nullptr));
#if QT_CONFIG(whatsthis)
        _xVector->setWhatsThis(QCoreApplication::translate("DataWizardPageDataPresentation", "The field to be used as the X-axis in XY curves.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _xAxisUseExisting->setText(QCoreApplication::translate("DataWizardPageDataPresentation", "Use existin&g:", nullptr));
        _xAxisCreateFromField->setText(QCoreApplication::translate("DataWizardPageDataPresentation", "Create fr&om field:", nullptr));
        _xAxisWarningLabel->setText(QString());
        _XAxisWarningLabel2->setText(QCoreApplication::translate("DataWizardPageDataPresentation", "Selecting 'Create from field' will create a new matching X vector.", nullptr));
        (void)DataWizardPageDataPresentation;
    } // retranslateUi

};

namespace Ui {
    class DataWizardPageDataPresentation: public Ui_DataWizardPageDataPresentation {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATAWIZARDPAGEDATAPRESENTATION_H
