/********************************************************************************
** Form generated from reading UI file 'datawizardpageplot.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATAWIZARDPAGEPLOT_H
#define UI_DATAWIZARDPAGEPLOT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWizardPage>
#include "combobox.h"

QT_BEGIN_NAMESPACE

class Ui_DataWizardPagePlot
{
public:
    QGridLayout *gridLayout_4;
    QGroupBox *_plotGroup;
    QGridLayout *gridLayout;
    QRadioButton *_onePlot;
    QRadioButton *_multiplePlots;
    QHBoxLayout *hboxLayout;
    QRadioButton *_cycleThrough;
    QSpinBox *_plotNumber;
    QRadioButton *_cycleExisting;
    QHBoxLayout *hboxLayout1;
    QRadioButton *_existingPlot;
    Kst::ComboBox *_existingPlotName;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *_currentTab;
    QRadioButton *_newTab;
    QRadioButton *_separateTabs;
    QGroupBox *_plotLayoutBox;
    QGridLayout *gridLayout1;
    QRadioButton *_autoLayout;
    QRadioButton *_customGrid;
    QHBoxLayout *hboxLayout2;
    QLabel *label_3;
    QSpinBox *_gridColumns;
    QRadioButton *_protectLayout;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_2;
    QCheckBox *_rescaleFonts;
    QCheckBox *_shareAxis;
    QGroupBox *buttonGroup1;
    QVBoxLayout *vboxLayout;
    QRadioButton *_drawLines;
    QRadioButton *_drawPoints;
    QRadioButton *_drawBoth;
    QGroupBox *_psdAxisGroup;
    QVBoxLayout *vboxLayout1;
    QCheckBox *_psdLogX;
    QCheckBox *_psdLogY;
    QSpacerItem *verticalSpacer;
    QGroupBox *buttonGroup7;
    QGridLayout *gridLayout_3;
    QRadioButton *_legendsOn;
    QRadioButton *_legendsOff;
    QRadioButton *_legendsAuto;
    QCheckBox *_legendsVertical;

    void setupUi(QWizardPage *DataWizardPagePlot)
    {
        if (DataWizardPagePlot->objectName().isEmpty())
            DataWizardPagePlot->setObjectName("DataWizardPagePlot");
        DataWizardPagePlot->resize(653, 376);
        gridLayout_4 = new QGridLayout(DataWizardPagePlot);
        gridLayout_4->setSpacing(6);
        gridLayout_4->setContentsMargins(11, 11, 11, 11);
        gridLayout_4->setObjectName("gridLayout_4");
        _plotGroup = new QGroupBox(DataWizardPagePlot);
        _plotGroup->setObjectName("_plotGroup");
        gridLayout = new QGridLayout(_plotGroup);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        _onePlot = new QRadioButton(_plotGroup);
        _onePlot->setObjectName("_onePlot");
        _onePlot->setChecked(false);

        gridLayout->addWidget(_onePlot, 0, 0, 1, 1);

        _multiplePlots = new QRadioButton(_plotGroup);
        _multiplePlots->setObjectName("_multiplePlots");
        _multiplePlots->setChecked(true);

        gridLayout->addWidget(_multiplePlots, 1, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        _cycleThrough = new QRadioButton(_plotGroup);
        _cycleThrough->setObjectName("_cycleThrough");

        hboxLayout->addWidget(_cycleThrough);

        _plotNumber = new QSpinBox(_plotGroup);
        _plotNumber->setObjectName("_plotNumber");
        _plotNumber->setEnabled(false);
        _plotNumber->setButtonSymbols(QAbstractSpinBox::PlusMinus);
        _plotNumber->setMinimum(2);
        _plotNumber->setMaximum(64);

        hboxLayout->addWidget(_plotNumber);


        gridLayout->addLayout(hboxLayout, 2, 0, 1, 1);

        _cycleExisting = new QRadioButton(_plotGroup);
        _cycleExisting->setObjectName("_cycleExisting");
        _cycleExisting->setChecked(false);

        gridLayout->addWidget(_cycleExisting, 3, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        _existingPlot = new QRadioButton(_plotGroup);
        _existingPlot->setObjectName("_existingPlot");

        hboxLayout1->addWidget(_existingPlot);

        _existingPlotName = new Kst::ComboBox(_plotGroup);
        _existingPlotName->setObjectName("_existingPlotName");
        _existingPlotName->setEnabled(false);

        hboxLayout1->addWidget(_existingPlotName);


        gridLayout->addLayout(hboxLayout1, 4, 0, 1, 1);


        gridLayout_4->addWidget(_plotGroup, 0, 0, 2, 1);

        groupBox = new QGroupBox(DataWizardPagePlot);
        groupBox->setObjectName("groupBox");
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        _currentTab = new QRadioButton(groupBox);
        _currentTab->setObjectName("_currentTab");
        _currentTab->setChecked(true);

        verticalLayout_3->addWidget(_currentTab);

        _newTab = new QRadioButton(groupBox);
        _newTab->setObjectName("_newTab");

        verticalLayout_3->addWidget(_newTab);

        _separateTabs = new QRadioButton(groupBox);
        _separateTabs->setObjectName("_separateTabs");

        verticalLayout_3->addWidget(_separateTabs);


        gridLayout_4->addWidget(groupBox, 0, 1, 1, 2);

        _plotLayoutBox = new QGroupBox(DataWizardPagePlot);
        _plotLayoutBox->setObjectName("_plotLayoutBox");
        gridLayout1 = new QGridLayout(_plotLayoutBox);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName("gridLayout1");
        _autoLayout = new QRadioButton(_plotLayoutBox);
        _autoLayout->setObjectName("_autoLayout");
        _autoLayout->setChecked(true);

        gridLayout1->addWidget(_autoLayout, 0, 0, 1, 2);

        _customGrid = new QRadioButton(_plotLayoutBox);
        _customGrid->setObjectName("_customGrid");

        gridLayout1->addWidget(_customGrid, 1, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        label_3 = new QLabel(_plotLayoutBox);
        label_3->setObjectName("label_3");

        hboxLayout2->addWidget(label_3);

        _gridColumns = new QSpinBox(_plotLayoutBox);
        _gridColumns->setObjectName("_gridColumns");
        _gridColumns->setEnabled(false);
        _gridColumns->setMinimum(1);
        _gridColumns->setMaximum(10);

        hboxLayout2->addWidget(_gridColumns);


        gridLayout1->addLayout(hboxLayout2, 1, 1, 1, 1);

        _protectLayout = new QRadioButton(_plotLayoutBox);
        _protectLayout->setObjectName("_protectLayout");

        gridLayout1->addWidget(_protectLayout, 2, 0, 1, 2);


        gridLayout_4->addWidget(_plotLayoutBox, 2, 0, 2, 1);

        groupBox_3 = new QGroupBox(DataWizardPagePlot);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout_2 = new QGridLayout(groupBox_3);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        _rescaleFonts = new QCheckBox(groupBox_3);
        _rescaleFonts->setObjectName("_rescaleFonts");
        _rescaleFonts->setChecked(true);

        gridLayout_2->addWidget(_rescaleFonts, 0, 0, 1, 1);

        _shareAxis = new QCheckBox(groupBox_3);
        _shareAxis->setObjectName("_shareAxis");

        gridLayout_2->addWidget(_shareAxis, 1, 0, 1, 1);


        gridLayout_4->addWidget(groupBox_3, 1, 1, 2, 1);

        buttonGroup1 = new QGroupBox(DataWizardPagePlot);
        buttonGroup1->setObjectName("buttonGroup1");
        vboxLayout = new QVBoxLayout(buttonGroup1);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        _drawLines = new QRadioButton(buttonGroup1);
        _drawLines->setObjectName("_drawLines");
        _drawLines->setChecked(true);

        vboxLayout->addWidget(_drawLines);

        _drawPoints = new QRadioButton(buttonGroup1);
        _drawPoints->setObjectName("_drawPoints");

        vboxLayout->addWidget(_drawPoints);

        _drawBoth = new QRadioButton(buttonGroup1);
        _drawBoth->setObjectName("_drawBoth");

        vboxLayout->addWidget(_drawBoth);


        gridLayout_4->addWidget(buttonGroup1, 3, 2, 1, 1);

        _psdAxisGroup = new QGroupBox(DataWizardPagePlot);
        _psdAxisGroup->setObjectName("_psdAxisGroup");
        vboxLayout1 = new QVBoxLayout(_psdAxisGroup);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        _psdLogX = new QCheckBox(_psdAxisGroup);
        _psdLogX->setObjectName("_psdLogX");

        vboxLayout1->addWidget(_psdLogX);

        _psdLogY = new QCheckBox(_psdAxisGroup);
        _psdLogY->setObjectName("_psdLogY");

        vboxLayout1->addWidget(_psdLogY);


        gridLayout_4->addWidget(_psdAxisGroup, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer, 4, 1, 1, 1);

        buttonGroup7 = new QGroupBox(DataWizardPagePlot);
        buttonGroup7->setObjectName("buttonGroup7");
        gridLayout_3 = new QGridLayout(buttonGroup7);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        _legendsOn = new QRadioButton(buttonGroup7);
        _legendsOn->setObjectName("_legendsOn");

        gridLayout_3->addWidget(_legendsOn, 0, 0, 1, 1);

        _legendsOff = new QRadioButton(buttonGroup7);
        _legendsOff->setObjectName("_legendsOff");

        gridLayout_3->addWidget(_legendsOff, 0, 1, 1, 1);

        _legendsAuto = new QRadioButton(buttonGroup7);
        _legendsAuto->setObjectName("_legendsAuto");
        _legendsAuto->setChecked(true);

        gridLayout_3->addWidget(_legendsAuto, 0, 2, 1, 1);

        _legendsVertical = new QCheckBox(buttonGroup7);
        _legendsVertical->setObjectName("_legendsVertical");

        gridLayout_3->addWidget(_legendsVertical, 1, 0, 1, 1);


        gridLayout_4->addWidget(buttonGroup7, 1, 2, 2, 1);

        QWidget::setTabOrder(_onePlot, _multiplePlots);
        QWidget::setTabOrder(_multiplePlots, _cycleThrough);
        QWidget::setTabOrder(_cycleThrough, _plotNumber);
        QWidget::setTabOrder(_plotNumber, _cycleExisting);
        QWidget::setTabOrder(_cycleExisting, _existingPlot);
        QWidget::setTabOrder(_existingPlot, _existingPlotName);
        QWidget::setTabOrder(_existingPlotName, _autoLayout);
        QWidget::setTabOrder(_autoLayout, _customGrid);
        QWidget::setTabOrder(_customGrid, _gridColumns);
        QWidget::setTabOrder(_gridColumns, _protectLayout);
        QWidget::setTabOrder(_protectLayout, _currentTab);
        QWidget::setTabOrder(_currentTab, _newTab);
        QWidget::setTabOrder(_newTab, _separateTabs);
        QWidget::setTabOrder(_separateTabs, _rescaleFonts);
        QWidget::setTabOrder(_rescaleFonts, _shareAxis);
        QWidget::setTabOrder(_shareAxis, _legendsOn);
        QWidget::setTabOrder(_legendsOn, _legendsVertical);
        QWidget::setTabOrder(_legendsVertical, _legendsOff);
        QWidget::setTabOrder(_legendsOff, _legendsAuto);
        QWidget::setTabOrder(_legendsAuto, _psdLogX);
        QWidget::setTabOrder(_psdLogX, _psdLogY);
        QWidget::setTabOrder(_psdLogY, _drawLines);
        QWidget::setTabOrder(_drawLines, _drawPoints);
        QWidget::setTabOrder(_drawPoints, _drawBoth);

        retranslateUi(DataWizardPagePlot);
        QObject::connect(_cycleThrough, &QRadioButton::toggled, _plotNumber, &QSpinBox::setEnabled);
        QObject::connect(_customGrid, &QRadioButton::toggled, _gridColumns, &QSpinBox::setEnabled);
        QObject::connect(_existingPlot, SIGNAL(toggled(bool)), _existingPlotName, SLOT(setEnabled(bool)));
        QObject::connect(_currentTab, &QRadioButton::toggled, _cycleExisting, &QRadioButton::setEnabled);
        QObject::connect(_currentTab, SIGNAL(toggled(bool)), _existingPlotName, SLOT(setEnabled(bool)));
        QObject::connect(_currentTab, &QRadioButton::toggled, _existingPlot, &QRadioButton::setEnabled);
        QObject::connect(_existingPlot, &QRadioButton::toggled, _plotLayoutBox, &QGroupBox::setDisabled);
        QObject::connect(_cycleExisting, &QRadioButton::toggled, _plotLayoutBox, &QGroupBox::setDisabled);
        QObject::connect(_existingPlot, &QRadioButton::toggled, groupBox_3, &QGroupBox::setDisabled);
        QObject::connect(_cycleExisting, &QRadioButton::toggled, groupBox_3, &QGroupBox::setDisabled);
        QObject::connect(_existingPlot, &QRadioButton::toggled, groupBox, &QGroupBox::setDisabled);
        QObject::connect(_existingPlot, &QRadioButton::toggled, _psdAxisGroup, &QGroupBox::setDisabled);
        QObject::connect(_cycleExisting, &QRadioButton::toggled, _psdAxisGroup, &QGroupBox::setDisabled);

        QMetaObject::connectSlotsByName(DataWizardPagePlot);
    } // setupUi

    void retranslateUi(QWizardPage *DataWizardPagePlot)
    {
        _plotGroup->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Curve Placement", nullptr));
        _onePlot->setText(QCoreApplication::translate("DataWizardPagePlot", "&All curves in one plot", nullptr));
        _multiplePlots->setText(QCoreApplication::translate("DataWizardPagePlot", "&1 new plot per curve", nullptr));
#if QT_CONFIG(shortcut)
        _multiplePlots->setShortcut(QCoreApplication::translate("DataWizardPagePlot", "Alt+1", nullptr));
#endif // QT_CONFIG(shortcut)
        _cycleThrough->setText(QCoreApplication::translate("DataWizardPagePlot", "Cycle t&hrough", nullptr));
        _plotNumber->setSuffix(QCoreApplication::translate("DataWizardPagePlot", " plots", nullptr));
        _cycleExisting->setText(QCoreApplication::translate("DataWizardPagePlot", "Cycle through exis&ting plots", nullptr));
        _existingPlot->setText(QCoreApplication::translate("DataWizardPagePlot", "All curv&es in plot", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Plot Placement (Tabs)", nullptr));
        _currentTab->setText(QCoreApplication::translate("DataWizardPagePlot", "All in &current tab", nullptr));
        _newTab->setText(QCoreApplication::translate("DataWizardPagePlot", "All in ne&w tab", nullptr));
        _separateTabs->setText(QCoreApplication::translate("DataWizardPagePlot", "XY and spectra in &separate new tabs", nullptr));
        _plotLayoutBox->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Plot Layout", nullptr));
        _autoLayout->setText(QCoreApplication::translate("DataWizardPagePlot", "Automatic &layout", nullptr));
        _customGrid->setText(QCoreApplication::translate("DataWizardPagePlot", "Custo&m grid", nullptr));
        label_3->setText(QCoreApplication::translate("DataWizardPagePlot", "Columns:", nullptr));
        _protectLayout->setText(QCoreApplication::translate("DataWizardPagePlot", "P&rotect existing layout", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Labels", nullptr));
        _rescaleFonts->setText(QCoreApplication::translate("DataWizardPagePlot", "Rescale all labels on tab", nullptr));
        _shareAxis->setText(QCoreApplication::translate("DataWizardPagePlot", "Share axis on tab", nullptr));
        buttonGroup1->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Cur&ve Style", nullptr));
        _drawLines->setText(QCoreApplication::translate("DataWizardPagePlot", "L&ines only", nullptr));
        _drawPoints->setText(QCoreApplication::translate("DataWizardPagePlot", "&Points only", nullptr));
        _drawBoth->setText(QCoreApplication::translate("DataWizardPagePlot", "Lines an&d points", nullptr));
        _psdAxisGroup->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Spectrum Axis Mode", nullptr));
        _psdLogX->setText(QCoreApplication::translate("DataWizardPagePlot", "Lo&g X-axis", nullptr));
        _psdLogY->setText(QCoreApplication::translate("DataWizardPagePlot", "&Log Y-axis", nullptr));
        buttonGroup7->setTitle(QCoreApplication::translate("DataWizardPagePlot", "Legends", nullptr));
        _legendsOn->setText(QCoreApplication::translate("DataWizardPagePlot", "O&n", nullptr));
        _legendsOff->setText(QCoreApplication::translate("DataWizardPagePlot", "&Off", nullptr));
        _legendsAuto->setText(QCoreApplication::translate("DataWizardPagePlot", "A&uto", nullptr));
        _legendsVertical->setText(QCoreApplication::translate("DataWizardPagePlot", "&Vertical", nullptr));
        (void)DataWizardPagePlot;
    } // retranslateUi

};

namespace Ui {
    class DataWizardPagePlot: public Ui_DataWizardPagePlot {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATAWIZARDPAGEPLOT_H
