/********************************************************************************
** Form generated from reading UI file 'vectortab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_VECTORTAB_H
#define UI_VECTORTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "combobox.h"
#include "datarange.h"
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_VectorTab
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *_generatedVectorGroup;
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QSpinBox *_numberOfSamples;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLineEdit *_from;
    QLabel *label_3;
    QLineEdit *_to;
    QGroupBox *_dataVectorGroup;
    QVBoxLayout *verticalLayout;
    QLabel *label_4;
    Kst::DataSourceSelector *_fileName;
    QLabel *label_5;
    Kst::ComboBox *_field;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *_connect;
    QPushButton *_configure;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_3;
    QLabel *_fileType_3;
    QComboBox *_updateBox;
    Kst::DataRange *_dataRange;

    void setupUi(QWidget *VectorTab)
    {
        if (VectorTab->objectName().isEmpty())
            VectorTab->setObjectName("VectorTab");
        VectorTab->resize(739, 546);
        verticalLayout_2 = new QVBoxLayout(VectorTab);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        _generatedVectorGroup = new QGroupBox(VectorTab);
        _generatedVectorGroup->setObjectName("_generatedVectorGroup");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_generatedVectorGroup->sizePolicy().hasHeightForWidth());
        _generatedVectorGroup->setSizePolicy(sizePolicy);
        _generatedVectorGroup->setCheckable(true);
        _generatedVectorGroup->setChecked(true);
        horizontalLayout_4 = new QHBoxLayout(_generatedVectorGroup);
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(_generatedVectorGroup);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);
        label_2->setWordWrap(false);

        horizontalLayout_2->addWidget(label_2);

        _numberOfSamples = new QSpinBox(_generatedVectorGroup);
        _numberOfSamples->setObjectName("_numberOfSamples");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_numberOfSamples->sizePolicy().hasHeightForWidth());
        _numberOfSamples->setSizePolicy(sizePolicy2);
        _numberOfSamples->setMaximum(1000000000);
        _numberOfSamples->setSingleStep(1000);
        _numberOfSamples->setValue(1000);

        horizontalLayout_2->addWidget(_numberOfSamples);


        verticalLayout_3->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(_generatedVectorGroup);
        label->setObjectName("label");
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setWordWrap(false);

        horizontalLayout->addWidget(label);

        _from = new QLineEdit(_generatedVectorGroup);
        _from->setObjectName("_from");
        _from->setText(QString::fromUtf8("-10.0"));

        horizontalLayout->addWidget(_from);

        label_3 = new QLabel(_generatedVectorGroup);
        label_3->setObjectName("label_3");
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);
        label_3->setWordWrap(false);

        horizontalLayout->addWidget(label_3);

        _to = new QLineEdit(_generatedVectorGroup);
        _to->setObjectName("_to");
        _to->setText(QString::fromUtf8("10.0"));

        horizontalLayout->addWidget(_to);


        verticalLayout_3->addLayout(horizontalLayout);


        horizontalLayout_4->addLayout(verticalLayout_3);


        verticalLayout_2->addWidget(_generatedVectorGroup);

        _dataVectorGroup = new QGroupBox(VectorTab);
        _dataVectorGroup->setObjectName("_dataVectorGroup");
        _dataVectorGroup->setCheckable(true);
        verticalLayout = new QVBoxLayout(_dataVectorGroup);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        label_4 = new QLabel(_dataVectorGroup);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        label_4->setWordWrap(false);

        verticalLayout->addWidget(label_4);

        _fileName = new Kst::DataSourceSelector(_dataVectorGroup);
        _fileName->setObjectName("_fileName");

        verticalLayout->addWidget(_fileName);

        label_5 = new QLabel(_dataVectorGroup);
        label_5->setObjectName("label_5");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy3);
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_5->setWordWrap(false);

        verticalLayout->addWidget(label_5);

        _field = new Kst::ComboBox(_dataVectorGroup);
        _field->setObjectName("_field");

        verticalLayout->addWidget(_field);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(21, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        _connect = new QPushButton(_dataVectorGroup);
        _connect->setObjectName("_connect");
        _connect->setEnabled(false);

        hboxLayout->addWidget(_connect);

        _configure = new QPushButton(_dataVectorGroup);
        _configure->setObjectName("_configure");
        _configure->setEnabled(false);

        hboxLayout->addWidget(_configure);


        verticalLayout->addLayout(hboxLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        _fileType_3 = new QLabel(_dataVectorGroup);
        _fileType_3->setObjectName("_fileType_3");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(10);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_fileType_3->sizePolicy().hasHeightForWidth());
        _fileType_3->setSizePolicy(sizePolicy4);
        _fileType_3->setWordWrap(false);

        horizontalLayout_3->addWidget(_fileType_3);

        _updateBox = new QComboBox(_dataVectorGroup);
        _updateBox->setObjectName("_updateBox");
        _updateBox->setMinimumSize(QSize(0, 0));

        horizontalLayout_3->addWidget(_updateBox);


        verticalLayout->addLayout(horizontalLayout_3);

        _dataRange = new Kst::DataRange(_dataVectorGroup);
        _dataRange->setObjectName("_dataRange");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(_dataRange->sizePolicy().hasHeightForWidth());
        _dataRange->setSizePolicy(sizePolicy5);

        verticalLayout->addWidget(_dataRange);


        verticalLayout_2->addWidget(_dataVectorGroup);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(_numberOfSamples);
        label->setBuddy(_from);
        label_3->setBuddy(_to);
        label_5->setBuddy(_field);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_generatedVectorGroup, _numberOfSamples);
        QWidget::setTabOrder(_numberOfSamples, _from);
        QWidget::setTabOrder(_from, _to);
        QWidget::setTabOrder(_to, _dataVectorGroup);
        QWidget::setTabOrder(_dataVectorGroup, _field);
        QWidget::setTabOrder(_field, _connect);
        QWidget::setTabOrder(_connect, _configure);
        QWidget::setTabOrder(_configure, _updateBox);

        retranslateUi(VectorTab);

        QMetaObject::connectSlotsByName(VectorTab);
    } // setupUi

    void retranslateUi(QWidget *VectorTab)
    {
        _generatedVectorGroup->setTitle(QCoreApplication::translate("VectorTab", "&Generate", nullptr));
        label_2->setText(QCoreApplication::translate("VectorTab", "&Number of samples:", nullptr));
#if QT_CONFIG(whatsthis)
        _numberOfSamples->setWhatsThis(QCoreApplication::translate("VectorTab", "The number of samples for the new independent variable.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label->setText(QCoreApplication::translate("VectorTab", "Fro&m:", nullptr));
#if QT_CONFIG(whatsthis)
        _from->setWhatsThis(QCoreApplication::translate("VectorTab", "The minimum value of the independent variable.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_3->setText(QCoreApplication::translate("VectorTab", "&to:", nullptr));
#if QT_CONFIG(whatsthis)
        _to->setWhatsThis(QCoreApplication::translate("VectorTab", "The maximum value of the independent variable.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _dataVectorGroup->setTitle(QCoreApplication::translate("VectorTab", "Read from &data source", nullptr));
        label_4->setText(QCoreApplication::translate("VectorTab", "File name:", nullptr));
        label_5->setText(QCoreApplication::translate("VectorTab", "F&ield or column:", nullptr));
        _connect->setText(QCoreApplication::translate("VectorTab", "&Connect", nullptr));
        _configure->setText(QCoreApplication::translate("VectorTab", "Co&nfigure...", nullptr));
        _fileType_3->setText(QCoreApplication::translate("VectorTab", "Update type. Use 'Time Interval' for non local data.", nullptr));
        (void)VectorTab;
    } // retranslateUi

};

namespace Ui {
    class VectorTab: public Ui_VectorTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_VECTORTAB_H
