/****************************************************************************
** Meta object code from reading C++ file 'cclineedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kst-plot/src/widgets/cclineedit.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cclineedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Kst10CCLineEditE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::CCLineEdit::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst10CCLineEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::CCLineEdit",
        "currentPrefixChanged",
        "",
        "fillKstObjects",
        "insert",
        "i",
        "stringIsCompletion",
        "divide",
        "x"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentPrefixChanged'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'fillKstObjects'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insert'
        QtMocHelpers::SlotData<void(const QString &, bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Slot 'insert'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'divide'
        QtMocHelpers::SlotData<void(QString)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'divide'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CCLineEdit, qt_meta_tag_ZN3Kst10CCLineEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::CCLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10CCLineEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10CCLineEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst10CCLineEditE_t>.metaTypes,
    nullptr
} };

void Kst::CCLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CCLineEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentPrefixChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->fillKstObjects(); break;
        case 2: _t->insert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->insert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->divide((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->divide(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CCLineEdit::*)(QString )>(_a, &CCLineEdit::currentPrefixChanged, 0))
            return;
    }
}

const QMetaObject *Kst::CCLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::CCLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10CCLineEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CCCommonEdit"))
        return static_cast< CCCommonEdit*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int Kst::CCLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Kst::CCLineEdit::currentPrefixChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3Kst12SVCCLineEditE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::SVCCLineEdit::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst12SVCCLineEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::SVCCLineEdit",
        "fillKstObjects",
        "",
        "newVector",
        "newScalar",
        "editItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'fillKstObjects'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newVector'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newScalar'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editItem'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SVCCLineEdit, qt_meta_tag_ZN3Kst12SVCCLineEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::SVCCLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<CCLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst12SVCCLineEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst12SVCCLineEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst12SVCCLineEditE_t>.metaTypes,
    nullptr
} };

void Kst::SVCCLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SVCCLineEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fillKstObjects(); break;
        case 1: _t->newVector(); break;
        case 2: _t->newScalar(); break;
        case 3: _t->editItem(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *Kst::SVCCLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::SVCCLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst12SVCCLineEditE_t>.strings))
        return static_cast<void*>(this);
    return CCLineEdit::qt_metacast(_clname);
}

int Kst::SVCCLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CCLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3Kst10CCTextEditE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::CCTextEdit::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst10CCTextEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::CCTextEdit",
        "currentPrefixChanged",
        "",
        "insert",
        "i",
        "stringIsCompletion",
        "fillKstObjects",
        "divide",
        "x"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentPrefixChanged'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'insert'
        QtMocHelpers::SlotData<void(const QString &, bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'insert'
        QtMocHelpers::SlotData<void(const QString &)>(3, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'fillKstObjects'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'divide'
        QtMocHelpers::SlotData<void(QString)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'divide'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CCTextEdit, qt_meta_tag_ZN3Kst10CCTextEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::CCTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10CCTextEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10CCTextEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst10CCTextEditE_t>.metaTypes,
    nullptr
} };

void Kst::CCTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CCTextEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentPrefixChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->insert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->insert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fillKstObjects(); break;
        case 4: _t->divide((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->divide(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CCTextEdit::*)(QString )>(_a, &CCTextEdit::currentPrefixChanged, 0))
            return;
    }
}

const QMetaObject *Kst::CCTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::CCTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst10CCTextEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "CCCommonEdit"))
        return static_cast< CCCommonEdit*>(this);
    return QTextEdit::qt_metacast(_clname);
}

int Kst::CCTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Kst::CCTextEdit::currentPrefixChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN3Kst12SVCCTextEditE_t {};
} // unnamed namespace

template <> constexpr inline auto Kst::SVCCTextEdit::qt_create_metaobjectdata<qt_meta_tag_ZN3Kst12SVCCTextEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kst::SVCCTextEdit",
        "fillKstObjects",
        "",
        "newVector",
        "newScalar",
        "newString",
        "editItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'fillKstObjects'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newVector'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newScalar'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newString'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editItem'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SVCCTextEdit, qt_meta_tag_ZN3Kst12SVCCTextEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kst::SVCCTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<CCTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst12SVCCTextEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst12SVCCTextEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Kst12SVCCTextEditE_t>.metaTypes,
    nullptr
} };

void Kst::SVCCTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SVCCTextEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fillKstObjects(); break;
        case 1: _t->newVector(); break;
        case 2: _t->newScalar(); break;
        case 3: _t->newString(); break;
        case 4: _t->editItem(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *Kst::SVCCTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kst::SVCCTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Kst12SVCCTextEditE_t>.strings))
        return static_cast<void*>(this);
    return CCTextEdit::qt_metacast(_clname);
}

int Kst::SVCCTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CCTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
