/********************************************************************************
** Form generated from reading UI file 'asciiconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ASCIICONFIG_H
#define UI_ASCIICONFIG_H

#include <QtCore/QDate>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AsciiConfig
{
public:
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_11;
    QCheckBox *_applyDefault;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *_previewButton;
    QLabel *_labelBeginning;
    QPlainTextEdit *_showBeginning;
    QFrame *line;
    QHBoxLayout *horizontalLayout_9;
    QGroupBox *headerButtonGroup;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *formLayout;
    QLabel *textLabel1_3;
    QSpinBox *_startLine;
    QCheckBox *_readFields;
    QSpinBox *_fieldsLine;
    QCheckBox *_readUnits;
    QSpinBox *_unitsLine;
    QLabel *textLabel1;
    QLineEdit *_delimiters;
    QHBoxLayout *horizontalLayout_15;
    QLabel *label;
    QRadioButton *_useDot;
    QRadioButton *_useComma;
    QSpacerItem *horizontalSpacer_7;
    QSpacerItem *verticalSpacer_3;
    QGroupBox *_dataColumnLayout;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *_whitespace;
    QHBoxLayout *horizontalLayout_12;
    QRadioButton *_custom;
    QLineEdit *_columnDelimiter;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *_colWidthConst;
    QHBoxLayout *horizontalLayout_13;
    QRadioButton *_fixed;
    QSpinBox *_columnWidth;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *_limitFileBuffer;
    QSpinBox *_limitFileBufferSize;
    QCheckBox *_useThreads;
    QHBoxLayout *horizontalLayout_16;
    QLabel *textLabel1_2;
    QLineEdit *_fileNamePattern;
    QLabel *label_4;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_6;
    QRadioButton *_nanNull;
    QRadioButton *_nanNAN;
    QRadioButton *_nanPrevious;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *horizontalSpacer_4;
    QGroupBox *_interpret;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_10;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label_6;
    QComboBox *_indexVector;
    QLabel *_indexTypeLabel;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_5;
    QRadioButton *_ctime;
    QRadioButton *_seconds;
    QSpacerItem *horizontalSpacer_10;
    QRadioButton *_indexFreq;
    QDoubleSpinBox *_dataRate;
    QHBoxLayout *horizontalLayout_4;
    QRadioButton *_formattedString;
    QLineEdit *_timeAsciiFormatString;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer_9;
    QLabel *label_3;
    QSpacerItem *verticalSpacer_5;
    QGroupBox *_useOffset;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    QRadioButton *_offsetDateTime;
    QDateTimeEdit *_dateTimeOffset;
    QRadioButton *_offsetFileDate;
    QHBoxLayout *horizontalLayout;
    QRadioButton *_offsetRelative;
    QDoubleSpinBox *_relativeOffset;
    QLabel *label_5;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_4;
    QSpacerItem *horizontalSpacer_11;

    void setupUi(QWidget *AsciiConfig)
    {
        if (AsciiConfig->objectName().isEmpty())
            AsciiConfig->setObjectName("AsciiConfig");
        AsciiConfig->resize(739, 524);
        AsciiConfig->setMaximumSize(QSize(1280, 1024));
        verticalLayout_7 = new QVBoxLayout(AsciiConfig);
        verticalLayout_7->setSpacing(4);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setSpacing(6);
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        _applyDefault = new QCheckBox(AsciiConfig);
        _applyDefault->setObjectName("_applyDefault");

        horizontalLayout_11->addWidget(_applyDefault);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_5);

        _previewButton = new QPushButton(AsciiConfig);
        _previewButton->setObjectName("_previewButton");

        horizontalLayout_11->addWidget(_previewButton);


        verticalLayout_7->addLayout(horizontalLayout_11);

        _labelBeginning = new QLabel(AsciiConfig);
        _labelBeginning->setObjectName("_labelBeginning");

        verticalLayout_7->addWidget(_labelBeginning);

        _showBeginning = new QPlainTextEdit(AsciiConfig);
        _showBeginning->setObjectName("_showBeginning");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_showBeginning->sizePolicy().hasHeightForWidth());
        _showBeginning->setSizePolicy(sizePolicy);
        _showBeginning->setMinimumSize(QSize(640, 100));

        verticalLayout_7->addWidget(_showBeginning);

        line = new QFrame(AsciiConfig);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_7->addWidget(line);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setSpacing(6);
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        headerButtonGroup = new QGroupBox(AsciiConfig);
        headerButtonGroup->setObjectName("headerButtonGroup");
        verticalLayout_5 = new QVBoxLayout(headerButtonGroup);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setContentsMargins(11, 11, 11, 11);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(-1, -1, -1, 3);
        formLayout = new QFormLayout();
        formLayout->setSpacing(6);
        formLayout->setObjectName("formLayout");
        textLabel1_3 = new QLabel(headerButtonGroup);
        textLabel1_3->setObjectName("textLabel1_3");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(3);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel1_3->sizePolicy().hasHeightForWidth());
        textLabel1_3->setSizePolicy(sizePolicy1);
        textLabel1_3->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, textLabel1_3);

        _startLine = new QSpinBox(headerButtonGroup);
        _startLine->setObjectName("_startLine");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(2);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_startLine->sizePolicy().hasHeightForWidth());
        _startLine->setSizePolicy(sizePolicy2);
        _startLine->setMinimum(1);
        _startLine->setMaximum(9999999);
        _startLine->setValue(1);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, _startLine);

        _readFields = new QCheckBox(headerButtonGroup);
        _readFields->setObjectName("_readFields");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, _readFields);

        _fieldsLine = new QSpinBox(headerButtonGroup);
        _fieldsLine->setObjectName("_fieldsLine");
        _fieldsLine->setEnabled(false);
        sizePolicy2.setHeightForWidth(_fieldsLine->sizePolicy().hasHeightForWidth());
        _fieldsLine->setSizePolicy(sizePolicy2);
        _fieldsLine->setMinimum(1);
        _fieldsLine->setMaximum(9999999);
        _fieldsLine->setValue(1);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, _fieldsLine);

        _readUnits = new QCheckBox(headerButtonGroup);
        _readUnits->setObjectName("_readUnits");
        _readUnits->setEnabled(false);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, _readUnits);

        _unitsLine = new QSpinBox(headerButtonGroup);
        _unitsLine->setObjectName("_unitsLine");
        _unitsLine->setEnabled(false);
        sizePolicy2.setHeightForWidth(_unitsLine->sizePolicy().hasHeightForWidth());
        _unitsLine->setSizePolicy(sizePolicy2);
        _unitsLine->setMinimum(1);
        _unitsLine->setMaximum(9999999);
        _unitsLine->setValue(1);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, _unitsLine);

        textLabel1 = new QLabel(headerButtonGroup);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1->setWordWrap(false);

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, textLabel1);

        _delimiters = new QLineEdit(headerButtonGroup);
        _delimiters->setObjectName("_delimiters");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, _delimiters);


        verticalLayout_5->addLayout(formLayout);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setSpacing(6);
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        label = new QLabel(headerButtonGroup);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_15->addWidget(label);

        _useDot = new QRadioButton(headerButtonGroup);
        _useDot->setObjectName("_useDot");
        _useDot->setChecked(false);

        horizontalLayout_15->addWidget(_useDot);

        _useComma = new QRadioButton(headerButtonGroup);
        _useComma->setObjectName("_useComma");
        _useComma->setChecked(true);

        horizontalLayout_15->addWidget(_useComma);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_7);


        verticalLayout_5->addLayout(horizontalLayout_15);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_3);


        horizontalLayout_9->addWidget(headerButtonGroup);

        _dataColumnLayout = new QGroupBox(AsciiConfig);
        _dataColumnLayout->setObjectName("_dataColumnLayout");
        verticalLayout_4 = new QVBoxLayout(_dataColumnLayout);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(-1, -1, -1, 3);
        _whitespace = new QRadioButton(_dataColumnLayout);
        _whitespace->setObjectName("_whitespace");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_whitespace->sizePolicy().hasHeightForWidth());
        _whitespace->setSizePolicy(sizePolicy3);
        _whitespace->setChecked(true);

        verticalLayout_4->addWidget(_whitespace);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setSpacing(6);
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        _custom = new QRadioButton(_dataColumnLayout);
        _custom->setObjectName("_custom");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_custom->sizePolicy().hasHeightForWidth());
        _custom->setSizePolicy(sizePolicy4);

        horizontalLayout_12->addWidget(_custom);

        _columnDelimiter = new QLineEdit(_dataColumnLayout);
        _columnDelimiter->setObjectName("_columnDelimiter");
        _columnDelimiter->setEnabled(false);
        sizePolicy3.setHeightForWidth(_columnDelimiter->sizePolicy().hasHeightForWidth());
        _columnDelimiter->setSizePolicy(sizePolicy3);
        _columnDelimiter->setMaximumSize(QSize(40, 16777215));

        horizontalLayout_12->addWidget(_columnDelimiter);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_2);


        verticalLayout_4->addLayout(horizontalLayout_12);

        _colWidthConst = new QCheckBox(_dataColumnLayout);
        _colWidthConst->setObjectName("_colWidthConst");
        sizePolicy4.setHeightForWidth(_colWidthConst->sizePolicy().hasHeightForWidth());
        _colWidthConst->setSizePolicy(sizePolicy4);

        verticalLayout_4->addWidget(_colWidthConst);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setSpacing(6);
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        _fixed = new QRadioButton(_dataColumnLayout);
        _fixed->setObjectName("_fixed");
        sizePolicy4.setHeightForWidth(_fixed->sizePolicy().hasHeightForWidth());
        _fixed->setSizePolicy(sizePolicy4);

        horizontalLayout_13->addWidget(_fixed);

        _columnWidth = new QSpinBox(_dataColumnLayout);
        _columnWidth->setObjectName("_columnWidth");
        _columnWidth->setEnabled(false);
        sizePolicy4.setHeightForWidth(_columnWidth->sizePolicy().hasHeightForWidth());
        _columnWidth->setSizePolicy(sizePolicy4);
        _columnWidth->setMinimum(1);
        _columnWidth->setValue(16);

        horizontalLayout_13->addWidget(_columnWidth);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_8);


        verticalLayout_4->addLayout(horizontalLayout_13);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        horizontalLayout_9->addWidget(_dataColumnLayout);

        groupBox_2 = new QGroupBox(AsciiConfig);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setMinimumSize(QSize(0, 0));
        verticalLayout_3 = new QVBoxLayout(groupBox_2);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(-1, -1, -1, 3);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _limitFileBuffer = new QCheckBox(groupBox_2);
        _limitFileBuffer->setObjectName("_limitFileBuffer");

        horizontalLayout_2->addWidget(_limitFileBuffer);

        _limitFileBufferSize = new QSpinBox(groupBox_2);
        _limitFileBufferSize->setObjectName("_limitFileBufferSize");
        _limitFileBufferSize->setEnabled(false);
        _limitFileBufferSize->setMinimum(0);
        _limitFileBufferSize->setMaximum(16000);
        _limitFileBufferSize->setSingleStep(100);
        _limitFileBufferSize->setValue(100);

        horizontalLayout_2->addWidget(_limitFileBufferSize);


        verticalLayout_3->addLayout(horizontalLayout_2);

        _useThreads = new QCheckBox(groupBox_2);
        _useThreads->setObjectName("_useThreads");

        verticalLayout_3->addWidget(_useThreads);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setSpacing(6);
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        textLabel1_2 = new QLabel(groupBox_2);
        textLabel1_2->setObjectName("textLabel1_2");
        sizePolicy3.setHeightForWidth(textLabel1_2->sizePolicy().hasHeightForWidth());
        textLabel1_2->setSizePolicy(sizePolicy3);
        textLabel1_2->setWordWrap(false);

        horizontalLayout_16->addWidget(textLabel1_2);

        _fileNamePattern = new QLineEdit(groupBox_2);
        _fileNamePattern->setObjectName("_fileNamePattern");

        horizontalLayout_16->addWidget(_fileNamePattern);


        verticalLayout_3->addLayout(horizontalLayout_16);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");

        verticalLayout_3->addWidget(label_4);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(6);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalSpacer_6 = new QSpacerItem(13, 16, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_6);

        _nanNull = new QRadioButton(groupBox_2);
        _nanNull->setObjectName("_nanNull");
        _nanNull->setChecked(true);

        horizontalLayout_6->addWidget(_nanNull);

        _nanNAN = new QRadioButton(groupBox_2);
        _nanNAN->setObjectName("_nanNAN");
        _nanNAN->setChecked(false);

        horizontalLayout_6->addWidget(_nanNAN);

        _nanPrevious = new QRadioButton(groupBox_2);
        _nanPrevious->setObjectName("_nanPrevious");
        _nanPrevious->setChecked(false);

        horizontalLayout_6->addWidget(_nanPrevious);


        verticalLayout_3->addLayout(horizontalLayout_6);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);


        horizontalLayout_9->addWidget(groupBox_2);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_4);


        verticalLayout_7->addLayout(horizontalLayout_9);

        _interpret = new QGroupBox(AsciiConfig);
        _interpret->setObjectName("_interpret");
        _interpret->setCheckable(true);
        _interpret->setChecked(false);
        verticalLayout_6 = new QVBoxLayout(_interpret);
        verticalLayout_6->setSpacing(6);
        verticalLayout_6->setContentsMargins(11, 11, 11, 11);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setSpacing(6);
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        groupBox = new QGroupBox(_interpret);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(-1, -1, -1, 3);
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setSpacing(6);
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");

        horizontalLayout_8->addWidget(label_6);

        _indexVector = new QComboBox(groupBox);
        _indexVector->setObjectName("_indexVector");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(1);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(_indexVector->sizePolicy().hasHeightForWidth());
        _indexVector->setSizePolicy(sizePolicy5);

        horizontalLayout_8->addWidget(_indexVector);

        _indexTypeLabel = new QLabel(groupBox);
        _indexTypeLabel->setObjectName("_indexTypeLabel");
        _indexTypeLabel->setWordWrap(false);

        horizontalLayout_8->addWidget(_indexTypeLabel);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_3);


        verticalLayout_2->addLayout(horizontalLayout_8);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(6);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        _ctime = new QRadioButton(groupBox);
        _ctime->setObjectName("_ctime");
        _ctime->setChecked(true);

        horizontalLayout_5->addWidget(_ctime);

        _seconds = new QRadioButton(groupBox);
        _seconds->setObjectName("_seconds");

        horizontalLayout_5->addWidget(_seconds);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_10);

        _indexFreq = new QRadioButton(groupBox);
        _indexFreq->setObjectName("_indexFreq");

        horizontalLayout_5->addWidget(_indexFreq);

        _dataRate = new QDoubleSpinBox(groupBox);
        _dataRate->setObjectName("_dataRate");
        _dataRate->setEnabled(false);
        _dataRate->setDecimals(6);
        _dataRate->setMaximum(10000000.000000000000000);
        _dataRate->setSingleStep(1.000000000000000);
        _dataRate->setValue(1000.000000000000000);

        horizontalLayout_5->addWidget(_dataRate);


        verticalLayout_2->addLayout(horizontalLayout_5);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        _formattedString = new QRadioButton(groupBox);
        _formattedString->setObjectName("_formattedString");
        sizePolicy4.setHeightForWidth(_formattedString->sizePolicy().hasHeightForWidth());
        _formattedString->setSizePolicy(sizePolicy4);

        horizontalLayout_4->addWidget(_formattedString);

        _timeAsciiFormatString = new QLineEdit(groupBox);
        _timeAsciiFormatString->setObjectName("_timeAsciiFormatString");
        _timeAsciiFormatString->setEnabled(false);
        _timeAsciiFormatString->setMinimumSize(QSize(200, 0));

        horizontalLayout_4->addWidget(_timeAsciiFormatString);


        verticalLayout_2->addLayout(horizontalLayout_4);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setSpacing(6);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_9);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");
        sizePolicy4.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy4);

        horizontalLayout_7->addWidget(label_3);


        verticalLayout_2->addLayout(horizontalLayout_7);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_5);


        horizontalLayout_10->addWidget(groupBox);

        _useOffset = new QGroupBox(_interpret);
        _useOffset->setObjectName("_useOffset");
        _useOffset->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        verticalLayout = new QVBoxLayout(_useOffset);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(-1, -1, -1, 3);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        _offsetDateTime = new QRadioButton(_useOffset);
        _offsetDateTime->setObjectName("_offsetDateTime");
        sizePolicy4.setHeightForWidth(_offsetDateTime->sizePolicy().hasHeightForWidth());
        _offsetDateTime->setSizePolicy(sizePolicy4);

        horizontalLayout_3->addWidget(_offsetDateTime);

        _dateTimeOffset = new QDateTimeEdit(_useOffset);
        _dateTimeOffset->setObjectName("_dateTimeOffset");
        _dateTimeOffset->setEnabled(false);
        _dateTimeOffset->setDateTime(QDateTime(QDate(2000, 1, 1), QTime(8, 0, 0)));
        _dateTimeOffset->setDate(QDate(2000, 1, 1));
        _dateTimeOffset->setTime(QTime(8, 0, 0));
        _dateTimeOffset->setCalendarPopup(true);

        horizontalLayout_3->addWidget(_dateTimeOffset);


        verticalLayout->addLayout(horizontalLayout_3);

        _offsetFileDate = new QRadioButton(_useOffset);
        _offsetFileDate->setObjectName("_offsetFileDate");
        _offsetFileDate->setChecked(false);

        verticalLayout->addWidget(_offsetFileDate);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        _offsetRelative = new QRadioButton(_useOffset);
        _offsetRelative->setObjectName("_offsetRelative");
        _offsetRelative->setChecked(false);

        horizontalLayout->addWidget(_offsetRelative);

        _relativeOffset = new QDoubleSpinBox(_useOffset);
        _relativeOffset->setObjectName("_relativeOffset");

        horizontalLayout->addWidget(_relativeOffset);

        label_5 = new QLabel(_useOffset);
        label_5->setObjectName("label_5");
        sizePolicy3.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(label_5);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer_4 = new QSpacerItem(20, 30, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_4);


        horizontalLayout_10->addWidget(_useOffset);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_11);


        verticalLayout_6->addLayout(horizontalLayout_10);


        verticalLayout_7->addWidget(_interpret);

        verticalLayout_7->setStretch(2, 10);
        verticalLayout_7->setStretch(3, 1);
        verticalLayout_7->setStretch(4, 1);
        verticalLayout_7->setStretch(5, 1);
#if QT_CONFIG(shortcut)
        textLabel1_3->setBuddy(_startLine);
        textLabel1->setBuddy(_delimiters);
#endif // QT_CONFIG(shortcut)

        retranslateUi(AsciiConfig);
        QObject::connect(_fixed, &QRadioButton::toggled, _columnWidth, &QSpinBox::setEnabled);
        QObject::connect(_readFields, &QCheckBox::toggled, _fieldsLine, &QSpinBox::setEnabled);
        QObject::connect(_readUnits, &QCheckBox::toggled, _unitsLine, &QSpinBox::setEnabled);
        QObject::connect(_custom, &QRadioButton::toggled, _columnDelimiter, &QLineEdit::setEnabled);
        QObject::connect(_readFields, &QCheckBox::toggled, _readUnits, &QCheckBox::setEnabled);
        QObject::connect(_offsetDateTime, &QRadioButton::toggled, _dateTimeOffset, &QDateTimeEdit::setEnabled);
        QObject::connect(_offsetRelative, &QRadioButton::toggled, _relativeOffset, &QDoubleSpinBox::setEnabled);
        QObject::connect(_offsetRelative, &QRadioButton::toggled, label_5, &QLabel::setEnabled);
        QObject::connect(_limitFileBuffer, &QCheckBox::toggled, _limitFileBufferSize, &QSpinBox::setEnabled);
        QObject::connect(_interpret, &QGroupBox::toggled, groupBox, &QGroupBox::setChecked);
        QObject::connect(_indexFreq, &QRadioButton::toggled, _dataRate, &QDoubleSpinBox::setEnabled);
        QObject::connect(_formattedString, &QRadioButton::toggled, _timeAsciiFormatString, &QLineEdit::setEnabled);
        QObject::connect(_readFields, &QCheckBox::toggled, _unitsLine, &QSpinBox::setEnabled);

        QMetaObject::connectSlotsByName(AsciiConfig);
    } // setupUi

    void retranslateUi(QWidget *AsciiConfig)
    {
        _applyDefault->setText(QCoreApplication::translate("AsciiConfig", "Apply these settings as default", nullptr));
        _previewButton->setText(QCoreApplication::translate("AsciiConfig", "Preview First 1000 Lines in New Window", nullptr));
        _labelBeginning->setText(QCoreApplication::translate("AsciiConfig", "First lines ", nullptr));
        _showBeginning->setPlainText(QCoreApplication::translate("AsciiConfig", "[Preview]", nullptr));
        headerButtonGroup->setTitle(QCoreApplication::translate("AsciiConfig", "Header", nullptr));
        textLabel1_3->setText(QCoreApplication::translate("AsciiConfig", "Data starts at line:", nullptr));
        _readFields->setText(QCoreApplication::translate("AsciiConfig", "Read field names from line:", nullptr));
        _readUnits->setText(QCoreApplication::translate("AsciiConfig", "Read units from line:", nullptr));
        textLabel1->setText(QCoreApplication::translate("AsciiConfig", "Comment indicators:", nullptr));
#if QT_CONFIG(tooltip)
        _delimiters->setToolTip(QCoreApplication::translate("AsciiConfig", "You can use more than one symbol here, like \"#!C\" for instance, but it will slow down reading ASCII files", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("AsciiConfig", "Decimal Separator:", nullptr));
        _useDot->setText(QCoreApplication::translate("AsciiConfig", "Dot", "the . character"));
        _useComma->setText(QCoreApplication::translate("AsciiConfig", "Comma", "the , character"));
        _dataColumnLayout->setTitle(QCoreApplication::translate("AsciiConfig", "Data Column Layout", nullptr));
        _whitespace->setText(QCoreApplication::translate("AsciiConfig", "Space/tab delimited", nullptr));
        _custom->setText(QCoreApplication::translate("AsciiConfig", "Custom delimiter", nullptr));
#if QT_CONFIG(tooltip)
        _colWidthConst->setToolTip(QCoreApplication::translate("AsciiConfig", "Reading is faster if each column can be assumed to have its own constant width.", nullptr));
#endif // QT_CONFIG(tooltip)
        _colWidthConst->setText(QCoreApplication::translate("AsciiConfig", "Each column has its\n"
"own constant width", nullptr));
        _fixed->setText(QCoreApplication::translate("AsciiConfig", "Fixed Width", nullptr));
        _columnWidth->setSuffix(QString());
        groupBox_2->setTitle(QCoreApplication::translate("AsciiConfig", "General Options", nullptr));
        _limitFileBuffer->setText(QCoreApplication::translate("AsciiConfig", "Limit Buffer Size:", nullptr));
        _limitFileBufferSize->setSuffix(QCoreApplication::translate("AsciiConfig", " MB", nullptr));
        _useThreads->setText(QCoreApplication::translate("AsciiConfig", "Use threads", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("AsciiConfig", "Always accept files matching:", nullptr));
        label_4->setText(QCoreApplication::translate("AsciiConfig", "Interpret empty value as:", nullptr));
        _nanNull->setText(QCoreApplication::translate("AsciiConfig", "Zero", nullptr));
        _nanNAN->setText(QCoreApplication::translate("AsciiConfig", "NaN", nullptr));
        _nanPrevious->setText(QCoreApplication::translate("AsciiConfig", "Previous value", nullptr));
        _interpret->setTitle(QCoreApplication::translate("AsciiConfig", "Date/Time Interpretation", nullptr));
        groupBox->setTitle(QCoreApplication::translate("AsciiConfig", "Value", nullptr));
        label_6->setText(QCoreApplication::translate("AsciiConfig", "Interpret field", nullptr));
        _indexTypeLabel->setText(QCoreApplication::translate("AsciiConfig", "as:", nullptr));
        _ctime->setText(QCoreApplication::translate("AsciiConfig", "C Time", nullptr));
        _seconds->setText(QCoreApplication::translate("AsciiConfig", "Seconds", nullptr));
        _indexFreq->setText(QCoreApplication::translate("AsciiConfig", "Index with frequency", nullptr));
        _dataRate->setSpecialValueText(QString());
        _dataRate->setSuffix(QCoreApplication::translate("AsciiConfig", "Hz", nullptr));
        _formattedString->setText(QCoreApplication::translate("AsciiConfig", "Formatted String.  Format:", nullptr));
#if QT_CONFIG(tooltip)
        _timeAsciiFormatString->setToolTip(QCoreApplication::translate("AsciiConfig", "Refer to the documentation of QDateTime for details.<br>You can use a combination of y,M,d for dates and h,m,s,z for times (z for milliseconds).<br>Space is allowed between date and time columns, but you must select the first column as X vector.", nullptr));
#endif // QT_CONFIG(tooltip)
        _timeAsciiFormatString->setText(QCoreApplication::translate("AsciiConfig", "hh:mm:ss.zzz", nullptr));
        label_3->setText(QCoreApplication::translate("AsciiConfig", "   d=day, M=month, y=year, h=hour, m=minute, s=second, z=millisecond", nullptr));
        _useOffset->setTitle(QCoreApplication::translate("AsciiConfig", "Offset", nullptr));
        _offsetDateTime->setText(QCoreApplication::translate("AsciiConfig", "Date/Time:", nullptr));
        _dateTimeOffset->setDisplayFormat(QCoreApplication::translate("AsciiConfig", "dd/MM/yyyy HH:mm:ss.zzz", nullptr));
        _offsetFileDate->setText(QCoreApplication::translate("AsciiConfig", "File date", nullptr));
        _offsetRelative->setText(QCoreApplication::translate("AsciiConfig", "Relative", "relative offset"));
        label_5->setText(QCoreApplication::translate("AsciiConfig", "Seconds", nullptr));
        (void)AsciiConfig;
    } // retranslateUi

};

namespace Ui {
    class AsciiConfig: public Ui_AsciiConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ASCIICONFIG_H
