/********************************************************************************
** Form generated from reading UI file 'labeltab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LABELTAB_H
#define UI_LABELTAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFontComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "cclineedit.h"
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_LabelTab
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QPushButton *_globalLabelItalic;
    QLabel *label;
    QFontComboBox *_globalLabelFontFamily;
    QDoubleSpinBox *_globalLabelFontSize;
    Kst::ColorButton *_globalLabelColor;
    QLabel *label_3;
    QPushButton *_globalLabelBold;
    QLabel *_Label_4;
    QCheckBox *_autoScaleNumberAxis;
    QSpacerItem *horizontalSpacer_2;
    QFrame *line_2;
    QGridLayout *gridLayout_2;
    QLabel *_bottomLabelLabel;
    QCheckBox *_topLabelAuto;
    Kst::SVCCTextEdit *_rightLabelText;
    QLabel *_rightLabelLabel;
    Kst::SVCCTextEdit *_leftLabelText;
    QCheckBox *_rightLabelAuto;
    Kst::SVCCTextEdit *_bottomLabelText;
    QLabel *_leftLabelLabel;
    Kst::SVCCTextEdit *_topLabelText;
    QLabel *_topLabelLabel;
    QCheckBox *_leftLabelAuto;
    QCheckBox *_bottomLabelAuto;
    QLabel *label_2;
    QFrame *line;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *_showLegend;
    QPushButton *_editLegendContents;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *LabelTab)
    {
        if (LabelTab->objectName().isEmpty())
            LabelTab->setObjectName("LabelTab");
        LabelTab->resize(1155, 621);
        verticalLayout = new QVBoxLayout(LabelTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        _globalLabelItalic = new QPushButton(LabelTab);
        _globalLabelItalic->setObjectName("_globalLabelItalic");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_globalLabelItalic->sizePolicy().hasHeightForWidth());
        _globalLabelItalic->setSizePolicy(sizePolicy);
        _globalLabelItalic->setCheckable(true);

        gridLayout->addWidget(_globalLabelItalic, 0, 4, 1, 1);

        label = new QLabel(LabelTab);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label, 0, 2, 1, 1);

        _globalLabelFontFamily = new QFontComboBox(LabelTab);
        _globalLabelFontFamily->setObjectName("_globalLabelFontFamily");

        gridLayout->addWidget(_globalLabelFontFamily, 1, 1, 1, 7);

        _globalLabelFontSize = new QDoubleSpinBox(LabelTab);
        _globalLabelFontSize->setObjectName("_globalLabelFontSize");
        _globalLabelFontSize->setDecimals(1);
        _globalLabelFontSize->setMinimum(-25.000000000000000);

        gridLayout->addWidget(_globalLabelFontSize, 0, 1, 1, 1);

        _globalLabelColor = new Kst::ColorButton(LabelTab);
        _globalLabelColor->setObjectName("_globalLabelColor");

        gridLayout->addWidget(_globalLabelColor, 0, 5, 1, 1);

        label_3 = new QLabel(LabelTab);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        _globalLabelBold = new QPushButton(LabelTab);
        _globalLabelBold->setObjectName("_globalLabelBold");
        sizePolicy.setHeightForWidth(_globalLabelBold->sizePolicy().hasHeightForWidth());
        _globalLabelBold->setSizePolicy(sizePolicy);
        _globalLabelBold->setCheckable(true);

        gridLayout->addWidget(_globalLabelBold, 0, 3, 1, 1);

        _Label_4 = new QLabel(LabelTab);
        _Label_4->setObjectName("_Label_4");
        sizePolicy.setHeightForWidth(_Label_4->sizePolicy().hasHeightForWidth());
        _Label_4->setSizePolicy(sizePolicy);
        _Label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(_Label_4, 0, 0, 1, 1);

        _autoScaleNumberAxis = new QCheckBox(LabelTab);
        _autoScaleNumberAxis->setObjectName("_autoScaleNumberAxis");
        _autoScaleNumberAxis->setLayoutDirection(Qt::LeftToRight);

        gridLayout->addWidget(_autoScaleNumberAxis, 0, 7, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 0, 6, 1, 1);


        verticalLayout->addLayout(gridLayout);

        line_2 = new QFrame(LabelTab);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        _bottomLabelLabel = new QLabel(LabelTab);
        _bottomLabelLabel->setObjectName("_bottomLabelLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_bottomLabelLabel->sizePolicy().hasHeightForWidth());
        _bottomLabelLabel->setSizePolicy(sizePolicy1);
        _bottomLabelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_bottomLabelLabel, 1, 0, 1, 1);

        _topLabelAuto = new QCheckBox(LabelTab);
        _topLabelAuto->setObjectName("_topLabelAuto");
        sizePolicy.setHeightForWidth(_topLabelAuto->sizePolicy().hasHeightForWidth());
        _topLabelAuto->setSizePolicy(sizePolicy);
        _topLabelAuto->setChecked(true);

        gridLayout_2->addWidget(_topLabelAuto, 0, 2, 1, 1);

        _rightLabelText = new Kst::SVCCTextEdit(LabelTab);
        _rightLabelText->setObjectName("_rightLabelText");
        _rightLabelText->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_rightLabelText->sizePolicy().hasHeightForWidth());
        _rightLabelText->setSizePolicy(sizePolicy2);
        _rightLabelText->setMinimumSize(QSize(0, 19));
        _rightLabelText->setMaximumSize(QSize(16777215, 22));
        _rightLabelText->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _rightLabelText->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _rightLabelText->setLineWrapMode(QTextEdit::NoWrap);

        gridLayout_2->addWidget(_rightLabelText, 3, 1, 1, 1);

        _rightLabelLabel = new QLabel(LabelTab);
        _rightLabelLabel->setObjectName("_rightLabelLabel");
        sizePolicy1.setHeightForWidth(_rightLabelLabel->sizePolicy().hasHeightForWidth());
        _rightLabelLabel->setSizePolicy(sizePolicy1);
        _rightLabelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_rightLabelLabel, 3, 0, 1, 1);

        _leftLabelText = new Kst::SVCCTextEdit(LabelTab);
        _leftLabelText->setObjectName("_leftLabelText");
        _leftLabelText->setEnabled(false);
        sizePolicy2.setHeightForWidth(_leftLabelText->sizePolicy().hasHeightForWidth());
        _leftLabelText->setSizePolicy(sizePolicy2);
        _leftLabelText->setMinimumSize(QSize(0, 19));
        _leftLabelText->setMaximumSize(QSize(16777215, 22));
        _leftLabelText->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _leftLabelText->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _leftLabelText->setLineWrapMode(QTextEdit::NoWrap);

        gridLayout_2->addWidget(_leftLabelText, 2, 1, 1, 1);

        _rightLabelAuto = new QCheckBox(LabelTab);
        _rightLabelAuto->setObjectName("_rightLabelAuto");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_rightLabelAuto->sizePolicy().hasHeightForWidth());
        _rightLabelAuto->setSizePolicy(sizePolicy3);
        _rightLabelAuto->setChecked(true);

        gridLayout_2->addWidget(_rightLabelAuto, 3, 2, 1, 1);

        _bottomLabelText = new Kst::SVCCTextEdit(LabelTab);
        _bottomLabelText->setObjectName("_bottomLabelText");
        _bottomLabelText->setEnabled(false);
        sizePolicy2.setHeightForWidth(_bottomLabelText->sizePolicy().hasHeightForWidth());
        _bottomLabelText->setSizePolicy(sizePolicy2);
        _bottomLabelText->setMinimumSize(QSize(0, 19));
        _bottomLabelText->setMaximumSize(QSize(16777215, 22));
        _bottomLabelText->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _bottomLabelText->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _bottomLabelText->setLineWrapMode(QTextEdit::NoWrap);

        gridLayout_2->addWidget(_bottomLabelText, 1, 1, 1, 1);

        _leftLabelLabel = new QLabel(LabelTab);
        _leftLabelLabel->setObjectName("_leftLabelLabel");
        sizePolicy1.setHeightForWidth(_leftLabelLabel->sizePolicy().hasHeightForWidth());
        _leftLabelLabel->setSizePolicy(sizePolicy1);
        _leftLabelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_leftLabelLabel, 2, 0, 1, 1);

        _topLabelText = new Kst::SVCCTextEdit(LabelTab);
        _topLabelText->setObjectName("_topLabelText");
        _topLabelText->setEnabled(false);
        sizePolicy2.setHeightForWidth(_topLabelText->sizePolicy().hasHeightForWidth());
        _topLabelText->setSizePolicy(sizePolicy2);
        _topLabelText->setMinimumSize(QSize(0, 19));
        _topLabelText->setMaximumSize(QSize(16777215, 22));
        _topLabelText->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _topLabelText->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        _topLabelText->setLineWrapMode(QTextEdit::NoWrap);

        gridLayout_2->addWidget(_topLabelText, 0, 1, 1, 1);

        _topLabelLabel = new QLabel(LabelTab);
        _topLabelLabel->setObjectName("_topLabelLabel");
        sizePolicy1.setHeightForWidth(_topLabelLabel->sizePolicy().hasHeightForWidth());
        _topLabelLabel->setSizePolicy(sizePolicy1);
        _topLabelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(_topLabelLabel, 0, 0, 1, 1);

        _leftLabelAuto = new QCheckBox(LabelTab);
        _leftLabelAuto->setObjectName("_leftLabelAuto");
        sizePolicy3.setHeightForWidth(_leftLabelAuto->sizePolicy().hasHeightForWidth());
        _leftLabelAuto->setSizePolicy(sizePolicy3);
        _leftLabelAuto->setChecked(true);

        gridLayout_2->addWidget(_leftLabelAuto, 2, 2, 1, 1);

        _bottomLabelAuto = new QCheckBox(LabelTab);
        _bottomLabelAuto->setObjectName("_bottomLabelAuto");
        sizePolicy3.setHeightForWidth(_bottomLabelAuto->sizePolicy().hasHeightForWidth());
        _bottomLabelAuto->setSizePolicy(sizePolicy3);
        _bottomLabelAuto->setChecked(true);

        gridLayout_2->addWidget(_bottomLabelAuto, 1, 2, 1, 1);

        label_2 = new QLabel(LabelTab);
        label_2->setObjectName("label_2");
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);
        QFont font;
        font.setItalic(true);
        label_2->setFont(font);

        gridLayout_2->addWidget(label_2, 4, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        line = new QFrame(LabelTab);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(0);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        _showLegend = new QCheckBox(LabelTab);
        _showLegend->setObjectName("_showLegend");
        sizePolicy.setHeightForWidth(_showLegend->sizePolicy().hasHeightForWidth());
        _showLegend->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(_showLegend);

        _editLegendContents = new QPushButton(LabelTab);
        _editLegendContents->setObjectName("_editLegendContents");
        sizePolicy.setHeightForWidth(_editLegendContents->sizePolicy().hasHeightForWidth());
        _editLegendContents->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(_editLegendContents);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        _Label_4->setBuddy(_globalLabelFontSize);
        _bottomLabelLabel->setBuddy(_bottomLabelText);
        _rightLabelLabel->setBuddy(_rightLabelText);
        _leftLabelLabel->setBuddy(_leftLabelText);
        _topLabelLabel->setBuddy(_topLabelText);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_globalLabelFontSize, _globalLabelBold);
        QWidget::setTabOrder(_globalLabelBold, _globalLabelItalic);
        QWidget::setTabOrder(_globalLabelItalic, _globalLabelColor);
        QWidget::setTabOrder(_globalLabelColor, _globalLabelFontFamily);
        QWidget::setTabOrder(_globalLabelFontFamily, _topLabelAuto);
        QWidget::setTabOrder(_topLabelAuto, _topLabelText);
        QWidget::setTabOrder(_topLabelText, _bottomLabelAuto);
        QWidget::setTabOrder(_bottomLabelAuto, _bottomLabelText);
        QWidget::setTabOrder(_bottomLabelText, _leftLabelAuto);
        QWidget::setTabOrder(_leftLabelAuto, _leftLabelText);
        QWidget::setTabOrder(_leftLabelText, _rightLabelAuto);
        QWidget::setTabOrder(_rightLabelAuto, _rightLabelText);
        QWidget::setTabOrder(_rightLabelText, _editLegendContents);

        retranslateUi(LabelTab);
        QObject::connect(_topLabelAuto, SIGNAL(toggled(bool)), _topLabelText, SLOT(setDisabled(bool)));
        QObject::connect(_bottomLabelAuto, SIGNAL(toggled(bool)), _bottomLabelText, SLOT(setDisabled(bool)));
        QObject::connect(_leftLabelAuto, SIGNAL(toggled(bool)), _leftLabelText, SLOT(setDisabled(bool)));
        QObject::connect(_rightLabelAuto, SIGNAL(toggled(bool)), _rightLabelText, SLOT(setDisabled(bool)));

        QMetaObject::connectSlotsByName(LabelTab);
    } // setupUi

    void retranslateUi(QWidget *LabelTab)
    {
        label->setText(QCoreApplication::translate("LabelTab", "points", nullptr));
        label_3->setText(QCoreApplication::translate("LabelTab", "Font Family:", nullptr));
        _Label_4->setText(QCoreApplication::translate("LabelTab", "Default plot font si&ze:", nullptr));
        _autoScaleNumberAxis->setText(QCoreApplication::translate("LabelTab", "Autoscale axis numbers", nullptr));
        _bottomLabelLabel->setText(QCoreApplication::translate("LabelTab", "Botto&m label:", nullptr));
        _topLabelAuto->setText(QCoreApplication::translate("LabelTab", "Auto", nullptr));
        _rightLabelLabel->setText(QCoreApplication::translate("LabelTab", "&Right label:", nullptr));
        _rightLabelAuto->setText(QCoreApplication::translate("LabelTab", "Auto", nullptr));
        _leftLabelLabel->setText(QCoreApplication::translate("LabelTab", "&Left label:", nullptr));
        _topLabelLabel->setText(QCoreApplication::translate("LabelTab", "Top &label:", nullptr));
        _leftLabelAuto->setText(QCoreApplication::translate("LabelTab", "Auto", nullptr));
        _bottomLabelAuto->setText(QCoreApplication::translate("LabelTab", "Auto", nullptr));
        label_2->setText(QCoreApplication::translate("LabelTab", "Supports scalars & strings (name in square brackets) - eg,  [INDEX:Max (X2)],\n"
"embedded equations  - eg, [=[INDEX:Max] / 2]\n"
"and LaTeX symbols - eg, \\theta.  Names and symbols are autocompleted.", nullptr));
        _showLegend->setText(QCoreApplication::translate("LabelTab", "Show le&gend", nullptr));
        _editLegendContents->setText(QCoreApplication::translate("LabelTab", "E&dit", nullptr));
        (void)LabelTab;
    } // retranslateUi

};

namespace Ui {
    class LabelTab: public Ui_LabelTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LABELTAB_H
