/********************************************************************************
** Form generated from reading UI file 'rangetab.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RANGETAB_H
#define UI_RANGETAB_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RangeTab
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QRadioButton *_xAuto;
    QRadioButton *_xBorder;
    QRadioButton *_xSpike;
    QHBoxLayout *horizontalLayout;
    QRadioButton *_xMean;
    QLabel *label;
    QLineEdit *_xRange;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *_xFixed;
    QLabel *label_2;
    QLineEdit *_xMin;
    QLabel *label_3;
    QLineEdit *_xMax;
    QRadioButton *_xHidden;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout;
    QRadioButton *_yAuto;
    QRadioButton *_yBorder;
    QRadioButton *_ySpike;
    QHBoxLayout *horizontalLayout_3;
    QRadioButton *_yMean;
    QLabel *label_4;
    QLineEdit *_yRange;
    QHBoxLayout *horizontalLayout_4;
    QRadioButton *_yFixed;
    QLabel *label_5;
    QLineEdit *_yMin;
    QLabel *label_6;
    QLineEdit *_yMax;
    QRadioButton *_yHidden;

    void setupUi(QWidget *RangeTab)
    {
        if (RangeTab->objectName().isEmpty())
            RangeTab->setObjectName("RangeTab");
        RangeTab->resize(517, 375);
        verticalLayout = new QVBoxLayout(RangeTab);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(RangeTab);
        groupBox->setObjectName("groupBox");
        groupBox->setFlat(true);
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        _xAuto = new QRadioButton(groupBox);
        _xAuto->setObjectName("_xAuto");
        _xAuto->setChecked(true);

        gridLayout_2->addWidget(_xAuto, 0, 0, 1, 1);

        _xBorder = new QRadioButton(groupBox);
        _xBorder->setObjectName("_xBorder");

        gridLayout_2->addWidget(_xBorder, 0, 1, 1, 1);

        _xSpike = new QRadioButton(groupBox);
        _xSpike->setObjectName("_xSpike");

        gridLayout_2->addWidget(_xSpike, 0, 2, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        _xMean = new QRadioButton(groupBox);
        _xMean->setObjectName("_xMean");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_xMean->sizePolicy().hasHeightForWidth());
        _xMean->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(_xMean);

        label = new QLabel(groupBox);
        label->setObjectName("label");
        label->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(label);

        _xRange = new QLineEdit(groupBox);
        _xRange->setObjectName("_xRange");
        _xRange->setEnabled(false);

        horizontalLayout->addWidget(_xRange);


        gridLayout_2->addLayout(horizontalLayout, 1, 0, 1, 3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _xFixed = new QRadioButton(groupBox);
        _xFixed->setObjectName("_xFixed");
        sizePolicy.setHeightForWidth(_xFixed->sizePolicy().hasHeightForWidth());
        _xFixed->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(_xFixed);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");
        label_2->setEnabled(false);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(label_2);

        _xMin = new QLineEdit(groupBox);
        _xMin->setObjectName("_xMin");
        _xMin->setEnabled(false);

        horizontalLayout_2->addWidget(_xMin);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");
        label_3->setEnabled(false);
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(label_3);

        _xMax = new QLineEdit(groupBox);
        _xMax->setObjectName("_xMax");
        _xMax->setEnabled(false);

        horizontalLayout_2->addWidget(_xMax);


        gridLayout_2->addLayout(horizontalLayout_2, 2, 0, 1, 3);

        _xHidden = new QRadioButton(groupBox);
        _xHidden->setObjectName("_xHidden");
        _xHidden->setMaximumSize(QSize(0, 0));

        gridLayout_2->addWidget(_xHidden, 3, 0, 1, 2);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(RangeTab);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setFlat(true);
        gridLayout = new QGridLayout(groupBox_2);
        gridLayout->setObjectName("gridLayout");
        _yAuto = new QRadioButton(groupBox_2);
        _yAuto->setObjectName("_yAuto");
        _yAuto->setChecked(true);

        gridLayout->addWidget(_yAuto, 0, 0, 1, 1);

        _yBorder = new QRadioButton(groupBox_2);
        _yBorder->setObjectName("_yBorder");

        gridLayout->addWidget(_yBorder, 0, 1, 1, 1);

        _ySpike = new QRadioButton(groupBox_2);
        _ySpike->setObjectName("_ySpike");

        gridLayout->addWidget(_ySpike, 0, 2, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        _yMean = new QRadioButton(groupBox_2);
        _yMean->setObjectName("_yMean");
        sizePolicy.setHeightForWidth(_yMean->sizePolicy().hasHeightForWidth());
        _yMean->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(_yMean);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");
        label_4->setEnabled(false);
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(label_4);

        _yRange = new QLineEdit(groupBox_2);
        _yRange->setObjectName("_yRange");
        _yRange->setEnabled(false);

        horizontalLayout_3->addWidget(_yRange);


        gridLayout->addLayout(horizontalLayout_3, 1, 0, 1, 3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        _yFixed = new QRadioButton(groupBox_2);
        _yFixed->setObjectName("_yFixed");
        sizePolicy.setHeightForWidth(_yFixed->sizePolicy().hasHeightForWidth());
        _yFixed->setSizePolicy(sizePolicy);

        horizontalLayout_4->addWidget(_yFixed);

        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");
        label_5->setEnabled(false);
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(label_5);

        _yMin = new QLineEdit(groupBox_2);
        _yMin->setObjectName("_yMin");
        _yMin->setEnabled(false);

        horizontalLayout_4->addWidget(_yMin);

        label_6 = new QLabel(groupBox_2);
        label_6->setObjectName("label_6");
        label_6->setEnabled(false);
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);

        horizontalLayout_4->addWidget(label_6);

        _yMax = new QLineEdit(groupBox_2);
        _yMax->setObjectName("_yMax");
        _yMax->setEnabled(false);

        horizontalLayout_4->addWidget(_yMax);


        gridLayout->addLayout(horizontalLayout_4, 2, 0, 1, 3);

        _yHidden = new QRadioButton(groupBox_2);
        _yHidden->setObjectName("_yHidden");
        _yHidden->setMaximumSize(QSize(0, 0));

        gridLayout->addWidget(_yHidden, 3, 0, 1, 2);


        verticalLayout->addWidget(groupBox_2);

#if QT_CONFIG(shortcut)
        label->setBuddy(_xRange);
        label_2->setBuddy(_xMin);
        label_3->setBuddy(_xMax);
        label_4->setBuddy(_yRange);
        label_5->setBuddy(_yMin);
        label_6->setBuddy(_yMax);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_xAuto, _xBorder);
        QWidget::setTabOrder(_xBorder, _xSpike);
        QWidget::setTabOrder(_xSpike, _xMean);
        QWidget::setTabOrder(_xMean, _xRange);
        QWidget::setTabOrder(_xRange, _xFixed);
        QWidget::setTabOrder(_xFixed, _xMin);
        QWidget::setTabOrder(_xMin, _xMax);
        QWidget::setTabOrder(_xMax, _xHidden);
        QWidget::setTabOrder(_xHidden, _yAuto);
        QWidget::setTabOrder(_yAuto, _yBorder);
        QWidget::setTabOrder(_yBorder, _ySpike);
        QWidget::setTabOrder(_ySpike, _yMean);
        QWidget::setTabOrder(_yMean, _yRange);
        QWidget::setTabOrder(_yRange, _yFixed);
        QWidget::setTabOrder(_yFixed, _yMin);
        QWidget::setTabOrder(_yMin, _yMax);
        QWidget::setTabOrder(_yMax, _yHidden);

        retranslateUi(RangeTab);

        QMetaObject::connectSlotsByName(RangeTab);
    } // setupUi

    void retranslateUi(QWidget *RangeTab)
    {
        groupBox->setTitle(QCoreApplication::translate("RangeTab", "X Zoom Mode ", nullptr));
        _xAuto->setText(QCoreApplication::translate("RangeTab", "A&uto", nullptr));
        _xBorder->setText(QCoreApplication::translate("RangeTab", "Auto with bor&der", nullptr));
        _xSpike->setText(QCoreApplication::translate("RangeTab", "S&pike insensitive auto", nullptr));
        _xMean->setText(QCoreApplication::translate("RangeTab", "Mean c&entered, ", nullptr));
        label->setText(QCoreApplication::translate("RangeTab", "ran&ge: ", nullptr));
        _xFixed->setText(QCoreApplication::translate("RangeTab", "F&ixed, ", nullptr));
        label_2->setText(QCoreApplication::translate("RangeTab", "fro&m: ", nullptr));
        label_3->setText(QCoreApplication::translate("RangeTab", "t&o: ", nullptr));
        _xHidden->setText(QCoreApplication::translate("RangeTab", "SHOULD BE HIDDEN", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("RangeTab", "Y Zoom Mode ", nullptr));
        _yAuto->setText(QCoreApplication::translate("RangeTab", "&Auto", nullptr));
        _yBorder->setText(QCoreApplication::translate("RangeTab", "Auto with &border", nullptr));
        _ySpike->setText(QCoreApplication::translate("RangeTab", "&Spike insensitive auto", nullptr));
        _yMean->setText(QCoreApplication::translate("RangeTab", "Mean &centered, ", nullptr));
        label_4->setText(QCoreApplication::translate("RangeTab", "&range: ", nullptr));
        _yFixed->setText(QCoreApplication::translate("RangeTab", "&Fixed, ", nullptr));
        label_5->setText(QCoreApplication::translate("RangeTab", "f&rom: ", nullptr));
        label_6->setText(QCoreApplication::translate("RangeTab", "&to: ", nullptr));
        _yHidden->setText(QCoreApplication::translate("RangeTab", "SHOULD BE HIDDEN", nullptr));
        (void)RangeTab;
    } // retranslateUi

};

namespace Ui {
    class RangeTab: public Ui_RangeTab {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RANGETAB_H
