/********************************************************************************
** Form generated from reading UI file 'changefiledialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHANGEFILEDIALOG_H
#define UI_CHANGEFILEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "combobox.h"
#include "datasourceselector.h"

QT_BEGIN_NAMESPACE

class Ui_ChangeFileDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QPushButton *_allFromFile;
    Kst::ComboBox *_files;
    QGroupBox *buttonGroup1;
    QHBoxLayout *horizontalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *_changeFilePrimitiveList;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *_addAll;
    QPushButton *_add;
    QPushButton *_remove;
    QPushButton *_removeAll;
    QSpacerItem *spacerItem;
    QListWidget *_selectedFilePrimitiveList;
    QGroupBox *buttonGroup2;
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QLabel *textLabel1;
    QHBoxLayout *hboxLayout;
    Kst::DataSourceSelector *_dataFile;
    QGridLayout *gridLayout2;
    QRadioButton *_changeSelected;
    QSpacerItem *spacerItem1;
    QGridLayout *gridLayout3;
    QSpacerItem *spacerItem2;
    QCheckBox *_duplicateDependents;
    QRadioButton *_duplicateSelected;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer;
    QLabel *_fileType;
    QPushButton *_configure;
    QDialogButtonBox *_buttonBox;

    void setupUi(QDialog *ChangeFileDialog)
    {
        if (ChangeFileDialog->objectName().isEmpty())
            ChangeFileDialog->setObjectName("ChangeFileDialog");
        ChangeFileDialog->resize(649, 452);
        verticalLayout_2 = new QVBoxLayout(ChangeFileDialog);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(ChangeFileDialog);
        groupBox->setObjectName("groupBox");
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        _allFromFile = new QPushButton(groupBox);
        _allFromFile->setObjectName("_allFromFile");
        _allFromFile->setEnabled(false);

        horizontalLayout->addWidget(_allFromFile);

        _files = new Kst::ComboBox(groupBox);
        _files->setObjectName("_files");
        _files->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_files->sizePolicy().hasHeightForWidth());
        _files->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(_files);


        verticalLayout_2->addWidget(groupBox);

        buttonGroup1 = new QGroupBox(ChangeFileDialog);
        buttonGroup1->setObjectName("buttonGroup1");
        horizontalLayout_3 = new QHBoxLayout(buttonGroup1);
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setContentsMargins(11, 11, 11, 11);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        _changeFilePrimitiveList = new QListWidget(buttonGroup1);
        _changeFilePrimitiveList->setObjectName("_changeFilePrimitiveList");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_changeFilePrimitiveList->sizePolicy().hasHeightForWidth());
        _changeFilePrimitiveList->setSizePolicy(sizePolicy1);
        _changeFilePrimitiveList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout_2->addWidget(_changeFilePrimitiveList);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        _addAll = new QPushButton(buttonGroup1);
        _addAll->setObjectName("_addAll");

        verticalLayout->addWidget(_addAll);

        _add = new QPushButton(buttonGroup1);
        _add->setObjectName("_add");

        verticalLayout->addWidget(_add);

        _remove = new QPushButton(buttonGroup1);
        _remove->setObjectName("_remove");

        verticalLayout->addWidget(_remove);

        _removeAll = new QPushButton(buttonGroup1);
        _removeAll->setObjectName("_removeAll");

        verticalLayout->addWidget(_removeAll);

        spacerItem = new QSpacerItem(80, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);


        horizontalLayout_2->addLayout(verticalLayout);

        _selectedFilePrimitiveList = new QListWidget(buttonGroup1);
        _selectedFilePrimitiveList->setObjectName("_selectedFilePrimitiveList");
        sizePolicy1.setHeightForWidth(_selectedFilePrimitiveList->sizePolicy().hasHeightForWidth());
        _selectedFilePrimitiveList->setSizePolicy(sizePolicy1);
        _selectedFilePrimitiveList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        horizontalLayout_2->addWidget(_selectedFilePrimitiveList);


        horizontalLayout_3->addLayout(horizontalLayout_2);


        verticalLayout_2->addWidget(buttonGroup1);

        buttonGroup2 = new QGroupBox(ChangeFileDialog);
        buttonGroup2->setObjectName("buttonGroup2");
        gridLayout = new QGridLayout(buttonGroup2);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        gridLayout1->setObjectName("gridLayout1");
        textLabel1 = new QLabel(buttonGroup2);
        textLabel1->setObjectName("textLabel1");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy2);
        textLabel1->setWordWrap(false);

        gridLayout1->addWidget(textLabel1, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        _dataFile = new Kst::DataSourceSelector(buttonGroup2);
        _dataFile->setObjectName("_dataFile");

        hboxLayout->addWidget(_dataFile);


        gridLayout1->addLayout(hboxLayout, 0, 1, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);

        gridLayout2 = new QGridLayout();
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        gridLayout2->setObjectName("gridLayout2");
        _changeSelected = new QRadioButton(buttonGroup2);
        _changeSelected->setObjectName("_changeSelected");
        _changeSelected->setChecked(true);

        gridLayout2->addWidget(_changeSelected, 0, 0, 1, 1);

        spacerItem1 = new QSpacerItem(16, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout2->addItem(spacerItem1, 0, 2, 1, 1);


        gridLayout->addLayout(gridLayout2, 2, 0, 1, 1);

        gridLayout3 = new QGridLayout();
        gridLayout3->setSpacing(6);
        gridLayout3->setContentsMargins(0, 0, 0, 0);
        gridLayout3->setObjectName("gridLayout3");
        spacerItem2 = new QSpacerItem(16, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout3->addItem(spacerItem2, 0, 2, 1, 1);

        _duplicateDependents = new QCheckBox(buttonGroup2);
        _duplicateDependents->setObjectName("_duplicateDependents");
        _duplicateDependents->setEnabled(false);
        _duplicateDependents->setChecked(true);

        gridLayout3->addWidget(_duplicateDependents, 0, 1, 1, 1);

        _duplicateSelected = new QRadioButton(buttonGroup2);
        _duplicateSelected->setObjectName("_duplicateSelected");

        gridLayout3->addWidget(_duplicateSelected, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout3, 3, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        _fileType = new QLabel(buttonGroup2);
        _fileType->setObjectName("_fileType");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(10);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_fileType->sizePolicy().hasHeightForWidth());
        _fileType->setSizePolicy(sizePolicy3);
        _fileType->setWordWrap(false);

        horizontalLayout_4->addWidget(_fileType);

        _configure = new QPushButton(buttonGroup2);
        _configure->setObjectName("_configure");
        _configure->setEnabled(false);
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_configure->sizePolicy().hasHeightForWidth());
        _configure->setSizePolicy(sizePolicy4);

        horizontalLayout_4->addWidget(_configure);


        gridLayout->addLayout(horizontalLayout_4, 1, 0, 1, 1);


        verticalLayout_2->addWidget(buttonGroup2);

        _buttonBox = new QDialogButtonBox(ChangeFileDialog);
        _buttonBox->setObjectName("_buttonBox");
        _buttonBox->setOrientation(Qt::Horizontal);
        _buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(_buttonBox);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(_dataFile);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_allFromFile, _files);
        QWidget::setTabOrder(_files, _changeFilePrimitiveList);
        QWidget::setTabOrder(_changeFilePrimitiveList, _selectedFilePrimitiveList);
        QWidget::setTabOrder(_selectedFilePrimitiveList, _addAll);
        QWidget::setTabOrder(_addAll, _add);
        QWidget::setTabOrder(_add, _remove);
        QWidget::setTabOrder(_remove, _removeAll);
        QWidget::setTabOrder(_removeAll, _configure);
        QWidget::setTabOrder(_configure, _changeSelected);
        QWidget::setTabOrder(_changeSelected, _duplicateSelected);
        QWidget::setTabOrder(_duplicateSelected, _duplicateDependents);
        QWidget::setTabOrder(_duplicateDependents, _buttonBox);

        retranslateUi(ChangeFileDialog);

        QMetaObject::connectSlotsByName(ChangeFileDialog);
    } // setupUi

    void retranslateUi(QDialog *ChangeFileDialog)
    {
        ChangeFileDialog->setWindowTitle(QCoreApplication::translate("ChangeFileDialog", "Change Data File", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ChangeFileDialog", "Select by Source File", nullptr));
        _allFromFile->setText(QCoreApplication::translate("ChangeFileDialog", "Add All &From:", nullptr));
        buttonGroup1->setTitle(QCoreApplication::translate("ChangeFileDialog", "Vector and Matrix Selection", nullptr));
#if QT_CONFIG(whatsthis)
        _changeFilePrimitiveList->setWhatsThis(QCoreApplication::translate("ChangeFileDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        _addAll->setWhatsThis(QCoreApplication::translate("ChangeFileDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _addAll->setText(QCoreApplication::translate("ChangeFileDialog", "Add A&ll", nullptr));
#if QT_CONFIG(whatsthis)
        _add->setWhatsThis(QCoreApplication::translate("ChangeFileDialog", "Select all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _add->setText(QCoreApplication::translate("ChangeFileDialog", "&Add", nullptr));
#if QT_CONFIG(whatsthis)
        _remove->setWhatsThis(QCoreApplication::translate("ChangeFileDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _remove->setText(QCoreApplication::translate("ChangeFileDialog", "&Remove", nullptr));
#if QT_CONFIG(whatsthis)
        _removeAll->setWhatsThis(QCoreApplication::translate("ChangeFileDialog", "Unselect all vectors in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _removeAll->setText(QCoreApplication::translate("ChangeFileDialog", "R&emove All", nullptr));
#if QT_CONFIG(whatsthis)
        _selectedFilePrimitiveList->setWhatsThis(QCoreApplication::translate("ChangeFileDialog", "The source file for the vectors selected in this list will be changed.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonGroup2->setTitle(QCoreApplication::translate("ChangeFileDialog", "New Data File", nullptr));
        textLabel1->setText(QCoreApplication::translate("ChangeFileDialog", "&File:", nullptr));
        _changeSelected->setText(QCoreApplication::translate("ChangeFileDialog", "C&hange selected vectors and matrices", nullptr));
        _duplicateDependents->setText(QCoreApplication::translate("ChangeFileDialog", "D&uplicate derived objects (curves, equations, ...)", nullptr));
        _duplicateSelected->setText(QCoreApplication::translate("ChangeFileDialog", "&Duplicate selected vectors and matrices", nullptr));
        _fileType->setText(QCoreApplication::translate("ChangeFileDialog", "Unknown File Type", nullptr));
        _configure->setText(QCoreApplication::translate("ChangeFileDialog", "Con&figure...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ChangeFileDialog: public Ui_ChangeFileDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHANGEFILEDIALOG_H
