/********************************************************************************
** Form generated from reading UI file 'curveplacement.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CURVEPLACEMENT_H
#define UI_CURVEPLACEMENT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CurvePlacement
{
public:
    QGridLayout *gridLayout_2;
    QGroupBox *_curvePlacementGroup;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QCheckBox *_scaleFonts;
    QRadioButton *_newPlot;
    QCheckBox *_newTab;
    QComboBox *_plotList;
    QRadioButton *_existingPlot;
    QRadioButton *_noPlot;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QGroupBox *_layoutGroup;
    QVBoxLayout *verticalLayout;
    QRadioButton *_autoLayout;
    QHBoxLayout *hboxLayout;
    QRadioButton *_customGrid;
    QSpinBox *_gridColumns;
    QLabel *_gridColumnsLabel;
    QRadioButton *_protectLayout;

    void setupUi(QWidget *CurvePlacement)
    {
        if (CurvePlacement->objectName().isEmpty())
            CurvePlacement->setObjectName("CurvePlacement");
        CurvePlacement->resize(438, 245);
        gridLayout_2 = new QGridLayout(CurvePlacement);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        _curvePlacementGroup = new QGroupBox(CurvePlacement);
        _curvePlacementGroup->setObjectName("_curvePlacementGroup");
        verticalLayout_2 = new QVBoxLayout(_curvePlacementGroup);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        _scaleFonts = new QCheckBox(_curvePlacementGroup);
        _scaleFonts->setObjectName("_scaleFonts");
        _scaleFonts->setChecked(true);

        gridLayout->addWidget(_scaleFonts, 2, 2, 1, 1);

        _newPlot = new QRadioButton(_curvePlacementGroup);
        _newPlot->setObjectName("_newPlot");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(_newPlot->sizePolicy().hasHeightForWidth());
        _newPlot->setSizePolicy(sizePolicy);
        _newPlot->setChecked(true);

        gridLayout->addWidget(_newPlot, 2, 0, 1, 1);

        _newTab = new QCheckBox(_curvePlacementGroup);
        _newTab->setObjectName("_newTab");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_newTab->sizePolicy().hasHeightForWidth());
        _newTab->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_newTab, 2, 1, 1, 1);

        _plotList = new QComboBox(_curvePlacementGroup);
        _plotList->setObjectName("_plotList");
        _plotList->setEnabled(false);
        sizePolicy.setHeightForWidth(_plotList->sizePolicy().hasHeightForWidth());
        _plotList->setSizePolicy(sizePolicy);
        _plotList->setEditable(true);
        _plotList->setInsertPolicy(QComboBox::InsertAtTop);
        _plotList->setSizeAdjustPolicy(QComboBox::AdjustToContents);
        _plotList->setDuplicatesEnabled(false);

        gridLayout->addWidget(_plotList, 1, 1, 1, 2);

        _existingPlot = new QRadioButton(_curvePlacementGroup);
        _existingPlot->setObjectName("_existingPlot");
        sizePolicy1.setHeightForWidth(_existingPlot->sizePolicy().hasHeightForWidth());
        _existingPlot->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(_existingPlot, 1, 0, 1, 1);

        _noPlot = new QRadioButton(_curvePlacementGroup);
        _noPlot->setObjectName("_noPlot");
        sizePolicy.setHeightForWidth(_noPlot->sizePolicy().hasHeightForWidth());
        _noPlot->setSizePolicy(sizePolicy);

        gridLayout->addWidget(_noPlot, 0, 0, 1, 3);


        verticalLayout_2->addLayout(gridLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        _layoutGroup = new QGroupBox(_curvePlacementGroup);
        _layoutGroup->setObjectName("_layoutGroup");
        sizePolicy.setHeightForWidth(_layoutGroup->sizePolicy().hasHeightForWidth());
        _layoutGroup->setSizePolicy(sizePolicy);
        _layoutGroup->setFlat(true);
        verticalLayout = new QVBoxLayout(_layoutGroup);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        _autoLayout = new QRadioButton(_layoutGroup);
        _autoLayout->setObjectName("_autoLayout");
        sizePolicy.setHeightForWidth(_autoLayout->sizePolicy().hasHeightForWidth());
        _autoLayout->setSizePolicy(sizePolicy);
        _autoLayout->setChecked(true);

        verticalLayout->addWidget(_autoLayout);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        _customGrid = new QRadioButton(_layoutGroup);
        _customGrid->setObjectName("_customGrid");
        sizePolicy1.setHeightForWidth(_customGrid->sizePolicy().hasHeightForWidth());
        _customGrid->setSizePolicy(sizePolicy1);

        hboxLayout->addWidget(_customGrid);

        _gridColumns = new QSpinBox(_layoutGroup);
        _gridColumns->setObjectName("_gridColumns");
        _gridColumns->setEnabled(false);
        _gridColumns->setMinimum(1);
        _gridColumns->setMaximum(10);

        hboxLayout->addWidget(_gridColumns);

        _gridColumnsLabel = new QLabel(_layoutGroup);
        _gridColumnsLabel->setObjectName("_gridColumnsLabel");
        _gridColumnsLabel->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(_gridColumnsLabel->sizePolicy().hasHeightForWidth());
        _gridColumnsLabel->setSizePolicy(sizePolicy2);

        hboxLayout->addWidget(_gridColumnsLabel);


        verticalLayout->addLayout(hboxLayout);

        _protectLayout = new QRadioButton(_layoutGroup);
        _protectLayout->setObjectName("_protectLayout");

        verticalLayout->addWidget(_protectLayout);


        horizontalLayout_2->addWidget(_layoutGroup);


        verticalLayout_2->addLayout(horizontalLayout_2);


        gridLayout_2->addWidget(_curvePlacementGroup, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        _gridColumnsLabel->setBuddy(_gridColumns);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(_noPlot, _existingPlot);
        QWidget::setTabOrder(_existingPlot, _newPlot);
        QWidget::setTabOrder(_newPlot, _newTab);
        QWidget::setTabOrder(_newTab, _scaleFonts);
        QWidget::setTabOrder(_scaleFonts, _autoLayout);
        QWidget::setTabOrder(_autoLayout, _customGrid);
        QWidget::setTabOrder(_customGrid, _gridColumns);
        QWidget::setTabOrder(_gridColumns, _protectLayout);

        retranslateUi(CurvePlacement);

        QMetaObject::connectSlotsByName(CurvePlacement);
    } // setupUi

    void retranslateUi(QWidget *CurvePlacement)
    {
        _curvePlacementGroup->setTitle(QCoreApplication::translate("CurvePlacement", "Placement", nullptr));
        _scaleFonts->setText(QCoreApplication::translate("CurvePlacement", "Scale fonts", nullptr));
        _newPlot->setText(QCoreApplication::translate("CurvePlacement", "Place in &new plot", nullptr));
        _newTab->setText(QCoreApplication::translate("CurvePlacement", "In ne&w tab", nullptr));
#if QT_CONFIG(whatsthis)
        _plotList->setWhatsThis(QCoreApplication::translate("CurvePlacement", "The name of the data file which will be read for this vector.", nullptr));
#endif // QT_CONFIG(whatsthis)
        _existingPlot->setText(QCoreApplication::translate("CurvePlacement", "Place in &existing plot ", nullptr));
        _noPlot->setText(QCoreApplication::translate("CurvePlacement", "Do not pl&ace in any plot", nullptr));
        _layoutGroup->setTitle(QString());
        _autoLayout->setText(QCoreApplication::translate("CurvePlacement", "Automati&c layout", nullptr));
        _customGrid->setText(QCoreApplication::translate("CurvePlacement", "Custom &grid", nullptr));
        _gridColumnsLabel->setText(QCoreApplication::translate("CurvePlacement", "Colu&mns", nullptr));
        _protectLayout->setText(QCoreApplication::translate("CurvePlacement", "Protect e&xisting layout", nullptr));
        (void)CurvePlacement;
    } // retranslateUi

};

namespace Ui {
    class CurvePlacement: public Ui_CurvePlacement {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CURVEPLACEMENT_H
