/********************************************************************************
** Form generated from reading UI file 'autocorrelationconfig.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_AUTOCORRELATIONCONFIG_H
#define UI_AUTOCORRELATIONCONFIG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "vectorselector.h"

QT_BEGIN_NAMESPACE

class Ui_AutoCorrelationConfig
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_3;
    Kst::VectorSelector *_vector;

    void setupUi(QWidget *AutoCorrelationConfig)
    {
        if (AutoCorrelationConfig->objectName().isEmpty())
            AutoCorrelationConfig->setObjectName("AutoCorrelationConfig");
        AutoCorrelationConfig->resize(429, 49);
        AutoCorrelationConfig->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(AutoCorrelationConfig);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_3 = new QLabel(AutoCorrelationConfig);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setWordWrap(false);

        horizontalLayout->addWidget(label_3);

        _vector = new Kst::VectorSelector(AutoCorrelationConfig);
        _vector->setObjectName("_vector");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_vector->sizePolicy().hasHeightForWidth());
        _vector->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(_vector);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(AutoCorrelationConfig);

        QMetaObject::connectSlotsByName(AutoCorrelationConfig);
    } // setupUi

    void retranslateUi(QWidget *AutoCorrelationConfig)
    {
        label_3->setText(QCoreApplication::translate("AutoCorrelationConfig", "Input Vector", nullptr));
        (void)AutoCorrelationConfig;
    } // retranslateUi

};

namespace Ui {
    class AutoCorrelationConfig: public Ui_AutoCorrelationConfig {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_AUTOCORRELATIONCONFIG_H
